@@ -24,7 +24,7 @@ else
     if ! id -Gn debusine-worker | grep -qFw incus-admin; then
         echo "Adding debusine-worker to incus-admin..."
         usermod --append --groups incus-admin debusine-worker
-        systemctl restart debusine-worker.service
+        systemctl restart debusine-worker.service || true
     fi
 fi
 
@@ -49,6 +49,12 @@ if ! incus profile list -f csv | grep -q ^debusine,; then
     incus profile show debusine
 fi
 
+if incus network list -f csv | grep -q ^incusbr0,; then
+    echo "Removing the incusbr0 network..."
+    incus profile device set default eth0 network=debusinebr0
+    incus network delete incusbr0
+fi
+
 if ! incus storage list -f csv | grep -q .; then
     echo "Creating default storage pool..."
     filesystem=$(findmnt --raw --output FSTYPE --noheadings --first-only --target /var/lib/incus/storage-pools/)
