@@ -12,7 +12,7 @@
 import abc
 import enum
 import urllib.parse
-from typing import Any, Literal, cast
+from typing import Any, Literal, TypeAlias, cast
 
 try:
     import pydantic.v1 as pydantic
@@ -56,7 +56,7 @@ class BaseArtifactDataModel(pydantic.BaseModel):
         """Set up stricter pydantic Config."""
 
         validate_assignment = True
-        extra = pydantic.Extra.forbid
+        extra = "forbid"
 
     def dict(self, **kwargs: Any) -> dict[str, Any]:
         """Use aliases by default when serializing."""
@@ -87,7 +87,7 @@ class DebusinePromise(BaseArtifactDataModel):
     class Config(BaseArtifactDataModel.Config):
         """Allow extra fields."""
 
-        extra = pydantic.Extra.allow
+        extra = "allow"
 
     @pydantic.root_validator(pre=True)
     @classmethod
@@ -292,7 +292,6 @@ class CollectionCategory(enum.StrEnum):
 
     ENVIRONMENTS = "debian:environments"
     SUITE = "debian:suite"
-    SUITE_LINTIAN = "debian:suite-lintian"
     PACKAGE_BUILD_LOGS = "debian:package-build-logs"
     WORKFLOW_INTERNAL = "debusine:workflow-internal"
     TASK_CONFIGURATION = "debusine:task-configuration"
@@ -831,10 +830,67 @@ def get_source_package_name(
     raise TypeError(f"Unexpected type: {type(artifact_data).__name__}")
 
 
-def get_binary_package_name(artifact_data: DebianBinaryPackage) -> str | None:
+def get_source_package_version(
+    artifact_data: (
+        DebianSourcePackage
+        | DebianBinaryPackage
+        | DebianBinaryPackages
+        | DebianUpload
+        | DebianPackageBuildLog
+    ),
+) -> str:
+    """Retrieve the source version of a package."""
+    match artifact_data:
+        case DebianSourcePackage():
+            return artifact_data.version
+        case DebianBinaryPackage() | DebianBinaryPackages():
+            return artifact_data.srcpkg_version
+        case DebianUpload():
+            return cast(str, artifact_data.changes_fields["Version"])
+        case DebianPackageBuildLog():
+            return artifact_data.version
+
+    raise TypeError(f"Unexpected type: {type(artifact_data).__name__}")
+
+
+def get_binary_package_name(artifact_data: DebianBinaryPackage) -> str:
     """Retrieve the binary package name."""
     match artifact_data:
         case DebianBinaryPackage():
-            return artifact_data.deb_fields.get("Package")
+            return cast(str, artifact_data.deb_fields.get("Package"))
+
+    raise TypeError(f"Unexpected type: {type(artifact_data).__name__}")
+
+
+def get_binary_package_version(
+    artifact_data: DebianUpload | DebianBinaryPackage,
+) -> str:
+    """Retrieve the binary package version."""
+    match artifact_data:
+        case DebianUpload():
+            return cast(str, artifact_data.changes_fields["Version"])
+        case DebianBinaryPackage():
+            return cast(str, artifact_data.deb_fields["Version"])
+
+    raise TypeError(f"Unexpected type: {type(artifact_data).__name__}")
+
+
+ArtifactDataWithArch: TypeAlias = (
+    DebianBinaryPackages
+    | DebianBinaryPackage
+    | DebianPackageBuildLog
+    | DebianUpload
+)
+
+
+def get_architecture(artifact_data: ArtifactDataWithArch) -> str:
+    """Retrieve the architecture of the package."""
+    match artifact_data:
+        case DebianBinaryPackages() | DebianPackageBuildLog():
+            return cast(str, artifact_data.architecture)
+        case DebianBinaryPackage():
+            return cast(str, artifact_data.deb_fields.get("Architecture"))
+        case DebianUpload():
+            return cast(str, artifact_data.changes_fields.get("Architecture"))
 
     raise TypeError(f"Unexpected type: {type(artifact_data).__name__}")
