@@ -1623,3 +1623,27 @@ class RepositoryIndexTests(TestCase):
                 data=data_models.DebianRepositoryIndex(path="Release"),
             ),
         )
+
+    def test_create_with_content_type(self) -> None:
+        """Create a RepositoryIndex with a specific Content-Type."""
+        directory = self.create_temporary_directory()
+        (
+            release := directory / "deb.debian.org_debian_dists_sid_Release"
+        ).write_text("A Release file\n")
+
+        artifact = RepositoryIndex.create(
+            file=release,
+            path="Release",
+            content_type="text/plain; charset=utf-8",
+        )
+
+        artifact.validate_model()
+        self.assertEqual(
+            artifact,
+            RepositoryIndex(
+                category=RepositoryIndex._category,
+                files={"Release": release},
+                content_types={"Release": "text/plain; charset=utf-8"},
+                data=data_models.DebianRepositoryIndex(path="Release"),
+            ),
+        )
