@@ -716,7 +716,6 @@ class GetSourcePackageNameTests(TestCase):
     """Tests for function get_source_package_name."""
 
     def test_DebianSourcePackage(self) -> None:
-        """Test artifact_data is a DebianSourcePackage."""
         artifact_data = data_models.DebianSourcePackage(
             name="hello", version="1.0", type="dpkg", dsc_fields={}
         )
@@ -725,7 +724,6 @@ class GetSourcePackageNameTests(TestCase):
         )
 
     def test_DebianUpload(self) -> None:
-        """Test artifact_data is a DebianUpload."""
         artifact_data = data_models.DebianUpload(
             type="dpkg",
             changes_fields={
@@ -739,7 +737,6 @@ class GetSourcePackageNameTests(TestCase):
         )
 
     def test_DebianBinaryPackage(self) -> None:
-        """Test artifact_data is a DebianBinaryPackage."""
         artifact_data = data_models.DebianBinaryPackage(
             srcpkg_name="hello",
             srcpkg_version="1.0-1",
@@ -755,7 +752,6 @@ class GetSourcePackageNameTests(TestCase):
         )
 
     def test_DebianBinaryPackages(self) -> None:
-        """Test artifact_data is a DebianBinaryPackages."""
         artifact_data = data_models.DebianBinaryPackages(
             srcpkg_name="hello",
             srcpkg_version="1.0-1",
@@ -768,7 +764,6 @@ class GetSourcePackageNameTests(TestCase):
         )
 
     def test_invalid_type(self) -> None:
-        """Test artifact_data is not supported."""
         artifact_data = data_models.EmptyArtifactData()
 
         with self.assertRaisesRegex(
@@ -779,11 +774,76 @@ class GetSourcePackageNameTests(TestCase):
             )
 
 
+class GetSourcePackageVersionTests(TestCase):
+    """Tests for function get_source_package_version."""
+
+    def test_DebianSourcePackage(self) -> None:
+        artifact_data = data_models.DebianSourcePackage(
+            name="hello", version="1.0", type="dpkg", dsc_fields={}
+        )
+        self.assertEqual(
+            data_models.get_source_package_version(artifact_data), "1.0"
+        )
+
+    def test_DebianUpload(self) -> None:
+        artifact_data = data_models.DebianUpload(
+            type="dpkg",
+            changes_fields={
+                "Source": "hello",
+                "Version": "1.0",
+                "Architecture": "source",
+                "Files": [{"name": "hello_1.0_source.dsc"}],
+            },
+        )
+        self.assertEqual(
+            data_models.get_source_package_version(artifact_data), "1.0"
+        )
+
+    def test_DebianBinaryPackage(self) -> None:
+        artifact_data = data_models.DebianBinaryPackage(
+            srcpkg_name="test",
+            srcpkg_version="1.0",
+            deb_fields={
+                "Package": "test-bin",
+                "Version": "1.0-1+b1",
+                "Architecture": "amd64",
+            },
+            deb_control_files=[],
+        )
+        self.assertEqual(artifact_data.get_label(), "test-bin_1.0-1+b1_amd64")
+
+        self.assertEqual(
+            data_models.get_source_package_version(artifact_data), "1.0"
+        )
+
+    def test_DebianBinaryPackages(self) -> None:
+        artifact_data = data_models.DebianBinaryPackages(
+            srcpkg_name="test",
+            srcpkg_version="1.0",
+            version="1.0-1",
+            architecture="amd64",
+            packages=[],
+        )
+
+        self.assertEqual(
+            data_models.get_source_package_version(artifact_data), "1.0"
+        )
+
+    def test_invalid_type(self) -> None:
+        artifact_data = data_models.EmptyArtifactData()
+
+        with self.assertRaisesRegex(
+            TypeError, "^Unexpected type: EmptyArtifactData$"
+        ):
+            data_models.get_source_package_version(
+                artifact_data  # type: ignore[arg-type]
+            )
+
+
 class GetBinaryPackageNameTests(TestCase):
     """Tests for function get_binary_package_name."""
 
     def test_DebianBinaryPackage(self) -> None:
-        """Test artifact_data is a DebianBinaryPackage."""
         artifact_data = data_models.DebianBinaryPackage(
             srcpkg_name="hello",
             srcpkg_version="1.0-1",
@@ -799,7 +859,6 @@ class GetBinaryPackageNameTests(TestCase):
         )
 
     def test_invalid_type(self) -> None:
-        """Test artifact_data is not supported."""
         artifact_data = data_models.EmptyArtifactData()
 
         with self.assertRaisesRegex(
@@ -808,3 +867,107 @@ class GetBinaryPackageNameTests(TestCase):
             data_models.get_binary_package_name(
                 artifact_data  # type: ignore[arg-type]
             )
+
+
+class GetBinaryPackageVersionTests(TestCase):
+    """Tests for function get_binary_package_version."""
+
+    def test_DebianUpload(self) -> None:
+        artifact_data = data_models.DebianUpload(
+            type="dpkg",
+            changes_fields={
+                "Source": "hello",
+                "Version": "1.0-1",
+                "Architecture": "amd64",
+                "Files": [{"name": "hello_1.0-1_amd64.deb"}],
+            },
+        )
+
+        self.assertEqual(
+            data_models.get_binary_package_version(artifact_data), "1.0-1"
+        )
+
+    def test_DebianBinaryPackage(self) -> None:
+        artifact_data = data_models.DebianBinaryPackage(
+            srcpkg_name="test",
+            srcpkg_version="1.0-1",
+            deb_fields={
+                "Package": "test-bin",
+                "Version": "1.0-1+b1",
+                "Architecture": "amd64",
+            },
+            deb_control_files=[],
+        )
+
+        self.assertEqual(
+            data_models.get_binary_package_version(artifact_data), "1.0-1+b1"
+        )
+
+    def test_invalid_type(self) -> None:
+        artifact_data = data_models.EmptyArtifactData()
+
+        with self.assertRaisesRegex(
+            TypeError, "^Unexpected type: EmptyArtifactData$"
+        ):
+            data_models.get_binary_package_version(
+                artifact_data  # type: ignore[arg-type]
+            )
+
+
+class GetArchitectureTests(TestCase):
+    """Tests for function get_architecture."""
+
+    def test_DebianBinaryPackages(self) -> None:
+        artifact_data = data_models.DebianBinaryPackage(
+            srcpkg_name="test",
+            srcpkg_version="1.0-1",
+            deb_fields={
+                "Package": "test-bin",
+                "Version": "1.0-1+b1",
+                "Architecture": "amd64",
+            },
+            deb_control_files=[],
+        )
+
+        self.assertEqual(data_models.get_architecture(artifact_data), "amd64")
+
+    def test_DebianPackageBuildLog(self) -> None:
+        data = data_models.DebianPackageBuildLog(
+            source="test",
+            version="1.0-1",
+            architecture="amd64",
+            filename="test_1.0-1_amd64.buildlog",
+        )
+        self.assertEqual(data_models.get_architecture(data), "amd64")
+
+    def test_DebianBinaryPackage(self) -> None:
+        data = data_models.DebianBinaryPackage(
+            srcpkg_name="test",
+            srcpkg_version="1.0-1",
+            deb_fields={
+                "Package": "test-bin",
+                "Version": "1.0-1+b1",
+                "Architecture": "amd64",
+            },
+            deb_control_files=[],
+        )
+        self.assertEqual(data_models.get_architecture(data), "amd64")
+
+    def test_DebianUpload(self) -> None:
+        data = data_models.DebianUpload(
+            type="dpkg",
+            changes_fields={
+                "Source": "hello",
+                "Architecture": "amd64",
+                "Files": [{"name": "hello_1.0_amd64.deb"}],
+            },
+        )
+        self.assertEqual(data_models.get_architecture(data), "amd64")
+
+    def test_invalid_type(self) -> None:
+        data = data_models.EmptyArtifactData()
+
+        with self.assertRaisesRegex(
+            TypeError, "^Unexpected type: EmptyArtifactData$"
+        ):
+            data_models.get_architecture(data)  # type: ignore[arg-type]
