@@ -21,6 +21,7 @@ import io
 import json
 import logging
 import os
+import os.path
 import shutil
 import subprocess
 import tempfile
@@ -195,12 +196,12 @@ class Debusine:
         )
 
     def webui_url(self, path: str) -> str:
-        """Generate a URL to the web UI given a path under the current scope."""
-        assert self.scope is not None
+        """Generate a URL to the web UI given a full path."""
         parsed = urlparse(self.base_api_url)
-        scope_path = (PurePath(parsed.path).parent / self.scope).as_posix()
-
-        parsed = parsed._replace(path=f"{scope_path}/{path.lstrip('/')}")
+        new_path = (
+            os.path.dirname(parsed.path).rstrip("/") + "/" + path.lstrip('/')
+        )
+        parsed = parsed._replace(path=new_path)
         return urlunparse(parsed)
 
     def work_request_iter(self) -> Iterable[WorkRequestResponse]:
