@@ -191,10 +191,8 @@ class TestCreateWorkflow(TestCase):
         profile["debusine_workspace"] = "base"
         profile["debusine_artifact_ids"] = {"changes": 42}
         error = DebusineError(
-            {
-                "title": "Something went wrong",
-                "detail": "Something went badly wrong",
-            }
+            title="Something went wrong",
+            detail="Something went badly wrong",
         )
 
         with (
@@ -210,5 +208,8 @@ class TestCreateWorkflow(TestCase):
             create_workflow(changes, profile, FakeInterface())
 
         self.assertIsInstance(raised.exception, HookException)
-        self.assertEqual(str(raised.exception), str(error))
+        self.assertEqual(
+            str(raised.exception),
+            f"Failed to create workflow upload-to-unstable: {error}",
+        )
         self.assertEqual(raised.exception.__cause__, error)
