@@ -62,6 +62,13 @@ class ServerObject(StrictBaseModel):
     #: URLs to related resources
     links: dict[str, str | Link] = {}
 
+    def get_link_url(self, name: str) -> str:
+        """Return the href URL for a link."""
+        res = self.links[name]
+        if isinstance(res, str):
+            return res
+        return res.href
+
 
 class PaginatedResponse(StrictBaseModel):
     """Paginated response from the API."""
