@@ -289,29 +289,26 @@ class DebusineTests(TestCase):
         )
 
     def test_webui_url(self) -> None:
-        for api_url, scope, path, expected in (
+        for api_url, path, expected in (
             (
                 "https://debusine.debian.org/api",
-                "scope",
                 "/a/b",
-                "https://debusine.debian.org/scope/a/b",
+                "https://debusine.debian.org/a/b",
             ),
             (
                 "https://debusine.debian.org/subdir/api",
-                "scope",
                 "/a/b",
-                "https://debusine.debian.org/subdir/scope/a/b",
+                "https://debusine.debian.org/subdir/a/b",
             ),
             (
                 "https://debusine.debian.org/",
-                "scope",
                 "/a/b",
-                "https://debusine.debian.org/scope/a/b",
+                "https://debusine.debian.org/a/b",
             ),
         ):
-            with self.subTest(api_url=api_url, scope=scope, path=path):
+            with self.subTest(api_url=api_url, path=path):
                 debusine = Debusine(
-                    api_url, self.token, logger=self.logger, scope=scope
+                    api_url, self.token, logger=self.logger, scope="scope"
                 )
                 self.assertEqual(debusine.webui_url(path), expected)
 
