@@ -10,7 +10,6 @@
 """Tests for client task configuration handling."""
 
 import logging
-import os
 import shutil
 import subprocess
 from collections.abc import Collection
@@ -78,7 +77,7 @@ class TaskConfigKeyTests(TestCase):
                 TaskConfigKey(TaskTypes.WORKER, "noop", "subject", "context"),
             ),
         ):
-            with self.subTest(item=item):
+            with self.subTest(item=repr(item)):
                 self.assertEqual(TaskConfigKey.from_item(item), expected)
 
 
@@ -188,11 +187,9 @@ class LocalTaskConfigurationRepositoryTestsBase(
     def list_files(self) -> list[Path]:
         """List files present in self.root."""
         res: list[Path] = []
-        for cur_root, dirs, files in os.walk(self.root):
+        for cur_root, dirs, files in self.root.walk():
             for name in files:
-                res.append(
-                    Path(os.path.join(cur_root, name)).relative_to(self.root)
-                )
+                res.append((cur_root / name).relative_to(self.root))
         return sorted(res)
 
     def assertFileContents(
