@@ -8,7 +8,7 @@
 # contained in the LICENSE file.
 
 import copy
-from collections.abc import Sequence
+from collections.abc import Collection, Sequence
 from functools import partial
 from typing import Any
 
@@ -106,18 +106,27 @@ def make_data_field_renamer(
 
 def make_work_request_task_data_field_renamer(
     task_type: TaskTypes,
-    task_name: str,
+    task_name: str | Collection[str],
     renames: list[tuple[str, str]],
 ) -> migrations.RunPython:
     """Make a migration operation to rename task_data in a WorkRequest."""
-    filters = {"task_type": task_type, "task_name": task_name}
+    filters: dict[str, Any] = {"task_type": task_type}
+    if isinstance(task_name, str):
+        filters["task_name"] = task_name
+    else:
+        filters["task_name__in"] = task_name
     return make_data_field_renamer("WorkRequest", "task_data", filters, renames)
 
 
 def make_workflow_template_task_data_field_renamer(
-    task_name: str, renames: list[tuple[str, str]]
+    task_name: str | Collection[str], renames: list[tuple[str, str]]
 ) -> migrations.RunPython:
     """Make a migration operation to rename task_data in a WorkflowTemplate."""
+    filters: dict[str, Any] = {}
+    if isinstance(task_name, str):
+        filters["task_name"] = task_name
+    else:
+        filters["task_name__in"] = task_name
     return make_data_field_renamer(
-        "WorkflowTemplate", "task_data", {"task_name": task_name}, renames
+        "WorkflowTemplate", "task_data", filters, renames
     )
