@@ -9,7 +9,7 @@
 
 """Data models for db artifacts."""
 
-from datetime import datetime, timedelta
+import datetime as dt
 from functools import partial
 from pathlib import Path
 from typing import Any, Generic, Optional, TYPE_CHECKING, TypeVar
@@ -35,9 +35,11 @@ from debusine.db.models.workspaces import Workspace
 from debusine.utils.typing_utils import copy_signature_from
 
 if TYPE_CHECKING:
+    from django.http import HttpRequest
     from django_stubs_ext.db.models import TypedModelMeta
 
     from debusine.db.models.work_requests import WorkRequest
+    from debusine.web.views.ui.artifacts import ArtifactUI, FileInArtifactUI
 else:
     TypedModelMeta = object
 
@@ -108,7 +110,7 @@ class ArtifactQuerySet(QuerySet["Artifact", A], Generic[A]):
             )
         )
 
-    def not_expired(self, at: datetime) -> "ArtifactQuerySet[Any]":
+    def not_expired(self, at: dt.datetime) -> "ArtifactQuerySet[Any]":
         """
         Return queryset with artifacts that have not expired.
 
@@ -122,7 +124,7 @@ class ArtifactQuerySet(QuerySet["Artifact", A], Generic[A]):
                 "workspace__default_expiration_delay",
             )
         ).filter(
-            Q(effective_expiration_delay=timedelta(0))
+            Q(effective_expiration_delay=dt.timedelta(0))
             | Q(
                 # https://github.com/typeddjango/django-stubs/issues/1548
                 created_at__gt=(
@@ -131,7 +133,7 @@ class ArtifactQuerySet(QuerySet["Artifact", A], Generic[A]):
             )
         )
 
-    def expired(self, at: datetime) -> "ArtifactQuerySet[Any]":
+    def expired(self, at: dt.datetime) -> "ArtifactQuerySet[Any]":
         """
         Return queryset with artifacts that have expired.
 
@@ -145,7 +147,7 @@ class ArtifactQuerySet(QuerySet["Artifact", A], Generic[A]):
                     "workspace__default_expiration_delay",
                 )
             )
-            .exclude(effective_expiration_delay=timedelta(0))
+            .exclude(effective_expiration_delay=dt.timedelta(0))
             .filter(
                 # https://github.com/typeddjango/django-stubs/issues/1548
                 created_at__lte=(
@@ -240,6 +242,7 @@ class Artifact(models.Model):
     objects = ArtifactManager.from_queryset(ArtifactQuerySet)()
 
     class Meta(TypedModelMeta):
+        base_manager_name = "objects"
         indexes = [
             # Used by debusine.server.open_metrics.DebusineCollector.  This
             # is imperfect (on debusine.debian.net at the time of writing it
@@ -323,7 +326,7 @@ class Artifact(models.Model):
             return label
         return str(self.category)
 
-    def effective_expiration_delay(self) -> timedelta:
+    def effective_expiration_delay(self) -> dt.timedelta:
         """Return expiration_delay, inherited if None."""
         expiration_delay = self.expiration_delay
         if self.expiration_delay is None:  # inherit
@@ -332,14 +335,14 @@ class Artifact(models.Model):
         return expiration_delay
 
     @property
-    def expire_at(self) -> datetime | None:
+    def expire_at(self) -> dt.datetime | None:
         """Return computed expiration date."""
         delay = self.effective_expiration_delay()
-        if delay == timedelta(0):
+        if delay == dt.timedelta(0):
             return None
         return self.created_at + delay
 
-    def expired(self, at: datetime) -> bool:
+    def expired(self, at: dt.datetime) -> bool:
         """
         Return True if this artifact has expired at a given datetime.
 
@@ -352,6 +355,12 @@ class Artifact(models.Model):
             return False
         return expire_at <= at
 
+    def ui(self, request: "HttpRequest") -> "ArtifactUI":
+        """Return a UI helper for this instance."""
+        from debusine.web.views.ui.artifacts import ArtifactUI
+
+        return ArtifactUI.get(request, self)
+
     def get_absolute_url(self) -> str:
         """Return the canonical URL to display the artifact."""
         from debusine.server.scopes import urlconf_scope
@@ -410,6 +419,12 @@ class FileInArtifact(models.Model):
             f"Path: {self.path} File: {self.file.id}"
         )
 
+    def ui(self, request: "HttpRequest") -> "FileInArtifactUI":
+        """Return a UI helper for this instance."""
+        from debusine.web.views.ui.artifacts import FileInArtifactUI
+
+        return FileInArtifactUI.get(request, self)
+
     def get_absolute_url(self) -> str:
         """Return an absolute URL to view this file."""
         from debusine.server.scopes import urlconf_scope
@@ -589,6 +604,7 @@ class ArtifactRelation(models.Model):
         return f"{self.artifact.id} {self.type} {self.target.id}"
 
     class Meta(TypedModelMeta):
+        base_manager_name = "objects"
         constraints = [
             UniqueConstraint(
                 fields=["artifact", "target", "type"],
