@@ -29,10 +29,12 @@ from debusine.db.models.permissions import (
 )
 
 if TYPE_CHECKING:
+    from django.http import HttpRequest
     from django_stubs_ext.db.models import TypedModelMeta
 
     from debusine.db.models.auth import User
     from debusine.server.file_backend.interface import FileBackendInterface
+    from debusine.web.views.ui.scopes import ScopeUI
 else:
     TypedModelMeta = object
 
@@ -115,7 +117,9 @@ class ScopeQuerySet(QuerySet["Scope", A], Generic[A]):
         """Keep only resources where the user has the given role."""
         if not user or not user.is_authenticated:
             return self.none()
-        return self.filter(role.q(user))
+        return self.filter(
+            pk__in=self.model.objects.filter(role.q(user)).distinct()
+        )
 
     @permission_filter(workers=Allow.ALWAYS, anonymous=Allow.ALWAYS)
     def can_display(self, user: PermissionUser) -> Self:  # noqa: U100
@@ -176,10 +180,19 @@ class Scope(models.Model):
         FileStore, related_name="scopes", through="db.FileStoreInScope"
     )
 
+    class Meta(TypedModelMeta):
+        base_manager_name = "objects"
+
     def __str__(self) -> str:
         """Return basic information of Scope."""
         return self.name
 
+    def ui(self, request: "HttpRequest") -> "ScopeUI":
+        """Return a UI helper for this instance."""
+        from debusine.web.views.ui.scopes import ScopeUI
+
+        return ScopeUI.get(request, self)
+
     @permission_check(
         "{user} cannot display scope {resource}",
         workers=Allow.ALWAYS,
