@@ -10,9 +10,9 @@
 """Unit tests for auth-related models."""
 
 import contextlib
+import datetime as dt
 import re
 from collections.abc import Generator
-from datetime import timedelta
 from typing import ClassVar
 from unittest import mock
 
@@ -21,6 +21,7 @@ from django.contrib.auth import get_user_model
 from django.contrib.auth.models import AnonymousUser
 from django.core.exceptions import ImproperlyConfigured
 from django.db import IntegrityError
+from django.test import RequestFactory
 from django.utils import timezone
 
 import debusine.db.models.auth as auth
@@ -92,7 +93,7 @@ class TokenTests(ChannelsHelpersMixin, TestCase):
         """get_token_or_none allows tokens with `expire_at` in the future."""
         token_hash = Token._generate_hash('some_key')
         token = Token.objects.create(
-            hash=token_hash, expire_at=timezone.now() + timedelta(days=1)
+            hash=token_hash, expire_at=timezone.now() + dt.timedelta(days=1)
         )
 
         self.assertEqual(token, Token.objects.get_token_or_none('some_key'))
@@ -101,7 +102,7 @@ class TokenTests(ChannelsHelpersMixin, TestCase):
         """get_token_or_none skips tokens with `expire_at` in the past."""
         token_hash = Token._generate_hash('some_key')
         Token.objects.create(
-            hash=token_hash, expire_at=timezone.now() - timedelta(seconds=1)
+            hash=token_hash, expire_at=timezone.now() - dt.timedelta(seconds=1)
         )
 
         self.assertIsNone(Token.objects.get_token_or_none('some_key'))
@@ -230,7 +231,7 @@ class TokenManagerTests(TestCase):
 
     def test_get_tokens_include_expired(self) -> None:
         """get_tokens only includes expired tokens if requested."""
-        self.token_john.expire_at = timezone.now() - timedelta(seconds=1)
+        self.token_john.expire_at = timezone.now() - dt.timedelta(seconds=1)
         self.token_john.save()
         self.assertQuerySetEqual(Token.objects.get_tokens(username="John"), [])
         self.assertQuerySetEqual(
@@ -245,10 +246,10 @@ class TokenManagerTests(TestCase):
             Token.objects.create(expire_at=expire_at)
             for expire_at in (
                 None,
-                now - timedelta(minutes=1),
-                now - timedelta(seconds=1),
+                now - dt.timedelta(minutes=1),
+                now - dt.timedelta(seconds=1),
                 now,
-                now + timedelta(seconds=1),
+                now + dt.timedelta(seconds=1),
             )
         ]
 
@@ -256,7 +257,7 @@ class TokenManagerTests(TestCase):
             Token.objects.expired(at=now), tokens[1:3], ordered=False
         )
         self.assertQuerySetEqual(
-            Token.objects.expired(at=now + timedelta(seconds=1)),
+            Token.objects.expired(at=now + dt.timedelta(seconds=1)),
             tokens[1:4],
             ordered=False,
         )
@@ -315,6 +316,13 @@ class UserTests(TestCase):
             denied=self.scenario.user,
         )
 
+    def test_ui(self) -> None:
+        from debusine.web.views.ui.auth import UserUI
+
+        request = RequestFactory().get("/")
+        ui = self.scenario.user.ui(request)
+        self.assertIsInstance(ui, UserUI)
+
     def test_get_absolute_url(self) -> None:
         """Test the get_absolute_url method."""
         self.assertEqual(
@@ -567,7 +575,7 @@ class GroupTests(TestCase):
     @contextlib.contextmanager
     def assertGroupAuditLog(
         self,
-    ) -> Generator[list[auth.GroupAuditLogEntry], None, None]:
+    ) -> Generator[list[auth.GroupAuditLogEntry]]:
         """
         Ensure add_audit_log is called.
 
@@ -617,6 +625,13 @@ class GroupTests(TestCase):
             "/other/-/groups/other/",
         )
 
+    def test_ui(self) -> None:
+        from debusine.web.views.ui.auth import GroupUI
+
+        request = RequestFactory().get("/")
+        ui = self.group.ui(request)
+        self.assertIsInstance(ui, GroupUI)
+
     def test_defaults(self) -> None:
         """Test default values."""
         self.assertFalse(self.group.ephemeral)
