@@ -1115,8 +1115,8 @@ class CollectionItemManagerTests(TestCase):
             dt.datetime(2025, 2, 1, tzinfo=dt.timezone.utc),
             dt.datetime(2025, 3, 1, tzinfo=dt.timezone.utc),
         ):
-            CollectionItem.objects.active().filter(
-                parent_collection=self.collection, data__codename="trixie"
+            self.collection.child_items.active().filter(
+                data__codename="trixie"
             ).update(
                 removed_by_user=self.playground.get_default_user(),
                 removed_at=timestamp,
@@ -1127,8 +1127,8 @@ class CollectionItemManagerTests(TestCase):
             item.created_at = timestamp
             item.save()
             items.append(item)
-        CollectionItem.objects.active().filter(
-            parent_collection=self.collection, data__codename="trixie"
+        self.collection.child_items.active().filter(
+            data__codename="trixie"
         ).update(
             removed_by_user=self.playground.get_default_user(),
             removed_at=dt.datetime(2025, 4, 1, tzinfo=dt.timezone.utc),
@@ -1165,9 +1165,7 @@ class CollectionItemManagerTests(TestCase):
             (dt.datetime(2025, 4, 1, tzinfo=dt.timezone.utc), [items[0]]),
         ):
             self.assertQuerySetEqual(
-                CollectionItem.objects.active_at(timestamp).filter(
-                    parent_collection=self.collection
-                ),
+                self.collection.child_items.active_at(timestamp),
                 expected_items,
                 ordered=False,
             )
