@@ -1481,7 +1481,7 @@ class WorkspaceTests(TestCase):
                 )
             )
             self.assertFalse(
-                self.scenario.workspace.can_create_experiment_workspace(
+                self.scenario.workspace.can_create_child_workspace(
                     self.scenario.user
                 )
             )
@@ -1603,19 +1603,17 @@ class WorkspaceTests(TestCase):
             workers=Allow.NEVER,
         )
 
-    def test_can_create_experiment_workspace_anonymous(self) -> None:
-        """Test the can_create_experiment_workspace with an anonymous user."""
+    def test_can_create_child_workspace_anonymous(self) -> None:
+        """Test the can_create_child_workspace with an anonymous user."""
         self.assertFalse(
-            self.scenario.workspace.can_create_experiment_workspace(
-                AnonymousUser()
-            )
+            self.scenario.workspace.can_create_child_workspace(AnonymousUser())
         )
 
-    def test_can_create_experiment_workspace(self) -> None:
-        """Test can_create_experiment_workspace."""
+    def test_can_create_child_workspace(self) -> None:
+        """Test can_create_child_workspace."""
         self.assertRolePredicate(
             self.scenario.workspace,
-            "can_create_experiment_workspace",
+            "can_create_child_workspace",
             Workspace.Roles.CONTRIBUTOR,
             workers=Allow.NEVER,
         )
@@ -1912,13 +1910,17 @@ class WorkspaceDeleteTests(TestCase):
 
         self.assertQuerySetEqual(
             operation.workflow_templates,
-            [self.scenario.template_sbuild, self.scenario.template_noop],
+            [
+                self.scenario.template_sbuild,
+                self.scenario.template_noop,
+                self.scenario.template_upload_to_sid,
+            ],
             ordered=False,
         )
-        self.assertEqual(operation.artifacts.count(), 26)
-        self.assertEqual(operation.work_requests.count(), 20)
-        self.assertEqual(operation.collection_items.count(), 13)
-        self.assertEqual(operation.collections.count(), 14)
+        self.assertEqual(operation.artifacts.count(), 54)
+        self.assertEqual(operation.work_requests.count(), 74)
+        self.assertEqual(operation.collection_items.count(), 71)
+        self.assertEqual(operation.collections.count(), 17)
         self.assertEqual(operation.file_in_artifacts.count(), 31)
         self.assertEqual(operation.artifact_relations.count(), 22)
         self.assertEqual(operation.asset_usages.count(), 1)
