@@ -36,9 +36,11 @@ except ImportError:
     import pydantic as pydantic  # type: ignore
 
 if TYPE_CHECKING:
+    from django.http import HttpRequest
     from django_stubs_ext.db.models import TypedModelMeta
 
     from debusine.db.models.workers import Worker
+    from debusine.web.views.ui.worker_pools import WorkerPoolUI
 else:
     TypedModelMeta = object
 
@@ -73,10 +75,19 @@ class WorkerPool(models.Model):
 
     objects = WorkerPoolManager()
 
+    class Meta(TypedModelMeta):
+        base_manager_name = "objects"
+
     def __str__(self) -> str:
         """Return the id and name of the WorkerPool."""
         return f"Id: {self.id} Name: {self.name}"
 
+    def ui(self, request: "HttpRequest") -> "WorkerPoolUI":
+        """Return a UI helper for this instance."""
+        from debusine.web.views.ui.worker_pools import WorkerPoolUI
+
+        return WorkerPoolUI.get(request, self)
+
     def get_absolute_url(self) -> str:
         """Return an absolute URL to view this WorkerPool."""
         return reverse("worker-pools:detail", kwargs={"name": self.name})
@@ -114,11 +125,16 @@ class WorkerPool(models.Model):
         """Return a WorkerPoolInterface instance for this pool."""
         return provider_interface(self)
 
-    def launch_workers(self, count: int) -> None:
+    def launch_workers(
+        self, count: int, override_disabled: bool = False
+    ) -> None:
         """Launch count additional worker instances."""
         from debusine.db.models.auth import Token
         from debusine.db.models.workers import Worker
 
+        if not (self.enabled or override_disabled):
+            raise ValueError("Pool is disabled, refusing to launch workers.")
+
         available = self.workers_stopped.count()
         if available < count:
             Worker.objects.create_pool_members(self, count - available)
