@@ -263,8 +263,8 @@ class WorkspaceQuerySet(QuerySet["Workspace", A], Generic[A]):
         return self.with_role(user, Workspace.Roles.CONTRIBUTOR)
 
     @permission_filter()
-    def can_create_experiment_workspace(self, user: PermissionUser) -> Self:
-        """Workspaces from which the user can create an experiment workspace."""
+    def can_create_child_workspace(self, user: PermissionUser) -> Self:
+        """Workspaces from which the user can create a child workspace."""
         return self.with_role(user, Workspace.Roles.CONTRIBUTOR)
 
     @permission_filter()
@@ -637,11 +637,9 @@ class Workspace(models.Model):
         # workspaces.
         return self.has_role(user, Workspace.Roles.CONTRIBUTOR)
 
-    @permission_check(
-        "{user} cannot create an experiment workspace from {resource}",
-    )
-    def can_create_experiment_workspace(self, user: PermissionUser) -> bool:
-        """Check if the user can create an experiment workspace from this."""
+    @permission_check("{user} cannot create a child workspace from {resource}")
+    def can_create_child_workspace(self, user: PermissionUser) -> bool:
+        """Check if the user can create a child workspace from this."""
         return self.has_role(user, Workspace.Roles.CONTRIBUTOR)
 
     @permission_check(
