@@ -36,7 +36,9 @@ from debusine.artifacts.models import (
     BareDataCategory,
     BaseArtifactDataModel,
     CollectionCategory,
+    DebianBinaryPackage,
     DebianPackageBuildLog,
+    DebianSourcePackage,
     DebusineSigningInput,
     DebusineSigningOutput,
     SigningResult,
@@ -487,6 +489,9 @@ class Playground(ArtifactPlayground):
         mark_running: bool = False,
         mark_aborted: bool = False,
         result: WorkRequest.Results | None = None,
+        internal_collection_artifacts: (
+            list[tuple[Artifact, dict[str, Any]]] | None
+        ) = None,
     ) -> WorkRequest:
         """
         Create a workflow.
@@ -494,6 +499,9 @@ class Playground(ArtifactPlayground):
         :param mark_running: if True call mark_running() method
         :param mark_aborted: if True call mark_aborted() method
         :param result: if not None call mark_completed(result)
+        :param internal_collection_artifacts: list of tuples with the artifacts
+           and arguments for the add_artifact() method. Adds the artifacts
+           with the same created_by user as the workflow
         """
         if created_by is None:
             created_by = self.get_default_user()
@@ -548,11 +556,16 @@ class Playground(ArtifactPlayground):
                     )
                 )
             work_request = WorkRequest.objects.create_workflow(
-                template=template,
-                data=task_data or {},
-                parent=parent,
-                status=WorkRequest.Statuses.BLOCKED,
+                template=template, data=task_data or {}, parent=parent
             )
+            if internal_collection_artifacts is not None:
+                assert work_request.internal_collection is not None
+
+                for artifact, add_args in internal_collection_artifacts:
+                    work_request.internal_collection.manager.add_artifact(
+                        artifact, user=created_by, **add_args
+                    )
+
             self.advance_work_request(
                 work_request,
                 mark_pending=mark_pending,
@@ -566,7 +579,6 @@ class Playground(ArtifactPlayground):
         self,
         work_request: WorkRequest,
         worker: Worker | None = None,
-        validate: bool = True,
     ) -> None:
         """
         Assign a worker to a WorkRequest.
@@ -588,14 +600,7 @@ class Playground(ArtifactPlayground):
                         " work request"
                     )
 
-        with contextlib.ExitStack() as stack:
-            if not validate:
-                stack.enter_context(
-                    mock.patch(
-                        "debusine.db.models.WorkRequest.configure_for_worker"
-                    )
-                )
-            work_request.assign_worker(worker)
+        work_request.assign_worker(worker)
 
     def advance_work_request(
         self,
@@ -614,11 +619,12 @@ class Playground(ArtifactPlayground):
         :param mark_aborted: if True call mark_aborted() method
         :param result: if not None call mark_completed(result)
         """
-        if mark_pending:
+        if mark_pending or (
+            work_request.status == WorkRequest.Statuses.BLOCKED
+            and (mark_running or result is not None)
+        ):
             work_request.mark_pending()
         if mark_running:
-            if work_request.status == WorkRequest.Statuses.BLOCKED:
-                work_request.mark_pending()
             if (
                 work_request.task_type
                 in (TaskTypes.WORKER, TaskTypes.SERVER, TaskTypes.SIGNING)
@@ -650,7 +656,6 @@ class Playground(ArtifactPlayground):
         completed_at: dt.datetime | None = None,
         validate: bool = True,
         worker: Worker | None = None,
-        status: WorkRequest.Statuses = WorkRequest.Statuses.PENDING,
         **kwargs: Any,
     ) -> WorkRequest:
         """
@@ -668,15 +673,6 @@ class Playground(ArtifactPlayground):
         :param kwargs: use them when creating the WorkRequest model
         """
         assert "task_type" not in kwargs
-        match status:
-            case WorkRequest.Statuses.ABORTED:
-                raise AssertionError(
-                    "use mark_aborted instead of status=ABORTED"
-                )
-            case WorkRequest.Statuses.COMPLETED:
-                raise AssertionError(
-                    "use mark_completed instead of status=COMPLETED"
-                )
 
         # FileStore and Workspace are usually created by the migrations
         # When using a TransactionTestCase with async methods
@@ -691,7 +687,6 @@ class Playground(ArtifactPlayground):
             "task_name": task_name,
             "workspace": workspace,
             "task_data": task_data,
-            "status": status,
         }
         if "created_by" not in kwargs:
             defaults["created_by"] = self.get_default_user()
@@ -727,7 +722,7 @@ class Playground(ArtifactPlayground):
 
         assert not (worker and assign_new_worker)
         if worker or assign_new_worker:
-            self.assign_worker(work_request, worker, validate=validate)
+            self.assign_worker(work_request, worker)
 
         self.advance_work_request(
             work_request,
@@ -745,22 +740,35 @@ class Playground(ArtifactPlayground):
         self,
         task_name: str = "servernoop",
         task_data: BaseTaskData | dict[str, Any] | None = None,
-        status: WorkRequest.Statuses = WorkRequest.Statuses.PENDING,
+        mark_pending: bool = False,
+        mark_running: bool = False,
+        mark_aborted: bool = False,
+        result: WorkRequest.Results | None = None,
     ) -> WorkRequest:
         """Return a new instance of WorkRequest, for a server task."""
-        return WorkRequest.objects.create_server(
+        work_request = WorkRequest.objects.create_server(
             task_name=task_name,
             task_data=task_data,
             created_by=self.get_default_user(),
             workspace=self.create_workspace(public=True),
-            status=status,
         )
+        self.advance_work_request(
+            work_request,
+            mark_pending=mark_pending,
+            mark_running=mark_running,
+            mark_aborted=mark_aborted,
+            result=result,
+        )
+        return work_request
 
     @context.disable_permission_checks()
     def create_internal_task(
         self,
         task_name: str = "noop",
-        status: WorkRequest.Statuses = WorkRequest.Statuses.PENDING,
+        mark_pending: bool = False,
+        mark_running: bool = False,
+        mark_aborted: bool = False,
+        result: WorkRequest.Results | None = None,
         validate: bool = True,
     ) -> WorkRequest:
         """Return a new instance of WorkRequest, for an internal task."""
@@ -769,17 +777,27 @@ class Playground(ArtifactPlayground):
                 stack.enter_context(
                     mock.patch("debusine.db.models.WorkRequest.full_clean")
                 )
-            return WorkRequest.objects.create_internal(
+            work_request = WorkRequest.objects.create_internal(
                 task_name=task_name,
                 workspace=self.create_workspace(public=True),
                 created_by=self.get_default_user(),
-                status=status,
             )
+            self.advance_work_request(
+                work_request,
+                mark_pending=mark_pending,
+                mark_running=mark_running,
+                mark_aborted=mark_aborted,
+                result=result,
+            )
+            return work_request
 
     @context.disable_permission_checks()
     def create_signing_task(
         self,
+        mark_pending: bool = False,
         mark_running: bool = False,
+        mark_aborted: bool = False,
+        result: WorkRequest.Results | None = None,
         workspace: Workspace | None = None,
         task_name: str = "noop",
         task_data: BaseTaskData | dict[str, Any] | None = None,
@@ -833,7 +851,10 @@ class Playground(ArtifactPlayground):
 
         self.advance_work_request(
             work_request,
+            mark_pending=mark_pending,
             mark_running=mark_running,
+            mark_aborted=mark_aborted,
+            result=result,
         )
 
         return work_request
@@ -844,11 +865,10 @@ class Playground(ArtifactPlayground):
         *,
         task_name: str = "noop",
         task_data: BaseTaskData | dict[str, Any] | None = None,
+        mark_pending: bool = False,
         mark_running: bool = False,
         mark_aborted: bool = False,
         result: WorkRequest.Results | None = None,
-        validate: bool = True,
-        status: WorkRequest.Statuses = WorkRequest.Statuses.PENDING,
         **kwargs: Any,
     ) -> WorkRequest:
         """
@@ -859,41 +879,21 @@ class Playground(ArtifactPlayground):
         :param kwargs: use them when creating the WorkRequest model
         """
         assert "task_type" not in kwargs
-        match status:
-            case WorkRequest.Statuses.ABORTED:
-                raise AssertionError(
-                    "use mark_aborted instead of status=ABORTED"
-                )
-            case WorkRequest.Statuses.COMPLETED:
-                raise AssertionError(
-                    "use mark_completed instead of status=COMPLETED"
-                )
 
         defaults: dict[str, Any] = {
             "task_name": task_name,
             "workspace": self.create_workspace(public=True),
             "task_data": task_data,
-            "status": status,
         }
         if "created_by" not in kwargs:
             defaults["created_by"] = self.get_default_user()
         defaults.update(kwargs)
 
-        with contextlib.ExitStack() as stack:
-            if not validate:
-                stack.enter_context(
-                    mock.patch("debusine.db.models.WorkRequest.full_clean")
-                )
-                stack.enter_context(
-                    mock.patch(
-                        "debusine.db.models.WorkRequest"
-                        ".apply_task_configuration"
-                    )
-                )
-            work_request = WorkRequest.objects.create_wait(**defaults)
+        work_request = WorkRequest.objects.create_wait(**defaults)
 
         self.advance_work_request(
             work_request,
+            mark_pending=mark_pending,
             mark_running=mark_running,
             mark_aborted=mark_aborted,
             result=result,
@@ -1249,16 +1249,15 @@ class Playground(ArtifactPlayground):
         *,
         paths: list[str] | dict[str, bytes] | None = None,
         create_files: bool = False,
+        dsc_fields: dict[str, Any] | None = None,
     ) -> Artifact:
         """Create a minimal `debian:source-package` artifact."""
+        dsc_fields = {"Package": name, "Version": version, **(dsc_fields or {})}
         artifact, _ = self.create_artifact(
             category=ArtifactCategory.SOURCE_PACKAGE,
-            data={
-                "name": name,
-                "version": version,
-                "type": "dpkg",
-                "dsc_fields": {},
-            },
+            data=DebianSourcePackage(
+                name=name, version=version, type="dpkg", dsc_fields=dsc_fields
+            ),
             paths=paths,
             create_files=create_files,
         )
@@ -1299,20 +1298,23 @@ class Playground(ArtifactPlayground):
         *,
         paths: list[str] | dict[str, bytes] | None = None,
         create_files: bool = False,
+        deb_fields: dict[str, Any] | None = None,
     ) -> Artifact:
         """Create a minimal `debian:binary-package` artifact."""
+        deb_fields = {
+            "Package": package,
+            "Version": version,
+            "Architecture": architecture,
+            **(deb_fields or {}),
+        }
         artifact, _ = self.create_artifact(
             category=ArtifactCategory.BINARY_PACKAGE,
-            data={
-                "srcpkg_name": srcpkg_name,
-                "srcpkg_version": srcpkg_version,
-                "deb_fields": {
-                    "Architecture": architecture,
-                    "Package": package,
-                    "Version": version,
-                },
-                "deb_control_files": [],
-            },
+            data=DebianBinaryPackage(
+                srcpkg_name=srcpkg_name,
+                srcpkg_version=srcpkg_version,
+                deb_fields=deb_fields,
+                deb_control_files=[],
+            ),
             paths=paths,
             create_files=create_files,
         )
@@ -1781,10 +1783,8 @@ class Playground(ArtifactPlayground):
             variables = {"backend": BackendType.UNSHARE, "variant": variant}
 
         try:
-            return CollectionItem.objects.active().get(
-                parent_collection=collection,
-                artifact=environment,
-                data__contains=variables,
+            return collection.child_items.active().get(
+                artifact=environment, data__contains=variables
             )
         except CollectionItem.DoesNotExist:
             pass
@@ -1871,8 +1871,7 @@ class Playground(ArtifactPlayground):
             workflow=workflow,
             workspace=workspace,
         )
-        if work_request.status == WorkRequest.Statuses.BLOCKED:
-            work_request.mark_pending()
+        work_request.mark_pending()
         if worker is None:
             worker = self.create_worker()
 
