@@ -806,7 +806,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
 
         self.assertEqual(wr.workspace, playground.get_default_workspace())
         self.assertEqual(wr.created_by, playground.get_default_user())
-        self.assertEqual(wr.status, WorkRequest.Statuses.PENDING)
+        self.assertEqual(wr.status, WorkRequest.Statuses.BLOCKED)
         self.assertEqual(wr.result, WorkRequest.Results.NONE)
         self.assertIsNone(wr.worker)
         self.assertEqual(wr.task_type, TaskTypes.WORKER)
