@@ -31,7 +31,10 @@ application = ProtocolTypeRouter(
     {
         "http": django_asgi_app,
         "websocket": ContextMiddlewareChannels(
-            TokenLastSeenAtMiddlewareChannels(URLRouter(websocket_urlpatterns)),
+            TokenLastSeenAtMiddlewareChannels(
+                # https://github.com/python/typeshed/pull/15080
+                URLRouter(websocket_urlpatterns)  # type: ignore[arg-type]
+            ),
         ),
     }
 )
