@@ -138,7 +138,7 @@ class TestConnectWorker(TransactionTestCase):
         self.app = make_app()
 
     @contextmanager
-    def patch_anon_nodename(self, hostname: str) -> Generator[None, None, None]:
+    def patch_anon_nodename(self, hostname: str) -> Generator[None]:
         """
         Patch Celery's hostname.
 
