@@ -37,7 +37,7 @@ from django.views.generic import RedirectView
 
 from debusine.web.views.auth import LoginView, LogoutView
 from debusine.web.views.task_status import TaskStatusView
-from debusine.web.views.views import ToastView
+from debusine.web.views.views import ThemeSelectionView, ToastView
 
 # from django.conf import settings
 
@@ -70,8 +70,18 @@ service_urlpatterns: list[URLPattern | URLResolver] = [
         include("debusine.web.urls.enroll", namespace="enroll"),
     ),
     path("toasts/", ToastView.as_view(), name="test-toasts"),
+    path("theme/", ThemeSelectionView.as_view(), name="theme-selection"),
 ]
 
+if (
+    settings.DEBUG and 'debug_toolbar' in settings.INSTALLED_APPS
+):  # pragma: no cover
+    import debug_toolbar
+
+    service_urlpatterns += [
+        path("__debug__/", include(debug_toolbar.urls)),
+    ]
+
 
 def make_urlpatterns(scope: str) -> list[Any]:
     """
