@@ -299,10 +299,8 @@ class CollectionManagerInterface(ABC):
         workflow: WorkRequest | None = None,
     ) -> None:
         """Remove items from the collection by name."""
-        for item in CollectionItem.objects.active().filter(
-            parent_collection=self.collection,
-            child_type__in=child_types,
-            name=name,
+        for item in self.collection.child_items.active().filter(
+            child_type__in=child_types, name=name
         ):
             self.remove_item(item, user=user, workflow=workflow)
 
@@ -325,10 +323,8 @@ class CollectionManagerInterface(ABC):
         """
         if m := re.match(r"^name:(.+)$", query):
             try:
-                return CollectionItem.objects.get(
-                    parent_collection=self.collection,
-                    removed_at__isnull=True,
-                    name=m.group(1),
+                return self.collection.child_items.get(
+                    removed_at__isnull=True, name=m.group(1)
                 )
             except CollectionItem.DoesNotExist:
                 return None
