@@ -169,9 +169,7 @@ class DebianArchiveManager(CollectionManagerInterface):
             "may_reuse_versions", False
         ):
             CollectionItemMatchConstraint.objects.filter(
-                collection_item_id__in=CollectionItem.objects.filter(
-                    parent_collection=item.collection
-                ),
+                collection_item_id__in=item.collection.child_items.all(),
                 constraint_name="pool-file",
             ).delete()
 
@@ -252,9 +250,8 @@ class DebianArchiveManager(CollectionManagerInterface):
         # so doing the sorting in Python should be fine in practice.
         if via_suite:
             suite_qs = (
-                CollectionItem.objects.active()
+                self.collection.child_items.active()
                 .filter(
-                    parent_collection=self.collection,
                     child_type=CollectionItem.Types.COLLECTION,
                     category=CollectionCategory.SUITE,
                 )
