@@ -12,7 +12,8 @@
 import datetime as dt
 import re
 from collections.abc import Generator
-from typing import Any
+from contextlib import contextmanager
+from typing import Any, override
 
 from debian.debian_support import Version
 from django.db import IntegrityError
@@ -80,19 +81,31 @@ class DebianSuiteManager(CollectionManagerInterface):
         }
     )
 
+    @override
+    def __init__(self, collection: Collection) -> None:
+        super().__init__(collection)
+        self.may_rewind_versions = False
+
+    def get_archive(self) -> Collection | None:
+        """Get the parent archive, if there is one."""
+        try:
+            return Collection.objects.get(
+                workspace=self.collection.workspace,
+                category=CollectionCategory.ARCHIVE,
+                name="_",
+            )
+        except Collection.DoesNotExist:
+            return None
+
     def get_signing_keys(self) -> QuerySet[Asset]:
         """Return signing keys for this suite."""
         if self.collection.data.get("signing_keys") is not None:
             fingerprints = self.collection.data["signing_keys"]
         else:
-            try:
-                archive = self.collection.parent_collections.get(
-                    category=CollectionCategory.ARCHIVE
-                )
-            except Collection.DoesNotExist:
-                return Asset.objects.none()
-            else:
+            if (archive := self.get_archive()) is not None:
                 fingerprints = archive.data.get("signing_keys", [])
+            else:
+                return Asset.objects.none()
         # TODO: This might not be quite right for snapshots; we always just
         # return the current signing keys.  However, at the moment we only
         # track the creation time of assets, but we don't track when asset
@@ -165,6 +178,32 @@ class DebianSuiteManager(CollectionManagerInterface):
                     value=hash_digest,
                 )
 
+    @contextmanager
+    def allow_rewinding_versions(self) -> Generator[None]:
+        """Temporarily allow rewinding versions in this suite."""
+        original_may_rewind_versions = self.may_rewind_versions
+        try:
+            self.may_rewind_versions = True
+            yield
+        finally:
+            self.may_rewind_versions = original_may_rewind_versions
+
+    def _check_existing_versions(
+        self,
+        new_version: Version,
+        existing_items: QuerySet[CollectionItem],
+        kind: str,
+    ) -> None:
+        if not self.may_rewind_versions and not self.collection.data.get(
+            "may_rewind_versions", False
+        ):
+            for existing_item in existing_items:
+                if new_version < Version(existing_item.data["version"]):
+                    raise ItemAdditionError(
+                        f"New {kind} version {new_version} < "
+                        f"existing version {existing_item.data["version"]}"
+                    )
+
     def do_add_artifact(
         self,
         artifact: Artifact,
@@ -205,6 +244,15 @@ class DebianSuiteManager(CollectionManagerInterface):
             }
             name = "{package}_{version}".format(**data)
             source_package_name = source_package_data.name
+            self._check_existing_versions(
+                Version(source_package_data.version),
+                self.collection.child_items.active().filter(
+                    child_type=CollectionItem.Types.ARTIFACT,
+                    category=ArtifactCategory.SOURCE_PACKAGE,
+                    data__package=source_package_data.name,
+                ),
+                "source",
+            )
         elif artifact.category == ArtifactCategory.BINARY_PACKAGE:
             binary_package_data = BinaryPackage.create_data(artifact.data)
             data = {
@@ -219,6 +267,18 @@ class DebianSuiteManager(CollectionManagerInterface):
             }
             name = "{package}_{version}_{architecture}".format(**data)
             source_package_name = binary_package_data.srcpkg_name
+            self._check_existing_versions(
+                Version(binary_package_data.deb_fields["Version"]),
+                self.collection.child_items.active().filter(
+                    child_type=CollectionItem.Types.ARTIFACT,
+                    category=ArtifactCategory.BINARY_PACKAGE,
+                    data__package=binary_package_data.deb_fields["Package"],
+                    data__architecture=binary_package_data.deb_fields[
+                        "Architecture"
+                    ],
+                ),
+                "binary",
+            )
         else:
             assert artifact.category == ArtifactCategory.REPOSITORY_INDEX
             data = {"path": variables["path"]}
@@ -295,9 +355,8 @@ class DebianSuiteManager(CollectionManagerInterface):
           possible for `source:` and `binary:` queries): return the one with
           the highest version.
         """
-        qs = CollectionItem.objects.active().filter(
-            parent_collection=self.collection,
-            child_type=CollectionItem.Types.ARTIFACT,
+        qs = self.collection.child_items.active().filter(
+            child_type=CollectionItem.Types.ARTIFACT
         )
         sort_by_version = True
 
