@@ -354,9 +354,7 @@ def _lookup_dict(
         workflow_root=workflow_root,
         expect_type=LookupChildType.COLLECTION,
     ).collection
-    objects = CollectionItem.objects.active().filter(
-        parent_collection=collection
-    )
+    objects = collection.child_items.active()
 
     # Prepare query conditions.  We don't need to check the workspace here;
     # it's good enough if the item is in a collection we can see.
