@@ -9,8 +9,8 @@
 
 """Manager for debusine:task-history collections."""
 
+import datetime as dt
 import re
-from datetime import datetime
 from typing import Any, TypedDict
 from urllib.parse import quote, unquote
 
@@ -49,7 +49,7 @@ class TaskHistoryManager(CollectionManagerInterface):
         data: dict[str, Any] | None = None,
         name: str | None = None,  # noqa: U100
         replace: bool = False,
-        created_at: datetime | None = None,
+        created_at: dt.datetime | None = None,
         replaced_by: CollectionItem | None = None,
     ) -> CollectionItem:
         """
@@ -112,8 +112,7 @@ class TaskHistoryManager(CollectionManagerInterface):
         # For each subject/context combination, keep the last success, the last
         # failure, and a configured number of most recent entries.  Clean up
         # other matching items.
-        old_items = CollectionItem.objects.filter(
-            parent_collection=self.collection,
+        old_items = self.collection.child_items.filter(
             child_type=CollectionItem.Types.BARE,
             category=category,
             data__task_type=task_type,
@@ -178,11 +177,8 @@ class TaskHistoryManager(CollectionManagerInterface):
           `last-failure:TASK_TYPE:TASK_NAME:SUBJECT:CONTEXT`.
         :raise LookupError: the query contains an unknown lookup format.
         """
-        objects = CollectionItem.objects.active()
-        query_filter = Q(
-            parent_collection=self.collection,
-            child_type=CollectionItem.Types.BARE,
-        )
+        objects = self.collection.child_items.active()
+        query_filter = Q(child_type=CollectionItem.Types.BARE)
 
         if m := re.match(
             r"^(last-entry|last-success|last-failure):(.+?):(.+?):(.*?):(.*)$",
