@@ -327,10 +327,7 @@ class DebianQAResultsManagerTests(TestCase):
         )
 
         self.assertQuerySetEqual(
-            CollectionItem.objects.active()
-            .filter(parent_collection=self.collection)
-            .order_by("id"),
-            items[3:],
+            self.collection.child_items.active().order_by("id"), items[3:]
         )
 
     def test_add_artifact_no_variables(self) -> None:
@@ -680,10 +677,7 @@ class DebianQAResultsManagerTests(TestCase):
         )
 
         self.assertQuerySetEqual(
-            CollectionItem.objects.active()
-            .filter(parent_collection=self.collection)
-            .order_by("id"),
-            items[3:],
+            self.collection.child_items.active().order_by("id"), items[3:]
         )
 
     def test_remove_item_bare_data(self) -> None:
