@@ -145,6 +145,19 @@ class DebianSuiteManagerTests(TestCase):
         with self.assertRaisesRegex(ValueError, msg):
             DebianSuiteManager(collection)
 
+    def test_get_archive(self) -> None:
+        self.assertEqual(
+            self.manager.get_archive(),
+            self.workspace.collections.get(category=CollectionCategory.ARCHIVE),
+        )
+
+    def test_get_archive_none(self) -> None:
+        self.workspace.collections.filter(
+            category=CollectionCategory.ARCHIVE
+        ).delete()
+
+        self.assertIsNone(self.manager.get_archive())
+
     def test_get_signing_keys_from_suite(self) -> None:
         """Signing keys may be configured directly on the suite."""
         self.assertQuerySetEqual(self.manager.get_signing_keys(), [])
@@ -192,7 +205,10 @@ class DebianSuiteManagerTests(TestCase):
 
     def test_get_signing_keys_no_parent_archive(self) -> None:
         """``get_signing_keys`` returns None if there is no parent archive."""
-        self.assertQuerySetEqual(self.collection.parent_collections.all(), [])
+        self.workspace.collections.filter(
+            category=CollectionCategory.ARCHIVE
+        ).delete()
+
         self.assertQuerySetEqual(self.manager.get_signing_keys(), [])
 
     def test_get_signing_keys_no_asset_usage(self) -> None:
@@ -439,6 +455,71 @@ class DebianSuiteManagerTests(TestCase):
             ],
         )
 
+    def test_add_artifact_source_package_older_version(self) -> None:
+        """Suites normally forbid rewinding source packages."""
+        source_package_artifact = self.create_source_package(
+            "hello", "1.1", ["hello_1.1.dsc", "hello_1.1.tar.xz"]
+        )
+        self.manager.add_artifact(
+            source_package_artifact,
+            user=self.user,
+            variables={"component": "main", "section": "devel"},
+        )
+        source_package_artifact2 = self.create_source_package(
+            "hello", "1.0", ["hello_1.0.dsc", "hello_1.0.tar.xz"]
+        )
+        with self.assertRaisesRegex(
+            ItemAdditionError,
+            r"New source version 1\.0 < existing version 1\.1",
+        ):
+            self.manager.add_artifact(
+                source_package_artifact2,
+                user=self.user,
+                variables={"component": "main", "section": "devel"},
+                replace=True,
+            )
+
+        # This can be overridden temporarily.
+        self.assertFalse(self.manager.may_rewind_versions)
+        with self.manager.allow_rewinding_versions():
+            self.assertTrue(self.manager.may_rewind_versions)
+            self.manager.add_artifact(
+                source_package_artifact2,
+                user=self.user,
+                variables={"component": "main", "section": "devel"},
+                replace=True,
+            )
+        self.assertFalse(self.manager.may_rewind_versions)
+
+    def test_add_artifact_source_package_may_rewind_versions(self) -> None:
+        """Suites can be configured to allow rewinding source packages."""
+        self.collection.data["may_rewind_versions"] = True
+        source_package_artifact = self.create_source_package(
+            "hello", "1.1", ["hello_1.1.dsc", "hello_1.1.tar.xz"]
+        )
+        collection_item = self.manager.add_artifact(
+            source_package_artifact,
+            user=self.user,
+            variables={"component": "main", "section": "devel"},
+        )
+        source_package_artifact2 = self.create_source_package(
+            "hello", "1.0", ["hello_1.0.dsc", "hello_1.0.tar.xz"]
+        )
+        collection_item2 = self.manager.add_artifact(
+            source_package_artifact2,
+            user=self.user,
+            variables={"component": "main", "section": "devel"},
+            replace=True,
+        )
+
+        collection_item.refresh_from_db()
+        self.assertEqual(collection_item.artifact, source_package_artifact)
+        self.assertIsNone(collection_item.removed_by_user)
+        self.assertIsNone(collection_item.removed_at)
+        self.assertEqual(collection_item2.artifact, source_package_artifact2)
+        self.assertIsNone(collection_item2.removed_by_user)
+        self.assertIsNone(collection_item2.removed_at)
+
     def test_add_artifact_binary_package(self) -> None:
         """`add_artifact` can add a binary package to the suite."""
         binary_package_artifact = self.create_binary_package(
@@ -717,6 +798,111 @@ class DebianSuiteManagerTests(TestCase):
             ],
         )
 
+    def test_add_artifact_binary_package_older_version(self) -> None:
+        """Suites normally forbid rewinding binary packages."""
+        binary_package_artifact = self.create_binary_package(
+            "hello",
+            "1.1",
+            "libhello1",
+            "1:1.1",
+            "amd64",
+            ["libhello1_1.1_amd64.deb"],
+        )
+        self.manager.add_artifact(
+            binary_package_artifact,
+            user=self.user,
+            variables={
+                "component": "main",
+                "section": "devel",
+                "priority": "optional",
+            },
+        )
+        binary_package_artifact2 = self.create_binary_package(
+            "hello",
+            "1.0",
+            "libhello1",
+            "1:1.0",
+            "amd64",
+            ["libhello1_1.0_amd64.deb"],
+        )
+        with self.assertRaisesRegex(
+            ItemAdditionError,
+            r"New binary version 1:1\.0 < existing version 1:1\.1",
+        ):
+            self.manager.add_artifact(
+                binary_package_artifact2,
+                user=self.user,
+                variables={
+                    "component": "main",
+                    "section": "devel",
+                    "priority": "optional",
+                },
+                replace=True,
+            )
+
+        # This can be overridden temporarily.
+        self.assertFalse(self.manager.may_rewind_versions)
+        with self.manager.allow_rewinding_versions():
+            self.assertTrue(self.manager.may_rewind_versions)
+            self.manager.add_artifact(
+                binary_package_artifact2,
+                user=self.user,
+                variables={
+                    "component": "main",
+                    "section": "devel",
+                    "priority": "optional",
+                },
+                replace=True,
+            )
+        self.assertFalse(self.manager.may_rewind_versions)
+
+    def test_add_artifact_binary_package_may_rewind_versions(self) -> None:
+        """Suites can be configured to allow rewinding binary packages."""
+        self.collection.data["may_rewind_versions"] = True
+        binary_package_artifact = self.create_binary_package(
+            "hello",
+            "1.1",
+            "libhello1",
+            "1:1.1",
+            "amd64",
+            ["libhello1_1.1_amd64.deb"],
+        )
+        collection_item = self.manager.add_artifact(
+            binary_package_artifact,
+            user=self.user,
+            variables={
+                "component": "main",
+                "section": "devel",
+                "priority": "optional",
+            },
+        )
+        binary_package_artifact2 = self.create_binary_package(
+            "hello",
+            "1.0",
+            "libhello1",
+            "1:1.0",
+            "amd64",
+            ["libhello1_1.0_amd64.deb"],
+        )
+        collection_item2 = self.manager.add_artifact(
+            binary_package_artifact2,
+            user=self.user,
+            variables={
+                "component": "main",
+                "section": "devel",
+                "priority": "optional",
+            },
+            replace=True,
+        )
+
+        collection_item.refresh_from_db()
+        self.assertEqual(collection_item.artifact, binary_package_artifact)
+        self.assertIsNone(collection_item.removed_by_user)
+        self.assertIsNone(collection_item.removed_at)
+        self.assertEqual(collection_item2.artifact, binary_package_artifact2)
+        self.assertIsNone(collection_item2.removed_by_user)
+        self.assertIsNone(collection_item2.removed_at)
+
     def test_add_artifact_repository_index(self) -> None:
         """``add_artifact`` can add a repository index to the suite."""
         packages_artifact = self.playground.create_repository_index(
