@@ -10,11 +10,13 @@
 """Implementation of in-memory file backend."""
 
 import contextlib
+import datetime as dt
 import io
 import logging
+import tempfile
 from collections import defaultdict
 from collections.abc import Callable, Generator
-from datetime import datetime
+from contextlib import contextmanager
 from pathlib import Path
 from typing import Self
 
@@ -53,6 +55,16 @@ class MemoryFileBackendEntry(
         """Return None: no local paths for in-memory files."""
         return None
 
+    @contextmanager
+    def get_temporary_local_path(self) -> Generator[Path]:
+        """Yield the path to a possibly-temporary local copy of the file."""
+        with tempfile.NamedTemporaryFile(
+            prefix="debusine-memory-"
+        ) as temp_file:
+            temp_file.write(self.backend.storage[self._identifier])
+            temp_file.flush()
+            yield Path(temp_file.name)
+
     def get_url(self, content_type: str | None = None) -> None:  # noqa: U100
         """Return None: no remote URLs for in-memory files."""
         return None
@@ -95,7 +107,7 @@ class MemoryFileBackend(FileBackendInterface[MemoryFileBackendConfiguration]):
         self.storage[hash_hex] = value
 
     @contextlib.contextmanager
-    def get_stream(self, fileobj: File) -> Generator[io.BytesIO, None, None]:
+    def get_stream(self, fileobj: File) -> Generator[io.BytesIO]:
         """Yield a file-like object that can be read."""
         hash_hex = fileobj.hash_digest.hex()
         with io.BytesIO(self.storage[hash_hex]) as out:
@@ -103,8 +115,8 @@ class MemoryFileBackend(FileBackendInterface[MemoryFileBackendConfiguration]):
 
     def list_entries(
         self,
-        mtime_filter: Callable[[datetime], bool] | None = None,  # noqa: U100
-    ) -> Generator[MemoryFileBackendEntry, None, None]:
+        mtime_filter: Callable[[dt.datetime], bool] | None = None,  # noqa: U100
+    ) -> Generator[MemoryFileBackendEntry]:
         """
         Yield entries in this backend.
 
