@@ -15,7 +15,7 @@ from django.core.management import CommandParser
 
 from debusine.db.models import WorkRequest
 from debusine.django.management.debusine_base_command import DebusineBaseCommand
-from debusine.server.management.utils import WorkRequests
+from debusine.server.management.management_utils import WorkRequests
 
 
 class Command(DebusineBaseCommand):
