@@ -10,7 +10,7 @@
 """Tests for the management command create_workflow."""
 
 import io
-from typing import ClassVar
+from typing import Any, ClassVar
 
 import yaml
 from django.core.management import CommandError
@@ -19,8 +19,10 @@ from debusine.artifacts.models import TaskTypes
 from debusine.client.models import RuntimeParameter
 from debusine.db.models import (
     Artifact,
+    User,
     WorkRequest,
     WorkflowTemplate,
+    Workspace,
     default_workspace,
     system_user,
 )
@@ -55,6 +57,26 @@ class CreateWorkflowCommandTests(TestCase):
             runtime_parameters=RuntimeParameter.ANY,
         )
 
+    def assert_workflow(
+        self,
+        workflow: WorkRequest,
+        *,
+        workspace: Workspace | None = None,
+        created_by: User | None = None,
+        runtime_parameters: dict[str, Any] | None = None,
+    ) -> None:
+        self.assertEqual(workflow.workspace, workspace or default_workspace())
+        self.assertEqual(workflow.created_by, created_by or system_user())
+        self.assertEqual(workflow.status, WorkRequest.Statuses.PENDING)
+        assert workflow.workflow_template is not None
+        self.assertEqual(
+            workflow.task_data,
+            {
+                **(runtime_parameters or {}),
+                **workflow.workflow_template.static_parameters,
+            },
+        )
+
     def test_data_from_file(self) -> None:
         """`create_workflow` accepts data from a file."""
         data = {"target_distribution": "debian:bookworm"}
@@ -72,11 +94,7 @@ class CreateWorkflowCommandTests(TestCase):
         workflow = WorkRequest.objects.get(
             task_type=TaskTypes.WORKFLOW, task_name="sbuild"
         )
-        self.assertEqual(workflow.workspace, default_workspace())
-        self.assertEqual(workflow.created_by, system_user())
-        self.assertEqual(
-            workflow.task_data, {**data, **self.template.static_parameters}
-        )
+        self.assert_workflow(workflow, runtime_parameters=data)
 
     def test_data_from_stdin(self) -> None:
         """`create_workflow` accepts data from stdin."""
@@ -92,11 +110,7 @@ class CreateWorkflowCommandTests(TestCase):
         workflow = WorkRequest.objects.get(
             task_type=TaskTypes.WORKFLOW, task_name="sbuild"
         )
-        self.assertEqual(workflow.workspace, default_workspace())
-        self.assertEqual(workflow.created_by, system_user())
-        self.assertEqual(
-            workflow.task_data, {**data, **self.template.static_parameters}
-        )
+        self.assert_workflow(workflow, runtime_parameters=data)
 
     def test_empty_data(self) -> None:
         """`create_workflow` defaults data to {}."""
@@ -114,19 +128,18 @@ class CreateWorkflowCommandTests(TestCase):
         workflow = WorkRequest.objects.get(
             task_type=TaskTypes.WORKFLOW, task_name="noop"
         )
-        self.assertEqual(workflow.task_data, {})
+        self.assert_workflow(workflow, runtime_parameters={})
 
     def test_different_created_by(self) -> None:
         """`create_workflow` can use a different created-by user."""
         user = self.playground.get_default_user()
+        data = {"target_distribution": "debian:bookworm"}
         stdout, stderr, exit_code = call_command(
             "create_workflow",
             "test",
             "--created-by",
             user.username,
-            stdin=io.StringIO(
-                yaml.safe_dump({"target_distribution": "debian:bookworm"})
-            ),
+            stdin=io.StringIO(yaml.safe_dump(data)),
         )
 
         self.assertEqual(stdout, "")
@@ -136,7 +149,7 @@ class CreateWorkflowCommandTests(TestCase):
         workflow = WorkRequest.objects.get(
             task_type=TaskTypes.WORKFLOW, task_name="sbuild"
         )
-        self.assertEqual(workflow.created_by, user)
+        self.assert_workflow(workflow, created_by=user, runtime_parameters=data)
 
     def test_different_workspace(self) -> None:
         """`create_workflow` can use a non-default workspace."""
@@ -147,14 +160,13 @@ class CreateWorkflowCommandTests(TestCase):
         workspace.set_inheritance([self.playground.get_default_workspace()])
         self.template.workspace = workspace
         self.template.save()
+        data = {"target_distribution": "debian:bookworm"}
         stdout, stderr, exit_code = call_command(
             "create_workflow",
             "test",
             "--workspace",
             workspace_name,
-            stdin=io.StringIO(
-                yaml.safe_dump({"target_distribution": "debian:bookworm"})
-            ),
+            stdin=io.StringIO(yaml.safe_dump(data)),
         )
 
         self.assertEqual(stdout, "")
@@ -164,7 +176,9 @@ class CreateWorkflowCommandTests(TestCase):
         workflow = WorkRequest.objects.get(
             task_type=TaskTypes.WORKFLOW, task_name="sbuild"
         )
-        self.assertEqual(workflow.workspace.name, workspace_name)
+        self.assert_workflow(
+            workflow, workspace=workspace, runtime_parameters=data
+        )
 
     def test_invalid_data_yaml(self) -> None:
         """`create_workflow` returns error: cannot parse data."""
