@@ -9,11 +9,11 @@
 
 """Tests for the management command vacuum_storage."""
 
+import datetime as dt
 import os
 import tempfile
 from collections.abc import Generator
 from contextlib import contextmanager
-from datetime import timedelta
 from io import BytesIO, StringIO
 from pathlib import Path
 from typing import Any, TypedDict
@@ -62,7 +62,7 @@ class VacuumStorageTestMixin(BaseDjangoTestCase):
     """Shared setup and teardown for vacuum_storage tests."""
 
     playground_memory_file_store = False
-    two_days_ago = timezone.now() - timedelta(days=2)
+    two_days_ago = timezone.now() - dt.timedelta(days=2)
 
     def setUp(self) -> None:
         """Set common objects and settings."""
