@@ -10,12 +10,12 @@
 
 import logging
 import re
-from collections.abc import Callable
 from typing import TYPE_CHECKING
 
 import django.http
 from django.conf import settings
 from django.utils import timezone
+from django.utils.deprecation import MiddlewareMixin
 
 from debusine.server.scopes import get_scope_urlconf
 
@@ -25,7 +25,7 @@ if TYPE_CHECKING:
 scope_prefix_re = re.compile(r'^/([^/]+)(/|$)')
 
 
-class ScopeMiddleware:
+class ScopeMiddleware(MiddlewareMixin):
     """
     Extract the current scope from the URL prefix.
 
@@ -34,19 +34,8 @@ class ScopeMiddleware:
     resolution.
     """
 
-    def __init__(
-        self,
-        get_response: Callable[
-            [django.http.HttpRequest], django.http.HttpResponse
-        ],
-    ) -> None:
-        """Middleware API entry point."""
-        self.get_response = get_response
-
-    def __call__(
-        self, request: django.http.HttpRequest
-    ) -> django.http.HttpResponse:
-        """Middleware entry point."""
+    def process_request(self, request: django.http.HttpRequest) -> None:
+        """Process request in middleware."""
         from debusine.db.context import context
         from debusine.db.models.scopes import is_valid_scope_name
 
@@ -69,8 +58,6 @@ class ScopeMiddleware:
         if getattr(request, "urlconf", None) is None:
             setattr(request, "urlconf", get_scope_urlconf(scope_name))
 
-        return self.get_response(request)
-
     def get_scope(self, name: str) -> "Scope":
         """Set the current scope to the given named one."""
         from django.shortcuts import get_object_or_404
@@ -80,7 +67,7 @@ class ScopeMiddleware:
         return get_object_or_404(Scope, name=name)
 
 
-class AuthorizationMiddleware:
+class AuthorizationMiddleware(MiddlewareMixin):
     """
     Check user access to the current scope.
 
@@ -91,15 +78,6 @@ class AuthorizationMiddleware:
     to validate the access of the worker token.
     """
 
-    def __init__(
-        self,
-        get_response: Callable[
-            [django.http.HttpRequest], django.http.HttpResponse
-        ],
-    ) -> None:
-        """Middleware API entry point."""
-        self.get_response = get_response
-
     def _validate_and_cache_token(
         self, request: django.http.HttpRequest
     ) -> None:
@@ -122,10 +100,10 @@ class AuthorizationMiddleware:
 
         setattr(request, "_debusine_token", token)
 
-    def __call__(
+    def process_request(
         self, request: django.http.HttpRequest
-    ) -> django.http.HttpResponse:
-        """Middleware entry point."""
+    ) -> django.http.HttpResponseForbidden | None:
+        """Process request in middleware."""
         from debusine.db.context import ContextConsistencyError, context
 
         self._validate_and_cache_token(request)
@@ -147,7 +125,7 @@ class AuthorizationMiddleware:
                     )
                 # Leave the user unset when using a worker token.
                 # TODO: see #523
-                return self.get_response(request)
+                return None
             elif user := token.user:
                 # If it's a user token, we may set it in context
                 if request.user.is_authenticated:
@@ -169,7 +147,7 @@ class AuthorizationMiddleware:
                     return django.http.HttpResponseForbidden(str(e))
 
                 # Quit processing, as we just checked request.user
-                return self.get_response(request)
+                return None
             else:
                 # Temporarily log only, later move to fail
                 logging.warning(
@@ -185,4 +163,4 @@ class AuthorizationMiddleware:
         except ContextConsistencyError as e:
             return django.http.HttpResponseForbidden(str(e))
 
-        return self.get_response(request)
+        return None
