@@ -14,7 +14,8 @@ import string
 from typing import Any, TYPE_CHECKING
 
 from asgiref.sync import async_to_sync
-from channels.layers import DEFAULT_CHANNEL_LAYER, channel_layers
+from channels import DEFAULT_CHANNEL_LAYER
+from channels.layers import channel_layers
 from channels_redis.core import RedisChannelLayer
 from django.core.mail import EmailMessage
 
