@@ -18,6 +18,7 @@ from functools import cached_property
 
 from celery import shared_task
 from django.contrib.postgres.functions import TransactionNow
+from django.db import transaction
 from django.db.models import F, IntegerField, Sum
 from django.db.models.functions import Cast, Extract, Greatest, Least
 from django.utils import timezone
@@ -350,13 +351,13 @@ class ScopeWorkerAllocator:
 
         # Add scope pools by decreasing priority
         pools.extend(
-            WorkerPool.objects.filter(
-                scopeworkerpool__scope=self.scope
-            ).order_by("-scopeworkerpool__priority")
+            WorkerPool.objects.enabled()
+            .filter(scopeworkerpool__scope=self.scope)
+            .order_by("-scopeworkerpool__priority")
         )
 
         # Add instance-wide pools at the end
-        pools.extend(WorkerPool.objects.filter(instance_wide=True))
+        pools.extend(WorkerPool.objects.enabled().filter(instance_wide=True))
 
         # TODO: enforce that ScopeWorkerPool cannot link to instance-wide
         # pools? Otherwise we may need to remove duplicates here
@@ -413,9 +414,16 @@ def create_new_workers() -> None:
         allocator.process_scope(scope)
 
 
+def scrub_workers() -> None:
+    """Scrub all WorkerPools."""
+    for pool in WorkerPool.objects.all():
+        pool.provider_interface.terminate_missing()
+
+
 # mypy complains that celery.shared_task is untyped, which is true, but we
 # can't fix that here.
 @shared_task()  # type: ignore[misc]
+@transaction.atomic
 def provision() -> None:
     """Provision or terminate workers from pools to handle pending tasks."""
     # NOTE: This is a minimum viable implementation of cloud compute
@@ -437,3 +445,17 @@ def provision() -> None:
         return
 
     create_new_workers()
+
+
+# mypy complains that celery.shared_task is untyped, which is true, but we
+# can't fix that here.
+@shared_task()  # type: ignore[misc]
+@transaction.atomic
+def scrub() -> None:
+    """
+    Scan the worker pool for failed workers and clean them up.
+
+    We expect some number of workers to die, especially if we're using cheap
+    "spot-priced" instances. Handle their death.
+    """
+    scrub_workers()
