@@ -216,7 +216,7 @@ class SignonTests(SignonTestCase):
             }
         )
         with self.assertLogs("debusine.server.signon") as log:
-            self.request.signon.activate_identity(identity)
+            self.assertTrue(self.request.signon.activate_identity(identity))
 
         self.assertEqual(
             self.request.signon.identities,
@@ -260,7 +260,7 @@ class SignonTests(SignonTestCase):
         user = get_user_model().objects.create_user("test")
         identity = self.make_identity(user=user)
         with self.assertNoLogs("debusine.server.signon"):
-            self.request.signon.activate_identity(identity)
+            self.assertTrue(self.request.signon.activate_identity(identity))
 
         self.assertEqual(
             self.request.signon.identities,
@@ -289,7 +289,7 @@ class SignonTests(SignonTestCase):
         ident2 = self.make_identity(user=user, issuer="salsa")
 
         with self.assertNoLogs("debusine.server.signon"):
-            self.request.signon.activate_identity(ident1)
+            self.assertTrue(self.request.signon.activate_identity(ident1))
 
         self.assertEqual(self.request.signon.identities, {"generic": ident1})
         self.assertEqual(self.request.user, user)
@@ -314,7 +314,7 @@ class SignonTests(SignonTestCase):
         ident1 = self.make_identity(user=user, issuer="generic")
         ident2 = self.make_identity(issuer="salsa")
         with self.assertNoLogs("debusine.server.signon"):
-            self.request.signon.activate_identity(ident1)
+            self.assertTrue(self.request.signon.activate_identity(ident1))
 
         self.assertEqual(self.request.signon.identities, {"generic": ident1})
         self.assertEqual(self.request.user, user)
@@ -345,7 +345,7 @@ class SignonTests(SignonTestCase):
         ident1 = self.make_identity(user=user1, issuer="generic")
         ident2 = self.make_identity(user=user2, issuer="salsa")
         with self.assertNoLogs("debusine.server.signon"):
-            self.request.signon.activate_identity(ident1)
+            self.assertTrue(self.request.signon.activate_identity(ident1))
 
         self.assertEqual(self.request.signon.identities, {"generic": ident1})
         self.assertEqual(self.request.user, user1)
@@ -391,7 +391,9 @@ class SignonTests(SignonTestCase):
 
         ident = self.make_identity()
         with self.assertLogs("debusine.server.signon") as log:
-            self.request.signon.activate_identity(ident, "bind")
+            self.assertTrue(
+                self.request.signon.activate_identity(ident, "bind")
+            )
 
         self.assertEqual(self.request.signon.identities, {"salsa": ident})
         self.assertEqual(self.request.user, user)
@@ -421,7 +423,9 @@ class SignonTests(SignonTestCase):
 
         ident = self.make_identity()
         with self.assertLogs("debusine.server.signon") as log:
-            self.request.signon.activate_identity(ident, "bind")
+            self.assertTrue(
+                self.request.signon.activate_identity(ident, "bind")
+            )
 
         self.assertEqual(self.request.signon.identities, {"salsa": ident})
         self.assertEqual(self.request.user, user)
@@ -448,11 +452,13 @@ class SignonTests(SignonTestCase):
         user = get_user_model().objects.create_user("test")
         ident1 = self.make_identity(user=user, issuer="generic")
         with self.assertNoLogs("debusine.server.signon"):
-            self.request.signon.activate_identity(ident1)
+            self.assertTrue(self.request.signon.activate_identity(ident1))
 
         ident2 = self.make_identity(issuer="salsa")
         with self.assertLogs("debusine.server.signon") as log:
-            self.request.signon.activate_identity(ident2, "bind")
+            self.assertTrue(
+                self.request.signon.activate_identity(ident2, "bind")
+            )
 
         self.assertEqual(
             self.request.signon.identities,
@@ -486,14 +492,16 @@ class SignonTests(SignonTestCase):
         )
         ident1 = self.make_identity(user=user1, issuer="generic")
         with self.assertNoLogs("debusine.server.signon"):
-            self.request.signon.activate_identity(ident1)
+            self.assertTrue(self.request.signon.activate_identity(ident1))
 
         user2 = get_user_model().objects.create_user(
             "test2", email="test2@example.org"
         )
         ident2 = self.make_identity(user=user2, issuer="salsa")
         with self.assertLogs("debusine.server.signon") as log:
-            self.request.signon.activate_identity(ident2, "bind")
+            self.assertTrue(
+                self.request.signon.activate_identity(ident2, "bind")
+            )
 
         self.assertEqual(
             self.request.signon.identities,
@@ -524,7 +532,7 @@ class SignonTests(SignonTestCase):
         user = get_user_model().objects.create_user("test")
         ident1 = self.make_identity(user=user)
         with self.assertNoLogs("debusine.server.signon"):
-            self.request.signon.activate_identity(ident1)
+            self.assertTrue(self.request.signon.activate_identity(ident1))
         self.assertEqual(self.request.user, user)
 
         # Log out identities
@@ -548,7 +556,9 @@ class SignonTests(SignonTestCase):
         # Also activate an identity mapped to user
         ident1 = self.make_identity(user=user)
         with self.assertLogs("debusine.server.signon"):
-            self.request.signon.activate_identity(ident1, "bind")
+            self.assertTrue(
+                self.request.signon.activate_identity(ident1, "bind")
+            )
         self.assertEqual(self.request.user, user)
 
         # Log out identities
@@ -576,7 +586,7 @@ class SignonTests(SignonTestCase):
         )
 
         with self.assertLogs("debusine.server.signon") as log:
-            self.request.signon.activate_identity(ident)
+            self.assertTrue(self.request.signon.activate_identity(ident))
 
         ident.refresh_from_db()
         self.assertTrue(self.request.user.is_authenticated)
@@ -606,7 +616,7 @@ class SignonTests(SignonTestCase):
         )
 
         with self.assertLogs("debusine.server.signon") as log:
-            self.request.signon.activate_identity(ident)
+            self.assertFalse(self.request.signon.activate_identity(ident))
 
         ident.refresh_from_db()
         self.assertFalse(self.request.user.is_authenticated)
@@ -633,7 +643,7 @@ class SignonTests(SignonTestCase):
         )
 
         with self.assertLogs("debusine.server.signon") as log:
-            self.request.signon.activate_identity(ident)
+            self.assertFalse(self.request.signon.activate_identity(ident))
 
         ident.refresh_from_db()
         self.assertFalse(self.request.user.is_authenticated)
@@ -661,7 +671,7 @@ class SignonTests(SignonTestCase):
         )
 
         with self.assertLogs("debusine.server.signon") as log:
-            self.request.signon.activate_identity(ident)
+            self.assertFalse(self.request.signon.activate_identity(ident))
 
         ident.refresh_from_db()
         self.assertFalse(self.request.user.is_authenticated)
