@@ -19,23 +19,21 @@ from debusine.server.tasks.base import BaseServerTask  # isort: split
 # BaseTask.__init_subclass__.
 from debusine.server.tasks.aptmirror import APTMirror
 from debusine.server.tasks.base import ServerTaskPermissionDenied
+from debusine.server.tasks.cloud_provisioning import CloudProvisioning
 from debusine.server.tasks.copy_collection_items import CopyCollectionItems
-from debusine.server.tasks.create_experiment_workspace import (
-    CreateExperimentWorkspace,
-)
+from debusine.server.tasks.create_child_workspace import CreateChildWorkspace
+from debusine.server.tasks.generate_suite_indexes import GenerateSuiteIndexes
 from debusine.server.tasks.noop import ServerNoop
 from debusine.server.tasks.package_upload import PackageUpload
-from debusine.server.tasks.update_suite_lintian_collection import (
-    UpdateSuiteLintianCollection,
-)
 
 __all__ = [
     "APTMirror",
     "BaseServerTask",
+    "CloudProvisioning",
     "CopyCollectionItems",
-    "CreateExperimentWorkspace",
+    "CreateChildWorkspace",
+    "GenerateSuiteIndexes",
     "PackageUpload",
     "ServerNoop",
     "ServerTaskPermissionDenied",
-    "UpdateSuiteLintianCollection",
 ]
