@@ -367,13 +367,18 @@ class GenerateSuiteIndexes(
         *,
         path_in_suite: PurePath,
         path: Path,
+        content_type: str | None = None,
         newer_item: CollectionItem | None,
     ) -> Artifact:
         assert self.work_request is not None
         assert self.workspace is not None
 
         artifact = Artifact.objects.create_from_local_artifact(
-            RepositoryIndex.create(file=path, path=path_in_suite.as_posix()),
+            RepositoryIndex.create(
+                file=path,
+                path=path_in_suite.as_posix(),
+                content_type=content_type,
+            ),
             self.workspace,
             created_by_work_request=self.work_request,
         )
@@ -869,6 +874,7 @@ class GenerateSuiteIndexes(
             path_in_suite=release_path_in_suite,
             path=release_path,
             newer_item=newer_item,
+            content_type="text/plain; charset=utf-8",
         )
         for target in related:
             ArtifactRelation.objects.create(
@@ -879,8 +885,7 @@ class GenerateSuiteIndexes(
 
     def _find_indexes(self) -> QuerySet[CollectionItem]:
         """Find index items in the suite collection."""
-        return CollectionItem.objects.filter(
-            parent_collection=self.suite,
+        return self.suite.child_items.filter(
             child_type=CollectionItem.Types.ARTIFACT,
             name__regex=(
                 r"^index:([^/]*/[^/]*/(Sources|Packages)[^/]*|Release)$"
