@@ -18,6 +18,7 @@ try:
 except ImportError:
     import pydantic as pydantic  # type: ignore
 
+from debusine.db.models.workspaces import workspace_name_regex
 from debusine.tasks.models import (
     BaseDynamicTaskData,
     BaseTaskData,
@@ -114,27 +115,42 @@ class CopyCollectionItemsData(BaseTaskData):
     copies: list[CopyCollectionItemsCopies] = pydantic.Field(min_items=1)
 
 
-class CreateExperimentWorkspaceData(BaseTaskData):
-    """In-memory task data for CreateExperimentWorkspace."""
+class CreateChildWorkspaceData(BaseTaskData):
+    """In-memory task data for CreateChildWorkspace."""
 
-    experiment_name: str
+    prefix: str
+    suffix: str
     public: bool = True
     owner_group: str | None = None
     workflow_template_names: list[str] = []
     expiration_delay: int | None = 60
 
-    @pydantic.validator("experiment_name")
+    @pydantic.validator("prefix")
     @classmethod
-    def validate_experiment_name(cls, data: str) -> str:
+    def validate_prefix(cls, data: str) -> str:
         """
-        Validate the experiment name.
+        Validate the prefix.
 
-        Valid characters are the sames as workspace/group names, minus the '-'
+        Valid characters are the same as workspace/group names.
         """
         # In pydantic 1.10 this could be done with constr, but mypy currently
         # complains about their syntax
-        if not re.match(r"^[A-Za-z][A-Za-z0-9+._]*$", data):
-            raise ValueError("experiment name contains invalid characters")
+        if not re.match(workspace_name_regex, data):
+            raise ValueError("prefix contains invalid characters")
+        return data
+
+    @pydantic.validator("suffix")
+    @classmethod
+    def validate_suffix(cls, data: str) -> str:
+        """
+        Validate the suffix.
+
+        Valid characters are the same as workspace/group names.
+        """
+        # In pydantic 1.10 this could be done with constr, but mypy currently
+        # complains about their syntax
+        if not re.match(workspace_name_regex, data):
+            raise ValueError("suffix contains invalid characters")
         return data
 
 
