@@ -420,7 +420,10 @@ class APTMirrorTests(TestCase):
 
     def test_parent_archive_no_parent_archive(self) -> None:
         """`parent_archive` returns None if there is no parent archive."""
-        self.create_suite_collection("bookworm")
+        bookworm = self.create_suite_collection("bookworm")
+        bookworm.workspace.collections.filter(
+            category=CollectionCategory.ARCHIVE
+        ).delete()
         task = self.create_apt_mirror_task("bookworm")
         self.assertIsNone(task.parent_archive)
 
@@ -948,6 +951,73 @@ class APTMirrorTests(TestCase):
         self.assertEqual(trixie_item_2.data["component"], "main")
         self.assertEqual(trixie_item_2.data["section"], "devel")
 
+    def test_add_source_can_rewind(self) -> None:
+        """There may already be a newer version of this source package."""
+        temp_path = self.create_temporary_directory()
+        collection = self.create_suite_collection("bookworm")
+        collection.manager.add_artifact(
+            self.playground.create_artifact(
+                category=ArtifactCategory.SOURCE_PACKAGE,
+                data=DebianSourcePackage(
+                    name="hello", version="1.1", type="dpkg", dsc_fields={}
+                ),
+            )[0],
+            user=self.playground.get_default_user(),
+            variables={"component": "main", "section": "devel"},
+        )
+        work_request = self.playground.create_worker_task(
+            assign_contributor_role=True
+        )
+        task = self.create_apt_mirror_task("bookworm")
+        task.set_work_request(work_request)
+
+        sources_entry = self.write_sample_source_package(
+            temp_path, "hello", "1.0"
+        )
+        dsc_contents = (temp_path / "hello_1.0.dsc").read_bytes()
+        tar_contents = (temp_path / "hello_1.0.tar.xz").read_bytes()
+        work_request.set_current()
+        with self.patch_download(
+            {
+                "hello=1.0": {
+                    "hello_1.0.dsc": dsc_contents,
+                    "hello_1.0.tar.xz": tar_contents,
+                }
+            }
+        ) as mock_run:
+            task.add_source(
+                temp_path,
+                name="hello_1.0",
+                source=sources_entry,
+                component="main",
+            )
+
+        self.assert_downloaded_source(mock_run, temp_path, "hello", "1.0")
+        source_item = collection.manager.lookup("source-version:hello_1.0")
+        assert source_item is not None
+        assert source_item.artifact is not None
+        self.assert_artifact_matches(
+            source_item.artifact,
+            ArtifactCategory.SOURCE_PACKAGE,
+            work_request.workspace,
+            {
+                "name": "hello",
+                "version": "1.0",
+                "type": "dpkg",
+                "dsc_fields": {
+                    "Format": "3.0 (native)",
+                    "Source": "hello",
+                    "Binary": "hello",
+                    "Architecture": "any",
+                    "Version": "1.0",
+                },
+            },
+            {"hello_1.0.dsc": dsc_contents, "hello_1.0.tar.xz": tar_contents},
+        )
+        self.assertEqual(source_item.created_by_user, work_request.created_by)
+        self.assertEqual(source_item.data["component"], "main")
+        self.assertEqual(source_item.data["section"], "devel")
+
     def test_update_sources(self) -> None:
         """`update_sources` executes a plan to update sources."""
         temp_path = self.create_temporary_directory()
@@ -1684,6 +1754,88 @@ class APTMirrorTests(TestCase):
         self.assertEqual(trixie_item_2.data["section"], "libs")
         self.assertEqual(trixie_item_2.data["priority"], "optional")
 
+    def test_add_binary_can_rewind(self) -> None:
+        """There may already be a newer version of this binary package."""
+        temp_path = self.create_temporary_directory()
+        collection = self.create_suite_collection("bookworm")
+        collection.manager.add_artifact(
+            self.playground.create_minimal_binary_package_artifact(
+                srcpkg_name="hello",
+                srcpkg_version="1:1.0-2",
+                package="libhello1",
+                version="1.0-2",
+                architecture="amd64",
+            ),
+            user=self.playground.get_default_user(),
+            variables={
+                "component": "main",
+                "section": "libs",
+                "priority": "optional",
+            },
+        )
+        work_request = self.playground.create_worker_task(
+            assign_contributor_role=True
+        )
+        task = self.create_apt_mirror_task("bookworm")
+        task.set_work_request(work_request)
+
+        packages_entry = self.write_sample_binary_package(
+            temp_path,
+            "libhello1",
+            "1.0-1",
+            "amd64",
+            srcpkg_name="hello",
+            srcpkg_version="1:1.0-1",
+            section="libs",
+        )
+        deb_contents = (temp_path / "libhello1_1.0-1_amd64.deb").read_bytes()
+        work_request.set_current()
+        with self.patch_download(
+            {
+                "libhello1:amd64=1.0-1": {
+                    "libhello1_1.0-1_amd64.deb": deb_contents
+                }
+            }
+        ) as mock_run:
+            task.add_binary(
+                temp_path,
+                name="libhello1_1.0-1_amd64",
+                binary=packages_entry,
+                component="main",
+            )
+
+        self.assert_downloaded_binary(
+            mock_run, temp_path, "libhello1", "1.0-1", "amd64"
+        )
+        binary_item = collection.manager.lookup(
+            "binary-version:libhello1_1.0-1_amd64"
+        )
+        assert binary_item is not None
+        assert binary_item.artifact is not None
+        self.assert_artifact_matches(
+            binary_item.artifact,
+            ArtifactCategory.BINARY_PACKAGE,
+            work_request.workspace,
+            {
+                "srcpkg_name": "hello",
+                "srcpkg_version": "1:1.0-1",
+                "deb_fields": {
+                    "Package": "libhello1",
+                    "Version": "1.0-1",
+                    "Architecture": "amd64",
+                    "Maintainer": "Example Maintainer <example@example.org>",
+                    "Description": "Example description",
+                    "Source": "hello (1:1.0-1)",
+                },
+                "deb_control_files": ["control"],
+            },
+            {"libhello1_1.0-1_amd64.deb": deb_contents},
+        )
+        self.assertEqual(binary_item.created_by_user, work_request.created_by)
+        self.assertEqual(binary_item.data["component"], "main")
+        self.assertEqual(binary_item.data["section"], "libs")
+        self.assertEqual(binary_item.data["priority"], "optional")
+
     def test_update_binaries(self) -> None:
         """`update_binaries` executes a plan to update binaries."""
         temp_path = self.create_temporary_directory()
