@@ -635,10 +635,8 @@ class GenerateSuiteIndexesTests(TestCase):
             for created_at in dts:
                 # Even during test setup, we have to remove old items to
                 # avoid constraint violations.
-                CollectionItem.objects.active().filter(
-                    parent_collection=collection,
-                    child_type=CollectionItem.Types.ARTIFACT,
-                    name=name,
+                collection.child_items.active().filter(
+                    child_type=CollectionItem.Types.ARTIFACT, name=name
                 ).update(
                     removed_by_user=self.playground.get_default_user(),
                     removed_at=created_at,
@@ -765,10 +763,8 @@ class GenerateSuiteIndexesTests(TestCase):
             for created_at in dts:
                 # Even during test setup, we have to remove old items to
                 # avoid constraint violations.
-                CollectionItem.objects.active().filter(
-                    parent_collection=collection,
-                    child_type=CollectionItem.Types.ARTIFACT,
-                    name=name,
+                collection.child_items.active().filter(
+                    child_type=CollectionItem.Types.ARTIFACT, name=name
                 ).update(
                     removed_by_user=self.playground.get_default_user(),
                     removed_at=created_at,
