@@ -72,7 +72,7 @@ class NotifyWorkRequestAssigned(ChannelsHelpersMixin, TestCase):
         super().setUpTestData()
         workspace = cls.playground.create_workspace(name="Tests")
         token = cls.playground.create_user_token()
-        cls.work_request = cls.playground.create_work_request(
+        cls.work_request = cls.playground.create_worker_task(
             workspace=workspace, created_by=token.user
         )
 
@@ -127,23 +127,15 @@ class NotifyWorkRequestCompletedTests(TestCase):
         """Assert WorkRequest completed in failure or error."""
         notification_channel_name = "deblts-email"
 
-        event_reactions = EventReactions.parse_obj(
-            {
-                event_name: [
-                    {
-                        "action": "send-notification",
-                        "channel": notification_channel_name,
-                    }
-                ]
-            }
-        )
+        action = ActionSendNotification(channel=notification_channel_name)
         if notification_data is not None:
-            getattr(event_reactions, event_name)[
-                0
-            ].data = notification_data.dict()
+            action.data = notification_data
 
-        work_request: WorkRequest = self.playground.create_work_request(
-            status=WorkRequest.Statuses.RUNNING, event_reactions=event_reactions
+        event_reactions = EventReactions()
+        setattr(event_reactions, event_name, [action])
+
+        work_request = self.playground.create_worker_task(
+            mark_running=True, event_reactions=event_reactions
         )
 
         if expected_to is None:
@@ -264,10 +256,9 @@ class NotifyWorkRequestCompletedTests(TestCase):
             WorkRequest.Results.SUCCESS,
             WorkRequest.Results.FAILURE,
         ):
-            work_request: WorkRequest = self.playground.create_work_request(
-                status=WorkRequest.Statuses.RUNNING,
+            work_request = self.playground.create_worker_task(
+                mark_running=True, event_reactions=EventReactions()
             )
-            work_request.event_reactions = EventReactions()
             with mock.patch(
                 'debusine.db.models.WorkRequest'
                 '.process_update_collection_with_artifacts'
@@ -288,8 +279,8 @@ class NotifyWorkRequestCompletedTests(TestCase):
                 )
             ]
         )
-        work_request: WorkRequest = self.playground.create_work_request(
-            status=WorkRequest.Statuses.RUNNING, event_reactions=event_reactions
+        work_request = self.playground.create_worker_task(
+            mark_running=True, event_reactions=event_reactions
         )
 
         expected_log = (
@@ -304,7 +295,7 @@ class NotifyWorkRequestCompletedTests(TestCase):
 
     def test_send_work_request_completed_fail_logs(self) -> None:
         """Send email fails: log failure."""
-        work_request = self.playground.create_work_request()
+        work_request = self.playground.create_worker_task()
         notification_channel = NotificationChannel.objects.create(
             name="deblts-email",
             method=NotificationChannel.Methods.EMAIL,
