@@ -30,7 +30,7 @@ from debusine.db.models import (
 )
 from debusine.server.open_metrics import DebusineCollector, ZeroGauge
 from debusine.server.signon.providers import Provider
-from debusine.tasks.models import BackendType, WorkerType
+from debusine.tasks.models import BackendType, NoopData, WorkerType
 from debusine.test.django import TestCase
 
 
@@ -317,23 +317,15 @@ class DebusineCollectorTests(TestCase):
         self.assertMetricEquals(gauge, 0, active="0", issuer="salsa")
 
     def test_count_work_requests(self) -> None:
+        self.playground.create_worker_task(task_name="noop", mark_pending=True)
+        self.playground.create_signing_task(task_name="noop", mark_running=True)
         self.playground.create_worker_task(
             task_name="noop",
-            status=WorkRequest.Statuses.PENDING,
-        )
-        self.playground.create_signing_task(
-            task_name="noop",
-            status=WorkRequest.Statuses.RUNNING,
-        )
-        # Disable validation to allow setting an arbitrary task data
-        self.playground.create_worker_task(
-            task_name="noop",
-            task_data={
-                "build_architecture": "amd64",
-                "backend": BackendType.UNSHARE,
-            },
-            status=WorkRequest.Statuses.RUNNING,
-            validate=False,
+            task_data=NoopData(
+                build_architecture="amd64",
+                backend=BackendType.UNSHARE,
+            ),
+            mark_running=True,
         )
         gauge = self.collector().count_work_requests()
         self.assertEqual(gauge.name, "work_requests")
@@ -410,9 +402,7 @@ class DebusineCollectorTests(TestCase):
         )
         worker3.mark_connected()
         self.playground.create_worker_task(
-            task_name="noop",
-            status=WorkRequest.Statuses.RUNNING,
-            worker=worker3,
+            task_name="noop", mark_running=True, worker=worker3
         )
 
         gauge = self.collector().count_workers()
