@@ -9,16 +9,17 @@
 
 """Tests for serializers."""
 
-from datetime import datetime, timezone
+import base64
+import datetime as dt
 from typing import Any, ClassVar
+from unittest import mock
 
-from django.conf import settings
-from django.db import connection, connections, transaction
+from django.db import transaction
 from django.test import RequestFactory, override_settings
 from django.urls import reverse
 from rest_framework.exceptions import ErrorDetail, ValidationError
 
-from debusine.artifacts.models import RuntimeStatistics
+from debusine.artifacts.models import CollectionCategory, RuntimeStatistics
 from debusine.assets import AssetCategory, KeyPurpose, SigningKeyData
 from debusine.client.models import (
     LookupChildType,
@@ -27,13 +28,14 @@ from debusine.client.models import (
 from debusine.db.context import context
 from debusine.db.models import (
     Artifact,
+    Collection,
+    CollectionItem,
     File,
     FileInArtifact,
     Token,
     WorkRequest,
     Worker,
     Workspace,
-    default_workspace,
 )
 from debusine.db.playground import scenarios
 from debusine.db.tests.utils import _calculate_hash_from_data
@@ -41,6 +43,7 @@ from debusine.server.serializers import (
     ArtifactSerializer,
     ArtifactSerializerResponse,
     AssetSerializer,
+    CollectionSerializer,
     FileSerializer,
     LookupMultipleSerializer,
     LookupSingleSerializer,
@@ -63,6 +66,7 @@ class WorkRequestSerializerTests(TestCase):
     work_request: ClassVar[WorkRequest]
     work_request_serializer: ClassVar[WorkRequestSerializer]
     workspace: ClassVar[Workspace]
+    source_artifact: ClassVar[Artifact]
 
     @classmethod
     def setUpTestData(cls) -> None:
@@ -70,24 +74,31 @@ class WorkRequestSerializerTests(TestCase):
         super().setUpTestData()
 
         cls.token = cls.playground.create_user_token()
+        cls.source_artifact = cls.playground.create_source_artifact()
+        cls.playground.create_debian_environment(
+            codename="sid", architecture="amd64"
+        )
 
         worker = Worker.objects.create_with_fqdn(
             'worker.lan', token=cls.playground.create_bare_token()
         )
 
         cls.work_request = cls.playground.create_worker_task(
-            started_at=datetime(2022, 2, 20, 15, 19, 1, 158424, timezone.utc),
-            completed_at=datetime(2022, 2, 20, 16, 2, 3, 558425, timezone.utc),
-            status=WorkRequest.Statuses.COMPLETED,
-            result=WorkRequest.Results.SUCCESS,
+            started_at=dt.datetime(
+                2022, 2, 20, 15, 19, 1, 158424, dt.timezone.utc
+            ),
+            completed_at=dt.datetime(
+                2022, 2, 20, 16, 2, 3, 558425, dt.timezone.utc
+            ),
             worker=worker,
             task_name='sbuild',
             task_data=SbuildData(
-                input=SbuildInput(source_artifact=1),
-                host_architecture="x64",
+                input=SbuildInput(source_artifact=cls.source_artifact.pk),
+                build_architecture="amd64",
                 environment="debian/match:codename=sid",
             ),
             created_by=cls.token.user,
+            result=WorkRequest.Results.SUCCESS,
         )
 
         cls.work_request_serializer = WorkRequestSerializer(cls.work_request)
@@ -102,12 +113,14 @@ class WorkRequestSerializerTests(TestCase):
                 'aborted_by',
                 'artifacts',
                 'completed_at',
+                'completed_by',
                 'created_at',
                 'created_by',
                 'duration',
                 'dynamic_task_data',
                 'event_reactions',
                 'id',
+                'links',
                 'priority_adjustment',
                 'priority_base',
                 'result',
@@ -492,6 +505,21 @@ class ArtifactSerializerResponseTests(TestCase):
             cls.files_to_add, create_files=True, skip_add_files_in_store=True
         )
 
+    @classmethod
+    def get_links(cls, artifact: Artifact) -> dict[str, str | dict[str, str]]:
+        return {
+            "self": reverse(
+                "api:artifact",
+                kwargs={"artifact_id": artifact.pk},
+            ),
+            "webui_self": artifact.get_absolute_url(),
+            "download_tar_gz": {
+                "href": artifact.get_absolute_url_download()
+                + "?archive=tar.gz",
+                "type": "application/tar+gzip",
+            },
+        }
+
     @override_settings(ALLOWED_HOSTS=["*"])
     def test_from_artifact(self) -> None:
         """from_artifact() return the expected ArtifactSerializerResponse."""
@@ -541,22 +569,20 @@ class ArtifactSerializerResponseTests(TestCase):
                 "content_type": content_type,
             }
 
-        self.assertEqual(
-            serializer.data,
-            {
-                "id": self.artifact.id,
-                "url": self.artifact.get_absolute_url(),
-                "scope": self.artifact.workspace.scope.name,
-                "workspace": self.artifact.workspace.name,
-                "category": str(self.artifact.category),
-                "data": self.artifact.data,
-                "created_at": created_at,
-                "expire_at": self.artifact.expire_at,
-                "download_tar_gz_url": download_url,
-                "files_to_upload": self.files_to_add,
-                "files": files,
-            },
-        )
+        assert serializer.data == {
+            "id": self.artifact.id,
+            "url": self.artifact.get_absolute_url(),
+            "scope": self.artifact.workspace.scope.name,
+            "workspace": self.artifact.workspace.name,
+            "category": str(self.artifact.category),
+            "data": self.artifact.data,
+            "created_at": created_at,
+            "expire_at": self.artifact.expire_at,
+            "download_tar_gz_url": download_url,
+            "files_to_upload": self.files_to_add,
+            "files": files,
+            "links": self.get_links(self.artifact),
+        }
 
     @override_settings(ALLOWED_HOSTS=["*"])
     def test_build_absolute_download_url(self) -> None:
@@ -613,11 +639,7 @@ class ArtifactSerializerResponseTransactionTests(TransactionTestCase):
                 skip_add_files_in_store=True,
             )
 
-            racing_alias = "racing"
-            racing_connection = connection.copy(racing_alias)
-            connections[racing_alias] = racing_connection
-
-            try:
+            with self.separate_connection(racing_alias := "racing"):
                 with (
                     transaction.atomic(using=racing_alias),
                     context.disable_permission_checks(),
@@ -646,9 +668,6 @@ class ArtifactSerializerResponseTransactionTests(TransactionTestCase):
                 self.assertEqual(
                     serializer.data["files_to_upload"], ["common-file"]
                 )
-            finally:
-                del connections[racing_alias]
-                racing_connection.close()
 
 
 class AssetSerializerTests(TestCase):
@@ -704,7 +723,7 @@ class AssetSerializerTests(TestCase):
                 "data": SigningKeyData(
                     description="A Test Key",
                     fingerprint="ABC123",
-                    public_key="PUBLIC KEY",
+                    public_key=base64.b64encode(b"PUBLIC KEY").decode(),
                     purpose=KeyPurpose.OPENPGP_UPLOAD,
                 ).dict(),
                 "work_request": work_request.id,
@@ -726,7 +745,7 @@ class AssetSerializerTests(TestCase):
                 "data": SigningKeyData(
                     description="A Test Key",
                     fingerprint="ABC123",
-                    public_key="PUBLIC KEY",
+                    public_key=base64.b64encode(b"PUBLIC KEY").decode(),
                     purpose=KeyPurpose.OPENPGP_UPLOAD,
                 ).dict(),
             },
@@ -807,65 +826,25 @@ class WorkflowTemplateSerializerTests(TestCase):
 
     def test_valid_data(self) -> None:
         """Test valid data."""
-        context.set_scope(self.playground.get_default_scope())
         data = {
             "name": "wt",
-            "workspace": settings.DEBUSINE_DEFAULT_WORKSPACE,
             "task_name": "noop",
         }
 
         serializer = WorkflowTemplateSerializer(data=data)
 
         self.assertTrue(serializer.is_valid())
-        self.assertEqual(
-            serializer.validated_data,
-            {**data, "workspace": default_workspace()},
-        )
+        self.assertEqual(serializer.validated_data, data)
 
     def test_invalid_task_name(self) -> None:
         """The serializer rejects an invalid task name."""
-        context.set_scope(self.playground.get_default_scope())
         data = {
             "name": "wt",
-            "workspace": settings.DEBUSINE_DEFAULT_WORKSPACE,
             "task_name": "nonexistent",
         }
-
         serializer = WorkflowTemplateSerializer(data=data)
-
         self.assertFalse(serializer.is_valid())
 
-    def test_workspace_name_resolved_in_current_scope(self) -> None:
-        """The given workspace name is resolved in the current scope."""
-        scope1 = self.playground.get_or_create_scope("scope1")
-        scope2 = self.playground.get_or_create_scope("scope2")
-        scope3 = self.playground.get_or_create_scope("scope3")
-        workspace1 = self.playground.create_workspace(
-            scope=scope1, name="common-name", public=True
-        )
-        workspace2 = self.playground.create_workspace(
-            scope=scope2, name="common-name", public=True
-        )
-        data = {
-            "name": "wt",
-            "workspace": "common-name",
-            "task_name": "noop",
-        }
-
-        for workspace in (workspace1, workspace2):
-            with context.local():
-                context.set_scope(workspace.scope)
-                serializer = WorkflowTemplateSerializer(data=data)
-                self.assertTrue(serializer.is_valid())
-                self.assertEqual(
-                    serializer.validated_data["workspace"], workspace
-                )
-
-        with context.local():
-            context.set_scope(scope3)
-            serializer = WorkflowTemplateSerializer(data=data)
-            self.assertFalse(serializer.is_valid())
-
 
 class WorkspaceChainItemFieldTests(TestCase):
     """Tests for :py:class:`WorkspaceChainItemField`."""
@@ -989,3 +968,121 @@ class WorkspaceChainSerializerTests(TestCase):
                 serializer = WorkspaceChainSerializer(data={"chain": data})
                 self.assertTrue(serializer.is_valid(raise_exception=True))
                 self.assertEqual(serializer.validated_data, {"chain": expected})
+
+
+class CollectionSerializerTests(TestCase):
+    """Tests for :py:class:`CollectionSerializer`."""
+
+    scenario = scenarios.DefaultContextAPI(set_current=True)
+
+    sid: ClassVar[Collection]
+
+    @classmethod
+    def setUpTestData(cls) -> None:
+        """Initialize test."""
+        super().setUpTestData()
+        cls.sid = cls.playground.create_collection(
+            name="sid", category=CollectionCategory.SUITE
+        )
+
+    def test_duration(self) -> None:
+        for value, expected in (
+            (None, None),
+            (dt.timedelta(microseconds=1), 1),
+            (dt.timedelta(milliseconds=1), 1),
+            (dt.timedelta(days=0), 0),
+            (dt.timedelta(days=1), 1),
+            (dt.timedelta(hours=1), 1),
+            (dt.timedelta(hours=25), 2),
+            (dt.timedelta(seconds=1), 1),
+            (dt.timedelta(days=3, seconds=1), 4),
+            (dt.timedelta(days=365, hours=24), 366),
+        ):
+            with self.subTest(value=str(value)):
+                # Test serialization
+                self.sid.full_history_retention_period = value
+                self.sid.metadata_only_retention_period = value
+                serializer = CollectionSerializer(self.sid)
+                self.assertEqual(
+                    serializer.data["full_history_retention_period"], expected
+                )
+                self.assertEqual(
+                    serializer.data["metadata_only_retention_period"], expected
+                )
+
+                # Test deserialization
+                serializer = CollectionSerializer(
+                    data={
+                        "id": self.sid.id,
+                        "name": self.sid.name,
+                        "category": self.sid.category,
+                        "full_history_retention_period": expected,
+                        "metadata_only_retention_period": expected,
+                        "data": {},
+                    }
+                )
+                self.assertTrue(serializer.is_valid(raise_exception=True))
+                self.assertEqual(
+                    serializer.validated_data["full_history_retention_period"],
+                    (
+                        dt.timedelta(days=expected)
+                        if expected is not None
+                        else None
+                    ),
+                )
+                self.assertEqual(
+                    serializer.validated_data["metadata_only_retention_period"],
+                    (
+                        dt.timedelta(days=expected)
+                        if expected is not None
+                        else None
+                    ),
+                )
+
+    def test_get_stats_disabled_in_context(self) -> None:
+        serializer = CollectionSerializer(self.sid)
+        with mock.patch.object(
+            self.sid,
+            "get_item_stats",
+            return_value={
+                ((CollectionItem.Types.BARE, True), 4),
+                ((CollectionItem.Types.BARE, False), 5),
+            },
+        ):
+            assert serializer.get_stats(self.sid) is None
+
+    def test_get_stats(self) -> None:
+        serializer = CollectionSerializer(self.sid)
+        serializer.context["with_stats"] = True
+        with mock.patch.object(
+            self.sid,
+            "get_item_stats",
+            return_value=[
+                {
+                    "type": CollectionItem.Types.BARE,
+                    "category": "category",
+                    "active": True,
+                    "count": 4,
+                },
+                {
+                    "type": CollectionItem.Types.BARE,
+                    "category": "category",
+                    "active": False,
+                    "count": 5,
+                },
+            ],
+        ):
+            assert serializer.get_stats(self.sid) == [
+                {
+                    "type": "bare",
+                    "category": "category",
+                    "active": True,
+                    "count": 4,
+                },
+                {
+                    "type": "bare",
+                    "category": "category",
+                    "active": False,
+                    "count": 5,
+                },
+            ]
