@@ -19,9 +19,12 @@ from debusine.server.status import (
     WorkerStatus,
     WorkerSummary,
 )
+from debusine.server.tasks.wait.models import ExternalDebsignData
+from debusine.server.workflows.models import WorkRequestWorkflowData
 from debusine.tasks.models import (
     MmDebstrapBootstrapOptions,
     MmDebstrapData,
+    NoopData,
     SystemBootstrapRepository,
     WorkerType,
 )
@@ -87,14 +90,8 @@ class WorkerStatusTests(TestCase):
 
     def test_counts_tasks_by_type(self) -> None:
         """Test that a tasks counts tasks by type."""
-        self.playground.create_worker_task(
-            task_name="noop",
-            status=WorkRequest.Statuses.PENDING,
-        )
-        self.playground.create_signing_task(
-            task_name="noop",
-            status=WorkRequest.Statuses.RUNNING,
-        )
+        self.playground.create_worker_task(task_name="noop", mark_pending=True)
+        self.playground.create_signing_task(task_name="noop", mark_running=True)
 
         status = WorkerStatus.get_status()
         self.assertTasks(status, TaskTypes.WORKER, "pending", 1)
@@ -136,27 +133,30 @@ class WorkerStatusTests(TestCase):
 
         For a WorkRequest with build_architecture specified in task_data.
         """
-        for wr_status in (
-            WorkRequest.Statuses.PENDING,
-            WorkRequest.Statuses.RUNNING,
-        ):
-            self.playground.create_worker_task(
-                task_name="noop",
-                task_data={"build_architecture": "amd64"},
-                status=wr_status,
-                validate=False,
-            )
+        self.playground.create_worker_task(
+            task_name="noop",
+            task_data=NoopData(build_architecture="amd64"),
+            mark_pending=True,
+        )
+        self.playground.create_worker_task(
+            task_name="noop",
+            task_data=NoopData(build_architecture="amd64"),
+            mark_running=True,
+        )
 
+        # Signing tasks don't really have build architectures, but force it
+        # for the sake of coverage.
         for wr_status in (
             WorkRequest.Statuses.PENDING,
             WorkRequest.Statuses.RUNNING,
         ):
-            self.playground.create_signing_task(
+            wr = self.playground.create_signing_task(
                 task_name="noop",
                 task_data={"build_architecture": "amd64"},
-                status=wr_status,
                 validate=False,
             )
+            wr.status = wr_status
+            wr.save()
 
         status = WorkerStatus.get_status()
         self.assertWorkerTasksArch(status, "amd64", "pending", 1)
@@ -192,14 +192,10 @@ class WorkerStatusTests(TestCase):
         expected_architecture = task_data.bootstrap_options.architecture
 
         self.playground.create_worker_task(
-            task_name="mmdebstrap",
-            task_data=task_data,
-            status=WorkRequest.Statuses.PENDING,
+            task_name="mmdebstrap", task_data=task_data, mark_pending=True
         )
         self.playground.create_worker_task(
-            task_name="mmdebstrap",
-            task_data=task_data,
-            status=WorkRequest.Statuses.RUNNING,
+            task_name="mmdebstrap", task_data=task_data, mark_running=True
         )
 
         status = WorkerStatus.get_status()
@@ -217,14 +213,8 @@ class WorkerStatusTests(TestCase):
         Work request without "build_architecture" in task_data and
         build_architecture() returns None.
         """
-        self.playground.create_worker_task(
-            task_name="noop",
-            status=WorkRequest.Statuses.PENDING,
-        )
-        self.playground.create_worker_task(
-            task_name="noop",
-            status=WorkRequest.Statuses.RUNNING,
-        )
+        self.playground.create_worker_task(task_name="noop", mark_pending=True)
+        self.playground.create_worker_task(task_name="noop", mark_running=True)
 
         status = WorkerStatus.get_status()
 
@@ -248,7 +238,7 @@ class WorkerStatusTests(TestCase):
         self.playground.create_worker_task(
             task_name="sbuild",
             task_data={"foo": "bar"},
-            status=WorkRequest.Statuses.PENDING,
+            mark_pending=True,
             validate=False,
         )
 
@@ -372,10 +362,9 @@ class WorkerStatusTests(TestCase):
 
         self.playground.create_worker_task(
             task_name="noop",
-            task_data={"build_architecture": "arch1"},
-            status=WorkRequest.Statuses.RUNNING,
+            task_data=NoopData(build_architecture="arch1"),
             worker=worker,
-            validate=False,
+            mark_running=True,
         )
 
         status = WorkerStatus.get_status()
@@ -399,10 +388,9 @@ class WorkerStatusTests(TestCase):
 
         self.playground.create_worker_task(
             task_name="noop",
-            task_data={"build_architecture": "arch1"},
-            status=WorkRequest.Statuses.PENDING,
+            task_data=NoopData(build_architecture="arch1"),
+            mark_pending=True,
             worker=worker,
-            validate=False,
         )
 
         status = WorkerStatus.get_status()
@@ -445,9 +433,8 @@ class WorkerStatusTests(TestCase):
         """We populate external_workers_arch for all queued architectures."""
         self.playground.create_worker_task(
             task_name="noop",
-            task_data={"build_architecture": "arch1"},
-            status=WorkRequest.Statuses.PENDING,
-            validate=False,
+            task_data=NoopData(build_architecture="arch1"),
+            mark_pending=True,
         )
         status = WorkerStatus.get_status()
         self.assertIn("arch1", status.external_workers_arch)
@@ -477,22 +464,19 @@ class WorkerStatusTests(TestCase):
         """Test the full data output of a non-trivial example state."""
         # Create three work requests of type sbuild
         self.playground.create_worker_task(
-            task_name="sbuild",
-            task_data={"build_architecture": "amd64"},
-            status=WorkRequest.Statuses.PENDING,
-            validate=False,
+            task_name="noop",
+            task_data=NoopData(build_architecture="amd64"),
+            mark_pending=True,
         )
         self.playground.create_worker_task(
-            task_name="sbuild",
-            task_data={"build_architecture": "amd64"},
-            status=WorkRequest.Statuses.PENDING,
-            validate=False,
+            task_name="noop",
+            task_data=NoopData(build_architecture="amd64"),
+            mark_pending=True,
         )
         self.playground.create_worker_task(
-            task_name="sbuild",
-            task_data={"build_architecture": "arm64"},
-            status=WorkRequest.Statuses.PENDING,
-            validate=False,
+            task_name="noop",
+            task_data=NoopData(build_architecture="arm64"),
+            mark_pending=True,
         )
 
         # Create three connected workers
@@ -535,18 +519,19 @@ class WorkerStatusTests(TestCase):
 
         # Create a running task
         self.playground.create_worker_task(
-            task_name="sbuild",
-            task_data={"build_architecture": "arm64"},
-            status=WorkRequest.Statuses.RUNNING,
+            task_name="noop",
+            task_data=NoopData(build_architecture="arm64"),
+            mark_running=True,
             worker=worker_arm64_1,
-            validate=False,
         )
 
         # Create a running wait task
+        unsigned = self.playground.create_upload_artifacts().upload
         self.playground.create_wait_task(
             task_name="externaldebsign",
-            status=WorkRequest.Statuses.RUNNING,
-            validate=False,
+            task_data=ExternalDebsignData(unsigned=unsigned.id),
+            workflow_data=WorkRequestWorkflowData(needs_input=True),
+            mark_running=True,
         )
 
         actual = WorkerStatus.get_status()
