@@ -108,7 +108,11 @@ class ArtifactView(GenericAPIViewBase[Artifact], BaseAPIView):
         )
 
         file_in_artifact = FileInArtifact.objects.create(
-            artifact=artifact, file=fileobj, path=path, complete=False
+            artifact=artifact,
+            file=fileobj,
+            path=path,
+            complete=False,
+            content_type=file_deserialized.get("content_type"),
         )
 
         if created or artifact.workspace.file_needs_upload(fileobj):
@@ -372,7 +376,7 @@ class UploadFileView(BaseAPIView):
         try:
             artifact = (
                 Artifact.objects.in_current_scope()
-                .can_display(context.require_user())
+                .can_display(context.user)
                 .get(id=artifact_id)
             )
         except Artifact.DoesNotExist:
@@ -465,6 +469,10 @@ class UploadFileView(BaseAPIView):
                     f"Received: {hash_digest.hex()}"
                 ),
                 status_code=status.HTTP_409_CONFLICT,
+                # Don't roll back; the caller wants to delete the FileUpload
+                # in this case, and we haven't made any other database
+                # changes yet.
+                rollback_transaction=False,
             )
 
         scope = file_in_artifact.artifact.workspace.scope
@@ -514,7 +522,7 @@ class ArtifactRelationsView(
                 )
             try:
                 Artifact.objects.in_current_scope().can_display(
-                    context.require_user()
+                    context.user
                 ).get(id=required_artifact_id)
             except Artifact.DoesNotExist:
                 raise DebusineAPIException(
@@ -535,4 +543,14 @@ class ArtifactRelationsView(
         # rather than on can_display.
         self.enforce(artifact.can_display)
         self.enforce(target.can_display)
+        if artifact.fileinartifact_set.filter(complete=False).exists():
+            raise DebusineAPIException(
+                f"Cannot create relation: source artifact {artifact.id} is "
+                f"incomplete"
+            )
+        if target.fileinartifact_set.filter(complete=False).exists():
+            raise DebusineAPIException(
+                f"Cannot create relation: target artifact {target.id} is "
+                f"incomplete"
+            )
         super().perform_create(serializer)
