@@ -623,7 +623,7 @@ class CollectionViewSetTests(TestCase):
 
     scenario = scenarios.DefaultContextAPI()
     sid: ClassVar[Collection]
-    sid_links: ClassVar[dict[str, str]]
+    sid_links: ClassVar[dict[str, str | dict[str, str]]]
     trixie: ClassVar[Collection]
 
     @classmethod
@@ -639,7 +639,9 @@ class CollectionViewSetTests(TestCase):
         cls.sid_links = cls.get_links(cls.sid)
 
     @classmethod
-    def get_links(cls, collection: Collection) -> dict[str, str]:
+    def get_links(
+        cls, collection: Collection
+    ) -> dict[str, str | dict[str, str]]:
         return {
             "self": reverse(
                 "api:collection-detail",
@@ -649,6 +651,16 @@ class CollectionViewSetTests(TestCase):
                 },
             ),
             "webui_self": collection.get_absolute_url(),
+            "webui_category": {
+                "href": reverse(
+                    "workspaces:collections:category_list",
+                    kwargs={
+                        "wname": collection.workspace.name,
+                        "ccat": collection.category,
+                    },
+                ),
+                "type": "category",
+            },
         }
 
     def _workspace_url_kwarg(self, workspace: Workspace | str | None) -> str:
@@ -793,6 +805,18 @@ class CollectionViewSetTests(TestCase):
             headers=self._headers(user_token),
         )
 
+    def delete(
+        self,
+        collection: int | Collection | None = None,
+        workspace: Workspace | str | None = None,
+        user_token: bool = True,
+    ) -> TestResponseType:
+        """DELETE request for a named collection."""
+        return self.client.delete(
+            self._reverse_detail(collection, workspace),
+            headers=self._headers(user_token),
+        )
+
     def assertSidUnchanged(self) -> None:
         """Ensure self.sid is unchanged."""
         self.sid.refresh_from_db()
@@ -804,7 +828,7 @@ class CollectionViewSetTests(TestCase):
 
     def test_unauthenticated(self) -> None:
         """Authentication is required."""
-        for method in ("lookup", "list", "post", "put", "patch"):
+        for method in ("lookup", "list", "post", "put", "patch", "delete"):
             with self.subTest(method=method):
                 response = getattr(self, method)(user_token=False)
                 self.assertResponseProblem(
@@ -818,7 +842,7 @@ class CollectionViewSetTests(TestCase):
 
     def test_workspace_not_found(self) -> None:
         """Workspace must exist."""
-        for method in ("lookup", "list", "post", "put", "patch"):
+        for method in ("lookup", "list", "post", "put", "patch", "delete"):
             with self.subTest(method=method):
                 response = getattr(self, method)(workspace="does-not-exist")
                 self.assertResponseProblem(
@@ -833,7 +857,7 @@ class CollectionViewSetTests(TestCase):
     def test_workspace_not_accessible(self) -> None:
         """User must be able to display the workspace."""
         workspace = self.playground.create_workspace(name="private")
-        for method in ("lookup", "list", "post", "put", "patch"):
+        for method in ("lookup", "list", "post", "put", "patch", "delete"):
             with self.subTest(method=method):
                 response = getattr(self, method)(workspace=workspace)
                 self.assertResponseProblem(
@@ -848,7 +872,7 @@ class CollectionViewSetTests(TestCase):
     def test_not_found(self) -> None:
         """The collection must exist."""
         max_id = Collection.objects.aggregate(Max('id'))['id__max']
-        for method in ("get", "put", "patch"):
+        for method in ("get", "put", "patch", "delete"):
             with self.subTest(method=method):
                 response = getattr(self, method)(collection=max_id + 1)
                 self.assertResponseProblem(
@@ -859,7 +883,7 @@ class CollectionViewSetTests(TestCase):
 
     def test_detail_unauthorized(self) -> None:
         """The user must be able to display the collection."""
-        for method in ("get", "put", "patch"):
+        for method in ("get", "put", "patch", "delete"):
             with (
                 self.subTest(method=method),
                 override_permission(Collection, "can_display", DenyAll),
@@ -896,6 +920,16 @@ class CollectionViewSetTests(TestCase):
                     status_code=status.HTTP_403_FORBIDDEN,
                 )
 
+    def test_delete_unauthorized(self) -> None:
+        """The user must be able to delete the collection."""
+        with override_permission(Collection, "can_delete", DenyAll):
+            response = self.delete()
+            self.assertResponseProblem(
+                response,
+                f"playground cannot delete collection {self.sid}",
+                status_code=status.HTTP_403_FORBIDDEN,
+            )
+
     def test_lookup(self) -> None:
         response = self.lookup()
         data = self.assertAPIResponseOk(response)
@@ -907,6 +941,7 @@ class CollectionViewSetTests(TestCase):
             "metadata_only_retention_period": None,
             "data": {"test": 1},
             "links": self.sid_links,
+            "stats": None,
         }
 
     def test_lookup_not_found(self) -> None:
@@ -917,6 +952,35 @@ class CollectionViewSetTests(TestCase):
             status_code=status.HTTP_404_NOT_FOUND,
         )
 
+    def test_list(self) -> None:
+        response = self.list()
+        data = self.assertAPIResponseOk(response)
+        names = [f"{x["name"]}@{x["category"]}" for x in data["results"]]
+        assert names == [
+            '_@debian:archive',
+            "_@debian:package-build-logs",
+            "sid@debian:suite",
+            'trixie@debian:suite',
+            'default@debusine:task-configuration',
+            '_@debusine:task-history',
+        ]
+
+    def test_list_omits_workflow_internal(self) -> None:
+        self.playground.create_collection(
+            name="1234", category=CollectionCategory.WORKFLOW_INTERNAL
+        )
+        response = self.list()
+        data = self.assertAPIResponseOk(response)
+        names = [f"{x["name"]}@{x["category"]}" for x in data["results"]]
+        assert names == [
+            '_@debian:archive',
+            "_@debian:package-build-logs",
+            "sid@debian:suite",
+            'trixie@debian:suite',
+            'default@debusine:task-configuration',
+            '_@debusine:task-history',
+        ]
+
     def test_detail(self) -> None:
         response = self.get()
         data = self.assertAPIResponseOk(response)
@@ -928,6 +992,7 @@ class CollectionViewSetTests(TestCase):
             "metadata_only_retention_period": None,
             "data": {"test": 1},
             "links": self.sid_links,
+            "stats": None,
         }
 
     @override_permission(Workspace, "can_create_collections", AllowAll)
@@ -960,6 +1025,7 @@ class CollectionViewSetTests(TestCase):
             "metadata_only_retention_period": None,
             "data": {"created": True},
             "links": self.get_links(new),
+            "stats": None,
         }
 
     @override_permission(Workspace, "can_create_collections", AllowAll)
@@ -1015,6 +1081,7 @@ class CollectionViewSetTests(TestCase):
             "metadata_only_retention_period": None,
             "data": self.sid.data,
             "links": self.sid_links,
+            "stats": None,
         }
         self.assertSidUnchanged()
 
@@ -1037,6 +1104,7 @@ class CollectionViewSetTests(TestCase):
             "metadata_only_retention_period": None,
             "data": self.sid.data,
             "links": self.sid_links,
+            "stats": None,
         }
         self.assertSidUnchanged()
 
@@ -1053,6 +1121,7 @@ class CollectionViewSetTests(TestCase):
             "metadata_only_retention_period": None,
             "data": self.sid.data,
             "links": self.get_links(self.sid),
+            "stats": None,
         }
         self.assertEqual(self.sid.name, "renamed")
         self.assertEqual(self.sid.category, CollectionCategory.SUITE)
@@ -1077,6 +1146,7 @@ class CollectionViewSetTests(TestCase):
             "metadata_only_retention_period": 456,
             "data": self.sid.data,
             "links": self.sid_links,
+            "stats": None,
         }
         self.sid.refresh_from_db()
         self.assertEqual(self.sid.name, "sid")
@@ -1103,6 +1173,7 @@ class CollectionViewSetTests(TestCase):
             "metadata_only_retention_period": None,
             "data": {"changed": True},
             "links": self.sid_links,
+            "stats": None,
         }
         self.sid.refresh_from_db()
         self.assertEqual(self.sid.name, "sid")
@@ -1131,6 +1202,7 @@ class CollectionViewSetTests(TestCase):
             "metadata_only_retention_period": 456,
             "data": {"changed": True},
             "links": self.get_links(self.sid),
+            "stats": None,
         }
         self.assertEqual(self.sid.name, "renamed")
         self.assertEqual(
@@ -1199,6 +1271,7 @@ class CollectionViewSetTests(TestCase):
             ),
             "data": self.sid.data,
             "links": self.get_links(self.sid),
+            "stats": None,
         }
 
     @override_permission(Collection, "can_configure", AllowAll)
@@ -1223,6 +1296,7 @@ class CollectionViewSetTests(TestCase):
             ),
             "data": collection.data,
             "links": self.get_links(collection),
+            "stats": None,
         }
 
     @override_permission(Collection, "can_configure", AllowAll)
@@ -1245,6 +1319,30 @@ class CollectionViewSetTests(TestCase):
         collection.refresh_from_db()
         self.assertEqual(collection.name, "_")
 
+    @override_permission(Collection, "can_delete", AllowAll)
+    def test_delete(self) -> None:
+        response = self.delete(collection=self.sid)
+        new_data = self.assertAPIResponseOk(
+            response, status_code=status.HTTP_204_NO_CONTENT
+        )
+        self.assertIsNone(new_data)
+        self.assertFalse(Collection.objects.filter(pk=self.sid.pk).exists())
+        self.assertTrue(Collection.objects.filter(pk=self.trixie.pk).exists())
+
+    def test_debusine_collection_show(self) -> None:
+        debusine = self.get_debusine()
+        result = debusine.collection_get(
+            self.scenario.workspace.name, CollectionCategory.SUITE, "sid"
+        )
+        self.assertIsNone(result.stats)
+        result = debusine.collection_get(
+            self.scenario.workspace.name,
+            CollectionCategory.SUITE,
+            "sid",
+            with_stats=True,
+        )
+        self.assertEqual(result.stats, [])
+
     def test_debusine_collection_iter(self) -> None:
         debusine = self.get_debusine()
         result = debusine.collection_iter(self.scenario.workspace.name)
@@ -1331,3 +1429,37 @@ class CollectionViewSetTests(TestCase):
         self.assertEqual(
             self.sid.full_history_retention_period, dt.timedelta(days=3)
         )
+
+    @override_permission(Collection, "can_delete", AllowAll)
+    def test_debusine_collection_delete(self) -> None:
+        debusine = self.get_debusine()
+        debusine.collection_delete(
+            self.scenario.workspace.name,
+            CollectionData(
+                id=self.sid.id,
+                name=self.sid.name,
+                category=self.sid.category,
+                full_history_retention_period=3,
+                data=self.sid.data,
+            ),
+        )
+        self.assertFalse(Collection.objects.filter(pk=self.sid.pk).exists())
+
+    @override_permission(Collection, "can_delete", DenyAll)
+    def test_debusine_collection_delete_not_allowed(self) -> None:
+        debusine = self.get_debusine()
+        with self.assertRaisesRegex(
+            DebusineError,
+            "playground cannot delete collection sid@debian:suite",
+        ):
+            debusine.collection_delete(
+                self.scenario.workspace.name,
+                CollectionData(
+                    id=self.sid.id,
+                    name=self.sid.name,
+                    category=self.sid.category,
+                    full_history_retention_period=3,
+                    data=self.sid.data,
+                ),
+            )
+        self.assertTrue(Collection.objects.filter(pk=self.sid.pk).exists())
