@@ -206,7 +206,7 @@ class GetNextWorkRequestViewTests(WorkRequestTestCase):
         )
 
     def create_work_request(
-        self, status: WorkRequest.Statuses, created_at: dt.datetime
+        self, created_at: dt.datetime, **kwargs: Any
     ) -> WorkRequest:
         """Return a new work_request as specified by the parameters."""
         environment_item = self.playground.create_debian_environment()
@@ -222,7 +222,7 @@ class GetNextWorkRequestViewTests(WorkRequestTestCase):
             worker=self.worker,
             task_name="sbuild",
             task_data=task_data,
-            status=status,
+            **kwargs,
         )
 
         work_request.created_at = created_at
@@ -242,14 +242,14 @@ class GetNextWorkRequestViewTests(WorkRequestTestCase):
         """A running work request is aborted and retried."""
         # Create WorkRequest pending
         work_request_pending = self.create_work_request(
-            WorkRequest.Statuses.PENDING,
             dt.datetime(2022, 1, 5, 10, 13, 20, 204242, dt.timezone.utc),
+            mark_pending=True,
         )
 
         # Create WorkRequest running
         work_request_running = self.create_work_request(
-            WorkRequest.Statuses.RUNNING,
             dt.datetime(2022, 1, 5, 11, 14, 22, 242178, dt.timezone.utc),
+            mark_running=True,
         )
 
         # Request
@@ -275,12 +275,12 @@ class GetNextWorkRequestViewTests(WorkRequestTestCase):
     def test_get_running_work_request_cannot_retry(self) -> None:
         """If a running work request cannot be retried, it is logged."""
         work_request_pending = self.create_work_request(
-            WorkRequest.Statuses.PENDING,
             dt.datetime(2022, 1, 5, 10, 13, 20, 204242, dt.timezone.utc),
+            mark_pending=True,
         )
         work_request_running = self.create_work_request(
-            WorkRequest.Statuses.RUNNING,
             dt.datetime(2022, 1, 5, 11, 14, 22, 242178, dt.timezone.utc),
+            mark_running=True,
         )
         work_request_running.task_data = {"invalid": True}
         work_request_running.configured_task_data = {"invalid": True}
@@ -310,8 +310,8 @@ class GetNextWorkRequestViewTests(WorkRequestTestCase):
     def test_get_work_request_change_status(self) -> None:
         """Get a WorkRequest change the status to running."""
         work_request = self.create_work_request(
-            WorkRequest.Statuses.PENDING,
             dt.datetime(2022, 1, 5, 10, 13, 20, 204242, dt.timezone.utc),
+            mark_pending=True,
         )
 
         # Request
@@ -331,8 +331,8 @@ class GetNextWorkRequestViewTests(WorkRequestTestCase):
     def test_get_work_request_configured_task_data(self) -> None:
         """Get a WorkRequest shows configured task data."""
         work_request = self.create_work_request(
-            WorkRequest.Statuses.PENDING,
             dt.datetime(2022, 1, 5, 10, 13, 20, 204242, dt.timezone.utc),
+            mark_pending=True,
         )
         work_request.task_data = {"test": 1}
         work_request.configured_task_data = {"test": 2}
@@ -353,13 +353,13 @@ class GetNextWorkRequestViewTests(WorkRequestTestCase):
         """View return the older pending work request."""
         # Create WorkRequest pending
         work_request_pending_01 = self.create_work_request(
-            WorkRequest.Statuses.PENDING,
             dt.datetime(2022, 1, 5, 10, 13, 20, 204242, dt.timezone.utc),
+            mark_pending=True,
         )
 
         work_request_pending_02 = self.create_work_request(
-            WorkRequest.Statuses.PENDING,
             dt.datetime(2022, 1, 5, 10, 13, 22, 204242, dt.timezone.utc),
+            mark_pending=True,
         )
         # Request
         response = self.client.get(
@@ -511,7 +511,7 @@ class UpdateWorkRequestAsCompletedTests(TestCase):
             worker=cls.worker,
             task_name='sbuild',
             task_data=task_data,
-            status=WorkRequest.Statuses.PENDING,
+            mark_pending=True,
         )
 
     def test_check_permissions(self) -> None:
@@ -627,7 +627,7 @@ class UpdateWorkRequestAsCompletedTests(TestCase):
             worker=another_worker,
             task_name='sbuild',
             task_data=task_data,
-            status=WorkRequest.Statuses.PENDING,
+            mark_pending=True,
         )
 
         response = self.client.put(
