@@ -22,12 +22,14 @@ from debusine.artifacts.models import ArtifactCategory, TaskTypes
 from debusine.client.models import (
     Link,
     RuntimeParameter,
+    RuntimeParameters,
     StrictBaseModel,
     WorkflowTemplateData,
     WorkflowTemplateDataNew,
     model_to_json_serializable_dict,
 )
 from debusine.db.models import (
+    Scope,
     WorkRequest,
     WorkflowTemplate,
     Workspace,
@@ -326,6 +328,25 @@ class WorkflowTemplateViewSetTests(TestCase):
                 status_code=status.HTTP_403_FORBIDDEN,
             )
 
+    def test_create_unauthorized_create_child_workspace(self) -> None:
+        """Only scope owners may create ``create_child_workspace`` templates."""
+        with (
+            override_permission(Workspace, "can_configure", AllowAll),
+            override_permission(Scope, "can_create_workspace", DenyAll),
+        ):
+            response = self.post(
+                data=WorkflowTemplateDataNew(
+                    name="test",
+                    task_name="create_child_workspace",
+                    static_parameters={},
+                )
+            )
+            self.assertResponseProblem(
+                response,
+                f"playground cannot create workspaces in {self.scenario.scope}",
+                status_code=status.HTTP_403_FORBIDDEN,
+            )
+
     def test_update_unauthorized(self) -> None:
         """The user must be able to update the collection."""
         for method in ("put", "patch"):
@@ -344,6 +365,35 @@ class WorkflowTemplateViewSetTests(TestCase):
                     status_code=status.HTTP_403_FORBIDDEN,
                 )
 
+    def test_update_unauthorized_create_child_workspace(self) -> None:
+        """Only scope owners may update ``create_child_workspace`` templates."""
+        template = self.playground.create_workflow_template(
+            name="test", task_name="create_child_workspace"
+        )
+
+        for method in ("put", "patch"):
+            with (
+                self.subTest(method=method),
+                override_permission(
+                    WorkflowTemplate, "can_configure", AllowAll
+                ),
+                override_permission(Scope, "can_create_workspace", DenyAll),
+            ):
+                response = getattr(self, method)(
+                    workflow_template=template,
+                    data=WorkflowTemplateDataNew(
+                        name="test",
+                        task_name="create_child_workspace",
+                        static_parameters={},
+                    ),
+                )
+                self.assertResponseProblem(
+                    response,
+                    "playground cannot create workspaces in"
+                    f" {self.scenario.scope}",
+                    status_code=status.HTTP_403_FORBIDDEN,
+                )
+
     def test_post_unauthorized(self) -> None:
         """The user must be able to create the workflow template."""
         with override_permission(Workspace, "can_configure", DenyAll):
@@ -512,6 +562,35 @@ class WorkflowTemplateViewSetTests(TestCase):
         self.assertEqual(template.static_parameters, static_parameters)
         self.assertEqual(template.runtime_parameters, RuntimeParameter.ANY)
 
+    @override_permission(Workspace, "can_configure", AllowAll)
+    @override_permission(Scope, "can_create_workspace", AllowAll)
+    def test_post_create_child_workspace(self) -> None:
+        """Create a new ``create_child_workspace`` workflow template."""
+        static_parameters = {"prefix": "experiment"}
+        runtime_parameters: RuntimeParameters = {
+            "suffix": RuntimeParameter.ANY,
+            "public": RuntimeParameter.ANY,
+            "owner_group": RuntimeParameter.ANY,
+            "workflow_template_names": RuntimeParameter.ANY,
+            "expiration_delay": RuntimeParameter.ANY,
+        }
+
+        response = self.post(
+            data=WorkflowTemplateDataNew(
+                name="test",
+                task_name="create_child_workspace",
+                static_parameters=static_parameters,
+                runtime_parameters=runtime_parameters,
+            )
+        )
+        self.assertAPIResponseOk(response, status.HTTP_201_CREATED)
+        template = WorkflowTemplate.objects.get(
+            name="test", workspace=self.scenario.workspace
+        )
+        self.assertEqual(template.task_name, "create_child_workspace")
+        self.assertEqual(template.static_parameters, static_parameters)
+        self.assertEqual(template.runtime_parameters, runtime_parameters)
+
     @override_permission(Workspace, "can_configure", AllowAll)
     def test_post_different_workspace(self) -> None:
         workspace = self.playground.create_workspace(
@@ -617,6 +696,23 @@ class WorkflowTemplateViewSetTests(TestCase):
         self.sample.refresh_from_db()
         self.assertEqual(self.sample.static_parameters, static_parameters)
 
+    @override_permission(WorkflowTemplate, "can_configure", AllowAll)
+    @override_permission(Scope, "can_create_workspace", AllowAll)
+    def test_patch_create_child_workspace(self) -> None:
+        template = self.playground.create_workflow_template(
+            "test",
+            task_name="create_child_workspace",
+            static_parameters={"prefix": "experiment"},
+        )
+        static_parameters = {"prefix": "new-experiment"}
+        response = self.patch(
+            workflow_template=template,
+            data={"static_parameters": static_parameters},
+        )
+        self.assertAPIResponseOk(response)
+        template.refresh_from_db()
+        self.assertEqual(template.static_parameters, static_parameters)
+
     @override_permission(WorkflowTemplate, "can_configure", AllowAll)
     def test_patch_positive_priority_without_permissions(self) -> None:
         response = self.patch(
@@ -844,6 +940,7 @@ class WorkflowViewTests(TestCase):
             {
                 "workspace": settings.DEBUSINE_DEFAULT_WORKSPACE,
                 "created_by": self.token.user_id,
+                "status": WorkRequest.Statuses.PENDING,
                 "task_type": TaskTypes.WORKFLOW,
                 "task_name": "sbuild",
                 "task_data": {
