@@ -9,7 +9,7 @@
 
 """Dummy WorkerPool implementation, for testing."""
 
-from typing import TYPE_CHECKING
+from typing import TYPE_CHECKING, override
 
 from django.utils import timezone
 
@@ -25,14 +25,18 @@ class DummyWorkerPool(
 ):
     """Dummy implementation of WorkerPoolInterface for testing."""
 
+    @override
     def launch_worker(self, worker: "Worker") -> None:
-        """Launch a cloud worker using worker."""
         worker.instance_created_at = timezone.now()
         worker.worker_pool_data = {"launched": True}
         worker.save()
 
+    @override
     def terminate_worker(self, worker: "Worker") -> None:
-        """Terminate a cloud worker using worker."""
         worker.instance_created_at = None
         worker.worker_pool_data = None
         worker.save()
+
+    @override
+    def terminate_missing(self) -> None:
+        pass
