@@ -10,8 +10,10 @@
 """Tests for AWS EC2 cloud workers."""
 
 from base64 import b64decode
-from typing import Any
-from unittest.mock import patch
+from collections import UserList
+from typing import Any, TYPE_CHECKING, TypeVar, override
+from unittest import TestCase as SimpleTestCase
+from unittest.mock import call, patch
 
 from botocore import stub
 from botocore.exceptions import ClientError
@@ -19,7 +21,7 @@ from botocore.exceptions import ClientError
 from debusine.assets import AWSProviderAccountData, CloudProvidersType
 from debusine.db.context import context
 from debusine.db.models import Asset, WorkerPool
-from debusine.server.worker_pools.amazon_ec2 import AWSEC2WorkerPool
+from debusine.server.worker_pools.amazon_ec2 import AWSEC2WorkerPool, log
 from debusine.server.worker_pools.cloud_init import FilesystemSetup
 from debusine.server.worker_pools.exceptions import InstanceLaunchException
 from debusine.server.worker_pools.models import (
@@ -31,6 +33,36 @@ from debusine.server.worker_pools.models import (
 )
 from debusine.test.django import TestCase
 
+if TYPE_CHECKING:
+    from mypy_boto3_ec2.type_defs import DescribeInstancesResultTypeDef
+else:
+    DescribeInstancesResultTypeDef = dict[str, Any]
+
+
+LI = TypeVar("LI")
+
+
+class UnsortedList(UserList[LI]):
+    """List that compares values in any order."""
+
+    @override
+    def __eq__(self, other: object) -> bool:
+        if isinstance(other, list):
+            return set(self.data) == set(other)
+        return super().__eq__(other)
+
+
+class TestUnsortedList(SimpleTestCase):
+    """Test UnsortedList."""
+
+    def test_equal(self) -> None:
+        self.assertEqual(UnsortedList([1, 2, 3]), [3, 2, 1])
+        self.assertEqual(UnsortedList(['a', 'b', 'c']), ['c', 'b', 'a'])
+
+    def test_not_equal(self) -> None:
+        self.assertNotEqual(UnsortedList(['a', 'b', 'c']), ['a', 'b', 'c', 'd'])
+        self.assertNotEqual(UnsortedList(['a', 'b', 'c']), 42)
+
 
 class TestAWSEC2WorkerPool(TestCase):
     """Test for AWSEC2WorkerPool."""
@@ -160,7 +192,7 @@ class TestAWSEC2WorkerPool(TestCase):
     def test_cloud_init_config_simple(self) -> None:
         token = self.playground.create_bare_token()
         config = self.get_worker_pool()._cloud_init_config(
-            self.launch_template(), token
+            self.launch_template(), "worker", token
         )
         self.assertEqual(config.device_aliases, {})
         self.assertEqual(config.fs_setup, [])
@@ -171,7 +203,7 @@ class TestAWSEC2WorkerPool(TestCase):
         token = self.playground.create_bare_token()
         launch_template = self.launch_template(swap_size=4)
         config = self.get_worker_pool()._cloud_init_config(
-            launch_template, token
+            launch_template, "worker", token
         )
         self.assertEqual(config.device_aliases, {"swap": "/dev/xvdb"})
         self.assertEqual(
@@ -246,7 +278,7 @@ class TestAWSEC2WorkerPool(TestCase):
         token = self.playground.create_bare_token()
         launch_template = self.launch_template()
         launch_template_data = self.get_worker_pool()._launch_template_data(
-            launch_template, token
+            launch_template, "worker", token
         )
         # Literal copies
         self.assertEqual(
@@ -280,7 +312,7 @@ class TestAWSEC2WorkerPool(TestCase):
             ),
         )
         launch_template_data = self.get_worker_pool()._launch_template_data(
-            launch_template, token
+            launch_template, "worker", token
         )
         self.assertNotIn("InstanceType", launch_template_data)
         assert launch_template.InstanceRequirements
@@ -293,7 +325,7 @@ class TestAWSEC2WorkerPool(TestCase):
         token = self.playground.create_bare_token()
         launch_template = self.launch_template(KeyName="my-key")
         launch_template_data = self.get_worker_pool()._launch_template_data(
-            launch_template, token
+            launch_template, "worker", token
         )
         self.assertEqual(
             launch_template_data["KeyName"], launch_template.KeyName
@@ -310,7 +342,7 @@ class TestAWSEC2WorkerPool(TestCase):
             ],
         )
         launch_template_data = self.get_worker_pool()._launch_template_data(
-            launch_template, token
+            launch_template, "worker", token
         )
         self.assertEqual(
             launch_template_data["NetworkInterfaces"],
@@ -334,7 +366,7 @@ class TestAWSEC2WorkerPool(TestCase):
             return_value="/dev/xvda",
         ):
             launch_template_data = worker_pool._launch_template_data(
-                launch_template, token
+                launch_template, "worker", token
             )
         self.assertEqual(
             launch_template_data["BlockDeviceMappings"],
@@ -354,7 +386,7 @@ class TestAWSEC2WorkerPool(TestCase):
         token = self.playground.create_bare_token()
         launch_template = self.launch_template(swap_size=8)
         launch_template_data = self.get_worker_pool()._launch_template_data(
-            launch_template, token
+            launch_template, "worker", token
         )
         self.assertEqual(
             launch_template_data["BlockDeviceMappings"],
@@ -374,7 +406,7 @@ class TestAWSEC2WorkerPool(TestCase):
         token = self.playground.create_bare_token()
         launch_template = self.launch_template(tags={"foo": "bar"})
         launch_template_data = self.get_worker_pool()._launch_template_data(
-            launch_template, token
+            launch_template, "worker", token
         )
         self.assertEqual(
             launch_template_data["TagSpecifications"],
@@ -513,6 +545,48 @@ class TestAWSEC2WorkerPool(TestCase):
             )
         self.assertEqual(instance_id, "i-id")
 
+    def test_create_fleet_spot_insufficient_capacity(self) -> None:
+        """EC2's harmless InsufficientInstanceCapacity is logged."""
+        worker_pool = self.get_worker_pool()
+        with (
+            stub.Stubber(worker_pool.client) as stubber,
+            self.assertLogsContains(
+                (
+                    "WARNING:debusine.server.worker_pools.amazon_ec2:Amazon "
+                    "EC2 reported InsufficientInstanceCapacity, when "
+                    "attempting to launch cloud0: We have cheaper instances "
+                    "in another AZ!"
+                ),
+                logger=log,
+            ),
+        ):
+            stubber.add_response(
+                method="create_fleet",
+                expected_params=self.get_expected_create_fleet_params(),
+                service_response={
+                    "Instances": [
+                        {
+                            "InstanceIds": ["i-id"],
+                        },
+                    ],
+                    "Errors": [
+                        {
+                            "ErrorCode": "InsufficientInstanceCapacity",
+                            "ErrorMessage": (
+                                "We have cheaper instances in another AZ!"
+                            ),
+                        }
+                    ],
+                },
+            )
+            instance_id = worker_pool._create_fleet(
+                ["my-template"],
+                AWSEC2InstanceMarketType.SPOT,
+                worker_name="cloud0",
+                max_spot_price_per_hour=None,
+            )
+        self.assertEqual(instance_id, "i-id")
+
     def test_create_fleet_no_templates(self) -> None:
         worker_pool = self.get_worker_pool()
         with self.assertRaisesRegex(
@@ -636,7 +710,7 @@ class TestAWSEC2WorkerPool(TestCase):
             worker_pool.launch_worker(worker)
 
         launch_template_data.assert_called_once_with(
-            launch_template, worker.activation_token
+            launch_template, worker.name, worker.activation_token
         )
         create_launch_template.assert_called_once_with(
             f"debusine_worker_{worker.name}_0",
@@ -674,3 +748,77 @@ class TestAWSEC2WorkerPool(TestCase):
         )
         self.assertIsNone(worker.instance_created_at)
         self.assertEqual(worker.worker_pool_data, {})
+
+    def test_terminate_missing(self) -> None:
+        worker_pool = self.get_worker_pool()
+        for instance_id in ("instance-1", "instance-2", "instance-3"):
+            worker = self.playground.create_worker(
+                worker_pool=worker_pool.worker_pool
+            )
+            worker.worker_pool_data = {"instance-id": instance_id}
+            worker.save()
+
+        # Inactive pool member
+        worker = self.playground.create_worker(
+            worker_pool=worker_pool.worker_pool
+        )
+        worker.instance_created_at = None
+        worker.save()
+
+        response: DescribeInstancesResultTypeDef = {
+            "Reservations": [
+                {
+                    "Instances": [
+                        {
+                            "InstanceId": "instance-1",
+                            "State": {
+                                "Name": "running",
+                            },
+                        },
+                        {
+                            "InstanceId": "instance-2",
+                            "State": {
+                                "Name": "terminated",
+                            },
+                        },
+                    ],
+                }
+            ],
+            "ResponseMetadata": {
+                "HTTPHeaders": {},
+                "HTTPStatusCode": 200,
+                "RequestId": "foo",
+                "RetryAttempts": 0,
+            },
+        }
+        with (
+            stub.Stubber(worker_pool.client) as stubber,
+            patch.object(
+                worker_pool, "_terminate_existing_instances"
+            ) as terminate_existing_instances,
+        ):
+            stubber.add_response(
+                method="describe_instances",
+                expected_params={
+                    "InstanceIds": UnsortedList(
+                        ["instance-1", "instance-2", "instance-3"]
+                    ),
+                },
+                service_response=response,
+            )
+            worker_pool.terminate_missing()
+
+        self.assertEqual(terminate_existing_instances.call_count, 2)
+        terminate_existing_instances.assert_has_calls(
+            [
+                call({"instance-id": "instance-2"}),
+                call({"instance-id": "instance-3"}),
+            ],
+            any_order=True,
+        )
+
+    def test_terminate_missing_empty(self) -> None:
+        worker_pool = self.get_worker_pool()
+        with stub.Stubber(worker_pool.client) as stubber:
+            worker_pool.terminate_missing()
+        stubber.assert_no_pending_responses()
