@@ -278,22 +278,22 @@ class AutopkgtestWorkflow(
 
     def _extract_artifact_details(
         self, artifact: Artifact | None
-    ) -> tuple[str | None, str | None, WorkRequest | None]:
+    ) -> tuple[str | None, int | None, WorkRequest | None]:
         """Extract details from a ``debian:autopkgtest`` artifact."""
         source_version: str | None
-        url: str | None
+        artifact_id: int | None
         wr: WorkRequest | None
         if artifact is None:
             source_version = None
-            url = None
+            artifact_id = None
             wr = None
         else:
             data = artifact.create_data()
             assert isinstance(data, DebianAutopkgtest)
             source_version = data.source_package.version
-            url = artifact.get_absolute_url()
+            artifact_id = artifact.id
             wr = artifact.created_by_work_request
-        return source_version, url, wr
+        return source_version, artifact_id, wr
 
     @staticmethod
     def compare_qa_results_fine_grained(
@@ -345,11 +345,11 @@ class AutopkgtestWorkflow(
             set(reference_artifacts) | set(new_artifacts)
         ):
             reference = reference_artifacts.get(architecture)
-            reference_source_version, reference_url, reference_wr = (
+            reference_source_version, reference_artifact_id, reference_wr = (
                 self._extract_artifact_details(reference)
             )
             new = new_artifacts.get(architecture)
-            new_source_version, new_url, new_wr = (
+            new_source_version, new_artifact_id, new_wr = (
                 self._extract_artifact_details(new)
             )
 
@@ -365,9 +365,9 @@ class AutopkgtestWorkflow(
 
             output_data.regression_analysis[architecture] = RegressionAnalysis(
                 original_source_version=reference_source_version,
-                original_url=reference_url,
+                original_artifact_id=reference_artifact_id,
                 new_source_version=new_source_version,
-                new_url=new_url,
+                new_artifact_id=new_artifact_id,
                 status=status,
                 details=details,
             )
