@@ -54,6 +54,7 @@ from debusine.tasks.models import (
     ActionUpdateCollectionWithData,
     BaseDynamicTaskData,
     BaseTaskData,
+    InputArtifact,
     LookupMultiple,
     LookupSingle,
     OutputData,
@@ -587,11 +588,23 @@ class Workflow(BaseServerTask[WD, DTD], metaclass=ABCMeta):
             expect_type=LookupChildType.COLLECTION,
         ).collection
 
-    def get_input_artifacts_ids(self) -> list[int]:
-        """Return the list of input artifact IDs used by this task."""
-        # All workflows use BaseDynamicTaskData
+    def get_input_artifacts(self) -> list[InputArtifact]:
+        """Return the list of input artifacts used by this workflow."""
         return []
 
+    def get_input_artifacts_ids(self) -> list[int]:
+        """
+        Return the list of artifact ids used by this workflow.
+
+        .. deprecated:: 2025-12
+           Use get_input_artifacts()
+        """
+        return [
+            artifact_id
+            for input_artifact in self.get_input_artifacts()
+            for artifact_id in input_artifact.resolved_ids()
+        ]
+
     def _execute(self) -> bool:
         """Unused abstract method from BaseTask: populate() is used instead."""
         raise NotImplementedError()
