@@ -9,17 +9,24 @@
 
 """debian_pipeline workflow."""
 
+from functools import partial
+from typing import assert_never
+
 from django.contrib.auth.models import AnonymousUser
+from django.db.models import Q
 
 try:
     import pydantic.v1 as pydantic
 except ImportError:
     import pydantic  # type: ignore
 
+from debusine.artifacts.models import ArtifactCategory, DebianSourcePackage
 from debusine.assets import KeyPurpose
 from debusine.client.models import LookupChildType
+from debusine.db.models import WorkRequest
 from debusine.server.collections.lookup import lookup_single
 from debusine.server.tasks.models import PackageUploadTarget
+from debusine.server.tasks.wait.models import ConfirmData
 from debusine.server.workflows import (
     Workflow,
     WorkflowValidationError,
@@ -30,6 +37,8 @@ from debusine.server.workflows.models import (
     MakeSignedSourceWorkflowData,
     PackagePublishWorkflowData,
     PackageUploadWorkflowData,
+    QAFailOn,
+    QAFailurePolicy,
     QAWorkflowData,
     SbuildWorkflowData,
     WorkRequestWorkflowData,
@@ -37,6 +46,10 @@ from debusine.server.workflows.models import (
 from debusine.tasks.models import (
     BackendType,
     DebianPipelineWorkflowDynamicData,
+    ExtraDebusineRepository,
+    ExtraRepository,
+    InputArtifact,
+    InputArtifactSingle,
     LintianFailOnSeverity,
     LookupMultiple,
     LookupSingle,
@@ -60,7 +73,7 @@ class DebianPipelineWorkflow(
                     self.data.qa_suite,
                     self.workspace,
                     user=self.work_request.created_by,
-                    workflow_root=self.work_request.get_workflow_root(),
+                    workflow_root=self.work_request.workflow_root,
                     expect_type=LookupChildType.COLLECTION,
                 )
             except LookupError as e:
@@ -80,7 +93,7 @@ class DebianPipelineWorkflow(
             self.data.source_artifact,
             self.workspace,
             user=self.work_request.created_by,
-            workflow_root=self.work_request.get_workflow_root(),
+            workflow_root=self.work_request.workflow_root,
             expect_type=LookupChildType.ARTIFACT,
         ).artifact.id
 
@@ -90,22 +103,62 @@ class DebianPipelineWorkflow(
             input_source_artifact_id=source_artifact_id,
         )
 
-    def get_input_artifacts_ids(self) -> list[int]:
-        """Return a list with the source package (or empty list)."""
-        if (
-            self.dynamic_data is None
-            or (artifact_id := self.dynamic_data.input_source_artifact_id)
-            is None
-        ):
-            return []
+    def get_input_artifacts(self) -> list[InputArtifact]:
+        """Return workflow input artifacts (the source package)."""
+        return [
+            InputArtifactSingle(
+                lookup=self.data.source_artifact,
+                label="source_artifact",
+                artifact_id=(
+                    None
+                    if self.dynamic_data is None
+                    else self.dynamic_data.input_source_artifact_id
+                ),
+            )
+        ]
 
-        return [artifact_id]
+    def suite_has_active_indexes(self, suite_lookup: LookupSingle) -> bool:
+        """Return True if this suite has an active ``InRelease`` file."""
+        suite = lookup_single(
+            suite_lookup,
+            self.workspace,
+            user=self.work_request.created_by,
+            workflow_root=self.work_request.workflow_root,
+            expect_type=LookupChildType.COLLECTION,
+        ).collection
+        return (
+            suite.child_items.active()
+            .filter(
+                category=ArtifactCategory.REPOSITORY_INDEX,
+                data__path="InRelease",
+            )
+            .exists()
+        )
+
+    def get_extra_repositories(self) -> list[ExtraRepository] | None:
+        """Decide which extra repositories to use for this workflow."""
+        extra_repositories = (
+            None
+            if self.data.extra_repositories is None
+            else list(self.data.extra_repositories)
+        )
+        if self.data.suite is not None and self.suite_has_active_indexes(
+            self.data.suite
+        ):
+            if extra_repositories is None:
+                extra_repositories = []
+            extra_repositories.insert(
+                0, ExtraDebusineRepository(suite=self.data.suite)
+            )
+        return extra_repositories
 
     def populate(self) -> None:  # noqa: C901
         """Create work requests."""
         assert self.dynamic_data is not None
         assert self.dynamic_data.subject is not None
 
+        source_data = workflow_utils.source_package_data(self)
+
         if (data_archs := self.data.architectures) is not None:
             architectures = set(data_archs)
         else:
@@ -127,12 +180,15 @@ class DebianPipelineWorkflow(
 
         effective_architectures = sorted(architectures)
 
+        extra_repositories = self.get_extra_repositories()
+
         self._populate_sbuild(
             source_artifact=self.data.source_artifact,
             target_distribution=target_distribution,
             backend=self.data.sbuild_backend,
             architectures=effective_architectures,
             environment_variant=self.data.sbuild_environment_variant,
+            extra_repositories=extra_repositories,
             signing_template_names=self.data.signing_template_names,
         )
 
@@ -163,6 +219,12 @@ class DebianPipelineWorkflow(
             )
         ]
 
+        # Work request that post-QA steps in the pipeline should depend on
+        # for QA confirmation.  Depending on the task data, this may be
+        # automatic or manual.
+        qa_confirmation: WorkRequest | None = None
+
+        qa: WorkRequest | None = None
         if (
             self.data.enable_autopkgtest
             or self.data.enable_reverse_dependencies_autopkgtest
@@ -171,14 +233,17 @@ class DebianPipelineWorkflow(
             or self.data.enable_debdiff
             or self.data.enable_blhc
         ):
-            self._populate_qa(
+            populate_qa = partial(
+                self._populate_qa,
                 source_artifact=self.data.source_artifact,
-                binary_artifacts=binary_artifacts,
                 package_build_logs=package_build_logs,
                 vendor=self.data.vendor,
                 codename=self.data.codename,
                 architectures=effective_architectures,
-                arch_all_host_architecture=self.data.arch_all_host_architecture,
+                arch_all_build_architecture=(
+                    self.data.arch_all_build_architecture
+                ),
+                extra_repositories=extra_repositories,
                 qa_suite=self.data.qa_suite,
                 enable_autopkgtest=self.data.enable_autopkgtest,
                 autopkgtest_backend=self.data.autopkgtest_backend,
@@ -191,10 +256,100 @@ class DebianPipelineWorkflow(
                 enable_piuparts=self.data.enable_piuparts,
                 piuparts_backend=self.data.piuparts_backend,
                 piuparts_environment=self.data.piuparts_environment,
-                enable_debdiff=self.data.enable_debdiff,
                 enable_blhc=self.data.enable_blhc,
             )
 
+            if self.data.enable_regression_tracking:
+                reference_source_lookup = (
+                    f"{self.data.qa_suite}/source:{source_data.name}"
+                )
+                try:
+                    reference_source = lookup_single(
+                        reference_source_lookup,
+                        self.workspace,
+                        user=self.work_request.created_by,
+                        workflow_root=self.work_request.workflow_root,
+                        expect_type=LookupChildType.ARTIFACT,
+                    ).artifact
+                except KeyError:
+                    # This source package does not exist in qa_suite.  Skip
+                    # regression tracking.
+                    pass
+                else:
+                    reference_source_data = reference_source.create_data()
+                    assert isinstance(
+                        reference_source_data, DebianSourcePackage
+                    )
+                    reference_name = reference_source_data.name
+                    reference_version = reference_source_data.version
+                    populate_qa(
+                        prefix="reference-qa-result|",
+                        source_artifact=reference_source.id,
+                        binary_artifacts=LookupMultiple.parse_obj(
+                            {
+                                "collection": self.data.qa_suite,
+                                "category": ArtifactCategory.BINARY_PACKAGE,
+                                "data__srcpkg_name": reference_name,
+                                "data__srcpkg_version": reference_version,
+                            }
+                        ),
+                        enable_regression_tracking=False,
+                        reference_qa_results=(
+                            self.data.regression_tracking_qa_results
+                        ),
+                        update_qa_results=True,
+                        # The debdiff workflow doesn't contribute any QA
+                        # results, so don't waste time running it here.
+                        enable_debdiff=False,
+                        fail_on=QAFailOn.NEVER,
+                        workflow_data=WorkRequestWorkflowData(
+                            display_name=(
+                                f"QA (reference tests for "
+                                f"{reference_name}_{reference_version})"
+                            ),
+                            step="qa-reference",
+                            expanded=False,
+                        ),
+                    )
+                    qa = populate_qa(
+                        source_artifact=self.data.source_artifact,
+                        binary_artifacts=binary_artifacts,
+                        reference_source_artifact=reference_source.id,
+                        enable_regression_tracking=True,
+                        reference_qa_results=(
+                            self.data.regression_tracking_qa_results
+                        ),
+                        reference_prefix="reference-qa-result|",
+                        fail_on=QAFailOn.REGRESSION,
+                        enable_debdiff=self.data.enable_debdiff,
+                    )
+
+            if qa is None:
+                qa = populate_qa(
+                    source_artifact=self.data.source_artifact,
+                    binary_artifacts=binary_artifacts,
+                    enable_debdiff=self.data.enable_debdiff,
+                )
+
+            match self.data.qa_failure_policy:
+                case QAFailurePolicy.IGNORE:
+                    pass
+                case QAFailurePolicy.FAIL:
+                    qa_confirmation = qa
+                case QAFailurePolicy.CONFIRM:
+                    qa_confirmation = self._populate_confirmation(
+                        auto_confirm_if_no_failure=True,
+                        confirm_label=(
+                            "Continue the workflow despite QA failures"
+                        ),
+                    )
+                    qa_confirmation.add_dependency(qa)
+                case _ as unreachable:
+                    assert_never(unreachable)
+
+        if qa_confirmation is None and self.data.enable_confirmation:
+            qa_confirmation = self._populate_confirmation()
+
         if (
             self.data.enable_make_signed_source
             and self.data.signing_template_names is not None
@@ -224,6 +379,7 @@ class DebianPipelineWorkflow(
                 key=self.data.make_signed_source_key,
                 sbuild_backend=self.data.sbuild_backend,
                 signing_template_names=self.data.signing_template_names,
+                qa_confirmation=qa_confirmation,
             ):
                 to_upload.append(
                     (
@@ -257,9 +413,10 @@ class DebianPipelineWorkflow(
                     vendor=self.data.vendor,
                     codename=self.data.codename,
                     subject=upload_subject,
+                    qa_confirmation=qa_confirmation,
                 )
 
-        if self.data.publish_target:
+        if self.data.suite:
             for (
                 upload_source_artifact,
                 upload_binary_artifacts,
@@ -269,9 +426,10 @@ class DebianPipelineWorkflow(
                 self._populate_package_publish(
                     source_artifact=upload_source_artifact,
                     binary_artifacts=upload_binary_artifacts,
-                    target_suite=self.data.publish_target,
+                    target_suite=self.data.suite,
                     replace=self.data.publish_replace,
                     subject=upload_subject,
+                    qa_confirmation=qa_confirmation,
                 )
 
     def _populate_sbuild(
@@ -282,6 +440,7 @@ class DebianPipelineWorkflow(
         backend: BackendType,
         architectures: list[str],
         environment_variant: str | None,
+        extra_repositories: list[ExtraRepository] | None = None,
         signing_template_names: dict[str, list[str]],
     ) -> None:
         """Create work request for sbuild workflow."""
@@ -292,10 +451,12 @@ class DebianPipelineWorkflow(
                 target_distribution=target_distribution,
                 backend=backend,
                 architectures=architectures,
-                arch_all_host_architecture=self.data.arch_all_host_architecture,
+                arch_all_build_architecture=(
+                    self.data.arch_all_build_architecture
+                ),
                 environment_variant=environment_variant,
+                extra_repositories=extra_repositories,
                 signing_template_names=signing_template_names,
-                extra_repositories=self.data.extra_repositories,
             ),
             workflow_data=WorkRequestWorkflowData(
                 display_name="sbuild",
@@ -307,14 +468,21 @@ class DebianPipelineWorkflow(
     def _populate_qa(
         self,
         *,
+        prefix: str = "",
+        reference_prefix: str = "",
         source_artifact: LookupSingle,
         binary_artifacts: LookupMultiple,
         package_build_logs: LookupMultiple,
+        reference_source_artifact: LookupSingle | None = None,
         vendor: str,
         codename: str,
         architectures: list[str],
-        arch_all_host_architecture: str,
+        arch_all_build_architecture: str,
+        extra_repositories: list[ExtraRepository] | None = None,
         qa_suite: LookupSingle | None,
+        reference_qa_results: LookupSingle | None = None,
+        enable_regression_tracking: bool = False,
+        update_qa_results: bool = False,
         enable_autopkgtest: bool,
         autopkgtest_backend: BackendType,
         enable_reverse_dependencies_autopkgtest: bool,
@@ -326,18 +494,26 @@ class DebianPipelineWorkflow(
         piuparts_environment: LookupSingle | None,
         enable_debdiff: bool,
         enable_blhc: bool,
-    ) -> None:
+        fail_on: QAFailOn = QAFailOn.FAILURE,
+        workflow_data: WorkRequestWorkflowData | None = None,
+    ) -> WorkRequest:
         """Create work request for qa workflow."""
         data = QAWorkflowData(
+            prefix=prefix,
+            reference_prefix=reference_prefix,
             source_artifact=source_artifact,
             binary_artifacts=binary_artifacts,
             package_build_logs=package_build_logs,
+            reference_source_artifact=reference_source_artifact,
             vendor=vendor,
             codename=codename,
             architectures=architectures,
-            arch_all_host_architecture=arch_all_host_architecture,
-            extra_repositories=self.data.extra_repositories,
+            arch_all_build_architecture=arch_all_build_architecture,
+            extra_repositories=extra_repositories,
             qa_suite=qa_suite,
+            reference_qa_results=reference_qa_results,
+            enable_regression_tracking=enable_regression_tracking,
+            update_qa_results=update_qa_results,
             enable_autopkgtest=enable_autopkgtest,
             autopkgtest_backend=autopkgtest_backend,
             enable_reverse_dependencies_autopkgtest=(
@@ -350,21 +526,43 @@ class DebianPipelineWorkflow(
             piuparts_backend=piuparts_backend,
             enable_debdiff=enable_debdiff,
             enable_blhc=enable_blhc,
+            fail_on=fail_on,
         )
         if piuparts_environment:
             data.piuparts_environment = piuparts_environment
+        if workflow_data is None:
+            workflow_data = WorkRequestWorkflowData(
+                display_name="QA", step="qa"
+            )
         wr = self.work_request_ensure_child_workflow(
             task_name="qa",
             task_data=data,
-            workflow_data=WorkRequestWorkflowData(
-                display_name="QA",
-                step="qa",
-            ),
+            task_data_filter=Q(task_data__update_qa_results=update_qa_results),
+            workflow_data=workflow_data,
         )
         # The qa workflow's children will have dependencies on the work
         # requests creating binary_artifacts, but the qa workflow itself
         # doesn't need that in order to populate itself.
         self.orchestrate_child(wr)
+        return wr
+
+    def _populate_confirmation(
+        self,
+        *,
+        auto_confirm_if_no_failure: bool = False,
+        confirm_label: str = "Continue the workflow",
+    ) -> WorkRequest:
+        """Create a work request acting as a confirmation step."""
+        return self.work_request_ensure_child_wait(
+            task_name="confirm",
+            task_data=ConfirmData(
+                auto_confirm_if_no_failure=auto_confirm_if_no_failure,
+                confirm_label=confirm_label,
+            ),
+            workflow_data=WorkRequestWorkflowData(
+                display_name="Confirm", step="confirm", needs_input=True
+            ),
+        )
 
     def _populate_make_signed_source(
         self,
@@ -377,6 +575,7 @@ class DebianPipelineWorkflow(
         key: str,
         sbuild_backend: BackendType,
         signing_template_names: dict[str, list[str]],
+        qa_confirmation: WorkRequest | None = None,
     ) -> list[tuple[LookupSingle, LookupMultiple, str]]:
         """Create work request for make signed source workflow."""
         signing_artifacts: list[LookupSingle] = []
@@ -392,7 +591,7 @@ class DebianPipelineWorkflow(
                     signing_artifact,
                     self.workspace,
                     user=self.work_request.created_by or AnonymousUser(),
-                    workflow_root=self.work_request.get_workflow_root(),
+                    workflow_root=self.work_request.workflow_root,
                     expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
                 )
 
@@ -433,6 +632,14 @@ class DebianPipelineWorkflow(
                 display_name="make signed source", step="make_signed_source"
             ),
         )
+
+        # If confirmation is enabled, then the make_signed_source workflow
+        # will not be populated until confirmation (to avoid having to poke
+        # around in its internals to find the first work request for each
+        # architecture).
+        if qa_confirmation is not None:
+            wr.add_dependency(qa_confirmation)
+
         # The make_signed_source workflow's children will have dependencies
         # on the work requests creating binary_artifacts and
         # signing_artifacts, but the make_signed_source workflow itself
@@ -459,6 +666,7 @@ class DebianPipelineWorkflow(
         vendor: str,
         codename: str,
         subject: str,
+        qa_confirmation: WorkRequest | None = None,
     ) -> None:
         """Create work request for package upload workflow."""
         wr = self.work_request_ensure_child_workflow(
@@ -482,7 +690,9 @@ class DebianPipelineWorkflow(
                 delayed_days=delayed_days,
                 vendor=vendor,
                 codename=codename,
-                arch_all_host_architecture=self.data.arch_all_host_architecture,
+                arch_all_build_architecture=(
+                    self.data.arch_all_build_architecture
+                ),
             ),
             workflow_data=WorkRequestWorkflowData(
                 display_name=f"package upload ({subject})",
@@ -490,6 +700,12 @@ class DebianPipelineWorkflow(
             ),
         )
 
+        # If confirmation is enabled, then the package_upload workflow will
+        # not be populated until confirmation (to avoid having to poke
+        # around in its internals to find the first work request).
+        if qa_confirmation is not None:
+            wr.add_dependency(qa_confirmation)
+
         # The package_upload workflow's children will have dependencies on
         # the work requests creating source_artifact and binary_artifacts,
         # but the package_upload workflow itself doesn't normally need that
@@ -510,6 +726,7 @@ class DebianPipelineWorkflow(
         target_suite: LookupSingle,
         replace: bool,
         subject: str,
+        qa_confirmation: WorkRequest | None = None,
     ) -> None:
         """Create work request for package publish workflow."""
         wr = self.work_request_ensure_child_workflow(
@@ -526,6 +743,12 @@ class DebianPipelineWorkflow(
             ),
         )
 
+        # If confirmation is enabled, then the package_publish workflow will
+        # not be populated until confirmation (to avoid having to poke
+        # around in its internals to find the first work request).
+        if qa_confirmation is not None:
+            wr.add_dependency(qa_confirmation)
+
         # The package_publish workflow needs to split up debian:upload
         # binary artifacts into individual debian:binary-package artifacts
         # when preparing data for the CopyCollectionItems task, so its input
