@@ -9,8 +9,8 @@
 
 """Pydantic models used by debusine workflows."""
 
+import datetime as dt
 import re
-from datetime import datetime
 from enum import StrEnum
 from typing import Any
 
@@ -21,6 +21,7 @@ try:
 except ImportError:
     import pydantic as pydantic  # type: ignore
 
+from debusine.db.models.workspaces import workspace_name_regex
 from debusine.server.tasks.models import PackageUploadTarget
 from debusine.tasks.models import (
     ActionRetryWithDelays,
@@ -28,6 +29,7 @@ from debusine.tasks.models import (
     AutopkgtestNeedsInternet,
     AutopkgtestTimeout,
     BackendType,
+    BaseDynamicTaskData,
     BaseTaskData,
     BlhcFlags,
     DebDiffFlags,
@@ -62,7 +64,7 @@ class WorkRequestManualUnblockLog(pydantic.BaseModel):
     """A log entry for a review of a manual unblock request."""
 
     user_id: int
-    timestamp: datetime
+    timestamp: dt.datetime
     notes: str | None = None
     action: WorkRequestManualUnblockAction | None = None
 
@@ -82,12 +84,18 @@ class WorkRequestWorkflowData(pydantic.BaseModel):
         """Set up stricter pydantic Config."""
 
         validate_assignment = True
-        extra = pydantic.Extra.forbid
+        extra = "forbid"
 
-    #: If the work request fails, if True the workflow can continue, if false
-    #: it is interrupted
+    #: Whether the parent workflow and reverse-dependencies should continue
+    #: if this work request fails.  If False, they are aborted instead.
     allow_failure: bool = pydantic.Field(default=False, allow_mutation=False)
 
+    #: Whether this work request should continue if any of its child work
+    #: requests or dependencies fails.  If False, it is aborted instead.
+    allow_dependency_failures: bool = pydantic.Field(
+        default=False, allow_mutation=False
+    )
+
     #: name of the step in the visual representation of the workflow
     display_name: str | None = pydantic.Field(
         default=None, allow_mutation=False
@@ -107,6 +115,7 @@ class WorkRequestWorkflowData(pydantic.BaseModel):
         default=None, allow_mutation=False
     )
 
+    #: Data stored for manual unblock requests.
     manual_unblock: WorkRequestManualUnblockData | None = None
 
     #: Whether this task requires user input.  Only relevant for WAIT tasks.
@@ -119,6 +128,10 @@ class WorkRequestWorkflowData(pydantic.BaseModel):
     #: workflow.
     visible: bool = pydantic.Field(default=True, allow_mutation=False)
 
+    #: Whether this task's descendants should be expanded by default in the
+    #: visual representation of the workflow.
+    expanded: bool = pydantic.Field(default=True, allow_mutation=False)
+
 
 class SbuildWorkflowData(BaseWorkflowData):
     """Sbuild workflow data."""
@@ -132,7 +145,7 @@ class SbuildWorkflowData(BaseWorkflowData):
         min_items=1,
         unique_items=True,
     )
-    arch_all_host_architecture: str = "amd64"
+    arch_all_build_architecture: str = "amd64"
     environment_variant: str | None = None
     extra_repositories: list[ExtraRepository] | None = None
     binnmu: SbuildBinNMU | None = None
@@ -155,6 +168,12 @@ class SbuildWorkflowData(BaseWorkflowData):
         return values
 
 
+class SbuildWorkflowDynamicData(BaseDynamicTaskData):
+    """Dynamic data for the Sbuild workflow."""
+
+    source_artifact_id: int | None = None
+
+
 class UpdateEnvironmentsWorkflowTarget(BaseWorkflowData):
     """A target for an update_environments workflow."""
 
@@ -199,7 +218,7 @@ class PackageUploadWorkflowData(BaseWorkflowData):
     delayed_days: int | None = None
     vendor: str | None = None
     codename: str | None = None
-    arch_all_host_architecture: str = "amd64"
+    arch_all_build_architecture: str = "amd64"
 
 
 class MakeSignedSourceWorkflowData(BaseWorkflowData):
@@ -269,7 +288,7 @@ class PiupartsWorkflowData(RegressionTrackingWorkflowData):
     architectures: list[str] | None = None
     backend: BackendType = BackendType.AUTO
     environment: LookupSingle | None = None
-    arch_all_host_architecture: str = "amd64"
+    arch_all_build_architecture: str = "amd64"
     extra_repositories: list[ExtraRepository] | None = None
 
 
@@ -286,7 +305,7 @@ class AutopkgtestWorkflowData(RegressionTrackingWorkflowData):
     codename: str
     backend: BackendType = BackendType.AUTO
     architectures: list[str] = pydantic.Field(default_factory=list)
-    arch_all_host_architecture: str = "amd64"
+    arch_all_build_architecture: str = "amd64"
     extra_repositories: list[ExtraRepository] | None = None
 
     include_tests: list[str] = pydantic.Field(default_factory=list)
@@ -319,7 +338,7 @@ class ReverseDependenciesAutopkgtestWorkflowData(
     codename: str
     backend: BackendType = BackendType.AUTO
     architectures: list[str] = []
-    arch_all_host_architecture: str = "amd64"
+    arch_all_build_architecture: str = "amd64"
     packages_allowlist: list[str] | None = None
     packages_denylist: list[str] = []
     extra_repositories: list[ExtraRepository] | None = None
@@ -338,7 +357,7 @@ class LintianWorkflowData(RegressionTrackingWorkflowData):
     backend: BackendType = BackendType.UNSHARE
 
     architectures: list[str] | None = None
-    arch_all_host_architecture: str = "amd64"
+    arch_all_build_architecture: str = "amd64"
     output: LintianOutput = pydantic.Field(default_factory=LintianOutput)
 
     include_tags: list[str] = pydantic.Field(default_factory=list)
@@ -346,6 +365,13 @@ class LintianWorkflowData(RegressionTrackingWorkflowData):
     fail_on_severity: LintianFailOnSeverity = LintianFailOnSeverity.ERROR
 
 
+class LintianWorkflowDynamicData(BaseDynamicTaskData):
+    """`lintian` workflow dynamic data."""
+
+    source_artifact_id: int | None = None
+    binary_artifacts_ids: list[int] | None = None
+
+
 class DebDiffWorkflowData(BaseWorkflowData):
     """`debdiff` workflow data."""
 
@@ -358,7 +384,7 @@ class DebDiffWorkflowData(BaseWorkflowData):
     vendor: str
     codename: str
 
-    arch_all_host_architecture: str = "amd64"
+    arch_all_build_architecture: str = "amd64"
 
 
 class BlhcWorkflowData(RegressionTrackingWorkflowData):
@@ -371,7 +397,7 @@ class BlhcWorkflowData(RegressionTrackingWorkflowData):
     vendor: str
     codename: str
 
-    arch_all_host_architecture: str = "amd64"
+    arch_all_build_architecture: str = "amd64"
 
     @pydantic.root_validator(allow_reuse=True)
     @classmethod
@@ -388,6 +414,14 @@ class BlhcWorkflowData(RegressionTrackingWorkflowData):
         return values
 
 
+class QAFailOn(StrEnum):
+    """Conditions to trigger a failure of the whole QA workflow."""
+
+    FAILURE = "failure"
+    REGRESSION = "regression"
+    NEVER = "never"
+
+
 class QAWorkflowData(RegressionTrackingWorkflowData):
     """`qa` workflow data."""
 
@@ -395,6 +429,10 @@ class QAWorkflowData(RegressionTrackingWorkflowData):
     binary_artifacts: LookupMultiple
     package_build_logs: LookupMultiple | None = None
 
+    # debian_pipeline sets this if enable_regression_tracking is set, but it
+    # wasn't present in older workflows.
+    reference_source_artifact: LookupSingle | None = None
+
     vendor: str
     codename: str
     architectures: list[str] | None = None
@@ -403,7 +441,7 @@ class QAWorkflowData(RegressionTrackingWorkflowData):
     architectures_allowlist: list[str] | None = None
     architectures_denylist: list[str] | None = None
 
-    arch_all_host_architecture: str = "amd64"
+    arch_all_build_architecture: str = "amd64"
 
     enable_autopkgtest: bool = True
     autopkgtest_backend: BackendType = BackendType.AUTO
@@ -423,6 +461,10 @@ class QAWorkflowData(RegressionTrackingWorkflowData):
     enable_debdiff: bool = False
     enable_blhc: bool = False
 
+    # This fake default is just to avoid too much mypy fallout.  The real
+    # default is set by `default_fail_on` below.
+    fail_on: QAFailOn = QAFailOn.FAILURE
+
     @pydantic.root_validator(allow_reuse=True)
     @classmethod
     def check_reverse_dependencies_autopkgtest_consistency(
@@ -464,19 +506,52 @@ class QAWorkflowData(RegressionTrackingWorkflowData):
             )
         return values
 
+    # TODO: Once we require pydantic v2, this dynamic default can be handled
+    # using default_factory instead of a validator.
+    @pydantic.root_validator(pre=True, allow_reuse=True)
+    @classmethod
+    def default_fail_on(cls, values: dict[str, Any]) -> dict[str, Any]:
+        """Set default for ``fail_on``."""
+        if "fail_on" not in values:
+            values["fail_on"] = (
+                QAFailOn.REGRESSION
+                if values.get("enable_regression_tracking")
+                else QAFailOn.FAILURE
+            )
+        return values
+
+
+class QAWorkflowDynamicData(BaseDynamicTaskData):
+    """Dynamic data for the QA workflow."""
+
+    source_artifact_id: int | None = None
+    binary_artifacts_ids: list[int] | None = None
+    reference_source_artifact_id: int | None = None
+    package_build_logs_ids: list[int] | None = None
+
+
+class QAFailurePolicy(StrEnum):
+    """Policy to apply if the QA workflow fails."""
+
+    IGNORE = "ignore"
+    FAIL = "fail"
+    CONFIRM = "confirm"
+
 
 class DebianPipelineWorkflowData(BaseWorkflowData):
     """`debian_pipeline` workflow data."""
 
     source_artifact: LookupSingle
 
+    suite: LookupSingle | None = None
+
     vendor: str
     codename: str
 
     architectures: list[str] | None = None
     architectures_allowlist: list[str] | None = None
     architectures_denylist: list[str] | None = None
-    arch_all_host_architecture: str = "amd64"
+    arch_all_build_architecture: str = "amd64"
     extra_repositories: list[ExtraRepository] | None = None
 
     signing_template_names: dict[str, list[str]] = {}
@@ -485,6 +560,9 @@ class DebianPipelineWorkflowData(BaseWorkflowData):
     sbuild_environment_variant: str | None = None
 
     qa_suite: LookupSingle | None = None
+    regression_tracking_qa_results: LookupSingle | None = None
+    enable_regression_tracking: bool = False
+    qa_failure_policy: QAFailurePolicy = QAFailurePolicy.IGNORE
 
     enable_autopkgtest: bool = True
     autopkgtest_backend: BackendType = BackendType.AUTO
@@ -505,6 +583,8 @@ class DebianPipelineWorkflowData(BaseWorkflowData):
 
     enable_blhc: bool = True
 
+    enable_confirmation: bool = False
+
     enable_make_signed_source: bool = False
     make_signed_source_purpose: KeyPurpose | None = None
     make_signed_source_key: str | None = None
@@ -524,7 +604,6 @@ class DebianPipelineWorkflowData(BaseWorkflowData):
     )
     upload_delayed_days: int | None = None
 
-    publish_target: LookupSingle | None = None
     publish_replace: bool = False
 
     @pydantic.root_validator(allow_reuse=True)
@@ -555,6 +634,25 @@ class DebianPipelineWorkflowData(BaseWorkflowData):
             )
         return values
 
+    @pydantic.root_validator(allow_reuse=True)
+    @classmethod
+    def check_regression_tracking_consistency(
+        cls, values: dict[str, Any]
+    ) -> dict[str, Any]:
+        """Check consistency of regression-tracking options."""
+        if values.get("enable_regression_tracking"):
+            if values.get("qa_suite") is None:
+                raise ValueError(
+                    '"qa_suite" is required if '
+                    '"enable_regression_tracking" is set'
+                )
+            if values.get("regression_tracking_qa_results") is None:
+                raise ValueError(
+                    '"regression_tracking_qa_results" is required if '
+                    '"enable_regression_tracking" is set'
+                )
+        return values
+
 
 class PackagePublishWorkflowData(BaseWorkflowData):
     """`package_publish` workflow data."""
@@ -585,27 +683,42 @@ class PackagePublishWorkflowData(BaseWorkflowData):
         return values
 
 
-class ExperimentWorkspaceData(BaseWorkflowData):
-    """``create_experiment_workspace`` workflow data."""
+class ChildWorkspaceData(BaseWorkflowData):
+    """``create_child_workspace`` workflow data."""
 
-    experiment_name: str
+    prefix: str | None = None
+    suffix: str
     public: bool = True
     owner_group: str | None = None
     workflow_template_names: list[str] = []
     expiration_delay: int | None = 60
 
-    @pydantic.validator("experiment_name")
+    @pydantic.validator("prefix")
     @classmethod
-    def validate_experiment_name(cls, data: str) -> str:
+    def validate_prefix(cls, data: str | None) -> str | None:
         """
-        Validate the experiment name.
+        Validate the prefix.
 
-        Valid characters are the sames as workspace/group names, minus the '-'
+        Valid characters are the same as workspace/group names.
         """
         # In pydantic 1.10 this could be done with constr, but mypy currently
         # complains about their syntax
-        if not re.match(r"^[A-Za-z][A-Za-z0-9+._]*$", data):
-            raise ValueError("experiment name contains invalid characters")
+        if data is not None and not re.match(workspace_name_regex, data):
+            raise ValueError("prefix contains invalid characters")
+        return data
+
+    @pydantic.validator("suffix")
+    @classmethod
+    def validate_suffix(cls, data: str) -> str:
+        """
+        Validate the suffix.
+
+        Valid characters are the same as workspace/group names.
+        """
+        # In pydantic 1.10 this could be done with constr, but mypy currently
+        # complains about their syntax
+        if not re.match(workspace_name_regex, data):
+            raise ValueError("suffix contains invalid characters")
         return data
 
 
