@@ -24,7 +24,3 @@ class NoopWorkflow(
 
     def populate(self) -> None:
         """Do nothing."""
-
-    def get_label(self) -> str:
-        """Return the task label."""
-        return "noop"
