@@ -68,7 +68,7 @@ class WorkflowTestBase(TestCase, Generic[W]):
             yield schedule()
         with (
             override_settings(CELERY_TASK_ALWAYS_EAGER=True),
-            mock.patch("debusine.server.celery.set_transaction"),
+            mock.patch("pgtransaction.Atomic.execute_set_transaction_modes"),
         ):
             for callback in on_commit:
                 callback()
@@ -117,7 +117,9 @@ class WorkflowTestBase(TestCase, Generic[W]):
         """
         sbuild_workflow_task = sbuild_workflow.get_task()
         assert isinstance(sbuild_workflow_task, SbuildWorkflow)
-        source_data = workflow_utils.source_package_data(sbuild_workflow_task)
+        source_data = workflow_utils.source_package_data(
+            sbuild_workflow_task, configuration_key="input.source_artifact"
+        )
 
         binaries: dict[str, list[Artifact]] = {}
         for sbuild in sbuild_workflow.children.filter(
