@@ -43,6 +43,7 @@ from debusine.tasks.models import (
     DebianPipelineWorkflowDynamicData,
     ExtraDebusineRepository,
     ExtraExternalRepository,
+    InputArtifactSingle,
     LintianFailOnSeverity,
     OutputData,
     OutputDataError,
@@ -168,8 +169,7 @@ class DebianPipelineWorkflowTests(WorkflowTestBase[DebianPipelineWorkflow]):
                 ),
             )
 
-    def test_get_input_artifacts_ids(self) -> None:
-        """Return source artifact id."""
+    def test_get_input_artifacts(self) -> None:
         upload_artifacts = self.playground.create_upload_artifacts()
         wr = self.playground.create_workflow(
             task_name="debian_pipeline",
@@ -181,20 +181,45 @@ class DebianPipelineWorkflowTests(WorkflowTestBase[DebianPipelineWorkflow]):
         )
         workflow = self.get_workflow(wr)
 
-        # workflow.dynamic_data is None: return []
+        # workflow.dynamic_data is None
         self.assertIsNone(workflow.dynamic_data)
-        self.assertEqual(workflow.get_input_artifacts_ids(), [])
+        self.assertEqual(
+            workflow.get_input_artifacts(),
+            [
+                InputArtifactSingle(
+                    lookup=workflow.data.source_artifact,
+                    label="source_artifact",
+                    artifact_id=None,
+                )
+            ],
+        )
 
-        # workflow.dynamic_data.source_artifact_id is None: return []
+        # workflow.dynamic_data.source_artifact_id is None
         workflow.dynamic_data = DebianPipelineWorkflowDynamicData()
-        self.assertEqual(workflow.get_input_artifacts_ids(), [])
+        self.assertEqual(
+            workflow.get_input_artifacts(),
+            [
+                InputArtifactSingle(
+                    lookup=workflow.data.source_artifact,
+                    label="source_artifact",
+                    artifact_id=None,
+                )
+            ],
+        )
 
         # workflow.dynamic_data.source_artifact_id is the id: return it
         workflow.dynamic_data.input_source_artifact_id = (
             upload_artifacts.upload.id
         )
         self.assertEqual(
-            workflow.get_input_artifacts_ids(), [upload_artifacts.upload.id]
+            workflow.get_input_artifacts(),
+            [
+                InputArtifactSingle(
+                    lookup=workflow.data.source_artifact,
+                    label="source_artifact",
+                    artifact_id=upload_artifacts.upload.id,
+                )
+            ],
         )
 
     def test_get_extra_repositories_none(self) -> None:
@@ -555,6 +580,7 @@ class DebianPipelineWorkflowTests(WorkflowTestBase[DebianPipelineWorkflow]):
                 "qa_suite": None,
                 "reference_prefix": "",
                 "reference_qa_results": None,
+                "reference_source_artifact": None,
                 "source_artifact": f"{self.source_artifact.id}@artifacts",
                 "update_qa_results": False,
                 "vendor": "debian",
@@ -613,6 +639,7 @@ class DebianPipelineWorkflowTests(WorkflowTestBase[DebianPipelineWorkflow]):
                 "qa_suite": None,
                 "reference_prefix": "",
                 "reference_qa_results": None,
+                "reference_source_artifact": None,
                 "source_artifact": self.source_artifact.id,
                 "update_qa_results": False,
                 "vendor": "debian",
@@ -672,6 +699,7 @@ class DebianPipelineWorkflowTests(WorkflowTestBase[DebianPipelineWorkflow]):
                 "qa_suite": None,
                 "reference_prefix": "",
                 "reference_qa_results": None,
+                "reference_source_artifact": None,
                 "source_artifact": self.source_artifact.id,
                 "update_qa_results": False,
                 "vendor": "debian",
@@ -731,6 +759,7 @@ class DebianPipelineWorkflowTests(WorkflowTestBase[DebianPipelineWorkflow]):
                 "qa_suite": None,
                 "reference_prefix": "",
                 "reference_qa_results": None,
+                "reference_source_artifact": None,
                 "source_artifact": self.source_artifact.id,
                 "update_qa_results": False,
                 "vendor": "debian",
@@ -798,6 +827,7 @@ class DebianPipelineWorkflowTests(WorkflowTestBase[DebianPipelineWorkflow]):
                 "qa_suite": str(trixie),
                 "reference_prefix": "",
                 "reference_qa_results": None,
+                "reference_source_artifact": None,
                 "source_artifact": self.source_artifact.id,
                 "update_qa_results": False,
                 "vendor": "debian",
@@ -869,6 +899,7 @@ class DebianPipelineWorkflowTests(WorkflowTestBase[DebianPipelineWorkflow]):
                 "qa_suite": None,
                 "reference_prefix": "",
                 "reference_qa_results": None,
+                "reference_source_artifact": None,
                 "source_artifact": self.source_artifact.id,
                 "update_qa_results": False,
                 "vendor": "debian",
@@ -924,6 +955,7 @@ class DebianPipelineWorkflowTests(WorkflowTestBase[DebianPipelineWorkflow]):
                 "prefix": "",
                 "reference_prefix": "",
                 "reference_qa_results": None,
+                "reference_source_artifact": None,
                 "source_artifact": source_artifact.id,
                 "update_qa_results": False,
             },
@@ -1016,6 +1048,7 @@ class DebianPipelineWorkflowTests(WorkflowTestBase[DebianPipelineWorkflow]):
                 "reference_qa_results": (
                     f"trixie@{CollectionCategory.QA_RESULTS}"
                 ),
+                "reference_source_artifact": reference_source_artifact.id,
                 "source_artifact": new_source_artifact.id,
                 "update_qa_results": False,
             },
@@ -1062,6 +1095,7 @@ class DebianPipelineWorkflowTests(WorkflowTestBase[DebianPipelineWorkflow]):
                 "reference_qa_results": (
                     f"trixie@{CollectionCategory.QA_RESULTS}"
                 ),
+                "reference_source_artifact": None,
                 "source_artifact": reference_source_artifact.id,
                 "update_qa_results": True,
             },
