@@ -731,7 +731,7 @@ class PiupartsWorkflowTests(
             work_request=child,
         )
         self.playground.advance_work_request(
-            child, mark_pending=True, result=WorkRequest.Results.SUCCESS
+            child, result=WorkRequest.Results.SUCCESS
         )
         self.assertEqual(
             lookup_single(
