@@ -40,6 +40,7 @@ from debusine.server.workflows.base import orchestrate_workflow
 from debusine.server.workflows.models import (
     BaseWorkflowData,
     QAWorkflowData,
+    QAWorkflowDynamicData,
     SbuildWorkflowData,
     WorkRequestWorkflowData,
 )
@@ -51,6 +52,8 @@ from debusine.tasks import TaskConfigError
 from debusine.tasks.models import (
     BackendType,
     BaseDynamicTaskData,
+    InputArtifactMultiple,
+    InputArtifactSingle,
     LintianFailOnSeverity,
     LookupMultiple,
     RegressionAnalysis,
@@ -120,7 +123,7 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
                 source_artifact_id = (
                     sbuild_workflow_source_artifact.id
                     if sbuild_workflow_source_artifact is not None
-                    else 20
+                    else self.source_artifact.id
                 )
                 sbuild = self_.work_request.create_child_workflow(
                     task_name="sbuild",
@@ -174,8 +177,12 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
 
                 data = {
                     "source_artifact": source_artifact_id,
-                    "binary_artifacts": [30, 31],
-                    "package_build_logs": [32, 33],
+                    "binary_artifacts": [
+                        self.binary_artifact.id,
+                    ],
+                    "package_build_logs": [
+                        self.package_build_log.id,
+                    ],
                     "enable_blhc": False,
                     "vendor": "debian",
                     "codename": "bookworm",
@@ -218,13 +225,10 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
             workflow.validate_input()
 
     def test_compute_dynamic_data(self) -> None:
-        source_artifact = self.playground.create_source_artifact(
-            name="hello", version="1.0-1"
-        )
         wr = self.playground.create_workflow(
             task_name="qa",
             task_data=QAWorkflowData(
-                source_artifact=source_artifact.id,
+                source_artifact=self.source_artifact.id,
                 binary_artifacts=LookupMultiple.parse_obj(
                     ["internal@collections/name:build-amd64"]
                 ),
@@ -234,13 +238,62 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
                 vendor="debian",
                 codename="sid",
             ),
+            internal_collection_artifacts=[
+                (self.binary_artifact, {"name": "build-amd64"}),
+                (self.package_build_log, {"name": "buildlog-amd64"}),
+            ],
         )
         workflow = self.get_workflow(wr)
 
         self.assertEqual(
             workflow.compute_dynamic_data(TaskDatabase(wr)),
-            BaseDynamicTaskData(
-                subject="hello", parameter_summary="hello_1.0-1"
+            QAWorkflowDynamicData(
+                subject="hello",
+                parameter_summary="hello_1.0-1",
+                source_artifact_id=self.source_artifact.id,
+                binary_artifacts_ids=[self.binary_artifact.id],
+                package_build_logs_ids=[self.package_build_log.id],
+            ),
+        )
+
+    def test_compute_dynamic_data_reference_source_artifact(self) -> None:
+        reference_source_artifact = self.playground.create_source_artifact(
+            name="hello", version="1.0-1"
+        )
+        new_source_artifact = self.playground.create_source_artifact(
+            name="hello", version="1.0-2"
+        )
+
+        wr = self.playground.create_workflow(
+            task_name="qa",
+            task_data=QAWorkflowData(
+                source_artifact=new_source_artifact.id,
+                binary_artifacts=LookupMultiple.parse_obj(
+                    ["internal@collections/name:build-amd64"]
+                ),
+                package_build_logs=LookupMultiple.parse_obj(
+                    ["internal@collections/name:buildlog-amd64"]
+                ),
+                reference_source_artifact=reference_source_artifact.id,
+                vendor="debian",
+                codename="sid",
+            ),
+            internal_collection_artifacts=[
+                (self.binary_artifact, {"name": "build-amd64"}),
+                (self.package_build_log, {"name": "buildlog-amd64"}),
+            ],
+        )
+        workflow = self.get_workflow(wr)
+
+        self.assertEqual(
+            workflow.compute_dynamic_data(TaskDatabase(wr)),
+            QAWorkflowDynamicData(
+                subject="hello",
+                parameter_summary="hello_1.0-2",
+                source_artifact_id=new_source_artifact.id,
+                reference_source_artifact_id=reference_source_artifact.id,
+                binary_artifacts_ids=[self.binary_artifact.id],
+                package_build_logs_ids=[self.package_build_log.id],
             ),
         )
 
@@ -269,9 +322,125 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
                 ),
             )
 
+    def test_get_input_artifacts_mandatory_data(self) -> None:
+        upload_artifacts = self.playground.create_upload_artifacts()
+        wr = self.playground.create_workflow(
+            task_name="qa",
+            task_data=QAWorkflowData(
+                source_artifact=upload_artifacts.upload.id,
+                binary_artifacts=LookupMultiple.parse_obj(
+                    [self.binary_artifact.id]
+                ),
+                enable_blhc=False,
+                vendor="debian",
+                codename="sid",
+            ),
+        )
+        workflow = self.get_workflow(wr)
+
+        self.assertIsNone(workflow.dynamic_data)
+        self.assertEqual(
+            workflow.get_input_artifacts(),
+            [
+                InputArtifactSingle(
+                    lookup=workflow.data.source_artifact,
+                    label="source_artifact",
+                    artifact_id=None,
+                ),
+                InputArtifactMultiple(
+                    lookup=workflow.data.binary_artifacts,
+                    label="binary_artifacts",
+                    artifact_ids=None,
+                ),
+            ],
+        )
+
+        workflow.dynamic_data = workflow.build_dynamic_data(TaskDatabase(wr))
+        self.assertIsNotNone(workflow.dynamic_data)
+
+        self.assertEqual(
+            workflow.get_input_artifacts(),
+            [
+                InputArtifactSingle(
+                    lookup=workflow.data.source_artifact,
+                    label="source_artifact",
+                    artifact_id=upload_artifacts.source.id,
+                ),
+                InputArtifactMultiple(
+                    lookup=workflow.data.binary_artifacts,
+                    label="binary_artifacts",
+                    artifact_ids=[self.binary_artifact.id],
+                ),
+            ],
+        )
+
+    def test_get_input_artifacts_with_optional_data(self) -> None:
+        upload_artifacts = self.playground.create_upload_artifacts()
+        reference_source_artifact = (
+            self.playground.create_minimal_source_package_artifact()
+        )
+
+        wr = self.playground.create_workflow(
+            task_name="qa",
+            task_data=QAWorkflowData(
+                source_artifact=upload_artifacts.upload.id,
+                binary_artifacts=LookupMultiple.parse_obj(
+                    ["internal@collections/name:build-amd64"]
+                ),
+                reference_source_artifact=reference_source_artifact.id,
+                package_build_logs=LookupMultiple.parse_obj(
+                    ["internal@collections/name:buildlog-amd64"]
+                ),
+                enable_blhc=False,
+                vendor="debian",
+                codename="sid",
+            ),
+            internal_collection_artifacts=[
+                (self.binary_artifact, {"name": "build-amd64"}),
+                (self.package_build_log, {"name": "buildlog-amd64"}),
+            ],
+        )
+        workflow = self.get_workflow(wr)
+        assert workflow.data.package_build_logs is not None
+
+        workflow.dynamic_data = QAWorkflowDynamicData(
+            source_artifact_id=upload_artifacts.source.id,
+            binary_artifacts_ids=[self.binary_artifact.id],
+            reference_source_artifact_id=reference_source_artifact.id,
+            package_build_logs_ids=[self.package_build_log.id],
+        )
+
+        assert workflow.data.reference_source_artifact is not None
+        self.assertEqual(
+            workflow.get_input_artifacts(),
+            [
+                InputArtifactSingle(
+                    lookup=workflow.data.source_artifact,
+                    label="source_artifact",
+                    artifact_id=upload_artifacts.source.id,
+                ),
+                InputArtifactMultiple(
+                    lookup=workflow.data.binary_artifacts,
+                    label="binary_artifacts",
+                    artifact_ids=workflow.dynamic_data.binary_artifacts_ids,
+                ),
+                InputArtifactSingle(
+                    lookup=workflow.data.reference_source_artifact,
+                    label="reference_source_artifact",
+                    artifact_id=reference_source_artifact.id,
+                ),
+                InputArtifactMultiple(
+                    lookup=workflow.data.package_build_logs,
+                    label="package_build_logs",
+                    artifact_ids=[self.package_build_log.id],
+                ),
+            ],
+        )
+
     def test_source_package_upload(self) -> None:
         """`source_artifact` can be an upload."""
         upload_artifacts = self.playground.create_upload_artifacts()
+
         wr = self.playground.create_workflow(
             task_name="qa",
             task_data=QAWorkflowData(
@@ -283,6 +452,9 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
                 vendor="debian",
                 codename="sid",
             ),
+            internal_collection_artifacts=[
+                (self.binary_artifact, {"name": "build-amd64"})
+            ],
         )
         workflow = self.get_workflow(wr)
         self.assertEqual(
@@ -1622,6 +1794,53 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
             },
         )
 
+    def test_summarize_status(self) -> None:
+        for statuses, expected_status in (
+            (
+                {
+                    RegressionAnalysisStatus.REGRESSION,
+                    RegressionAnalysisStatus.ERROR,
+                    RegressionAnalysisStatus.IMPROVEMENT,
+                    RegressionAnalysisStatus.STABLE,
+                    RegressionAnalysisStatus.NO_RESULT,
+                },
+                RegressionAnalysisStatus.REGRESSION,
+            ),
+            (
+                {
+                    RegressionAnalysisStatus.ERROR,
+                    RegressionAnalysisStatus.IMPROVEMENT,
+                    RegressionAnalysisStatus.STABLE,
+                    RegressionAnalysisStatus.NO_RESULT,
+                },
+                RegressionAnalysisStatus.ERROR,
+            ),
+            (
+                {
+                    RegressionAnalysisStatus.IMPROVEMENT,
+                    RegressionAnalysisStatus.STABLE,
+                    RegressionAnalysisStatus.NO_RESULT,
+                },
+                RegressionAnalysisStatus.IMPROVEMENT,
+            ),
+            (
+                {
+                    RegressionAnalysisStatus.STABLE,
+                    RegressionAnalysisStatus.NO_RESULT,
+                },
+                RegressionAnalysisStatus.STABLE,
+            ),
+            (
+                {RegressionAnalysisStatus.NO_RESULT},
+                RegressionAnalysisStatus.NO_RESULT,
+            ),
+            (set(), RegressionAnalysisStatus.NO_RESULT),
+        ):
+            with self.subTest(statuses=repr(statuses)):
+                self.assertEqual(
+                    QAWorkflow._summarize_status(statuses), expected_status
+                )
+
     def _prepare_test_callback_regression_analysis(
         self,
     ) -> tuple[WorkRequest, Artifact, Artifact]:
@@ -1688,6 +1907,7 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
                 # in qa_suite, but that doesn't really matter for testing
                 # purposes.
                 "binary_artifacts": ["internal@collections/name:build-amd64"],
+                "reference_source_artifact": reference_source_artifact.id,
                 "architectures": ["amd64"],
                 "qa_suite": f"sid@{CollectionCategory.SUITE}",
                 "reference_qa_results": f"sid@{CollectionCategory.QA_RESULTS}",
@@ -1776,16 +1996,19 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
         self.assertEqual(
             workflow.output_data.regression_analysis,
             {
+                "": RegressionAnalysis(
+                    original_source_version="1.0-1",
+                    new_source_version="1.0-2",
+                    status=RegressionAnalysisStatus.STABLE,
+                ),
                 "autopkgtest:hello:amd64": RegressionAnalysis(
                     original_source_version="1.0-1",
-                    original_url=(
-                        reference_autopkgtest_artifact.get_absolute_url()
-                    ),
+                    original_artifact_id=reference_autopkgtest_artifact.id,
                     new_source_version="1.0-2",
-                    new_url=new_autopkgtest_artifact.get_absolute_url(),
+                    new_artifact_id=new_autopkgtest_artifact.id,
                     status=RegressionAnalysisStatus.STABLE,
                     details={"upstream": "stable"},
-                )
+                ),
             },
         )
 
@@ -1857,15 +2080,18 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
         self.assertEqual(
             workflow.output_data.regression_analysis,
             {
+                "": RegressionAnalysis(
+                    original_source_version="1.0-1",
+                    new_source_version="1.0-2",
+                    status=RegressionAnalysisStatus.REGRESSION,
+                ),
                 "autopkgtest:hello:amd64": RegressionAnalysis(
                     original_source_version="1.0-1",
-                    original_url=(
-                        reference_autopkgtest_artifact.get_absolute_url()
-                    ),
+                    original_artifact_id=reference_autopkgtest_artifact.id,
                     new_source_version="1.0-2",
-                    new_url=new_autopkgtest_artifact.get_absolute_url(),
+                    new_artifact_id=new_autopkgtest_artifact.id,
                     status=RegressionAnalysisStatus.REGRESSION,
                     details={"upstream": "regression"},
-                )
+                ),
             },
         )
