@@ -12,7 +12,7 @@
 from collections.abc import Iterable, Mapping, Sequence
 from typing import Any, ClassVar
 
-from debusine.artifacts.local_artifact import BinaryPackages, Upload
+from debusine.artifacts.local_artifact import Upload
 from debusine.artifacts.models import (
     ArtifactCategory,
     BareDataCategory,
@@ -50,7 +50,7 @@ from debusine.tasks.models import (
     AutopkgtestNeedsInternet,
     BackendType,
     BaseDynamicTaskData,
-    ExtraRepository,
+    ExtraExternalRepository,
     LookupMultiple,
     LookupSingle,
     SbuildData,
@@ -65,6 +65,7 @@ class ReverseDependenciesAutopkgtestWorkflowTests(
     """Unit tests for :py:class:`ReverseDependenciesAutopkgtestWorkflow`."""
 
     sid: ClassVar[Collection]
+    source_artifact: ClassVar[Artifact]
 
     @classmethod
     def setUpTestData(cls) -> None:
@@ -73,6 +74,9 @@ class ReverseDependenciesAutopkgtestWorkflowTests(
         cls.sid = cls.playground.create_collection(
             name="sid", category=CollectionCategory.SUITE
         )
+        cls.source_artifact = cls.playground.create_source_artifact(
+            name="hello"
+        )
 
     def create_rdep_autopkgtest_workflow(
         self,
@@ -82,7 +86,7 @@ class ReverseDependenciesAutopkgtestWorkflowTests(
     ) -> ReverseDependenciesAutopkgtestWorkflow:
         """Create a reverse_dependencies_autopkgtest workflow."""
         task_data = {
-            "source_artifact": 1,
+            "source_artifact": self.source_artifact.pk,
             "binary_artifacts": ["internal@collections/name:build-amd64"],
             "qa_suite": "sid@debian:suite",
             "vendor": "debian",
@@ -176,7 +180,7 @@ class ReverseDependenciesAutopkgtestWorkflowTests(
         self.playground.create_collection(
             name="trixie", category=CollectionCategory.SUITE
         )
-        source_artifact = 2
+        source_artifact = self.source_artifact.pk
         binary_artifacts = ["internal@collections/name:build-arm64"]
         qa_suite = "trixie@debian:suite"
         vendor = "debian"
@@ -204,7 +208,7 @@ class ReverseDependenciesAutopkgtestWorkflowTests(
     def test_create_orchestrator_explicit_backend(self) -> None:
         """A ReverseDependenciesAutopkgtestWorkflow can take a backend."""
         workflow = self.create_rdep_autopkgtest_workflow(
-            extra_task_data={"backend": "incus-lxc"}
+            extra_task_data={"backend": BackendType.INCUS_LXC}
         )
 
         self.assertEqual(workflow.data.backend, BackendType.INCUS_LXC)
@@ -258,32 +262,6 @@ class ReverseDependenciesAutopkgtestWorkflowTests(
 
         self.assertEqual(workflow.get_binary_names(), {"single"})
 
-    def test_get_binary_names_binary_packages(self) -> None:
-        """Get binary names from a `debian:binary-packages` artifact."""
-        temp_dir = self.create_temporary_directory()
-        paths = [
-            temp_dir / "hello_1.0_amd64.deb",
-            temp_dir / "hello-dev_1.0_amd64.deb",
-        ]
-        for path in paths:
-            self.playground.write_deb_file(
-                path, source_name="hello", source_version="1.0"
-            )
-        binary_packages = self.playground.create_artifact_from_local(
-            BinaryPackages.create(
-                srcpkg_name="hello",
-                srcpkg_version="1.0",
-                version="1.0",
-                architecture="amd64",
-                files=paths,
-            )
-        )
-        workflow = self.create_rdep_autopkgtest_workflow(
-            extra_task_data={"binary_artifacts": [binary_packages.id]}
-        )
-
-        self.assertEqual(workflow.get_binary_names(), {"hello", "hello-dev"})
-
     def test_get_binary_names_upload(self) -> None:
         """Get binary names from a `debian:upload` artifact."""
         temp_dir = self.create_temporary_directory()
@@ -324,16 +302,15 @@ class ReverseDependenciesAutopkgtestWorkflowTests(
         artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.TEST
         )
-        workflow = self.create_rdep_autopkgtest_workflow(
-            extra_task_data={"source_artifact": artifact.id}
-        )
 
         with self.assertRaisesRegex(
             TaskConfigError,
             r"^source_artifact: unexpected artifact category: 'debusine:test'. "
             r"Valid categories: \['debian:source-package', 'debian:upload'\]$",
         ):
-            workflow.get_reverse_dependencies()
+            self.create_rdep_autopkgtest_workflow(
+                extra_task_data={"source_artifact": artifact.id}
+            )
 
     def assert_reverse_dependencies(
         self,
@@ -348,7 +325,7 @@ class ReverseDependenciesAutopkgtestWorkflowTests(
                 workflow.workspace,
                 user=workflow.work_request.created_by,
                 default_category=CollectionCategory.SUITE,
-                workflow_root=workflow.work_request.get_workflow_root(),
+                workflow_root=workflow.work_request.workflow_root,
                 expect_type=LookupChildType.ARTIFACT,
             ).artifact
             source_data = source.create_data()
@@ -365,7 +342,7 @@ class ReverseDependenciesAutopkgtestWorkflowTests(
                     workflow.workspace,
                     user=workflow.work_request.created_by,
                     default_category=CollectionCategory.SUITE,
-                    workflow_root=workflow.work_request.get_workflow_root(),
+                    workflow_root=workflow.work_request.workflow_root,
                     expect_type=LookupChildType.ARTIFACT,
                 ).artifact
                 binary_data = binary.create_data()
@@ -599,78 +576,84 @@ class ReverseDependenciesAutopkgtestWorkflowTests(
         *,
         extra_data: dict[str, Any] | None = None,
         priority_base: int = 0,
+        parent: WorkRequest | None = None,
+        pipeline_task_name: str = "examplepipeline",
     ) -> WorkRequest:
         """Create a workflow and call ``orchestrate_workflow``."""
-        with preserve_task_registry():
 
-            class ExamplePipeline(
-                SampleWorkflow[BaseWorkflowData, BaseDynamicTaskData]
-            ):
-                """Pipeline workflow that runs sbuild and rdep-autopkgtest."""
+        class ExamplePipeline(
+            SampleWorkflow[BaseWorkflowData, BaseDynamicTaskData]
+        ):
+            """Pipeline workflow that runs sbuild and rdep-autopkgtest."""
 
-                def populate(self) -> None:
-                    """Populate the pipeline."""
-                    sbuild = self.work_request.create_child_workflow(
+            TASK_NAME = pipeline_task_name
+
+            def populate(self_) -> None:
+                """Populate the pipeline."""
+                sbuild = self_.work_request.create_child_workflow(
+                    task_name="sbuild",
+                    task_data=SbuildWorkflowData(
+                        input=SbuildInput(source_artifact=source_artifact.id),
+                        target_distribution="debian:sid",
+                        architectures=["all", *architectures],
+                    ),
+                )
+                for architecture in ("all", *architectures):
+                    child = sbuild.create_child_worker(
                         task_name="sbuild",
-                        task_data=SbuildWorkflowData(
+                        task_data=SbuildData(
                             input=SbuildInput(
                                 source_artifact=source_artifact.id
                             ),
-                            target_distribution="debian:sid",
-                            architectures=["all", *architectures],
+                            build_architecture=architecture,
+                            environment="debian/match:codename=sid",
                         ),
                     )
-                    for architecture in ("all", *architectures):
-                        child = sbuild.create_child_worker(
-                            task_name="sbuild",
-                            task_data=SbuildData(
-                                input=SbuildInput(
-                                    source_artifact=source_artifact.id
-                                ),
-                                host_architecture=architecture,
-                                environment="debian/match:codename=sid",
-                            ),
-                        )
-                        self.provides_artifact(
-                            child,
-                            ArtifactCategory.UPLOAD,
-                            f"build-{architecture}",
-                            data={
-                                "binary_names": ["hello"],
-                                "architecture": architecture,
-                            },
-                        )
-
-                    rdep_autopkgtest = self.work_request.create_child_workflow(
-                        task_name="reverse_dependencies_autopkgtest",
-                        task_data=ReverseDependenciesAutopkgtestWorkflowData(
-                            source_artifact=source_artifact.id,
-                            binary_artifacts=LookupMultiple.parse_obj(
-                                [
-                                    f"internal@collections/"
-                                    f"name:build-{architecture}"
-                                    for architecture in ("all", *architectures)
-                                ]
-                            ),
-                            qa_suite=f"sid@{CollectionCategory.SUITE}",
-                            vendor="debian",
-                            codename="sid",
-                            **(extra_data or {}),
-                        ),
-                        status=WorkRequest.Statuses.PENDING,
+                    self_.provides_artifact(
+                        child,
+                        ArtifactCategory.UPLOAD,
+                        f"build-{architecture}",
+                        data={
+                            "binary_names": ["hello"],
+                            "architecture": architecture,
+                        },
                     )
-                    self.orchestrate_child(rdep_autopkgtest)
 
-            root = self.playground.create_workflow(task_name="examplepipeline")
-            root.priority_base = priority_base
-            root.save()
-            orchestrate_workflow(root)
+                rdep_autopkgtest = self_.work_request.create_child_workflow(
+                    task_name="reverse_dependencies_autopkgtest",
+                    task_data=ReverseDependenciesAutopkgtestWorkflowData(
+                        source_artifact=source_artifact.id,
+                        binary_artifacts=LookupMultiple.parse_obj(
+                            [
+                                f"internal@collections/"
+                                f"name:build-{architecture}"
+                                for architecture in ("all", *architectures)
+                            ]
+                        ),
+                        qa_suite=f"sid@{CollectionCategory.SUITE}",
+                        vendor="debian",
+                        codename="sid",
+                        **(extra_data or {}),
+                    ),
+                )
+                self.playground.advance_work_request(
+                    rdep_autopkgtest, mark_pending=True
+                )
+                self_.orchestrate_child(rdep_autopkgtest)
 
-            return root.children.get(
-                task_type=TaskTypes.WORKFLOW,
-                task_name="reverse_dependencies_autopkgtest",
-            )
+        root = self.playground.create_workflow(
+            task_name=pipeline_task_name, parent=parent
+        )
+        root.priority_base = priority_base
+        root.save()
+        self.assertTrue(orchestrate_workflow(root))
 
+        return root.children.get(
+            task_type=TaskTypes.WORKFLOW,
+            task_name="reverse_dependencies_autopkgtest",
+        )
+
+    @preserve_task_registry()
     def test_populate(self) -> None:
         """The workflow populates child work requests."""
         depends = self.create_source_package(
@@ -706,7 +689,7 @@ class ReverseDependenciesAutopkgtestWorkflowTests(
             architectures,
             extra_data={
                 "extra_repositories": [
-                    ExtraRepository.parse_obj(
+                    ExtraExternalRepository.parse_obj(
                         {
                             "url": "http://example.com/",
                             "suite": "bookworm",
@@ -730,6 +713,7 @@ class ReverseDependenciesAutopkgtestWorkflowTests(
                 child.task_data,
                 {
                     "prefix": f"{source}|",
+                    "reference_prefix": f"{source}|",
                     "source_artifact": (
                         f"sid@{CollectionCategory.SUITE}/"
                         f"source-version:{source}"
@@ -744,12 +728,13 @@ class ReverseDependenciesAutopkgtestWorkflowTests(
                     ],
                     "qa_suite": f"sid@{CollectionCategory.SUITE}",
                     "reference_qa_results": None,
+                    "enable_regression_tracking": False,
                     "update_qa_results": False,
                     "vendor": "debian",
                     "codename": "sid",
                     "backend": BackendType.UNSHARE,
                     "architectures": [],
-                    "arch_all_host_architecture": "amd64",
+                    "arch_all_build_architecture": "amd64",
                     "debug_level": 0,
                     "extra_repositories": [
                         {
@@ -783,7 +768,7 @@ class ReverseDependenciesAutopkgtestWorkflowTests(
                             for architecture in ("all", "amd64")
                         ],
                     },
-                    "host_architecture": "amd64",
+                    "build_architecture": "amd64",
                     "environment": "debian/match:codename=sid",
                     "backend": BackendType.UNSHARE,
                     "include_tests": [],
@@ -821,7 +806,7 @@ class ReverseDependenciesAutopkgtestWorkflowTests(
                     WorkRequest.objects.filter(
                         task_type=TaskTypes.WORKER,
                         task_name="sbuild",
-                        task_data__host_architecture__in={"all", "amd64"},
+                        task_data__build_architecture__in={"all", "amd64"},
                     )
                 ),
             )
@@ -838,8 +823,9 @@ class ReverseDependenciesAutopkgtestWorkflowTests(
         children = list(WorkRequest.objects.filter(parent=rdep_autopkgtest))
         self.assertEqual(len(children), 2)
 
-    def test_populate_passes_qa_results_parameters(self) -> None:
-        """The workflow passes on parameters related to QA results."""
+    @preserve_task_registry()
+    def test_populate_passes_regression_tracking_parameters(self) -> None:
+        """The workflow passes on parameters related to regression tracking."""
         self.playground.create_collection(
             name="sid", category=CollectionCategory.QA_RESULTS
         )
@@ -860,7 +846,8 @@ class ReverseDependenciesAutopkgtestWorkflowTests(
         )
         architectures = ("amd64", "i386")
 
-        rdep_autopkgtest = self.orchestrate(
+        root = self.playground.create_workflow(mark_running=True)
+        reference = self.orchestrate(
             source_artifact,
             architectures,
             extra_data={
@@ -868,16 +855,30 @@ class ReverseDependenciesAutopkgtestWorkflowTests(
                 "reference_qa_results": f"sid@{CollectionCategory.QA_RESULTS}",
                 "update_qa_results": True,
             },
+            parent=root,
+            pipeline_task_name="referencepipeline",
+        )
+        assert reference.parent is not None
+        new = self.orchestrate(
+            source_artifact,
+            architectures,
+            extra_data={
+                "reference_prefix": "reference-qa-result|",
+                "reference_qa_results": f"sid@{CollectionCategory.QA_RESULTS}",
+                "enable_regression_tracking": True,
+            },
+            parent=root,
         )
 
-        child = WorkRequest.objects.get(parent=rdep_autopkgtest)
-        self.assertEqual(child.status, WorkRequest.Statuses.RUNNING)
-        self.assertEqual(child.task_type, TaskTypes.WORKFLOW)
-        self.assertEqual(child.task_name, "autopkgtest")
+        reference_child = reference.children.get()
+        self.assertEqual(reference_child.status, WorkRequest.Statuses.RUNNING)
+        self.assertEqual(reference_child.task_type, TaskTypes.WORKFLOW)
+        self.assertEqual(reference_child.task_name, "autopkgtest")
         self.assertEqual(
-            child.task_data,
+            reference_child.task_data,
             {
                 "prefix": "reference-qa-result|depends_1.0|",
+                "reference_prefix": "depends_1.0|",
                 "source_artifact": (
                     f"sid@{CollectionCategory.SUITE}/source-version:depends_1.0"
                 ),
@@ -891,23 +892,28 @@ class ReverseDependenciesAutopkgtestWorkflowTests(
                 ],
                 "qa_suite": f"sid@{CollectionCategory.SUITE}",
                 "reference_qa_results": f"sid@{CollectionCategory.QA_RESULTS}",
+                "enable_regression_tracking": False,
                 "update_qa_results": True,
                 "vendor": "debian",
                 "codename": "sid",
                 "backend": BackendType.UNSHARE,
                 "architectures": [],
-                "arch_all_host_architecture": "amd64",
+                "arch_all_build_architecture": "amd64",
                 "debug_level": 0,
                 "extra_repositories": None,
             },
         )
 
-        grandchild = WorkRequest.objects.get(parent=child)
-        self.assertEqual(grandchild.status, WorkRequest.Statuses.BLOCKED)
-        self.assertEqual(grandchild.task_type, TaskTypes.WORKER)
-        self.assertEqual(grandchild.task_name, "autopkgtest")
+        reference_grandchild = reference_child.children.get(
+            task_type=TaskTypes.WORKER, task_name="autopkgtest"
+        )
         self.assertEqual(
-            grandchild.task_data,
+            reference_grandchild.status, WorkRequest.Statuses.BLOCKED
+        )
+        self.assertEqual(reference_grandchild.task_type, TaskTypes.WORKER)
+        self.assertEqual(reference_grandchild.task_name, "autopkgtest")
+        self.assertEqual(
+            reference_grandchild.task_data,
             {
                 "input": {
                     "source_artifact": (
@@ -922,7 +928,7 @@ class ReverseDependenciesAutopkgtestWorkflowTests(
                         for architecture in ("all", "amd64")
                     ],
                 },
-                "host_architecture": "amd64",
+                "build_architecture": "amd64",
                 "environment": "debian/match:codename=sid",
                 "backend": BackendType.UNSHARE,
                 "include_tests": [],
@@ -942,12 +948,12 @@ class ReverseDependenciesAutopkgtestWorkflowTests(
                 "version": "1.0",
                 "architecture": "amd64",
                 "timestamp": int(binary_item.created_at.timestamp()),
-                "work_request_id": grandchild.id,
+                "work_request_id": reference_grandchild.id,
             },
             artifact_filters={"category": ArtifactCategory.AUTOPKGTEST},
         )
         self.assert_work_request_event_reactions(
-            grandchild,
+            reference_grandchild,
             on_assignment=[
                 ActionSkipIfLookupResultChanged(
                     lookup=(
@@ -973,17 +979,15 @@ class ReverseDependenciesAutopkgtestWorkflowTests(
             ],
         )
         self.assertQuerySetEqual(
-            grandchild.dependencies.all(),
-            list(
-                WorkRequest.objects.filter(
-                    task_type=TaskTypes.WORKER,
-                    task_name="sbuild",
-                    task_data__host_architecture__in={"all", "amd64"},
-                )
+            reference_grandchild.dependencies.all(),
+            reference.parent.children.get(
+                task_type=TaskTypes.WORKFLOW, task_name="sbuild"
+            ).children.filter(
+                task_data__build_architecture__in={"all", "amd64"}
             ),
         )
         self.assertEqual(
-            grandchild.workflow_data_json,
+            reference_grandchild.workflow_data_json,
             {
                 "allow_failure": True,
                 "display_name": "autopkgtest amd64",
@@ -991,10 +995,105 @@ class ReverseDependenciesAutopkgtestWorkflowTests(
             },
         )
 
+        new_child = new.children.get()
+        self.assertEqual(new_child.status, WorkRequest.Statuses.RUNNING)
+        self.assertEqual(new_child.task_type, TaskTypes.WORKFLOW)
+        self.assertEqual(new_child.task_name, "autopkgtest")
+        self.assertEqual(
+            new_child.task_data,
+            {
+                "prefix": "depends_1.0|",
+                "reference_prefix": "reference-qa-result|depends_1.0|",
+                "source_artifact": (
+                    f"sid@{CollectionCategory.SUITE}/source-version:depends_1.0"
+                ),
+                "binary_artifacts": [
+                    f"sid@{CollectionCategory.SUITE}/"
+                    f"binary-version:depends_1.0_all"
+                ],
+                "context_artifacts": [
+                    f"internal@collections/name:build-{architecture}"
+                    for architecture in ("all", *architectures)
+                ],
+                "qa_suite": f"sid@{CollectionCategory.SUITE}",
+                "reference_qa_results": f"sid@{CollectionCategory.QA_RESULTS}",
+                "enable_regression_tracking": True,
+                "update_qa_results": False,
+                "vendor": "debian",
+                "codename": "sid",
+                "backend": BackendType.UNSHARE,
+                "architectures": [],
+                "arch_all_build_architecture": "amd64",
+                "debug_level": 0,
+                "extra_repositories": None,
+            },
+        )
+
+        new_grandchild = new_child.children.get(
+            task_type=TaskTypes.WORKER, task_name="autopkgtest"
+        )
+        self.assertEqual(new_grandchild.status, WorkRequest.Statuses.BLOCKED)
+        self.assertEqual(new_grandchild.task_type, TaskTypes.WORKER)
+        self.assertEqual(new_grandchild.task_name, "autopkgtest")
+        self.assertEqual(
+            new_grandchild.task_data,
+            {
+                "input": {
+                    "source_artifact": (
+                        f"sid@{CollectionCategory.SUITE}/"
+                        f"source-version:depends_1.0"
+                    ),
+                    "binary_artifacts": [
+                        f"sid@{CollectionCategory.SUITE}/name:depends_1.0_all"
+                    ],
+                    "context_artifacts": [
+                        f"internal@collections/name:build-{architecture}"
+                        for architecture in ("all", "amd64")
+                    ],
+                },
+                "build_architecture": "amd64",
+                "environment": "debian/match:codename=sid",
+                "backend": BackendType.UNSHARE,
+                "include_tests": [],
+                "exclude_tests": [],
+                "debug_level": 0,
+                "extra_environment": {},
+                "extra_repositories": None,
+                "needs_internet": AutopkgtestNeedsInternet.RUN,
+                "fail_on": {},
+                "timeout": None,
+            },
+        )
+        self.assert_work_request_event_reactions(
+            new_grandchild,
+            on_success=[
+                ActionUpdateCollectionWithArtifacts(
+                    collection="internal@collections",
+                    name_template="depends_1.0|autopkgtest-amd64",
+                    artifact_filters={"category": ArtifactCategory.AUTOPKGTEST},
+                )
+            ],
+        )
+        self.assertQuerySetEqual(
+            new_grandchild.dependencies.all(),
+            reference.parent.children.get(
+                task_type=TaskTypes.WORKFLOW, task_name="sbuild"
+            ).children.filter(
+                task_data__build_architecture__in={"all", "amd64"}
+            ),
+        )
+        self.assertEqual(
+            new_grandchild.workflow_data_json,
+            {"display_name": "autopkgtest amd64", "step": "autopkgtest-amd64"},
+        )
+
         # Population is idempotent.
-        self.get_workflow(rdep_autopkgtest).populate()
-        children = list(WorkRequest.objects.filter(parent=rdep_autopkgtest))
-        self.assertEqual(len(children), 1)
+        self.get_workflow(reference).populate()
+        reference_children = list(WorkRequest.objects.filter(parent=reference))
+        self.assertEqual(len(reference_children), 1)
+        self.get_workflow(new).populate()
+        new_children = list(WorkRequest.objects.filter(parent=new))
+        self.assertEqual(len(new_children), 1)
 
     def test_compute_dynamic_data(self) -> None:
         source_artifact = self.playground.create_source_artifact(
