@@ -76,10 +76,14 @@ class WorkflowUtilsTests(TestCase):
         workflow = work_request.get_task()
         assert isinstance(workflow, SbuildWorkflow)
 
-        artifact = workflow_utils.source_package(workflow)
+        artifact = workflow_utils.source_package(
+            workflow, configuration_key="input.source_artifact"
+        )
         self.assertEqual(artifact, source_package)
 
-        source_package_data = workflow_utils.source_package_data(workflow)
+        source_package_data = workflow_utils.source_package_data(
+            workflow, configuration_key="input.source_artifact"
+        )
         self.assertEqual(source_package_data.name, "hello")
         self.assertEqual(source_package_data.version, "1.0")
 
