@@ -11,6 +11,7 @@
 import functools
 from collections.abc import Collection as AbcCollection
 from collections.abc import Iterable, Sequence
+from operator import attrgetter
 from typing import Any, TYPE_CHECKING, get_args
 
 from django.db.models import QuerySet
@@ -57,27 +58,19 @@ if TYPE_CHECKING:
 
 
 @functools.lru_cache(maxsize=100)
-def source_package(workflow: "Workflow[Any, Any]") -> Artifact:
+def source_package(
+    workflow: "Workflow[Any, Any]",
+    *,
+    configuration_key: str = "source_artifact",
+) -> Artifact:
     """
-    Retrieve the source package artifact.
-
-    If ``workflow.data.input`` exists, use
-    ``workflow.data.input.source_artifact``, otherwise
-    ``workflow.data.source_artifact``.
+    Retrieve the source package artifact from ``configuration_key``.
 
     If the source artifact is a :artifact:`debian:upload`, returns its
     :artifact:`debian:source-package`.
     """
-    if hasattr(workflow.data, "input"):
-        lookup = workflow.data.input.source_artifact
-        configuration_key = "input.source_artifact"
-    else:
-        assert hasattr(workflow.data, "source_artifact")
-        lookup = workflow.data.source_artifact
-        configuration_key = "source_artifact"
-
     artifact = lookup_single(
-        lookup,
+        attrgetter(configuration_key)(workflow.data),
         workflow.workspace,
         user=workflow.work_request.created_by,
         workflow_root=workflow.work_request.workflow_root,
@@ -87,9 +80,15 @@ def source_package(workflow: "Workflow[Any, Any]") -> Artifact:
 
 
 @functools.lru_cache(maxsize=100)
-def source_package_data(workflow: "Workflow[Any, Any]") -> DebianSourcePackage:
+def source_package_data(
+    workflow: "Workflow[Any, Any]",
+    *,
+    configuration_key: str = "source_artifact",
+) -> DebianSourcePackage:
     """Return source package artifact data for the workflow."""
-    return SourcePackage.create_data(source_package(workflow).data)
+    return SourcePackage.create_data(
+        source_package(workflow, configuration_key=configuration_key).data
+    )
 
 
 def lookup_result_artifact_category(result: LookupResult) -> str:
