@@ -177,45 +177,57 @@ class KeyManagerTests(TestCase, DjangoTestCase):
         storage_private_key = PrivateKey.generate()
         log_file = io.BytesIO()
 
-        with (
-            mock.patch(
-                "gpg.Context.create_key", return_value=create_result
-            ) as mock_create_key,
-            mock.patch(
-                "gpg.Context.key_export_secret", return_value=secret_key
-            ) as mock_key_export_secret,
-            mock.patch(
-                "gpg.Context.key_export_minimal", return_value=public_key
-            ) as mock_key_export_minimal,
-            mock.patch(
-                "gpg.Context.key_import", return_value=import_result
-            ) as mock_key_import,
-            self.settings(DEBUSINE_SIGNING_PRIVATE_KEYS=[storage_private_key]),
+        for purpose in (
+            Key.Purpose.OPENPGP_UPLOAD,
+            Key.Purpose.OPENPGP_REPOSITORY,
         ):
-            key = Key.objects.generate(
-                Key.Purpose.OPENPGP, "test key", 123, log_file
-            )
+            with self.subTest(purpose=purpose.value):
+                with (
+                    mock.patch(
+                        "gpg.Context.create_key", return_value=create_result
+                    ) as mock_create_key,
+                    mock.patch(
+                        "gpg.Context.key_export_secret", return_value=secret_key
+                    ) as mock_key_export_secret,
+                    mock.patch(
+                        "gpg.Context.key_export_minimal",
+                        return_value=public_key,
+                    ) as mock_key_export_minimal,
+                    mock.patch(
+                        "gpg.Context.key_import", return_value=import_result
+                    ) as mock_key_import,
+                    self.settings(
+                        DEBUSINE_SIGNING_PRIVATE_KEYS=[storage_private_key]
+                    ),
+                ):
+                    key = Key.objects.generate(
+                        purpose, "test key", 123, log_file
+                    )
 
-        mock_create_key.assert_called_once_with(
-            "test key", algorithm="ed25519", expires=False, sign=True
-        )
-        mock_key_export_secret.assert_called_once_with(fingerprint)
-        mock_key_export_minimal.assert_called_once_with(fingerprint)
-        mock_key_import.assert_called_once_with(public_key)
-        self.assertEqual(key.purpose, Key.Purpose.OPENPGP)
-        self.assertEqual(key.fingerprint, fingerprint)
-        assert isinstance(key.stored_private_key.__root__, ProtectedKeyNaCl)
-        self.assertEqual(
-            key.stored_private_key.__root__.decrypt([storage_private_key]),
-            secret_key,
-        )
-        self.assertEqual(key.public_key, public_key)
-        audit_log = AuditLog.objects.latest("created_at")
-        self.assertEqual(audit_log.purpose, Key.Purpose.OPENPGP)
-        self.assertEqual(audit_log.fingerprint, fingerprint)
-        self.assertEqual(audit_log.event, AuditLog.Event.GENERATE)
-        self.assertEqual(audit_log.data, {"description": "test key"})
-        self.assertEqual(audit_log.created_by_work_request_id, 123)
+                mock_create_key.assert_called_once_with(
+                    "test key", algorithm="ed25519", expires=False, sign=True
+                )
+                mock_key_export_secret.assert_called_once_with(fingerprint)
+                mock_key_export_minimal.assert_called_once_with(fingerprint)
+                mock_key_import.assert_called_once_with(public_key)
+                self.assertEqual(key.purpose, purpose)
+                self.assertEqual(key.fingerprint, fingerprint)
+                assert isinstance(
+                    key.stored_private_key.__root__, ProtectedKeyNaCl
+                )
+                self.assertEqual(
+                    key.stored_private_key.__root__.decrypt(
+                        [storage_private_key]
+                    ),
+                    secret_key,
+                )
+                self.assertEqual(key.public_key, public_key)
+                audit_log = AuditLog.objects.latest("created_at")
+                self.assertEqual(audit_log.purpose, purpose)
+                self.assertEqual(audit_log.fingerprint, fingerprint)
+                self.assertEqual(audit_log.event, AuditLog.Event.GENERATE)
+                self.assertEqual(audit_log.data, {"description": "test key"})
+                self.assertEqual(audit_log.created_by_work_request_id, 123)
 
     def test_generate_openpgp_create_key_no_primary(self) -> None:
         """Key generation error: failed to generate an OpenPGP secret key."""
@@ -229,7 +241,9 @@ class KeyManagerTests(TestCase, DjangoTestCase):
                 GnuPGError, "Failed to generate a secret key"
             ),
         ):
-            Key.objects.generate(Key.Purpose.OPENPGP, "test key", 123, log_file)
+            Key.objects.generate(
+                Key.Purpose.OPENPGP_UPLOAD, "test key", 123, log_file
+            )
 
     def test_generate_openpgp_create_key_unexpected_subkey(self) -> None:
         """Key generation error: unexpectedly got OpenPGP encryption subkey."""
@@ -244,7 +258,9 @@ class KeyManagerTests(TestCase, DjangoTestCase):
                 GnuPGError, "Unexpectedly got an encryption subkey"
             ),
         ):
-            Key.objects.generate(Key.Purpose.OPENPGP, "test key", 123, log_file)
+            Key.objects.generate(
+                Key.Purpose.OPENPGP_UPLOAD, "test key", 123, log_file
+            )
 
     def test_generate_openpgp_key_export_secret_failed(self) -> None:
         """Key generation error: failed to export OpenPGP secret key."""
@@ -260,7 +276,9 @@ class KeyManagerTests(TestCase, DjangoTestCase):
             mock.patch("gpg.Context.key_export_secret", return_value=None),
             self.assertRaisesRegex(GnuPGError, "Failed to export secret key"),
         ):
-            Key.objects.generate(Key.Purpose.OPENPGP, "test key", 123, log_file)
+            Key.objects.generate(
+                Key.Purpose.OPENPGP_UPLOAD, "test key", 123, log_file
+            )
 
     def test_generate_openpgp_key_export_public_failed(self) -> None:
         """Key generation error: failed to export OpenPGP public key."""
@@ -280,7 +298,9 @@ class KeyManagerTests(TestCase, DjangoTestCase):
             mock.patch("gpg.Context.key_export_minimal", return_value=None),
             self.assertRaisesRegex(GnuPGError, "Failed to export public key"),
         ):
-            Key.objects.generate(Key.Purpose.OPENPGP, "test key", 123, log_file)
+            Key.objects.generate(
+                Key.Purpose.OPENPGP_UPLOAD, "test key", 123, log_file
+            )
 
     def test_generate_openpgp_key_import_failed(self) -> None:
         """Key generation error: failed to import OpenPGP public key."""
@@ -308,7 +328,9 @@ class KeyManagerTests(TestCase, DjangoTestCase):
                 GnuPGError, "Failed to get fingerprint of new key"
             ),
         ):
-            Key.objects.generate(Key.Purpose.OPENPGP, "test key", 123, log_file)
+            Key.objects.generate(
+                Key.Purpose.OPENPGP_UPLOAD, "test key", 123, log_file
+            )
 
     def test_sign_uefi_attached(self) -> None:
         """Sign data using a UEFI key, returning an attached signature."""
@@ -456,57 +478,66 @@ class KeyManagerTests(TestCase, DjangoTestCase):
         # sequence of operations.  Integration tests will make sure that it
         # actually works.
         storage_private_key = PrivateKey.generate()
-        key, secret_key = self.create_nacl_key(
-            storage_private_key=storage_private_key, purpose=Key.Purpose.OPENPGP
-        )
-        import_status = ImportStatus(None)
-        import_status.fpr = key.fingerprint
-        import_result = ImportResult(None)
-        import_result.secret_imported = 1
-        import_result.imports = [import_status]
-        keylist = [object()]
-        temp_path = self.create_temporary_directory()
-        data_bytes = b"data to sign"
-        (data_path := temp_path / "data").write_bytes(data_bytes)
-        signature_path = temp_path / "data.asc"
-        signature_bytes = random.randbytes(64)
-
-        def gpg_sign(
-            ctx: gpg.Context,
-            data: gpg.Data,
-            sink: BinaryIO | None = None,
-            mode: int = gpg.constants.SIG_MODE_NORMAL,
-        ) -> None:
-            self.assertTrue(ctx.armor)
-            self.assertEqual(data.read(), data_bytes)
-            self.assertEqual(mode, gpg.constants.SIG_MODE_CLEAR)
-            assert sink is not None
-            sink.write(signature_bytes)
-
-        with (
-            self.settings(DEBUSINE_SIGNING_PRIVATE_KEYS=[storage_private_key]),
-            mock.patch(
-                "gpg.Context.key_import", return_value=import_result
-            ) as mock_key_import,
-            mock.patch(
-                "gpg.Context.keylist", return_value=iter(keylist)
-            ) as mock_keylist,
-            mock.patch(
-                "gpg.Context.signers", new_callable=mock.PropertyMock
-            ) as mock_signers,
-            mock.patch(
-                "gpg.Context.sign", autospec=True, side_effect=gpg_sign
-            ) as mock_sign,
+        for purpose in (
+            Key.Purpose.OPENPGP_UPLOAD,
+            Key.Purpose.OPENPGP_REPOSITORY,
         ):
-            sign([key], data_path, signature_path, SigningMode.CLEAR, 123)
+            with self.subTest(purpose=purpose.value):
+                key, secret_key = self.create_nacl_key(
+                    storage_private_key=storage_private_key, purpose=purpose
+                )
+                import_status = ImportStatus(None)
+                import_status.fpr = key.fingerprint
+                import_result = ImportResult(None)
+                import_result.secret_imported = 1
+                import_result.imports = [import_status]
+                keylist = [object()]
+                temp_path = self.create_temporary_directory()
+                data_bytes = b"data to sign"
+                (data_path := temp_path / "data").write_bytes(data_bytes)
+                signature_path = temp_path / "data.asc"
+                signature_bytes = random.randbytes(64)
 
-        mock_key_import.assert_called_once_with(secret_key)
-        mock_keylist.assert_called_once_with(
-            pattern=key.fingerprint, secret=True
-        )
-        mock_signers.assert_called_with(keylist)
-        mock_sign.assert_called_once()
-        self.assertEqual(signature_path.read_bytes(), signature_bytes)
+                def gpg_sign(
+                    ctx: gpg.Context,
+                    data: gpg.Data,
+                    sink: BinaryIO | None = None,
+                    mode: int = gpg.constants.SIG_MODE_NORMAL,
+                ) -> None:
+                    self.assertTrue(ctx.armor)
+                    self.assertEqual(data.read(), data_bytes)
+                    self.assertEqual(mode, gpg.constants.SIG_MODE_CLEAR)
+                    assert sink is not None
+                    sink.write(signature_bytes)
+
+                with (
+                    self.settings(
+                        DEBUSINE_SIGNING_PRIVATE_KEYS=[storage_private_key]
+                    ),
+                    mock.patch(
+                        "gpg.Context.key_import", return_value=import_result
+                    ) as mock_key_import,
+                    mock.patch(
+                        "gpg.Context.keylist", return_value=iter(keylist)
+                    ) as mock_keylist,
+                    mock.patch(
+                        "gpg.Context.signers", new_callable=mock.PropertyMock
+                    ) as mock_signers,
+                    mock.patch(
+                        "gpg.Context.sign", autospec=True, side_effect=gpg_sign
+                    ) as mock_sign,
+                ):
+                    sign(
+                        [key], data_path, signature_path, SigningMode.CLEAR, 123
+                    )
+
+                mock_key_import.assert_called_once_with(secret_key)
+                mock_keylist.assert_called_once_with(
+                    pattern=key.fingerprint, secret=True
+                )
+                mock_signers.assert_called_with(keylist)
+                mock_sign.assert_called_once()
+                self.assertEqual(signature_path.read_bytes(), signature_bytes)
 
     def test_sign_openpgp_detached(self) -> None:
         """Sign data using an OpenPGP key, returning a detached signature."""
@@ -514,57 +545,70 @@ class KeyManagerTests(TestCase, DjangoTestCase):
         # sequence of operations.  Integration tests will make sure that it
         # actually works.
         storage_private_key = PrivateKey.generate()
-        key, secret_key = self.create_nacl_key(
-            storage_private_key=storage_private_key, purpose=Key.Purpose.OPENPGP
-        )
-        import_status = ImportStatus(None)
-        import_status.fpr = key.fingerprint
-        import_result = ImportResult(None)
-        import_result.secret_imported = 1
-        import_result.imports = [import_status]
-        keylist = [object()]
-        temp_path = self.create_temporary_directory()
-        data_bytes = b"data to sign"
-        (data_path := temp_path / "data").write_bytes(data_bytes)
-        signature_path = temp_path / "data.asc"
-        signature_bytes = random.randbytes(64)
-
-        def gpg_sign(
-            ctx: gpg.Context,
-            data: gpg.Data,
-            sink: BinaryIO | None = None,
-            mode: int = gpg.constants.SIG_MODE_NORMAL,
-        ) -> None:
-            self.assertTrue(ctx.armor)
-            self.assertEqual(data.read(), data_bytes)
-            self.assertEqual(mode, gpg.constants.SIG_MODE_DETACH)
-            assert sink is not None
-            sink.write(signature_bytes)
-
-        with (
-            self.settings(DEBUSINE_SIGNING_PRIVATE_KEYS=[storage_private_key]),
-            mock.patch(
-                "gpg.Context.key_import", return_value=import_result
-            ) as mock_key_import,
-            mock.patch(
-                "gpg.Context.keylist", return_value=iter(keylist)
-            ) as mock_keylist,
-            mock.patch(
-                "gpg.Context.signers", new_callable=mock.PropertyMock
-            ) as mock_signers,
-            mock.patch(
-                "gpg.Context.sign", autospec=True, side_effect=gpg_sign
-            ) as mock_sign,
+        for purpose in (
+            Key.Purpose.OPENPGP_UPLOAD,
+            Key.Purpose.OPENPGP_REPOSITORY,
         ):
-            sign([key], data_path, signature_path, SigningMode.DETACHED, 123)
+            with self.subTest(purpose=purpose.value):
+                key, secret_key = self.create_nacl_key(
+                    storage_private_key=storage_private_key, purpose=purpose
+                )
+                import_status = ImportStatus(None)
+                import_status.fpr = key.fingerprint
+                import_result = ImportResult(None)
+                import_result.secret_imported = 1
+                import_result.imports = [import_status]
+                keylist = [object()]
+                temp_path = self.create_temporary_directory()
+                data_bytes = b"data to sign"
+                (data_path := temp_path / "data").write_bytes(data_bytes)
+                signature_path = temp_path / "data.asc"
+                signature_bytes = random.randbytes(64)
 
-        mock_key_import.assert_called_once_with(secret_key)
-        mock_keylist.assert_called_once_with(
-            pattern=key.fingerprint, secret=True
-        )
-        mock_signers.assert_called_with(keylist)
-        mock_sign.assert_called_once()
-        self.assertEqual(signature_path.read_bytes(), signature_bytes)
+                def gpg_sign(
+                    ctx: gpg.Context,
+                    data: gpg.Data,
+                    sink: BinaryIO | None = None,
+                    mode: int = gpg.constants.SIG_MODE_NORMAL,
+                ) -> None:
+                    self.assertTrue(ctx.armor)
+                    self.assertEqual(data.read(), data_bytes)
+                    self.assertEqual(mode, gpg.constants.SIG_MODE_DETACH)
+                    assert sink is not None
+                    sink.write(signature_bytes)
+
+                with (
+                    self.settings(
+                        DEBUSINE_SIGNING_PRIVATE_KEYS=[storage_private_key]
+                    ),
+                    mock.patch(
+                        "gpg.Context.key_import", return_value=import_result
+                    ) as mock_key_import,
+                    mock.patch(
+                        "gpg.Context.keylist", return_value=iter(keylist)
+                    ) as mock_keylist,
+                    mock.patch(
+                        "gpg.Context.signers", new_callable=mock.PropertyMock
+                    ) as mock_signers,
+                    mock.patch(
+                        "gpg.Context.sign", autospec=True, side_effect=gpg_sign
+                    ) as mock_sign,
+                ):
+                    sign(
+                        [key],
+                        data_path,
+                        signature_path,
+                        SigningMode.DETACHED,
+                        123,
+                    )
+
+                mock_key_import.assert_called_once_with(secret_key)
+                mock_keylist.assert_called_once_with(
+                    pattern=key.fingerprint, secret=True
+                )
+                mock_signers.assert_called_with(keylist)
+                mock_sign.assert_called_once()
+                self.assertEqual(signature_path.read_bytes(), signature_bytes)
 
     def test_sign_openpgp_multiple(self) -> None:
         """Sign data using multiple OpenPGP keys."""
@@ -575,7 +619,7 @@ class KeyManagerTests(TestCase, DjangoTestCase):
         key_pairs = [
             self.create_nacl_key(
                 storage_private_key=storage_private_key,
-                purpose=Key.Purpose.OPENPGP,
+                purpose=Key.Purpose.OPENPGP_REPOSITORY,
             )
             for _ in range(2)
         ]
@@ -649,7 +693,8 @@ class KeyManagerTests(TestCase, DjangoTestCase):
         """Key signing error: signing mode not detached or clear."""
         storage_private_key = PrivateKey.generate()
         key, secret_key = self.create_nacl_key(
-            storage_private_key=storage_private_key, purpose=Key.Purpose.OPENPGP
+            storage_private_key=storage_private_key,
+            purpose=Key.Purpose.OPENPGP_UPLOAD,
         )
         temp_path = self.create_temporary_directory()
         (data_path := temp_path / "data").write_bytes(b"data to sign")
@@ -674,7 +719,7 @@ class KeyManagerTests(TestCase, DjangoTestCase):
         public_key = random.randbytes(64)
         fingerprint = make_fake_fingerprint()
         key = Key.objects.create(
-            purpose=Key.Purpose.OPENPGP,
+            purpose=Key.Purpose.OPENPGP_UPLOAD,
             fingerprint=fingerprint,
             private_key=ProtectedKeyPKCS11Static.create(
                 pkcs11_uri="pkcs11:id=1"
@@ -698,7 +743,8 @@ class KeyManagerTests(TestCase, DjangoTestCase):
         # actually works.
         storage_private_key = PrivateKey.generate()
         key, secret_key = self.create_nacl_key(
-            storage_private_key=storage_private_key, purpose=Key.Purpose.OPENPGP
+            storage_private_key=storage_private_key,
+            purpose=Key.Purpose.OPENPGP_UPLOAD,
         )
         import_status = ImportStatus(None)
         import_status.fpr = key.fingerprint
@@ -722,7 +768,8 @@ class KeyManagerTests(TestCase, DjangoTestCase):
         # actually works.
         storage_private_key = PrivateKey.generate()
         key, secret_key = self.create_nacl_key(
-            storage_private_key=storage_private_key, purpose=Key.Purpose.OPENPGP
+            storage_private_key=storage_private_key,
+            purpose=Key.Purpose.OPENPGP_UPLOAD,
         )
         import_status = ImportStatus(None)
         import_status.fpr = key.fingerprint
@@ -789,7 +836,7 @@ class KeyManagerTests(TestCase, DjangoTestCase):
         public_key = random.randbytes(64)
         fingerprint = make_fake_fingerprint()
         key = Key.objects.create(
-            purpose=Key.Purpose.OPENPGP,
+            purpose=Key.Purpose.OPENPGP_UPLOAD,
             fingerprint=fingerprint,
             private_key=ProtectedKeyPKCS11Static.create(
                 pkcs11_uri="pkcs11:id=1"
@@ -822,7 +869,8 @@ class KeyManagerTests(TestCase, DjangoTestCase):
         # actually works.
         storage_private_key = PrivateKey.generate()
         key, secret_key = self.create_nacl_key(
-            storage_private_key=storage_private_key, purpose=Key.Purpose.OPENPGP
+            storage_private_key=storage_private_key,
+            purpose=Key.Purpose.OPENPGP_UPLOAD,
         )
         import_status = ImportStatus(None)
         import_status.fpr = key.fingerprint
@@ -855,7 +903,8 @@ class KeyManagerTests(TestCase, DjangoTestCase):
         # actually works.
         storage_private_key = PrivateKey.generate()
         key, secret_key = self.create_nacl_key(
-            storage_private_key=storage_private_key, purpose=Key.Purpose.OPENPGP
+            storage_private_key=storage_private_key,
+            purpose=Key.Purpose.OPENPGP_UPLOAD,
         )
         import_status = ImportStatus(None)
         import_status.fpr = key.fingerprint
