@@ -18,6 +18,7 @@ from debusine.artifacts.models import ArtifactCategory, CollectionCategory
 from debusine.assets import AssetCategory, KeyPurpose
 from debusine.client.models import AssetPermissionCheckResponse, RelationType
 from debusine.signing.db.models import Key, sign
+from debusine.signing.models import SigningMode
 from debusine.signing.tasks import BaseSigningTask
 from debusine.signing.tasks.models import (
     SignRepositoryIndexData,
@@ -218,8 +219,23 @@ class SignRepositoryIndex(
         # latter case we won't get here.
         assert execution_success
 
+        content_type: str | None
+        match self.data.mode:
+            case SigningMode.DETACHED:
+                content_type = "application/pgp-signature; charset=utf-8"
+            case SigningMode.CLEAR:
+                # This is unregistered in
+                # https://www.iana.org/assignments/media-types/media-types.xhtml,
+                # but it's what file(1) claims.
+                content_type = "text/PGP; charset=utf-8"
+            case _:  # pragma: no cover
+                # Checked by SignRepositoryIndexData.validate_mode.
+                raise AssertionError(f"Unexpected mode: {self.data.mode}")
+
         signed_artifact = RepositoryIndex.create(
-            file=self._signed_path, path=self.data.signed_name
+            file=self._signed_path,
+            path=self.data.signed_name,
+            content_type=content_type,
         )
         uploaded_signed_artifact = self.debusine.upload_artifact(
             signed_artifact,
