@@ -196,10 +196,14 @@ class SignTests(SignTestMixin, TestCase, DjangoTestCase):
             )
         )
 
-    def test_can_run_on_openpgp(self) -> None:
-        """can_run_on always returns True for OpenPGP."""
+    def test_can_run_on_openpgp_upload(self) -> None:
+        """can_run_on always returns True for OpenPGP upload keys."""
         task = Sign(
-            task_data={"purpose": KeyPurpose.OPENPGP, "unsigned": [2], "key": 1}
+            task_data={
+                "purpose": KeyPurpose.OPENPGP_UPLOAD,
+                "unsigned": [2],
+                "key": 1,
+            }
         )
         self.assertTrue(
             task.can_run_on(
@@ -410,7 +414,7 @@ class SignTests(SignTestMixin, TestCase, DjangoTestCase):
 
         task = Sign(
             task_data={
-                "purpose": KeyPurpose.OPENPGP,
+                "purpose": KeyPurpose.OPENPGP_UPLOAD,
                 "unsigned": [signing_input_id],
                 "key": fingerprint,
             },
@@ -978,8 +982,8 @@ class SignIntegrationTests(SignTestMixin, TestCase, TransactionTestCase):
                 stdout=subprocess.DEVNULL,
             )
 
-    def test_openpgp(self) -> None:
-        """Integration test: sign using a real OpenPGP key."""
+    def test_openpgp_upload(self) -> None:
+        """Integration test: sign using a real OpenPGP upload key."""
         service_private_key = PrivateKey.generate()
         with (
             override_settings(
@@ -988,7 +992,7 @@ class SignIntegrationTests(SignTestMixin, TestCase, TransactionTestCase):
             open(os.devnull, "wb") as log_file,
         ):
             key = Key.objects.generate(
-                Key.Purpose.OPENPGP, "An OpenPGP key", 1, log_file
+                Key.Purpose.OPENPGP_UPLOAD, "An OpenPGP upload key", 1, log_file
             )
         temp_path = self.create_temporary_directory()
         (changes_path := temp_path / "foo.changes").write_text(
@@ -1022,7 +1026,7 @@ class SignIntegrationTests(SignTestMixin, TestCase, TransactionTestCase):
 
         task = Sign(
             task_data={
-                "purpose": KeyPurpose.OPENPGP,
+                "purpose": KeyPurpose.OPENPGP_UPLOAD,
                 "unsigned": [signing_input_id],
                 "key": key.fingerprint,
             },
@@ -1073,7 +1077,7 @@ class SignIntegrationTests(SignTestMixin, TestCase, TransactionTestCase):
                             "foo.changes.sig": "application/octet-stream"
                         },
                         data=DebusineSigningOutput(
-                            purpose=KeyPurpose.OPENPGP,
+                            purpose=KeyPurpose.OPENPGP_UPLOAD,
                             fingerprint=key.fingerprint,
                             results=[
                                 SigningResult(
