@@ -65,7 +65,10 @@ class SignRepositoryIndexTests(SignTestMixin, TestCase, DjangoTestCase):
             single_collection_lookups={
                 (f"sid@{CollectionCategory.SUITE}", None): CollectionInfo(
                     id=1,
+                    scope_name="scope",
+                    workspace_name="workspace",
                     category=CollectionCategory.SUITE,
+                    name="sid",
                     data={"signing_keys": ["ABC123"]},
                 )
             },
@@ -97,7 +100,12 @@ class SignRepositoryIndexTests(SignTestMixin, TestCase, DjangoTestCase):
             },
             single_collection_lookups={
                 ("sid", None): CollectionInfo(
-                    id=1, category=CollectionCategory.TEST, data={}
+                    id=1,
+                    scope_name="scope",
+                    workspace_name="workspace",
+                    category=CollectionCategory.TEST,
+                    name="sid",
+                    data={},
                 )
             },
         )
@@ -130,11 +138,19 @@ class SignRepositoryIndexTests(SignTestMixin, TestCase, DjangoTestCase):
             single_collection_lookups={
                 (f"_@{CollectionCategory.ARCHIVE}", None): CollectionInfo(
                     id=1,
+                    scope_name="scope",
+                    workspace_name="workspace",
                     category=CollectionCategory.ARCHIVE,
+                    name="_",
                     data={"signing_keys": ["ABC123"]},
                 ),
                 (f"sid@{CollectionCategory.SUITE}", None): CollectionInfo(
-                    id=2, category=CollectionCategory.SUITE, data={}
+                    id=2,
+                    scope_name="scope",
+                    workspace_name="workspace",
+                    category=CollectionCategory.SUITE,
+                    name="sid",
+                    data={},
                 ),
             },
         )
@@ -165,7 +181,12 @@ class SignRepositoryIndexTests(SignTestMixin, TestCase, DjangoTestCase):
             },
             single_collection_lookups={
                 (f"sid@{CollectionCategory.SUITE}", None): CollectionInfo(
-                    id=1, category=CollectionCategory.SUITE, data={}
+                    id=1,
+                    scope_name="scope",
+                    workspace_name="workspace",
+                    category=CollectionCategory.SUITE,
+                    name="sid",
+                    data={},
                 ),
             },
         )
@@ -547,7 +568,9 @@ class SignRepositoryIndexIntegrationTests(
                     local_artifact.files.items()
                 )
                 uploaded_signed = RepositoryIndex.create(
-                    file=signed_path, path=signed_path_in_suite
+                    file=signed_path,
+                    path=signed_path_in_suite,
+                    content_type="application/pgp-signature; charset=utf-8",
                 )
             return create_remote_artifact(
                 id=uploaded_artifact_ids[local_artifact.category],
@@ -695,7 +718,9 @@ class SignRepositoryIndexIntegrationTests(
                     local_artifact.files.items()
                 )
                 uploaded_signed = RepositoryIndex.create(
-                    file=signed_path, path=signed_path_in_suite
+                    file=signed_path,
+                    path=signed_path_in_suite,
+                    content_type="text/PGP; charset=utf-8",
                 )
             return create_remote_artifact(
                 id=uploaded_artifact_ids[local_artifact.category],
