@@ -12,7 +12,7 @@ import logging
 import re
 import shlex
 from pathlib import Path
-from typing import Any, TypeAlias, cast
+from typing import Any, cast
 
 try:
     import pydantic.v1 as pydantic
@@ -33,18 +33,23 @@ from debusine.artifacts.models import (
     DebianSystemTarball,
     DebianUpload,
     get_source_package_name,
+    get_source_package_version,
 )
 from debusine.tasks import (
     BaseTaskWithExecutor,
     ExtraRepositoryMixin,
     RunCommandTask,
 )
-from debusine.tasks.models import AutopkgtestData, AutopkgtestDynamicData
+from debusine.tasks.models import (
+    AutopkgtestData,
+    AutopkgtestDynamicData,
+    BackendType,
+)
 from debusine.tasks.server import TaskDatabaseInterface
 
 log = logging.getLogger(__name__)
 
-ParsedSummaryFile: TypeAlias = dict[str, DebianAutopkgtestResult]
+type ParsedSummaryFile = dict[str, DebianAutopkgtestResult]
 
 
 class Autopkgtest(
@@ -104,7 +109,8 @@ class Autopkgtest(
         Resolve artifact lookups for this task.
 
         :subject: source package name out of ``input:source_artifact``
-        :runtime_context: ``host_architecture:executor_backend``
+        :parameter_summary: ``source_package_name_version (architecture)``
+        :runtime_context: ``build_architecture:executor_backend``
         :configuration_context: codename of ``environment``
         """
         input_source_artifact = task_database.lookup_single_artifact(
@@ -126,6 +132,7 @@ class Autopkgtest(
             input_source_artifact.data, (DebianSourcePackage, DebianUpload)
         )
         package_name = get_source_package_name(input_source_artifact.data)
+        package_version = get_source_package_version(input_source_artifact.data)
 
         for i, binary_artifact in enumerate(input_binary_artifacts):
             self.ensure_artifact_categories(
@@ -133,7 +140,6 @@ class Autopkgtest(
                 category=binary_artifact.category,
                 expected=(
                     ArtifactCategory.BINARY_PACKAGE,
-                    ArtifactCategory.BINARY_PACKAGES,
                     ArtifactCategory.UPLOAD,
                 ),
             )
@@ -144,7 +150,6 @@ class Autopkgtest(
                 category=context_artifact.category,
                 expected=(
                     ArtifactCategory.BINARY_PACKAGE,
-                    ArtifactCategory.BINARY_PACKAGES,
                     ArtifactCategory.UPLOAD,
                 ),
             )
@@ -168,11 +173,17 @@ class Autopkgtest(
 
         return AutopkgtestDynamicData(
             environment_id=environment.id,
+            extra_repositories=self.resolve_extra_repositories(task_database),
             input_source_artifact_id=input_source_artifact.id,
             input_binary_artifacts_ids=input_binary_artifacts.get_ids(),
             input_context_artifacts_ids=input_context_artifacts.get_ids(),
             subject=package_name,
-            runtime_context=f"{self.host_architecture()}:{self.backend}",
+            # TODO: It would probably be better to get the architecture of
+            # the input binary artifacts and use that in the parameter
+            # summary.
+            parameter_summary=f"{package_name}_{package_version} "
+            f"({self.data.build_architecture})",
+            runtime_context=f"{self.build_architecture()}:{self.backend}",
             configuration_context=environment.data.codename,
         )
 
@@ -337,7 +348,10 @@ class Autopkgtest(
                 if timeout is not None:
                     cmd.append(f"--timeout-{key}={timeout}")
 
-        if self.backend == "unshare" and Path("/etc/resolv.conf").is_file():
+        if (
+            self.backend == BackendType.UNSHARE
+            and Path("/etc/resolv.conf").is_file()
+        ):
             # autopkgtest >= 5.31 copies /etc/resolv.conf from the host, but
             # earlier versions don't.  Force this, since we can't rely on
             # resolver configuration built into the system tarball.
@@ -470,7 +484,11 @@ class Autopkgtest(
                         pydantic.AnyUrl, self._source_package_url
                     ),
                 ),
-                architecture=self.data.host_architecture,
+                # TODO: Should this be the architecture of the binary
+                # packages instead?  If we're testing `Architecture: all`
+                # binary packages on amd64, "all" seems like a better value
+                # here.
+                architecture=self.data.build_architecture,
                 distribution=f"{img_data['vendor']}:{img_data['codename']}",
             ),
         )
@@ -480,11 +498,3 @@ class Autopkgtest(
             workspace=self.workspace_name,
             work_request=self.work_request_id,
         )
-
-    def get_label(self) -> str:
-        """Return the task label."""
-        return (
-            f"autopkgtest {subject}"
-            if (subject := self.get_subject())
-            else "autopkgtest"
-        )
