@@ -8,6 +8,7 @@
 # contained in the LICENSE file.
 
 """Interface for executor backends."""
+
 import os
 from abc import ABC, abstractmethod
 from enum import StrEnum
@@ -19,9 +20,9 @@ from typing import Any, AnyStr, ClassVar, Literal, Optional, overload
 from debusine.artifacts.models import ArtifactCategory
 from debusine.client.debusine import Debusine
 from debusine.client.models import ArtifactResponse
-from debusine.tasks.models import BaseTaskDataModel
+from debusine.tasks.models import BackendType, BaseTaskDataModel
 
-_backends: dict[str, type["ExecutorInterface"]] = {}
+_backends: dict[BackendType, type["ExecutorInterface"]] = {}
 
 
 class InstanceRunning(Exception):
@@ -83,10 +84,12 @@ class ExecutorInterface(ABC):
     """
 
     system_image: ArtifactResponse
-    backend_name: str
+    backend_name: BackendType
     image_category: ClassVar[ExecutorImageCategory]
 
-    def __init_subclass__(cls, /, backend_name: str, **kwargs: Any) -> None:
+    def __init_subclass__(
+        cls, /, backend_name: BackendType, **kwargs: Any
+    ) -> None:
         """Register the backend in _backends."""
         super().__init_subclass__(**kwargs)
         cls.backend_name = backend_name
@@ -335,8 +338,7 @@ class InstanceInterface(ABC):
         mode = S_IMODE(source.stat().st_mode)
         self.do_mkdir(target / source.name, uid, gid, mode, parents=False)
 
-        for absolute_path_str, dir_names, file_names in os.walk(source):
-            dir_path = Path(absolute_path_str)
+        for dir_path, dir_names, file_names in source.walk():
             target_relative_dir_path = dir_path.relative_to(source.parent)
 
             for dir_name in dir_names:
@@ -412,6 +414,7 @@ class InstanceInterface(ABC):
         args: list[str],
         *,
         text: Literal[True],
+        cwd: str | os.PathLike[str] | None = None,
         run_as_root: bool = False,
         **kwargs: Any,
     ) -> CompletedProcess[str]: ...
@@ -422,6 +425,7 @@ class InstanceInterface(ABC):
         args: list[str],
         *,
         text: Literal[False] | None = None,
+        cwd: str | os.PathLike[str] | None = None,
         run_as_root: bool = False,
         **kwargs: Any,
     ) -> CompletedProcess[bytes]: ...
@@ -450,6 +454,7 @@ class InstanceInterface(ABC):
         self,
         args: list[str],
         text: Literal[True],
+        cwd: str | os.PathLike[str] | None = None,
         run_as_root: bool = False,
         **kwargs: Any,
     ) -> CompletedProcess[str]: ...
@@ -460,6 +465,7 @@ class InstanceInterface(ABC):
         self,
         args: list[str],
         text: Literal[False] | None = None,
+        cwd: str | os.PathLike[str] | None = None,
         run_as_root: bool = False,
         **kwargs: Any,
     ) -> CompletedProcess[bytes]: ...
@@ -469,6 +475,7 @@ class InstanceInterface(ABC):
         self,
         args: list[str],
         text: Literal[False] | Literal[True] | None = None,
+        cwd: str | os.PathLike[str] | None = None,
         run_as_root: bool = False,
         **kwargs: Any,
     ) -> CompletedProcess[AnyStr]:
@@ -540,7 +547,7 @@ class InstanceInterface(ABC):
 
 
 def executor_class(
-    backend: str,
+    backend: BackendType,
 ) -> type[ExecutorInterface]:
     """
     Get the Executor class for a backend name.
@@ -556,7 +563,7 @@ def executor_class(
     return _backends[backend]
 
 
-def executor_backends() -> list[str]:
+def executor_backends() -> list[BackendType]:
     """List all registered executor backends."""
     return list(_backends.keys())
 
