@@ -10,14 +10,14 @@
 """The incus executor backend."""
 import importlib.resources
 import logging
+import os
 import random
 import string
 import subprocess
 import tarfile
 import textwrap
-import time
+import time as tm
 from abc import abstractmethod
-from copy import copy
 from io import BytesIO
 from pathlib import Path, PurePath
 from subprocess import (
@@ -46,6 +46,7 @@ from debusine.tasks.executors.base import (
     InstanceNotRunning,
 )
 from debusine.tasks.executors.images import ImageCache
+from debusine.tasks.models import BackendType
 
 # https://linuxcontainers.org/incus/docs/main/architectures/
 DEB_TO_INCUS_ARCH = {
@@ -254,7 +255,7 @@ class IncusImageCache(ImageCache):
 class IncusLXCImageCache(IncusImageCache):
     """Process LXC container images on import."""
 
-    backend = "incus-lxc"
+    backend = BackendType.INCUS_LXC
 
     def prepare_incus_image(
         self, source: Path, dest: TarFile, artifact: ArtifactResponse
@@ -288,9 +289,7 @@ class IncusLXCImageCache(IncusImageCache):
                 if dest_path in parent_directories:
                     parent_directories[dest_path] = True
 
-                # Can be replaced with entry.replace(name=...) in Python 3.12
-                dest_entry = copy(entry)
-                dest_entry.name = str(dest_path)
+                dest_entry = entry.replace(name=str(dest_path))
                 if entry.isreg():
                     dest.addfile(dest_entry, source_tar.extractfile(entry))
                 elif entry.islnk():
@@ -345,7 +344,7 @@ class IncusLXCImageCache(IncusImageCache):
 class IncusVMImageCache(IncusImageCache):
     """Process VM container images on import."""
 
-    backend = "incus-vm"
+    backend = BackendType.INCUS_VM
 
     def convert_raw_image(self, source: Path) -> Path:
         """Convert a raw image to qcow2."""
@@ -361,7 +360,9 @@ class IncusVMImageCache(IncusImageCache):
                 raise AssertionError(
                     "Raw image tarballs must not include any directories"
                 )
-            tar.extract(name, path=workdir, set_attrs=False)
+            # TODO: Remove filter="data" once we require Python >= 3.14,
+            # since it's the new default.
+            tar.extract(name, path=workdir, set_attrs=False, filter="data")
             raw = workdir / name
 
         dest = workdir / "image.qcow2"
@@ -540,7 +541,7 @@ class IncusExecutorMixin:
 
 
 class IncusLXCExecutor(
-    IncusExecutorMixin, ExecutorInterface, backend_name="incus-lxc"
+    IncusExecutorMixin, ExecutorInterface, backend_name=BackendType.INCUS_LXC
 ):
     """Support the Incus LXC Container executor."""
 
@@ -555,7 +556,7 @@ class IncusLXCExecutor(
 
 
 class IncusVMExecutor(
-    IncusExecutorMixin, ExecutorInterface, backend_name="incus-vm"
+    IncusExecutorMixin, ExecutorInterface, backend_name=BackendType.INCUS_VM
 ):
     """Support the Incus VM executor."""
 
@@ -609,9 +610,9 @@ class IncusInstance(InstanceInterface):
     ) -> None:
         """Wait for an instance to become ready."""
         assert self._instance is not None
-        start_time = time.monotonic()
+        start_time = tm.monotonic()
         end_time = start_time + timeout
-        while time.monotonic() < end_time:
+        while tm.monotonic() < end_time:
             p = subprocess.run(
                 [
                     "incus",
@@ -646,7 +647,7 @@ class IncusInstance(InstanceInterface):
                     p.stdout,
                     p.stderr,
                 )
-            time.sleep(sleep_time)
+            tm.sleep(sleep_time)
         raise InstanceNotRunning(
             f"Incus Instance {self._instance} hasn't booted "
             "or is missing incus-agent"
@@ -744,6 +745,7 @@ class IncusInstance(InstanceInterface):
         self,
         args: list[str],
         text: Literal[True],
+        cwd: str | os.PathLike[str] | None = None,
         run_as_root: bool = False,
         **kwargs: Any,
     ) -> CompletedProcess[str]: ...
@@ -753,6 +755,7 @@ class IncusInstance(InstanceInterface):
         self,
         args: list[str],
         text: Literal[False] | None = None,
+        cwd: str | os.PathLike[str] | None = None,
         run_as_root: bool = False,
         **kwargs: Any,
     ) -> CompletedProcess[bytes]: ...
@@ -761,6 +764,7 @@ class IncusInstance(InstanceInterface):
         self,
         args: list[str],
         text: Literal[False] | Literal[True] | None = None,
+        cwd: str | os.PathLike[str] | None = None,
         run_as_root: bool = False,
         **kwargs: Any,
     ) -> CompletedProcess[AnyStr]:
@@ -772,6 +776,13 @@ class IncusInstance(InstanceInterface):
         assert self._instance is not None
         cmd = ["incus", "exec", self._instance]
 
+        if cwd is not None:
+            # --cwd defaults to /root, so interpret relative paths as
+            # relative to that.
+            if not PurePath(cwd).is_absolute():
+                cwd = PurePath("/root", cwd)
+            cmd += ["--cwd", str(cwd)]
+
         if not run_as_root:
             uid = self.create_user()
             cmd += ["--user", str(uid)]
