@@ -8,6 +8,8 @@
 # contained in the LICENSE file.
 
 """Unit tests for the Executor basic interface."""
+
+import os
 from pathlib import Path, PurePath
 from subprocess import CalledProcessError, CompletedProcess, PIPE
 from typing import Any, AnyStr, Literal
@@ -62,6 +64,7 @@ class SampleInstance(InstanceInterface):
         self,
         args: list[str],
         text: Literal[True] | Literal[False] | None = None,
+        cwd: str | os.PathLike[str] | None = None,
         run_as_root: bool = False,
         **kwargs: Any,
     ) -> CompletedProcess[AnyStr]:
@@ -94,7 +97,7 @@ class ExecutorTests(TestCase):
 
     def test_analyze_worker_all_executors_unshare_available(self) -> None:
         """analyze_worker_all_executors reports available backends."""
-        self.mock_is_command_available({"unshare": True})
+        self.mock_is_command_available({"unshare": True, "ip": True})
         executors_metadata = analyze_worker_all_executors()
         self.assertTrue(executors_metadata["executor:unshare:available"])
 
