@@ -47,8 +47,3 @@ class NoopTests(TestCase):
         error_msg = "extra fields not permitted"
         with self.assertRaisesRegex(TaskConfigError, error_msg):
             Noop({"result": "True", "input": {}})
-
-    def test_get_label(self) -> None:
-        """Test get_label."""
-        noop = Noop({"result": True})
-        self.assertEqual(noop.get_label(), "noop")
