@@ -19,7 +19,6 @@ from unittest.mock import MagicMock, call
 from debusine import utils
 from debusine.artifacts import (
     BinaryPackage,
-    BinaryPackages,
     PackageBuildLog,
     SigningInputArtifact,
     Upload,
@@ -28,7 +27,6 @@ from debusine.artifacts.models import (
     ArtifactCategory,
     CollectionCategory,
     DebianBinaryPackage,
-    DebianBinaryPackages,
     DebianSourcePackage,
     DoseDistCheck,
     EmptyArtifactData,
@@ -38,11 +36,12 @@ from debusine.client.models import ArtifactResponse, RemoteArtifact
 from debusine.tasks import Sbuild, TaskConfigError
 from debusine.tasks.executors import ExecutorInterface
 from debusine.tasks.models import (
+    BackendType,
     SbuildBuildComponent,
     SbuildDynamicData,
     WorkerType,
 )
-from debusine.tasks.server import ArtifactInfo
+from debusine.tasks.server import ArtifactInfo, CollectionInfo
 from debusine.tasks.tests.helper_mixin import (
     ExternalTaskHelperMixin,
     FakeTaskDatabase,
@@ -62,8 +61,8 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
         "input": {
             "source_artifact": 5,
         },
-        "environment": "debian/match:codename=bullseye",
-        "host_architecture": "amd64",
+        "environment": "debian/match:codename=bookworm",
+        "build_architecture": "amd64",
         "build_components": [
             "any",
             "all",
@@ -93,10 +92,32 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
 
         super().configure_task(task_data, override, remove)
 
-        self.task.dynamic_data = SbuildDynamicData(
-            input_source_artifact_id=int(self.task.data.input.source_artifact),
-            binnmu_maintainer="Debusine <noreply@debusine.example.com>",
+        self.task_db = FakeTaskDatabase(
+            single_lookups={
+                # environment
+                (
+                    "debian/match:codename=bookworm:architecture=amd64:"
+                    "format=tarball:backend=unshare:variant=sbuild",
+                    CollectionCategory.ENVIRONMENTS,
+                ): ArtifactInfo(
+                    id=1,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(),
+                ),
+                # input.source_artifact
+                (5, None): ArtifactInfo(
+                    id=5,
+                    category=ArtifactCategory.SOURCE_PACKAGE,
+                    data=DebianSourcePackage(
+                        name="hello", version="1.0", type="dpkg", dsc_fields={}
+                    ),
+                ),
+            },
+            settings={
+                "DEBUSINE_FQDN": "debusine.example.com",
+            },
         )
+
         self.task.logger.disabled = True
         self.addCleanup(setattr, self.task.logger, "disabled", False)
 
@@ -114,55 +135,25 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
 
     def test_compute_dynamic_data(self) -> None:
         """Dynamic data receives relevant artifact IDs."""
-        self.configure_task(
-            override={
-                "backend": "unshare",
-                "environment": "debian/match:codename=bookworm",
-            },
-        )
-        task_db = FakeTaskDatabase(
-            single_lookups={
-                # environment
-                (
-                    "debian/match:codename=bookworm:architecture=amd64:"
-                    "format=tarball:backend=unshare:variant=sbuild",
-                    CollectionCategory.ENVIRONMENTS,
-                ): ArtifactInfo(
-                    id=1,
-                    category=ArtifactCategory.SYSTEM_TARBALL,
-                    data=create_system_tarball_data(),
-                ),
-                # input.source_artifact
-                (5, None): ArtifactInfo(
-                    id=5,
-                    category=ArtifactCategory.SOURCE_PACKAGE,
-                    data=DebianSourcePackage(
-                        name="hello", version="1.0", type="dpkg", dsc_fields={}
-                    ),
-                ),
-            },
-            settings={
-                "DEBUSINE_FQDN": "debusine.example.com",
-            },
-        )
+        self.configure_task(override={"backend": BackendType.UNSHARE})
 
         self.assertEqual(
-            self.task.compute_dynamic_data(task_db),
+            self.task.compute_dynamic_data(self.task_db),
             SbuildDynamicData(
                 environment_id=1,
                 input_source_artifact_id=5,
                 input_extra_binary_artifacts_ids=[],
                 binnmu_maintainer="Debusine <noreply@debusine.example.com>",
                 subject="hello",
-                parameter_summary="hello_1.0",
+                parameter_summary="hello_1.0 (amd64)",
                 runtime_context="all+any:amd64:amd64",
                 configuration_context="sid",
             ),
         )
 
-    def test_build_architecture(self) -> None:
+    def test_host_architecture(self) -> None:
         cases: list[tuple[dict[str, Any], str]] = [
-            ({"host_architecture": "arm64"}, "arm64"),
+            ({"build_architecture": "arm64"}, "arm64"),
             ({"build_components": [SbuildBuildComponent.ALL]}, "all"),
             ({"build_components": [SbuildBuildComponent.SOURCE]}, "source"),
         ]
@@ -170,7 +161,7 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
         for override, expected in cases:
             with self.subTest(override=repr(override)):
                 self.configure_task(override=override)
-                self.assertEqual(self.task._build_architecture, expected)
+                self.assertEqual(self.task._host_architecture, expected)
 
     def test_compute_dynamic_data_build_profiles_runtime_context(self) -> None:
         """
@@ -181,74 +172,29 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
         self.configure_task(
             override={
                 "build_profiles": ["nodoc", "nocheck"],
-                "backend": "unshare",
-                "environment": "debian/match:codename=bookworm",
-            },
-        )
-
-        task_db = FakeTaskDatabase(
-            single_lookups={
-                # environment
-                (
-                    "debian/match:codename=bookworm:architecture=amd64:"
-                    "format=tarball:backend=unshare:variant=sbuild",
-                    CollectionCategory.ENVIRONMENTS,
-                ): ArtifactInfo(
-                    id=1,
-                    category=ArtifactCategory.SYSTEM_TARBALL,
-                    data=create_system_tarball_data(),
-                ),
-                # input.source_artifact
-                (5, None): ArtifactInfo(
-                    id=5,
-                    category=ArtifactCategory.SOURCE_PACKAGE,
-                    data=DebianSourcePackage(
-                        name="hello", version="1.0", type="dpkg", dsc_fields={}
-                    ),
-                ),
-            },
-            settings={
-                "DEBUSINE_FQDN": "debusine.example.com",
+                "backend": BackendType.UNSHARE,
             },
         )
 
         self.assertIsNone(
-            self.task.compute_dynamic_data(task_db).runtime_context
+            self.task.compute_dynamic_data(self.task_db).runtime_context
         )
 
     def test_compute_dynamic_data_raise_task_config_error(self) -> None:
         """Raise TaskConfigError: invalid input_source_artifact."""
-        task_db = FakeTaskDatabase(
-            single_lookups={
-                # environment
-                (
-                    "debian/match:codename=bookworm:architecture=amd64:"
-                    "format=tarball:backend=unshare:variant=sbuild",
-                    CollectionCategory.ENVIRONMENTS,
-                ): ArtifactInfo(
-                    id=1,
-                    category=ArtifactCategory.SYSTEM_TARBALL,
-                    data=create_system_tarball_data(),
-                ),
-                # input.source_artifact
-                (5, None): ArtifactInfo(
-                    id=5,
-                    category=ArtifactCategory.BINARY_PACKAGES,
-                    data=EmptyArtifactData(),
-                ),
-            },
-            settings={
-                "DEBUSINE_FQDN": "debusine.example.com",
-            },
+        self.task_db.single_lookups[(5, None)] = ArtifactInfo(
+            id=5,
+            category=ArtifactCategory.BINARY_PACKAGE,
+            data=EmptyArtifactData(),
         )
 
         with self.assertRaisesRegex(
             TaskConfigError,
             "^input.source_artifact: unexpected artifact category: "
-            r"'debian:binary-packages'. Valid categories: "
+            r"'debian:binary-package'. Valid categories: "
             r"\['debian:source-package'\]$",
         ):
-            self.task.compute_dynamic_data(task_db)
+            self.task.compute_dynamic_data(self.task_db)
 
     def test_configure_with_unknown_data(self) -> None:
         """Configure fails if a non-recognised key is in task_data."""
@@ -257,7 +203,7 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
 
     def test_configure_fails_with_missing_required_data(self) -> None:
         """Configure fails with missing required keys in task_data."""
-        for key in ("input", "host_architecture"):
+        for key in ("input", "build_architecture"):
             with self.subTest(f"Configure with key {key} missing"):
                 with self.assertRaises(TaskConfigError):
                     self.configure_task(remove=[key])
@@ -529,8 +475,7 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
             (
                 "input.extra_binary_artifacts includes artifact 6, a "
                 "debian:source-package, not the expected "
-                "debian:binary-package, debian:binary-packages, or "
-                "debian:upload.\n"
+                "debian:binary-package, or debian:upload.\n"
             ),
         )
 
@@ -561,6 +506,7 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
     def test_cmdline_fixed_parameters(self) -> None:
         """Test fixed command line parameters."""
         self.configure_task()
+        self.set_dynamic_data()
         self.patch_executor()
 
         cmdline = self.task._cmdline()
@@ -571,7 +517,19 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
 
     def test_cmdline_contains_arch_parameter(self) -> None:
         """Ensure --arch parameter is computed from data."""
-        self.configure_task(override={"host_architecture": "mipsel"})
+        self.configure_task(override={"build_architecture": "mipsel"})
+        self.task_db.single_lookups[
+            (
+                "debian/match:codename=bookworm:architecture=mipsel:"
+                "format=tarball:backend=unshare:variant=sbuild",
+                CollectionCategory.ENVIRONMENTS,
+            )
+        ] = ArtifactInfo(
+            id=1,
+            category=ArtifactCategory.SYSTEM_TARBALL,
+            data=create_system_tarball_data(),
+        )
+        self.set_dynamic_data()
         self.patch_executor()
         cmdline = self.task._cmdline()
 
@@ -580,6 +538,7 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
     def test_cmdline_contains_dist_parameter(self) -> None:
         """Ensure --dist parameter is computed from data."""
         self.configure_task()
+        self.set_dynamic_data()
         self.patch_executor(codename="jessie")
 
         cmdline = self.task._cmdline()
@@ -603,6 +562,7 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
                 ],
             }
         )
+        self.set_dynamic_data()
         self.patch_executor(codename="jessie")
         env = self.task._cmd_env()
         self.assertIsNone(env)
@@ -629,6 +589,7 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
                 ],
             }
         )
+        self.set_dynamic_data()
         self.patch_executor(codename="bullseye")
         env = self.task._cmd_env()
         self.assertIsNone(env)
@@ -668,6 +629,7 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
                 ],
             }
         )
+        self.set_dynamic_data()
         self.patch_executor(codename="bookworm")
         env = self.task._cmd_env()
         self.assertIsNone(env)
@@ -685,6 +647,66 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
             cmdline,
         )
 
+    def test_cmdline_extra_debusine_repositories(self) -> None:
+        """Extra repositories can be hosted by Debusine."""
+        self.configure_task(
+            override={
+                "extra_repositories": [{"suite": "trixie-local@debian:suite"}]
+            }
+        )
+        self.task_db.single_collection_lookups[
+            ("trixie-local@debian:suite", None)
+        ] = CollectionInfo(
+            id=1,
+            scope_name="scope",
+            workspace_name="workspace",
+            category=CollectionCategory.SUITE,
+            name="trixie-local",
+            data={"components": ["main"]},
+        )
+        self.task_db.settings["DEBUSINE_DEBIAN_ARCHIVE_PRIMARY_FQDN"] = (
+            "deb.example.com"
+        )
+        self.task_db.suite_signing_keys[1] = "\n".join(
+            (
+                "-----BEGIN PGP PUBLIC KEY BLOCK-----",
+                "",
+                "ABCDEFGHI",
+                "-----END PGP PUBLIC KEY BLOCK-----",
+            )
+        )
+        self.set_dynamic_data()
+        self.patch_executor(codename="trixie")
+        destination = self.create_temporary_directory()
+
+        self.task.write_extra_repository_config("trixie", destination)
+        cmdline = self.task._cmdline()
+
+        self.assertEqual(self.task.extra_repository_keys, [])
+        [extra_repository_source] = self.task.extra_repository_sources
+        self.assertEqual(
+            extra_repository_source.read_text(),
+            textwrap.dedent(
+                """\
+                Types: deb
+                URIs: http://deb.example.com/scope/workspace
+                Suites: trixie-local
+                Components: main
+                Signed-By:
+                 -----BEGIN PGP PUBLIC KEY BLOCK-----
+                 .
+                 ABCDEFGHI
+                 -----END PGP PUBLIC KEY BLOCK-----
+                """
+            ),
+        )
+        self.assertIn(
+            f"--pre-build-commands=cat {extra_repository_source} | "
+            f"%SBUILD_CHROOT_EXEC sh -c 'cat > "
+            f"/etc/apt/sources.list.d/{extra_repository_source.name}'",
+            cmdline,
+        )
+
     def test_cmdline_binnmu(self) -> None:
         """Check binnmu command line."""
         self.configure_task(
@@ -696,6 +718,7 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
                 "build_components": ["any"],
             }
         )
+        self.set_dynamic_data()
         self.patch_executor(codename="jessie")
         cmdline = self.task._cmdline()
         self.assertIn("--make-binNMU=Rebuild for 64bit time_t", cmdline)
@@ -715,6 +738,7 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
                 "build_components": ["any"],
             }
         )
+        self.set_dynamic_data()
         self.patch_executor(codename="jessie")
         cmdline = self.task._cmdline()
         self.assertIn("--make-binNMU=Rebuild for 64bit time_t", cmdline)
@@ -728,6 +752,7 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
     def test_cmdline_no_build_profiles(self) -> None:
         """Check that missing build_profiles key is handled."""
         self.configure_task()
+        self.set_dynamic_data()
         self.patch_executor(codename="jessie")
         cmdline = self.task._cmdline()
         for argument in cmdline:
@@ -736,6 +761,7 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
     def test_cmdline_empty_build_profiles(self) -> None:
         """Check that empty build_profiles don't appear on the command line."""
         self.configure_task(override={"build_profiles": []})
+        self.set_dynamic_data()
         self.patch_executor(codename="jessie")
         cmdline = self.task._cmdline()
         for argument in cmdline:
@@ -744,6 +770,7 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
     def test_cmdline_build_profiles(self) -> None:
         """Check that build_profiles make it onto the command line."""
         self.configure_task(override={"build_profiles": ["nodoc", "nocheck"]})
+        self.set_dynamic_data()
         self.patch_executor(codename="jessie")
         cmdline = self.task._cmdline()
         self.assertIn("--profiles=nodoc,nocheck", cmdline)
@@ -751,6 +778,7 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
     def test_cmd_env_no_build_profiles(self) -> None:
         """Check that environment isn't modified, without build_profiles."""
         self.configure_task(override={"build_profiles": ["cross"]})
+        self.set_dynamic_data()
         self.patch_executor(codename="jessie")
         env = self.task._cmd_env()
         self.assertIsNone(env)
@@ -760,6 +788,7 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
         self.configure_task(
             override={"build_profiles": ["nodoc", "nocheck", "cross"]}
         )
+        self.set_dynamic_data()
         self.patch_executor(codename="jessie")
         env = self.task._cmd_env()
         self.assertIsInstance(env, dict)
@@ -786,6 +815,7 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
                 self.configure_task(
                     override={"build_components": list(combination)}
                 )
+                self.set_dynamic_data()
                 self.patch_executor()
                 cmdline = self.task._cmdline()
                 for key, value in option_mapping.items():
@@ -799,6 +829,7 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
     def test_cmdline_default_build_dep_resolver(self) -> None:
         """The default build_dep_resolver doesn't generate arguments."""
         self.configure_task()
+        self.set_dynamic_data()
         self.patch_executor(codename="jessie")
         cmdline = self.task._cmdline()
         for argument in cmdline:
@@ -807,6 +838,7 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
     def test_cmdline_aspcud_build_dep_resolver(self) -> None:
         """The aspcud build_dep_resolver generates arguments."""
         self.configure_task(override={"build_dep_resolver": "aspcud"})
+        self.set_dynamic_data()
         self.patch_executor(codename="jessie")
         cmdline = self.task._cmdline()
         self.assertIn("--build-dep-resolver=aspcud", cmdline)
@@ -814,6 +846,7 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
     def test_cmdline_null_aspcud_criteria(self) -> None:
         """The default aspcud_criteria doesn't generate arguments."""
         self.configure_task()
+        self.set_dynamic_data()
         self.patch_executor(codename="jessie")
         cmdline = self.task._cmdline()
         for argument in cmdline:
@@ -828,6 +861,7 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
                 ),
             }
         )
+        self.set_dynamic_data()
         self.patch_executor(codename="jessie")
         cmdline = self.task._cmdline()
         self.assertIn(
@@ -835,9 +869,17 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
             cmdline,
         )
 
+    def test_cmdline_resolve_alternatives(self) -> None:
+        self.configure_task(override={"resolve_alternatives": True})
+        self.set_dynamic_data()
+        self.patch_executor(codename="jessie")
+        cmdline = self.task._cmdline()
+        self.assertIn("--resolve-alternatives", cmdline)
+
     def test_cmdline_contains_extra_packages(self) -> None:
         """Ensure cmdline has the arguments --extra-package=."""
         self.configure_task()
+        self.set_dynamic_data()
         self.patch_executor()
         self.task._extra_packages = [Path("/tmp/extra.deb")]
         cmdline = self.task._cmdline()
@@ -845,17 +887,8 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
 
     def test_cmdline_contains_executor_unshare_build_args(self) -> None:
         """Ensure cmdline builds correct arguments for unshare executor."""
-        self.configure_task(
-            override={
-                "backend": "unshare",
-                "environment": "debian/match:codename=bookworm",
-            },
-        )
-        self.task.dynamic_data = SbuildDynamicData(
-            environment_id=42,
-            input_source_artifact_id=1,
-            input_extra_binary_artifacts_ids=[],
-        )
+        self.configure_task(override={"backend": BackendType.UNSHARE})
+        self.set_dynamic_data()
 
         debusine_mock = self.mock_debusine()
         self.mock_image_download(debusine_mock)
@@ -879,9 +912,20 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
     def test_cmdline_uses_autopkgtest_virtserver_build_args(self) -> None:
         """Ensure cmdline supports autopkgtest-virt-server executors."""
         data = self.configuration("amd64", ["any"])
-        data["backend"] = "incus-lxc"
-        data["environment"] = 1
+        data["backend"] = BackendType.INCUS_LXC
         self.configure_task(data)
+        self.task_db.single_lookups[
+            (
+                "debian/match:codename=bookworm:architecture=amd64:"
+                "format=tarball:backend=incus-lxc:variant=sbuild",
+                CollectionCategory.ENVIRONMENTS,
+            )
+        ] = ArtifactInfo(
+            id=1,
+            category=ArtifactCategory.SYSTEM_TARBALL,
+            data=create_system_tarball_data(),
+        )
+        self.set_dynamic_data()
 
         executor_mocked = self.patch_executor()
 
@@ -914,7 +958,7 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
         """execute() backend "unshare" succeeds."""
         self.configure_task(
             task_data=self.configuration("amd64", ["any"]),
-            override={"backend": "unshare", "environment": 42},
+            override={"backend": BackendType.UNSHARE, "environment": 42},
         )
         self.task.work_request_id = 5
         self.patch_sbuild_debusine()
@@ -1280,7 +1324,7 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
         return {
             "system:worker_type": WorkerType.EXTERNAL,
             "system:architectures": ["amd64"],
-            "system:host_architecture": "amd64",
+            "system:native_architecture": "amd64",
             "sbuild:version": self.task.TASK_VERSION,
             "sbuild:available": True,
         }
@@ -1305,14 +1349,18 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
         """can_run_on returns True: using unshare."""
         worker_metadata = self.worker_metadata()
         worker_metadata["executor:unshare:available"] = True
-        self.configure_task(override={"backend": "unshare", "environment": 42})
+        self.configure_task(
+            override={"backend": BackendType.UNSHARE, "environment": 42}
+        )
 
         self.assertTrue(self.task.can_run_on(worker_metadata))
 
     def test_can_run_on_unshare_requires_unshare(self) -> None:
         """can_run_on returns False: missing unshare executor."""
         worker_metadata = self.worker_metadata()
-        self.configure_task(override={"backend": "unshare", "environment": 42})
+        self.configure_task(
+            override={"backend": BackendType.UNSHARE, "environment": 42}
+        )
 
         self.assertFalse(self.task.can_run_on(worker_metadata))
 
@@ -1322,7 +1370,7 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
         worker_metadata["executor:incus-lxc:available"] = True
         worker_metadata["autopkgtest:available"] = True
         self.configure_task(
-            override={"backend": "incus-lxc", "environment": 42}
+            override={"backend": BackendType.INCUS_LXC, "environment": 42}
         )
 
         self.assertTrue(self.task.can_run_on(worker_metadata))
@@ -1332,7 +1380,7 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
         worker_metadata = self.worker_metadata()
         worker_metadata["executor:incus-lxc:available"] = True
         self.configure_task(
-            override={"backend": "incus-lxc", "environment": 42}
+            override={"backend": BackendType.INCUS_LXC, "environment": 42}
         )
 
         self.assertFalse(self.task.can_run_on(worker_metadata))
@@ -1412,7 +1460,7 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
                         file=log_file,
                         source=dsc["Source"],
                         version=dsc["Version"],
-                        architecture=self.task._build_architecture,
+                        architecture=self.task._host_architecture,
                     ),
                     workspace=self.task.workspace_name,
                     work_request=None,
@@ -1458,7 +1506,7 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
                         file=log_file,
                         source=dsc["Source"],
                         version=dsc["Version"],
-                        architecture=self.task._build_architecture,
+                        architecture=self.task._host_architecture,
                     ),
                     workspace=self.task.workspace_name,
                     work_request=self.task.work_request_id,
@@ -1500,7 +1548,7 @@ broken-packages: 4
                         file=log_file,
                         source=dsc["Source"],
                         version=dsc["Version"],
-                        architecture=self.task._build_architecture,
+                        architecture=self.task._host_architecture,
                         bd_uninstallable=DoseDistCheck.parse_obj(
                             {
                                 "output-version": "1.2",
@@ -1857,7 +1905,7 @@ broken-packages: 4
 
         source = dsc_data["Source"]
         version = dsc_data["Version"]
-        architecture = self.task._build_architecture
+        architecture = self.task._host_architecture
 
         calls = [
             call(
@@ -2010,7 +2058,7 @@ broken-packages: 4
             build_directory,
             dsc_data["Source"],
             dsc_data["Version"],
-            self.task._build_architecture,
+            self.task._host_architecture,
             execution_success=False,
         )
 
@@ -2039,7 +2087,7 @@ broken-packages: 4
         """upload_artifacts() called with a .deb but no .changes file."""
         build_directory = self.create_temporary_directory()
         self.task._dsc_file = self.create_temporary_file(suffix=".dsc")
-        dsc_data = self.write_dsc_example_file(self.task._dsc_file)
+        self.write_dsc_example_file(self.task._dsc_file)
         self.write_deb_file(build_directory / "hello_1.0_amd64.deb")
         debusine_mock = self.patch_sbuild_debusine()
         self.task.work_request_id = 8
@@ -2055,17 +2103,6 @@ broken-packages: 4
                 workspace=workspace_name,
                 work_request=self.task.work_request_id,
             ),
-            call(
-                BinaryPackages.create(
-                    srcpkg_name=dsc_data["Source"],
-                    srcpkg_version=dsc_data["Version"],
-                    version=dsc_data["Version"],
-                    architecture="amd64",
-                    files=[build_directory / "hello_1.0_amd64.deb"],
-                ),
-                workspace=workspace_name,
-                work_request=self.task.work_request_id,
-            ),
         ]
         self.assertEqual(
             debusine_mock.upload_artifact.call_count,
@@ -2090,7 +2127,7 @@ broken-packages: 4
 
         debusine_mock = self.patch_sbuild_debusine()
 
-        # Add two files: BinaryPackages is uploaded with them
+        # Add two files: BinaryPackages are uploaded with them
         self.write_deb_file(file1 := temp_directory / "hello_1.0_amd64.deb")
         self.write_deb_file(file2 := temp_directory / "hello_1.0_amd64.udeb")
 
@@ -2110,7 +2147,7 @@ broken-packages: 4
                     file=build_log_file,
                     source=dsc_data["Source"],
                     version=dsc_data["Version"],
-                    architecture=self.task._build_architecture,
+                    architecture=self.task._host_architecture,
                 ),
                 workspace=workspace_name,
                 work_request=self.task.work_request_id,
@@ -2125,17 +2162,6 @@ broken-packages: 4
                 workspace=workspace_name,
                 work_request=self.task.work_request_id,
             ),
-            call(
-                BinaryPackages.create(
-                    srcpkg_name=dsc_data["Source"],
-                    srcpkg_version=dsc_data["Version"],
-                    version=dsc_data["Version"],
-                    architecture="amd64",
-                    files=[file1, file2],
-                ),
-                workspace=workspace_name,
-                work_request=self.task.work_request_id,
-            ),
             call(
                 Upload.create(
                     changes_file=changes_file,
@@ -2256,15 +2282,15 @@ broken-packages: 4
 
     @staticmethod
     def configuration(
-        host_architecture: str, build_components: list[str]
+        build_architecture: str, build_components: list[str]
     ) -> dict[str, Any]:
-        """Return configuration with host_architecture and build_components."""
+        """Return configuration with build_architecture and build_components."""
         return {
             "input": {
                 "source_artifact": 5,
             },
-            "environment": "debian/match:codename=bullseye",
-            "host_architecture": host_architecture,
+            "environment": "debian/match:codename=bookworm",
+            "build_architecture": build_architecture,
             "build_components": build_components,
         }
 
@@ -2277,7 +2303,7 @@ broken-packages: 4
 
     def create_and_upload_binary_packages(
         self,
-        host_architecture: str,
+        build_architecture: str,
         build_components: list[str],
         *,
         create_debs_for_architectures: list[str],
@@ -2299,7 +2325,7 @@ broken-packages: 4
             )
 
         self.configure_task(
-            self.configuration(host_architecture, build_components)
+            self.configuration(build_architecture, build_components)
         )
 
         dsc = utils.read_dsc(dsc_file)
@@ -2307,12 +2333,12 @@ broken-packages: 4
 
         debusine_mock = self.patch_sbuild_debusine()
 
-        self.task._upload_binary_packages(temp_directory, dsc)
+        self.task._upload_binary_packages(temp_directory)
 
         return debusine_mock.upload_artifact, temp_directory
 
     def assert_upload_binary_packages_with_files(
-        self, upload_artifact_mock: MagicMock, expected_files: list[list[Path]]
+        self, upload_artifact_mock: MagicMock, expected_files: list[Path]
     ) -> None:
         """
         Assert debusine.upload_artifact() was called with expected artifacts.
@@ -2322,41 +2348,35 @@ broken-packages: 4
         sub-lists of paths; and assert that the file names on each
         upload_artifact() are the expected ones (expected_files[i]).
         """
-        self.assertEqual(
-            upload_artifact_mock.call_count,
-            sum(len(files) for files in expected_files) + len(expected_files),
-        )
+        self.assertEqual(upload_artifact_mock.call_count, len(expected_files))
 
         # coverage 7.6.0 complains that the zero-item case is uncovered,
         # which is true, but this is a test helper method so we don't care.
-        for call_args, files in zip(
+        for call_args, file in zip(
             upload_artifact_mock.call_args_list,
-            itertools.chain.from_iterable(
-                [[file] for file in files] + [files] for files in expected_files
-            ),
+            expected_files,
         ):  # pragma: no cover
-            expected_filenames = {file.name for file in files}
-            self.assertEqual(call_args[0][0].files.keys(), expected_filenames)
+            self.assertEqual(list(call_args[0][0].files.keys()), [file.name])
 
     def assert_upload_binary_packages_architectures(
-        self, upload_artifact_mock: MagicMock, expected_architectures: list[str]
+        self, upload_artifact_mock: MagicMock, expected_architectures: set[str]
     ) -> None:
         """
         Assert debusine.upload_artifact() was called with expected arches.
 
         Assert that the debian:binary-packages artifacts uploaded via
-        upload_artifact_mock have the architecture data fields given in
-        expected_architectures.
+        upload_artifact_mock match expected_architectures.
         """
         artifacts = [
             call_args[0][0]
             for call_args in upload_artifact_mock.call_args_list
-            if call_args[0][0].category == ArtifactCategory.BINARY_PACKAGES
+            if call_args[0][0].category == ArtifactCategory.BINARY_PACKAGE
         ]
-        self.assertEqual(
-            [artifact.data.architecture for artifact in artifacts],
-            expected_architectures,
-        )
+        seen_architectures = set()
+        for artifact in artifacts:
+            seen_architectures.add(artifact.data.deb_fields["Architecture"])
+
+        self.assertEqual(seen_architectures, expected_architectures)
 
     def test_upload_binary_packages_build_components_any(self) -> None:
         r"""
@@ -2374,10 +2394,10 @@ broken-packages: 4
         expected_files = sorted(temp_directory.glob("*_amd64.*deb"))
 
         self.assert_upload_binary_packages_with_files(
-            upload_artifact_mock, [expected_files]
+            upload_artifact_mock, expected_files
         )
         self.assert_upload_binary_packages_architectures(
-            upload_artifact_mock, ["amd64"]
+            upload_artifact_mock, {"amd64"}
         )
 
     def test_upload_binary_packages_build_components_all(self) -> None:
@@ -2397,10 +2417,10 @@ broken-packages: 4
         expected_files = sorted(temp_directory.glob("*_all.*deb"))
 
         self.assert_upload_binary_packages_with_files(
-            upload_artifact_mock, [expected_files]
+            upload_artifact_mock, expected_files
         )
         self.assert_upload_binary_packages_architectures(
-            upload_artifact_mock, ["all"]
+            upload_artifact_mock, {"all"}
         )
 
     def test_upload_binary_packages_build_components_any_all(self) -> None:
@@ -2418,10 +2438,10 @@ broken-packages: 4
         expected_files_all = sorted(temp_directory.glob("*_all.*deb"))
 
         self.assert_upload_binary_packages_with_files(
-            upload_artifact_mock, [expected_files_amd64, expected_files_all]
+            upload_artifact_mock, expected_files_amd64 + expected_files_all
         )
         self.assert_upload_binary_packages_architectures(
-            upload_artifact_mock, ["amd64", "all"]
+            upload_artifact_mock, {"amd64", "all"}
         )
 
     def test_upload_binary_packages_build_components_any_all_without_all(
@@ -2442,18 +2462,17 @@ broken-packages: 4
         expected_files_amd64 = sorted(temp_directory.glob("*_amd64.*deb"))
 
         self.assert_upload_binary_packages_with_files(
-            upload_artifact_mock, [expected_files_amd64]
+            upload_artifact_mock, expected_files_amd64
         )
         self.assert_upload_binary_packages_architectures(
-            upload_artifact_mock, ["amd64"]
+            upload_artifact_mock, {"amd64"}
         )
 
     def test_create_binary_package_local_artifacts(self) -> None:
         """
-        _create_binary_package_local_artifact(): return BinaryPackage(s).
+        _create_binary_package_local_artifact(): return BinaryPackages.
 
-        Assert that it returns the expected BinaryPackage and BinaryPackages
-        artifacts.
+        Assert that it returns the expected BinaryPackage artifacts.
         """
         build_directory = self.create_temporary_directory()
 
@@ -2472,8 +2491,6 @@ broken-packages: 4
 
         artifacts = self.task._create_binary_package_local_artifacts(
             build_directory,
-            dsc,
-            architecture="amd64",
             suffixes=[".deb"],
         )
 
@@ -2503,19 +2520,6 @@ broken-packages: 4
                         }
                     ),
                 ),
-                BinaryPackages(
-                    category=BinaryPackages._category,
-                    files={package_file.name: package_file},
-                    data=DebianBinaryPackages.parse_obj(
-                        {
-                            "srcpkg_name": dsc_data["Source"],
-                            "srcpkg_version": dsc_data["Version"],
-                            "version": dsc_data["Version"],
-                            "architecture": "amd64",
-                            "packages": ["package"],
-                        }
-                    ),
-                ),
             ],
         )
 
@@ -2544,19 +2548,3 @@ broken-packages: 4
             input_extra_binary_artifacts_ids=[3, 4],
         )
         self.assertEqual(self.task.get_input_artifacts_ids(), [2, 1, 3, 4])
-
-    def test_get_label_subject_is_none(self) -> None:
-        """Test get_label if dynamic_data.subject is None."""
-        self.assertEqual(self.task.get_label(), "build a package")
-
-    def test_get_label_dynamic_data_is_none(self) -> None:
-        """Test get_label if dynamic_data is None."""
-        self.task.dynamic_data = None
-        self.assertEqual(self.task.get_label(), "build a package")
-
-    def test_get_label_package_subject(self) -> None:
-        """Test get_label if dynamic_data.subject is set."""
-        self.task.dynamic_data = SbuildDynamicData(
-            input_source_artifact_id=1, environment_id=2, subject="hello"
-        )
-        self.assertEqual(self.task.get_label(), "build hello")
