@@ -31,7 +31,6 @@ from typing import (
     Generic,
     Protocol,
     TYPE_CHECKING,
-    TypeAlias,
     TypeVar,
     Union,
     cast,
@@ -47,7 +46,7 @@ import requests
 from asgiref.sync import async_to_sync
 from channels.layers import get_channel_layer
 from django.conf import settings
-from django.db import transaction
+from django.db import OperationalError, connection, connections, transaction
 from django.db.models import Model, QuerySet
 from django.http.response import HttpResponseBase
 from django.test.client import JSON_CONTENT_TYPE_RE
@@ -72,6 +71,7 @@ from debusine.db.models.permissions import (
     get_resource_workspace,
     resolve_roles_list,
 )
+from debusine.db.models.work_requests import WorkRequestQuerySet
 from debusine.db.playground.scenarios import Scenario
 from debusine.tasks.models import BaseTaskDataModel
 from debusine.test.playground import Playground
@@ -90,11 +90,9 @@ M = TypeVar("M", bound=Model)
 R = TypeVar("R", bound=Roles)
 
 #: Type alias for response objects returned by the Django test client
-TestResponseType: TypeAlias = "_MonkeyPatchedWSGIResponse"
+type TestResponseType = "_MonkeyPatchedWSGIResponse"
 #: Type alias for any response type
-AnyResponseType: TypeAlias = Union[
-    HttpResponseBase, "_MonkeyPatchedWSGIResponse"
-]
+type AnyResponseType = Union[HttpResponseBase, "_MonkeyPatchedWSGIResponse"]
 
 
 class PermissionOverride(abc.ABC, Generic[M]):
@@ -187,7 +185,7 @@ def override_permission(
     pred: str,
     override_cls: type[PermissionOverride[M]],
     **kwargs: Any,
-) -> Generator[None, None, None]:
+) -> Generator[None]:
     """
     Override a permission.
 
@@ -312,7 +310,7 @@ class BaseDjangoTestCase(django.test.SimpleTestCase, BaseTestCase):
                 loader.reset()
 
     @contextlib.contextmanager
-    def ephemeral_savepoint(self) -> Generator[Any, None, None]:
+    def ephemeral_savepoint(self) -> Generator[Any]:
         """
         Run the code inside a savepoint, rolled back at the end.
 
@@ -325,11 +323,22 @@ class BaseDjangoTestCase(django.test.SimpleTestCase, BaseTestCase):
         yield sid
         transaction.savepoint_rollback(sid)
 
+    @contextlib.contextmanager
+    def separate_connection(self, alias: str) -> Generator[None]:
+        """Temporarily open a separate database connection."""
+        connections[alias] = temp_connection = connection.copy(alias)
+        try:
+            yield
+        finally:
+            del connections[alias]
+            temp_connection.close()
+
     def assertResponseHTML(
         self,
         response: TestResponseType,
         status_code: int = status.HTTP_200_OK,
         content_type: str = "text/html; charset=utf-8",
+        dump_on_error: bool = False,
     ) -> lxml.objectify.ObjectifiedElement:
         """
         Check that the response is valid HTML.
@@ -341,7 +350,9 @@ class BaseDjangoTestCase(django.test.SimpleTestCase, BaseTestCase):
         """
         self.assertEqual(response.status_code, status_code)
         self.assertEqual(response.headers["Content-Type"], content_type)
-        return self.assertHTMLValid(response.content)
+        return self.assertHTMLValid(
+            response.content, dump_on_error=dump_on_error
+        )
 
     def assertResponse400(
         self, response: "_MonkeyPatchedWSGIResponse", error: str
@@ -360,10 +371,11 @@ class BaseDjangoTestCase(django.test.SimpleTestCase, BaseTestCase):
         status_description = http.client.responses.get(
             response.status_code, 'unknown'
         )
-        content_type = response["content-type"]
+        content_type = response.get("content-type")
         res = f"{response.status_code} {status_description} ({content_type}): "
 
         if hasattr(response, "content"):
+            assert content_type is not None
             content = response.content.decode(response.charset)
             if content_type == "application/problem+json":
                 # Format a ProblemResponse
@@ -406,6 +418,11 @@ class BaseDjangoTestCase(django.test.SimpleTestCase, BaseTestCase):
         try:
             self.assertEqual(response.status_code, status_code)
 
+            if status_code == status.HTTP_204_NO_CONTENT:
+                assert (
+                    not hasattr(response, "content") or response.content == b""
+                )
+                return None
             content_type = response.get("Content-Type")
             if content_type is None:
                 self.fail("response has no content type")
@@ -753,7 +770,7 @@ class BaseDjangoTestCase(django.test.SimpleTestCase, BaseTestCase):
     @contextlib.contextmanager
     def assert_model_count_unchanged(
         self, model_cls: type[Model]
-    ) -> Generator[None, None, None]:
+    ) -> Generator[None]:
         """Context manager to assert count of rows in a model is unchanged."""
         initial_count = getattr(model_cls, "objects").count()
         try:
@@ -764,7 +781,7 @@ class BaseDjangoTestCase(django.test.SimpleTestCase, BaseTestCase):
             )
 
     @contextlib.contextmanager
-    def template_dir(self) -> Generator[Path, None, None]:
+    def template_dir(self) -> Generator[Path]:
         """Prepend a temporary directory to the template search paths."""
         template_dir = self.create_temporary_directory()
         template_settings = copy.deepcopy(settings.TEMPLATES)
@@ -834,7 +851,7 @@ class PlaygroundTestCase(BaseDjangoTestCase):
     @classmethod
     def list_playground_scenarios(
         cls,
-    ) -> Generator[tuple[str, Scenario], None, None]:
+    ) -> Generator[tuple[str, Scenario]]:
         """
         List the scenarios configured in the test case.
 
@@ -1036,6 +1053,34 @@ class TransactionTestCase(
                 )
                 setattr(self, name, method_scenario)
 
+    def assert_work_requests_locked(
+        self, *, locked: Sequence[WorkRequest], unlocked: Sequence[WorkRequest]
+    ) -> None:
+        """Check that the given work requests are locked/unlocked for update."""
+        with self.separate_connection(alias := "test-locks"):
+            qs: QuerySet[WorkRequest] = WorkRequestQuerySet(
+                WorkRequest, using=alias
+            )
+            for wr in locked:
+                with (
+                    transaction.atomic(using=alias),
+                    self.assertRaisesRegex(
+                        OperationalError, "could not obtain lock"
+                    ),
+                ):
+                    # .exists() is enough here, since the query set contains
+                    # at most one row.
+                    qs.select_for_update(nowait=True, no_key=True).filter(
+                        id=wr.id
+                    ).exists()
+            for wr in unlocked:
+                with transaction.atomic(using=alias):
+                    # .exists() is enough here, since the query set contains
+                    # at most one row.
+                    qs.select_for_update(nowait=True, no_key=True).filter(
+                        id=wr.id
+                    ).exists()
+
 
 class TestCase(
     PlaygroundTestCase,
@@ -1120,19 +1165,17 @@ class ChannelsHelpersMixin(TestMixinBase):
         """Flush the channel layer."""
         super().tearDown()
         channel_layer = get_channel_layer()
+        assert channel_layer is not None
         async_to_sync(channel_layer.flush)()
 
-    # TODO: coverage is confused by something here, possibly
-    # https://github.com/python/cpython/issues/106749
-    async def create_channel(
-        self, group_name: str
-    ) -> dict[str, Any]:  # pragma: no cover
+    async def create_channel(self, group_name: str) -> dict[str, Any]:
         """
         Create a channel and add it to the group named ``group_name``.
 
         Return dict with layer and name.
         """
         channel_layer = get_channel_layer()
+        assert channel_layer is not None
         channel_name = await channel_layer.new_channel()
         await channel_layer.group_add(group_name, channel_name)
 
