@@ -33,7 +33,6 @@ from debusine.client.models import (
     RemoteArtifact,
     StrictBaseModel,
     WorkRequestResponse,
-    WorkflowTemplateResponse,
 )
 from debusine.tasks import BaseTask
 
@@ -176,33 +175,6 @@ def create_work_request_response(
     return WorkRequestResponse(**defaults)
 
 
-def create_workflow_template_response(
-    base_url: str = "https://example.com/",
-    scope: str = "debusine",
-    **kwargs: Any,
-) -> WorkflowTemplateResponse:
-    """Return a WorkflowTemplateResponse. Use defaults for certain fields."""
-    defaults: dict[str, Any] = {
-        "id": 11,
-        "name": "noop",
-        "task_name": "noop",
-        "task_data": {},
-        "priority": 0,
-        "workspace": "Testing",
-    }
-    defaults.update(kwargs)
-    defaults.setdefault(
-        "url",
-        urljoin(
-            base_url,
-            f"{quote(scope)}/{quote(defaults['workspace'])}/"
-            f"workflow-template/{quote(defaults['name'])}/",
-        ),
-    )
-
-    return WorkflowTemplateResponse(**defaults)
-
-
 def create_listing_response(
     *args: StrictBaseModel, next_url: str | None = None
 ) -> PaginatedResponse:
@@ -242,7 +214,7 @@ def create_system_tarball_data(
 
 
 @contextlib.contextmanager
-def preserve_task_registry() -> Generator[None, None, None]:
+def preserve_task_registry() -> Generator[None]:
     """Make task registry changes ephemeral."""
     orig = BaseTask._sub_tasks
     BaseTask._sub_tasks = copy.deepcopy(BaseTask._sub_tasks)
@@ -253,7 +225,7 @@ def preserve_task_registry() -> Generator[None, None, None]:
 
 
 @contextlib.contextmanager
-def print_sql_and_stack() -> Generator[None, None, None]:
+def print_sql_and_stack() -> Generator[None]:
     """Print each SQL query executed, with python stack trace."""
     from django.db import connection
 
