@@ -10,8 +10,8 @@
 """Tests for the test utility functions."""
 
 import contextlib
+import datetime as dt
 import io
-from datetime import datetime, timedelta, timezone
 from unittest import TestCase
 
 from django.test import TestCase as DjangoTestCase
@@ -57,26 +57,36 @@ class UtilsTest(TestCase):
         yesterday_date = yesterday()
 
         # time zone aware
-        self.assertEqual(yesterday_date.tzinfo, timezone.utc)
+        self.assertEqual(yesterday_date.tzinfo, dt.timezone.utc)
 
-        delta = yesterday_date - datetime.now(timezone.utc) - timedelta(days=1)
+        delta = (
+            yesterday_date
+            - dt.datetime.now(dt.timezone.utc)
+            - dt.timedelta(days=1)
+        )
 
-        self.assertLess(delta, timedelta(seconds=1))
+        self.assertLess(delta, dt.timedelta(seconds=1))
 
     def test_tomorrow(self) -> None:
         """Test utils.tomorrow: return a time older than 24 from now in UTC."""
         tomorrow_date = tomorrow()
 
         # time zone aware
-        self.assertEqual(tomorrow_date.tzinfo, timezone.utc)
+        self.assertEqual(tomorrow_date.tzinfo, dt.timezone.utc)
 
-        delta = datetime.now(timezone.utc) + timedelta(days=1) - tomorrow_date
+        delta = (
+            dt.datetime.now(dt.timezone.utc)
+            + dt.timedelta(days=1)
+            - tomorrow_date
+        )
 
-        self.assertLessEqual(delta, timedelta(seconds=1))
+        self.assertLessEqual(delta, dt.timedelta(seconds=1))
 
     def test_date_time_to_isoformat_rest_framework_utc(self) -> None:
         """Test method when timezone is UTC."""
-        value = datetime(2023, 4, 21, 13, 14, 43, 999949, tzinfo=timezone.utc)
+        value = dt.datetime(
+            2023, 4, 21, 13, 14, 43, 999949, tzinfo=dt.timezone.utc
+        )
         self.assertEqual(
             date_time_to_isoformat_rest_framework(value),
             "2023-04-21T13:14:43.999949Z",
@@ -86,8 +96,15 @@ class UtilsTest(TestCase):
         self,
     ) -> None:
         """Test method when timezone is not UTC."""
-        value = datetime(
-            2023, 4, 21, 13, 14, 43, 999949, tzinfo=timezone(timedelta(hours=1))
+        value = dt.datetime(
+            2023,
+            4,
+            21,
+            13,
+            14,
+            43,
+            999949,
+            tzinfo=dt.timezone(dt.timedelta(hours=1)),
         )
         self.assertEqual(
             date_time_to_isoformat_rest_framework(value),
