@@ -30,7 +30,9 @@ class YamlEditorTests(TestCase):
     ) -> YamlEditor[dict[str, Any]]:
         if value is None:
             value = {}
-        return YamlEditor(value, help_text=help_text)
+        editor = YamlEditor(value, help_text=help_text)
+        self.addCleanup(editor.cleanup)
+        return editor
 
     def test_init(self) -> None:
         editor = self._editor()
@@ -236,9 +238,22 @@ class YamlEditorTests(TestCase):
             mock.patch(
                 "debusine.utils.input.YamlEditor.confirm", return_value=True
             ),
+            mock.patch(
+                "debusine.utils.input.YamlEditor.message",
+                autospec=True,
+            ) as mock_message,
         ):
             self.assertTrue(editor.edit())
         self.assertEqual(editor.value, {"test": 42})
 
         mock_open_editor.assert_has_calls([mock.call()] * 2)
         self.assertFalse(editor.editor_file.exists())
+
+        mock_message.assert_has_calls(
+            [mock.call(mock.ANY, "Error reading metadata: test")]
+        )
+
+    def test_cleanup(self) -> None:
+        editor = self._editor()
+        editor.cleanup()
+        self.assertFalse(editor.editor_file.exists())
