@@ -9,9 +9,11 @@
 
 """Template tag library to render Debusine UI elements."""
 
+import abc
 import datetime as dt
 import logging
-from typing import Any, TypeVar
+from collections.abc import Callable
+from typing import Any, Never, TypeVar, override
 
 from django import template
 from django.conf import settings
@@ -22,6 +24,7 @@ from django.template.base import FilterExpression, Parser, Token
 from django.template.loader import render_to_string
 from django.utils.html import conditional_escape, format_html
 from django.utils.safestring import SafeString
+from more_itertools import peekable
 
 import debusine.web.views.view_utils
 from debusine.db.context import context
@@ -30,6 +33,7 @@ from debusine.server.scopes import urlconf_scope
 from debusine.web.helps import HELPS
 from debusine.web.icons import Icons
 from debusine.web.views.base import Widget
+from debusine.web.views.places import Place
 from debusine.web.views.ui.base import UI
 from debusine.web.views.ui_shortcuts import UIShortcut
 
@@ -51,15 +55,36 @@ def ui_shortcuts(obj: Model) -> list[UIShortcut]:
     return stored
 
 
-class WidgetNode(Node):
-    """Template node implementing the {% widget %} tag."""
+class DebusineNodeRenderError(Exception):
+    """Signal an error during rendering of a DebusineNode."""
 
-    def __init__(self, value: FilterExpression) -> None:
-        """Store node arguments."""
-        self.value = value
+    def __init__(
+        self,
+        *,
+        devel_message: str,
+        user_message: str | None = None,
+        exception: Exception | None = None,
+    ) -> None:
+        """
+        Store exception arguments.
+
+        :param devel_message: message targeted at developers, shown in testing
+          and logs but not to the user, to prevent leaking sensitive information
+        :param user_message: message shown to the user. If not provided a
+          default message is used.
+        :param exception: exception (if any) that caused the error
+        """
+        self.devel_message = devel_message
+        self.user_message = user_message or "template rendering error"
+        self.exception = exception
+
+
+class DebusineNode(Node, abc.ABC):
+    """Django Node element with added helper functions."""
 
     def _render_error(
         self,
+        *,
         user_message: str,
         devel_message: str,
         exception: Exception | None = None,
@@ -81,41 +106,92 @@ class WidgetNode(Node):
         # information while providing enough information for a potential bug
         # report to locate the stack trace in the logs
         logger.warning(
-            "Widget rendering error: %s", devel_message, exc_info=exception
+            "%s rendering error: %s",
+            self.__class__.__name__,
+            devel_message,
+            exc_info=exception,
         )
         return format_html(
-            "<span data-role='debusine-widget-error'"
+            "<span data-role='debusine-template-error'"
             " class='bg-danger text-white'>{ts} UTC: {message}</span>",
             ts=dt.datetime.now(dt.timezone.utc).isoformat(),
             message=user_message,
         )
 
-    def render(self, context: Context) -> str | SafeString:
-        """Render the template node."""
-        # Resolve the argument token to a value
+    def resolve_filter(
+        self,
+        context: Context,
+        value: FilterExpression,
+        value_name: str,
+        fail_if_none: bool = True,
+    ) -> Any:
+        """
+        Resolve a filter expression.
+
+        If ``fail_if_none`` is False, returns None if the filter expression
+        failed. Raises DebusineNodeRenderError on other kind of failures.
+        """
         try:
             # ignore_failures seems counterintuitive to me.
             # If False, then django will *handle* failures, generating a value
             # using `string_if_invalid` configured in the template `Engine`.
             # If True, then django will *not* handle failures, and use None for
             # an undefined variable, which is something that we can look for.
-            value: str | Widget | None = self.value.resolve(
-                context, ignore_failures=True
-            )
+            res = value.resolve(context, ignore_failures=True)
         except Exception as e:
-            return self._render_error(
-                user_message="widget argument malformed",
-                devel_message=f"Invalid widget argument: {self.value!r}",
+            raise DebusineNodeRenderError(
+                user_message="template argument malformed",
+                devel_message=f"Invalid {value_name} argument: {str(value)!r}",
                 exception=e,
             )
 
+        if res is None and fail_if_none:
+            raise DebusineNodeRenderError(
+                user_message="template argument lookup failed",
+                devel_message=f"{value_name} argument {str(value)!r}"
+                " failed to resolve",
+            )
+
+        return res
+
+    @abc.abstractmethod
+    def render_checked(self, context: Context) -> str | SafeString:
+        """
+        Render the template node.
+
+        Exceptions raised in this method are caught and rendered and logged as
+        appropriate.
+        """
+
+    @override
+    def render(self, context: Context) -> str | SafeString:
+        try:
+            return self.render_checked(context)
+        except DebusineNodeRenderError as de:
+            return self._render_error(
+                devel_message=de.devel_message,
+                user_message=de.user_message,
+                exception=de.exception,
+            )
+        except Exception as e:
+            return self._render_error(
+                user_message="template rendering error",
+                devel_message="unhandled exception in render_checked",
+                exception=e,
+            )
+
+
+class WidgetNode(DebusineNode):
+    """Template node implementing the {% widget %} tag."""
+
+    def __init__(self, value: FilterExpression) -> None:
+        """Store node arguments."""
+        self.value = value
+
+    @override
+    def render_checked(self, context: Context) -> str | SafeString:
         # Validate the value as a widget
-        match value:
-            case None:
-                return self._render_error(
-                    user_message="invalid or undefined widget value",
-                    devel_message=f"widget {self.value!r} resolved to None",
-                )
+        match value := self.resolve_filter(context, self.value, "widget"):
             case str():
                 if context.autoescape:
                     return conditional_escape(value)
@@ -125,7 +201,7 @@ class WidgetNode(Node):
                 try:
                     return value.render(context)
                 except Exception as e:
-                    return self._render_error(
+                    raise DebusineNodeRenderError(
                         user_message=(
                             f"{value.__class__.__name__} failed to render"
                         ),
@@ -136,7 +212,7 @@ class WidgetNode(Node):
                         exception=e,
                     )
             case _:
-                return self._render_error(
+                raise DebusineNodeRenderError(
                     user_message="invalid widget type",
                     devel_message=(
                         f"widget {self.value!r} {value!r} has invalid type"
@@ -282,7 +358,7 @@ def _help(name: str) -> str:
     return render_to_string("web/_help.html", HELPS[name]._asdict())
 
 
-class UINode(Node):
+class UINode(DebusineNode):
     """Template node implementing the {% ui %} tag."""
 
     def __init__(self, objvar: FilterExpression, asvar: str) -> None:
@@ -290,74 +366,29 @@ class UINode(Node):
         self.objvar = objvar
         self.asvar = asvar
 
-    def _lookup_error(
-        self,
-        devel_message: str,
-        exception: Exception | None = None,
-    ) -> None:
-        """
-        Render a description for a rendering error.
-
-        Depending on settings, it can either raise an exception or render a
-        description suitable for template output.
-        """
-        if settings.DEBUG or getattr(settings, "TEST_MODE", False):
-            if exception is not None:
-                exception.add_note(devel_message)
-                raise exception
-            else:
-                raise ValueError(devel_message)
-
-        # When running in production, avoid leaking possibly sensitive error
-        # information while providing enough information for a potential bug
-        # report to locate the stack trace in the logs
-        logger.warning(
-            "UI helper lookup failed: %s", devel_message, exc_info=exception
-        )
-
-    def render(self, context: Context) -> SafeString:
+    def render_checked(self, context: Context) -> SafeString:
         """Lookup the helper and store it in the context."""
-        try:
-            # ignore_failures seems counterintuitive to me.
-            # If False, then django will *handle* failures, generating a value
-            # using `string_if_invalid` configured in the template `Engine`.
-            # If True, then django will *not* handle failures, and use None for
-            # an undefined variable, which is something that we can look for.
-            instance: Model | None = self.objvar.resolve(
-                context, ignore_failures=True
-            )
-        except Exception as e:
-            self._lookup_error(
-                f"Invalid ui argument: {self.objvar!r}",
-                exception=e,
-            )
-            context[self.asvar] = None
-            return SafeString()
-
-        match instance:
-            case None:
-                self._lookup_error(
-                    f"ui model instance {self.objvar!r} resolved to None"
-                )
-                context[self.asvar] = None
+        context[self.asvar] = None
+        match instance := self.resolve_filter(context, self.objvar, "resource"):
             case Model():
                 try:
                     context[self.asvar] = UI.for_instance(
                         context["request"], instance
                     )
                 except Exception as e:
-                    self._lookup_error(
-                        f"ui helper lookup for {self.objvar!r} failed",
+                    raise DebusineNodeRenderError(
+                        user_message="template argument lookup failed",
+                        devel_message=f"ui helper lookup for {instance!r}"
+                        " failed",
                         exception=e,
                     )
-                    context[self.asvar] = None
 
             case _:
-                self._lookup_error(
-                    f"ui model instance {self.objvar!r} {instance!r}"
-                    " has invalid type"
+                raise DebusineNodeRenderError(
+                    user_message="template argument lookup failed",
+                    devel_message=f"ui model instance {self.objvar!r}"
+                    f" {instance!r} has invalid type",
                 )
-                context[self.asvar] = None
         return SafeString()
 
 
@@ -384,6 +415,166 @@ def ui(parser: Parser, token: Token) -> UINode:
     return UINode(parser.compile_filter(objvar), asvar)
 
 
+class PlaceNode(DebusineNode):
+    """Template node implementing the {% place %} tag."""
+
+    def __init__(
+        self,
+        objvar: FilterExpression,
+        place_type: str | None,
+        widget_type: str,
+        kwargs: list[tuple[str, FilterExpression]],
+    ) -> None:
+        """Store node arguments."""
+        self.objvar = objvar
+        self.place_type = place_type
+        self.widget_type = widget_type
+        self.kwargs = kwargs
+
+    def _resolve_place(self, context: Context) -> Place:
+        """Resolve self.objvar into a Place."""
+        place_method_name = (
+            "place" if self.place_type is None else f"place_{self.place_type}"
+        )
+
+        instance = self.resolve_filter(context, self.objvar, "object")
+
+        if isinstance(instance, Model):
+            try:
+                instance = UI.for_instance(context["request"], instance)
+            except Exception as e:
+                raise DebusineNodeRenderError(
+                    user_message="ui lookup failed",
+                    devel_message=f"ui helper lookup for {self.objvar!r}"
+                    " failed",
+                    exception=e,
+                )
+
+        if isinstance(instance, UI):
+            try:
+                instance = getattr(instance, place_method_name)
+            except Exception as e:
+                raise DebusineNodeRenderError(
+                    user_message="place lookup failed",
+                    devel_message=f"place method {place_method_name!r}"
+                    f" for {instance!r} failed",
+                    exception=e,
+                )
+
+        if isinstance(instance, Place):
+            return instance
+
+        raise DebusineNodeRenderError(
+            user_message="invalid object type",
+            devel_message=f"object {self.objvar!r} resolved to unsupported"
+            f" object type {instance.__class__.__name__} ({instance!r})",
+        )
+
+    def _resolve_kwargs(self, context: Context) -> dict[str, Any]:
+        """Resolve self.kwargs into a dict of keyword arguments."""
+        kwargs: dict[str, Any] = {}
+        for key, value_lookup in self.kwargs:
+            kwargs[key] = self.resolve_filter(
+                context, value_lookup, "kwarg value"
+            )
+        return kwargs
+
+    @override
+    def render_checked(self, context: Context) -> str:
+        place = self._resolve_place(context)
+
+        # Resolve render method
+        renderer: Callable[..., str] | None = getattr(
+            place, self.widget_type, None
+        )
+        if renderer is None:
+            raise DebusineNodeRenderError(
+                user_message="invalid place render type",
+                devel_message=f"place {place!r} does not have"
+                f" an {self.widget_type} method",
+            )
+
+        # Resolve kwargs
+        kwargs = self._resolve_kwargs(context)
+
+        # Call the render method
+        try:
+            return renderer(**kwargs)
+        except Exception as e:
+            raise DebusineNodeRenderError(
+                user_message="place rendering failed",
+                devel_message=f"place {place!r} {self.widget_type}"
+                " method failed",
+                exception=e,
+            )
+
+
+def _place_template_syntax_error(message: str) -> Never:
+    """Raise TemplateSyntaxError for ``{% place %}`` tags."""
+    raise TemplateSyntaxError(
+        message + ". Usage: {% place {object} [place_type] as_{widget_type}"
+        " [key=value, ...] %}"
+    )
+
+
+@register.tag
+def place(parser: Parser, token: Token) -> PlaceNode:
+    """
+    Render Place elements.
+
+    Usage::
+
+        {% place {Model|UI|Place} [type] as_[widget] [key=value, ...] %}
+
+    The first argument is used to obtain a Place instance:
+
+    * if it's a ``Place`` instance, it is used
+    * if it's a UI helper instance, ``place`` or ``place_{type}`` is invoked
+    * if it's a ``Model`` instance, its UI helper is instantiated and used to
+      get a ``Place`` as in the previous point
+
+    Then the ``as_[widget]`` method is called on the ``Place`` instance to
+    render it. Any trailing ``key=value`` assignments are passed to the
+    ``as_[widget]`` method.
+    """
+    bits = token.split_contents()
+
+    args = peekable(bits)
+    next(args)  # discard "place"
+
+    # Get resource/ui helper/place
+    if not args:
+        _place_template_syntax_error("object not provided")
+    obj: str = next(args)
+
+    # Get optional place type
+    place_type: str | None = None
+    if not args:
+        _place_template_syntax_error("as_[widget] not provided")
+    if not args.peek().startswith("as_"):
+        place_type = next(args)
+
+    # Get widget type
+    if not args:
+        _place_template_syntax_error("as_[widget] not provided")
+    widget_type: str = next(args)
+    if not widget_type.startswith("as_"):
+        _place_template_syntax_error("widget type does not begin with `as_`")
+
+    kwargs: list[tuple[str, FilterExpression]] = []
+    for kwarg in args:
+        if "=" not in kwarg:
+            _place_template_syntax_error("missing = in key=value assignment")
+        key, arg = kwarg.split("=", 1)
+        if not key:
+            _place_template_syntax_error("empty key in key=value assignment")
+        kwargs.append((key, parser.compile_filter(arg)))
+
+    return PlaceNode(
+        parser.compile_filter(obj), place_type, widget_type, kwargs
+    )
+
+
 @register.filter
 def message_toast_color_class(message: messages.storage.base.Message) -> str:
     """Get the bootstrap color class to use for showing Django messages."""
