@@ -14,7 +14,6 @@ from django.utils.safestring import SafeString
 
 from debusine.artifacts.models import TaskTypes
 from debusine.db.models import WorkRequest
-from debusine.db.models.work_requests import workflow_flattened
 
 register = template.Library()
 
@@ -111,9 +110,10 @@ def _first_workrequest_needs_input(workflow: WorkRequest) -> WorkRequest | None:
     assert workflow.task_type == TaskTypes.WORKFLOW
 
     for work_request in sorted(
-        workflow_flattened(workflow),
+        WorkRequest.objects.filter(pk=workflow.pk).workflow_flattened(),
         key=lambda workrequest: workrequest.created_at,
     ):
+        assert isinstance(work_request, WorkRequest)
         if (
             work_request.task_type == TaskTypes.WAIT
             and work_request.status == WorkRequest.Statuses.RUNNING
