@@ -11,11 +11,13 @@
 import datetime as dt
 import textwrap
 from pathlib import Path
-from typing import Any, ClassVar
+from typing import Any, ClassVar, override
+from unittest import mock
 
 from django import forms
 from django.contrib.auth import get_user_model
 from django.core.files.uploadedfile import SimpleUploadedFile
+from django.forms.fields import InvalidJSONInput
 from django.test import RequestFactory
 from django.utils.datastructures import MultiValueDict
 
@@ -51,6 +53,9 @@ from debusine.web.forms import (
     ArtifactForm,
     BaseFormSetBase,
     BootstrapMixin,
+    CollectionCreateForm,
+    CollectionFormBase,
+    CollectionUpdateForm,
     DaysField,
     GroupAddUserForm,
     TaskConfigurationInspectorForm,
@@ -61,6 +66,7 @@ from debusine.web.forms import (
     WorkspaceForm,
     WorkspaceInheritanceForm,
     WorkspaceInheritanceFormSet,
+    YAMLField,
 )
 from debusine.web.views.ui.workspaces import WorkspaceUI
 
@@ -200,6 +206,82 @@ class BootstrapMixinTests(TestCase):
         self.assertEqual(char_field_required.label_suffix, "something *")
 
 
+class YAMLFieldTests(TestCase):
+    """Tests for :py:class:`YAMLField`."""
+
+    def test_init_with_mutable(self) -> None:
+        field = YAMLField(empty_value=3)
+        self.assertEqual(field.empty_value, 3)
+        field = YAMLField(empty_value=dict)
+        self.assertEqual(field.empty_value, {})
+        field = YAMLField(empty_value=list)
+        self.assertEqual(field.empty_value, [])
+
+    def test_to_python(self) -> None:
+        field = YAMLField(empty_value=dict)
+        for value, expected in (
+            (None, {}),
+            ("", {}),
+            ("3", 3),
+            ("test", "test"),
+            ("test: 1", {"test": 1}),
+        ):
+            with self.subTest(value=value):
+                self.assertEqual(field.to_python(value), expected)
+
+    def test_to_python_disabled(self) -> None:
+        field = YAMLField(empty_value=dict, disabled=True)
+        for value in (None, "", "3", "test", "test: 1"):
+            with self.subTest(value=value):
+                self.assertEqual(field.to_python(value), value)
+
+    def test_bound_data(self) -> None:
+        field = YAMLField(empty_value=dict)
+        for value, expected in (
+            (None, None),
+            ("", None),
+            ("3", 3),
+            ("test", "test"),
+            ("test: 1", {"test": 1}),
+            ("{", "{"),
+        ):
+            with self.subTest(value=value):
+                self.assertEqual(field.bound_data(value, 42), expected)
+
+    def test_bound_data_disabled(self) -> None:
+        field = YAMLField(empty_value=dict, disabled=True)
+        for value in (None, "", "3", "test", "test: 1"):
+            with self.subTest(value=value):
+                self.assertEqual(field.bound_data(value, 42), 42)
+
+    def test_prepare_value(self) -> None:
+        field = YAMLField(empty_value=dict)
+        for value, expected in (
+            (None, "{}\n"),
+            ("", "{}\n"),
+            (3, "3\n...\n"),
+            ("test", "test\n...\n"),
+            ({"test": 1}, "test: 1\n"),
+            ("{", "'{'\n"),
+            (InvalidJSONInput("{"), "{"),
+        ):
+            with self.subTest(value=repr(value)):
+                self.assertEqual(field.prepare_value(value), expected)
+
+    def test_has_changed(self) -> None:
+        field = YAMLField(empty_value=dict)
+        for initial, data, expected in (
+            (None, None, True),
+            (None, "", True),
+            ([1, 2, 3], [1, 2, 3], False),
+            ([1, 2, 3], [1, 2, 4], True),
+            ({"one": 1, "two": 2}, {"two": 2, "one": 1}, False),
+            ({"one": 1, "two": 2}, {"two": 3, "one": 1}, True),
+        ):
+            with self.subTest(initial=initial, data=data):
+                self.assertEqual(field.has_changed(initial, data), expected)
+
+
 class TokenFormTests(TestCase):
     """Tests for TokenForm."""
 
@@ -334,7 +416,9 @@ class WorkRequestFormTests(TestCase):
         )
 
         self.assertFalse(form.is_valid())
-        self.assertRegex(str(form.errors["task_data"][0]), "^Invalid YAML")
+        self.assertRegex(
+            str(form.errors["task_data"][0]), "^Enter valid YAML data"
+        )
 
     def test_clean_task_name_must_be_valid(self) -> None:
         """Raise forms.ValidationError: invalid task name."""
@@ -1141,3 +1225,239 @@ class TaskConfigurationInspectorFormTests(TestCase):
             form.fields["context"].widget.choices,
             [unspecified, ('context1', 'context1'), ('context2', 'context2')],
         )
+
+
+class CollectionFormBaseTests(TestCase):
+    """Tests for :py:class:`CollectionFormBase`."""
+
+    scenario = scenarios.DefaultContext(set_current=True)
+    sid: ClassVar[Collection]
+
+    @override
+    @classmethod
+    def setUpTestData(cls) -> None:
+        super().setUpTestData()
+        cls.sid = cls.playground.create_collection(
+            "sid", CollectionCategory.SUITE
+        )
+
+    def make_form_class(self, **meta: Any) -> type[CollectionFormBase]:
+        """Create a concrete subclass of CollectionFormBase."""
+        meta.setdefault(
+            "fields",
+            [
+                "name",
+                "category",
+                "full_history_retention_period",
+                "metadata_only_retention_period",
+                "data",
+            ],
+        )
+        meta.setdefault(
+            "field_classes",
+            {
+                "full_history_retention_period": DaysField,
+                "metadata_only_retention_period": DaysField,
+            },
+        )
+
+        class ConcreteForm(CollectionFormBase):
+            """Concrete version of CollectionFormBase used for testing."""
+
+            Meta = type("Meta", (), {"model": Collection, **meta})
+
+        return ConcreteForm
+
+    def form(self, **meta: Any) -> CollectionFormBase:
+        """Instantiate a test form."""
+        form_class = self.make_form_class(**meta)
+        form = form_class()
+        # Simulate being inside full_clean
+        form.cleaned_data = {}
+        return form
+
+    def test_check_timedelta_units(self) -> None:
+        form = self.form()
+        self.assertRegex(
+            str(form.fields["full_history_retention_period"].help_text),
+            r".+ \(in days\)$",
+        )
+        self.assertRegex(
+            str(form.fields["metadata_only_retention_period"].help_text),
+            r".+ \(in days\)$",
+        )
+        self.assertNotRegex(
+            str(form.fields["name"].help_text), r".+ \(in days\)$"
+        )
+
+    def test_check_timedelta_units_without_fields(self) -> None:
+        form = self.form(
+            fields=["name", "category", "metadata_only_retention_period"]
+        )
+        self.assertNotIn("full_history_retention_period", form.fields)
+        self.assertRegex(
+            str(form.fields["metadata_only_retention_period"].help_text),
+            r".+ \(in days\)$",
+        )
+
+    def test_check_name(self) -> None:
+        form = self.form()
+        form.check_name(
+            self.scenario.workspace, "foo", CollectionCategory.SUITE
+        )
+        self.assertEqual(form.errors, {})
+
+    def test_check_name_avoid_name_conflict(self) -> None:
+        form = self.form()
+        form.check_name(
+            self.scenario.workspace, "sid", CollectionCategory.SUITE
+        )
+        self.assertEqual(
+            form.errors,
+            {
+                "name": [
+                    "A collection called sid@debian:suite"
+                    f" already exists in workspace {self.scenario.workspace}",
+                ]
+            },
+        )
+
+    def test_check_name_avoid_name_conflict_spares_instance(self) -> None:
+        form = self.form()
+        form.check_name(
+            self.scenario.workspace,
+            "sid",
+            CollectionCategory.SUITE,
+            instance=self.sid,
+        )
+        self.assertEqual(form.errors, {})
+
+    def test_check_name_wrong_name_for_singleton(self) -> None:
+        form = self.form()
+        form.check_name(
+            self.scenario.workspace,
+            "wrong",
+            CollectionCategory.PACKAGE_BUILD_LOGS,
+        )
+        self.assertEqual(
+            form.errors,
+            {"name": ['A singleton collection needs to be named "_"']},
+        )
+
+    def test_check_name_wrong_name_for_non_singleton(self) -> None:
+        for name in "_", "_wrong":
+            with self.subTest(name=name):
+                form = self.form()
+                form.check_name(
+                    self.scenario.workspace, name, CollectionCategory.SUITE
+                )
+                self.assertEqual(
+                    form.errors,
+                    {
+                        "name": [
+                            "The name of a non-singleton collection"
+                            ' cannot start with "_"'
+                        ]
+                    },
+                )
+
+
+class CollectionCreateFormTests(TestCase):
+    """Tests for :py:class:`CollectionCreateForm`."""
+
+    scenario = scenarios.DefaultContext(set_current=True)
+
+    sid: ClassVar[Collection]
+    default_fields: ClassVar[dict[str, Any]]
+
+    @override
+    @classmethod
+    def setUpTestData(cls) -> None:
+        super().setUpTestData()
+        cls.sid = cls.playground.create_collection(
+            "sid", CollectionCategory.SUITE
+        )
+        cls.default_fields = {
+            "full_history_retention_period": 0,
+            "metadata_only_retention_period": 0,
+            "data": {},
+        }
+
+    def form(self, **kwargs: Any) -> CollectionCreateForm:
+        """Instantiate the form."""
+        return CollectionCreateForm(data={**self.default_fields, **kwargs})
+
+    def test_clean_without_category(self) -> None:
+        form = self.form(name="foo", category="invalid")
+        with mock.patch.object(form, "check_name") as check_name:
+            self.assertFalse(form.is_valid())
+        check_name.assert_not_called()
+
+    def test_clean_checks_name(self) -> None:
+        form = self.form(name="foo", category=CollectionCategory.SUITE)
+        with mock.patch.object(form, "check_name") as check_name:
+            self.assertTrue(form.is_valid())
+        check_name.assert_called_once_with(
+            self.scenario.workspace, "foo", CollectionCategory.SUITE
+        )
+
+    def test_save(self) -> None:
+        form = self.form(name="foo", category=CollectionCategory.SUITE)
+        self.assertTrue(form.is_valid())
+        instance = form.save()
+        assert isinstance(instance.pk, int)
+        self.assertEqual(instance.workspace, self.scenario.workspace)
+        self.assertEqual(
+            instance.retains_artifacts, Collection.RetainsArtifacts.ALWAYS
+        )
+
+    def test_save_commit_false(self) -> None:
+        form = self.form(name="foo", category=CollectionCategory.SUITE)
+        self.assertTrue(form.is_valid())
+        instance = form.save(commit=False)
+        self.assertIsNone(instance.pk)
+
+
+class CollectionUpdateFormTests(TestCase):
+    """Tests for :py:class:`CollectionUpdateForm`."""
+
+    scenario = scenarios.DefaultContext(set_current=True)
+
+    sid: ClassVar[Collection]
+    experimental: ClassVar[Collection]
+    default_fields: ClassVar[dict[str, Any]]
+
+    @override
+    @classmethod
+    def setUpTestData(cls) -> None:
+        super().setUpTestData()
+        cls.sid = cls.playground.create_collection(
+            "sid", CollectionCategory.SUITE
+        )
+        cls.experimental = cls.playground.create_collection(
+            "experimental", CollectionCategory.SUITE
+        )
+        cls.default_fields = {
+            "full_history_retention_period": 0,
+            "metadata_only_retention_period": 0,
+            "data": {},
+        }
+
+    def form(self, instance: Collection, **kwargs: Any) -> CollectionUpdateForm:
+        """Instantiate the form."""
+        return CollectionUpdateForm(
+            data={**self.default_fields, **kwargs}, instance=instance
+        )
+
+    def test_clean_checks_name(self) -> None:
+        form = self.form(
+            self.experimental, name="foo", category=CollectionCategory.SUITE
+        )
+        with mock.patch.object(form, "check_name") as check_name:
+            self.assertTrue(form.is_valid())
+        check_name.assert_called_once_with(
+            self.scenario.workspace,
+            "foo",
+            CollectionCategory.SUITE,
+            instance=self.experimental,
+        )
