@@ -15,6 +15,7 @@ from enum import StrEnum
 from functools import cached_property
 from typing import Any, assert_never
 
+from django.contrib import messages
 from django.core.exceptions import PermissionDenied
 from django.db.models import F, QuerySet
 from django.db.models.functions import Lower
@@ -50,6 +51,10 @@ from debusine.web.views.places import Place
 from debusine.web.views.view_utils import format_yaml
 
 
+class IncompleteArtifactError(Exception):
+    """We attempted to read an incomplete artifact."""
+
+
 class ArtifactView(WorkspaceView, UIShortcutsMixin, abc.ABC):
     """Base view for UIs showing an artifact or part of it."""
 
@@ -145,10 +150,13 @@ class ArtifactDetailView(ArtifactView, DetailViewBase[Artifact]):
 
         if plugin_view is not None:
             # Add specific specialized plugin information
-            ctx.update(
-                **plugin_view(self.artifact).get_context_data(),
-                **{"specialized_view": True},
-            )
+            try:
+                ctx.update(
+                    **plugin_view(self.artifact).get_context_data(),
+                    **{"specialized_view": True},
+                )
+            except IncompleteArtifactError as e:
+                messages.add_message(self.request, messages.ERROR, str(e))
 
         for file_in_artifact in self.file_list:
             self.add_object_ui_shortcuts(
@@ -491,6 +499,25 @@ class ArtifactPlugin(abc.ABC):
         """Return ArtifactPlugin for artifact_category or None."""
         return cls._artifact_plugins.get(artifact_category)
 
+    @staticmethod
+    def read_file(file_in_artifact: FileInArtifact) -> bytes:
+        """Read contents of the file."""
+        artifact = file_in_artifact.artifact
+        scope = artifact.workspace.scope
+        file = file_in_artifact.file
+
+        if not file_in_artifact.complete:
+            raise IncompleteArtifactError(
+                f"File {file_in_artifact.path!r} has not completed uploading."
+            )
+
+        with (
+            scope.download_file_backend(file)
+            .get_entry(file)
+            .get_temporary_local_path() as path
+        ):
+            return path.read_bytes()
+
     @abc.abstractmethod
     def get_context_data(self) -> dict[str, Any]:
         """Return context data for rendering the artifact view."""
