@@ -11,7 +11,6 @@
 import itertools
 import re
 from enum import Enum
-from functools import cached_property
 from typing import Any, Optional, assert_never
 
 from more_itertools import peekable
@@ -36,14 +35,7 @@ class AutopkgtestViewWorkRequestPlugin(WorkRequestPlugin):
 
     task_type = TaskTypes.WORKER
     task_name = "autopkgtest"
-
-    @cached_property
-    def task(self) -> Autopkgtest:
-        """Return the task to display."""
-        return Autopkgtest(
-            self.work_request.used_task_data,
-            self.work_request.dynamic_task_data,
-        )
+    task: Autopkgtest
 
     @staticmethod
     def _get_fail_on_scenarios(fail_on: dict[str, bool]) -> list[str]:
