@@ -59,7 +59,7 @@ class DebDiffBinaryResult(TypedDict):
     control_diffs_packages_no_changes: list[str]
 
 
-class DebDiffView(ArtifactPlugin):
+class DebDiffViewArtifactPlugin(ArtifactPlugin):
     """View for DebDiff Artifact."""
 
     artifact_category = ArtifactCategory.DEBDIFF
@@ -76,6 +76,9 @@ class DebDiffView(ArtifactPlugin):
 
         summary: list[DebDiffFileSummary] = []
 
+        lines = None
+        last_binary_line = 0
+
         for file in patch:
             if file.is_added_file:
                 operation = OperationEnum.ADDED
@@ -86,19 +89,49 @@ class DebDiffView(ArtifactPlugin):
             else:
                 raise ValueError(f"Unexpected file status in diff: {file.path}")
 
-            # first diff line number of the first hunk of the file
-            #
-            # subtracts 3: skip back over the one-line hunk header (@@) and the
-            # two-line file header (--- and +++)
-            diff_line_number = file[0][0].diff_line_no - 3
+            if file.is_binary_file:
+                # find the line in the debdiff report of this binary change
+                # Unidiff is not giving this information at the moment
+                # See https://github.com/matiasb/python-unidiff/issues/122
+                if lines is None:
+                    # Avoid splitting for each binary file
+                    lines = debdiff.splitlines()
 
-            summary.append(
-                {
-                    "path": file.path,
-                    "diff_line_number": diff_line_number,
-                    "operation": operation,
-                }
-            )
+                # Always true in practice, but the type system doesn't know
+                # that.
+                assert file.patch_info is not None
+
+                diff_line_number = (
+                    lines.index(file.patch_info[0].rstrip(), last_binary_line)
+                    + 1
+                )
+                # keep searching from previous one
+                last_binary_line = diff_line_number
+                summary.append(
+                    {
+                        "path": file.path,
+                        "diff_line_number": diff_line_number,
+                        "operation": operation,
+                    }
+                )
+            else:
+                # Always true in practice, but the type system doesn't know
+                # that.
+                assert file[0][0].diff_line_no is not None
+
+                # first diff line number of the first hunk of the file
+                #
+                # subtracts 3: skip back over the one-line hunk header (@@)
+                # and the two-line file header (--- and +++)
+                diff_line_number = file[0][0].diff_line_no - 3
+
+                summary.append(
+                    {
+                        "path": file.path,
+                        "diff_line_number": diff_line_number,
+                        "operation": operation,
+                    }
+                )
 
         return summary
 
@@ -223,34 +256,8 @@ class DebDiffView(ArtifactPlugin):
 
         return result
 
-    @staticmethod
-    def _read_file(file_in_artifact: FileInArtifact) -> str:
-        """Read debdiff.txt file from debdiff_artifact and return it."""
-        artifact = file_in_artifact.artifact
-        scope = artifact.workspace.scope
-        file = file_in_artifact.file
-
-        with (
-            scope.download_file_backend(file)
-            .get_entry(file)
-            .get_temporary_local_path() as path
-        ):
-            return path.read_text()
-
     def get_context_data(self) -> dict[str, Any]:
         """Return the context."""
-        debdiff_txt_file_in_artifact = self.artifact.fileinartifact_set.get(
-            path=DebDiff.CAPTURE_OUTPUT_FILENAME
-        )
-        debdiff_contents = self._read_file(debdiff_txt_file_in_artifact)
-
-        debdiff_absolute_path = debdiff_txt_file_in_artifact.get_absolute_url()
-
-        related_to = self.artifact.relations.filter(
-            type=ArtifactRelation.Relations.RELATES_TO
-        ).first()
-        assert related_to is not None
-
         slug = "debdiff"
 
         specialized_tab = {
@@ -261,6 +268,33 @@ class DebDiffView(ArtifactPlugin):
             }
         }
 
+        try:
+            debdiff_txt_file_in_artifact = self.artifact.fileinartifact_set.get(
+                path=DebDiff.CAPTURE_OUTPUT_FILENAME
+            )
+        except FileInArtifact.DoesNotExist:
+            return {
+                **specialized_tab,
+                "debdiff_artifact_error": "Artifact does not contain "
+                f"{DebDiff.CAPTURE_OUTPUT_FILENAME} ",
+            }
+
+        debdiff_contents = self.read_file(debdiff_txt_file_in_artifact).decode(
+            errors="replace"
+        )
+        debdiff_absolute_path = debdiff_txt_file_in_artifact.get_absolute_url()
+
+        related_to = self.artifact.relations.filter(
+            type=ArtifactRelation.Relations.RELATES_TO
+        ).first()
+
+        if related_to is None:
+            return {
+                **specialized_tab,
+                "debdiff_artifact_error": "Artifact is not complete "
+                "(missing 'relates-to' relation)",
+            }
+
         if related_to.target.category == ArtifactCategory.SOURCE_PACKAGE:
             # The original and new artifacts are source artifacts
             debdiff_parser_error = None
@@ -286,6 +320,7 @@ class DebDiffView(ArtifactPlugin):
                 "debdiff_differences_reported": debdiff_source_summary
                 is not None
                 and len(debdiff_source_summary) != 0,
+                "debdiff_file_tag": slug,
             }
 
         else:
