@@ -26,7 +26,7 @@ from django.http import FileResponse
 from django.http.response import HttpResponseBase
 from django.shortcuts import redirect
 from django.template import Context
-from django.utils.html import format_html, format_html_join
+from django.utils.html import escape, format_html
 from django.utils.http import parse_header_parameters
 from django.utils.safestring import SafeString
 from django.views.generic.base import View
@@ -151,35 +151,44 @@ class FileUI(NamedTuple):
         return cls(content_type, widget_class, pygments_lexer)
 
 
+def add_line_number(
+    *, line_number: int, suffix: str, line: SafeString
+) -> SafeString:
+    """Return HTML to render the line number and contents."""
+    return format_html(
+        '<a id="L{line_number}{suffix}" href="#L{line_number}{suffix}">'
+        '{line_number}</a>'
+        '<div>{line}</div>',
+        line_number=line_number,
+        suffix=suffix,
+        line=line,
+    )
+
+
 class LinenoHtmlFormatter(HtmlFormatter[Any]):
     """HtmlFormatter that keeps a count of line numbers."""
 
-    def __init__(self, *args: Any, **kwargs: Any) -> None:
+    def __init__(self, *args: Any, suffix: str, **kwargs: Any) -> None:
         """Keep a count of line numbers."""
+        self.suffix = suffix
         super().__init__(*args, **kwargs)
-        self.line_count = 0
 
-    def wrap(self, *args: Any, **kwargs: Any) -> Generator[tuple[int, str]]:
-        """Count rendered lines."""
-        # mypy complains that pygments.formatters.HtmlFormatter.wrap is
-        # untyped, which is true, but we can't fix that here.
-        for val in super().wrap(
-            *args, **kwargs
-        ):  # type: ignore[no-untyped-call]
-            self.line_count += val[0]
-            yield val
-
-    def render_linenos(self, file_tag: str | None = None) -> str:
-        """Render line numbers to go alongside rendered contents."""
-        width = len(str(self.line_count))
-        suffix = f"-{file_tag}" if file_tag else ""
-        return SafeString(
-            "\n".join(
-                f"<a id='L{n}{suffix}' href='#L{n}{suffix}'>"
-                f"{str(n).rjust(width)}</a>"
-                for n in range(1, self.line_count + 1)
-            )
-        )
+    def wrap(  # type: ignore[no-untyped-def]
+        self, source
+    ) -> Generator[tuple[int, str]]:
+        """Format each line with the line number."""
+        lineno = 0
+        for formatted, line in source:
+            # f can be 1 (line of formatted code) or 0 (Pygments wrapping)
+            if formatted:
+                lineno += 1
+                yield 1, add_line_number(
+                    line_number=lineno,
+                    suffix=self.suffix,
+                    line=SafeString(line),
+                )
+            else:
+                yield 0, line
 
 
 class FileWidget(Widget):
@@ -231,6 +240,10 @@ class FileWidget(Widget):
         with file_backend.get_stream(self.file_in_artifact.file) as stream:
             yield stream
 
+    def suffix(self) -> str:
+        """Return suffix for the ids: "-{self.file_tag}" or ""."""
+        return f"-{self.file_tag}" if self.file_tag else ""
+
     def get_context_data(self) -> dict[str, Any]:
         """Return the context for this widget."""
         return {
@@ -250,7 +263,7 @@ class LogFileWidget(FileWidget):
     """Show log files."""
 
     class Section(pydantic.BaseModel):
-        """Define the title and contents of a section."""
+        """Information from a section of the log.."""
 
         title: str
         start_line: int
@@ -258,12 +271,11 @@ class LogFileWidget(FileWidget):
         expanded: bool
 
     class RenderedSection(pydantic.BaseModel):
-        """Rendered section."""
+        """Rendered section: content and other required information."""
 
-        id: str
+        start_line: int
         title: str | None
         expanded: bool
-        linenumbers: SafeString
         content: SafeString
 
     template_name = "web/_file_log.html"
@@ -280,60 +292,22 @@ class LogFileWidget(FileWidget):
 
         self._sections = sections
 
-    def _suffix(self) -> str:
-        """Return suffix for the ids: "-{self.file_tag}" or ""."""
-        return f"-{self.file_tag}" if self.file_tag else ""
+        LogFileWidget.RenderedSection.update_forward_refs()
 
-    def _generate_linenumbers_list(
-        self, number_of_lines: int
-    ) -> list[SafeString]:
-        width = len(str(number_of_lines))
-
-        return [
-            format_html(
-                "<a id='L{n}{suffix}' href='#L{n}{suffix}'>{n_padded}</a>",
-                n=n,
-                suffix=self._suffix(),
-                n_padded=str(n).rjust(width),
+    @staticmethod
+    def _add_line_numbers(
+        lines: list[str], *, start: int, end: int, suffix: str
+    ) -> SafeString:
+        return SafeString(
+            "".join(
+                add_line_number(line_number=n, suffix=suffix, line=escape(line))
+                for n, line in enumerate(lines[start - 1 : end], start=start)
             )
-            for n in range(1, number_of_lines + 1)
-        ]
-
-    def _create_section(
-        self,
-        *,
-        title: str | None = None,
-        start_line: int,
-        end_line: int,
-        expanded: bool,
-        linenumbers: list[SafeString],
-        content: list[str],
-    ) -> RenderedSection:
-        lines = content[start_line - 1 : end_line]
-        return LogFileWidget.RenderedSection(
-            title=title,
-            expanded=expanded,
-            linenumbers=SafeString(
-                "\n".join(linenumbers[start_line - 1 : end_line])
-            ),
-            id=f"L{start_line}{self._suffix()}-section",
-            content=format_html(
-                '<div class="file_highlighted">'
-                '<pre class="m-0 p-0"><span></span>'
-                "{}"
-                '</pre>'
-                '</div>',
-                format_html_join("\n", "{}", ((line,) for line in lines)),
-            ),
         )
 
-    def _render(self, file_contents: str) -> list[RenderedSection]:
+    def _render(self, file_contents: str, suffix: str) -> list[RenderedSection]:
         file_contents_list = file_contents.split("\n")
 
-        linenumbers_list = self._generate_linenumbers_list(
-            len(file_contents_list)
-        )
-
         rendered_sections = []
         line: int = 1
         for section in self._sections:
@@ -341,38 +315,49 @@ class LogFileWidget(FileWidget):
                 # Some lines of the file are not in any Section. Display them
                 # (without title, so are not collapsible)
                 rendered_sections.append(
-                    self._create_section(
+                    LogFileWidget.RenderedSection(
                         start_line=line,
-                        end_line=section.start_line - 1,
+                        title=None,
                         expanded=True,
-                        linenumbers=linenumbers_list,
-                        content=file_contents_list,
+                        content=self._add_line_numbers(
+                            file_contents_list,
+                            start=line,
+                            end=section.start_line - 1,
+                            suffix=suffix,
+                        ),
                     )
                 )
 
             rendered_sections.append(
-                self._create_section(
-                    title=section.title,
+                LogFileWidget.RenderedSection(
                     start_line=section.start_line,
-                    end_line=section.end_line,
+                    title=section.title,
                     expanded=section.expanded,
-                    linenumbers=linenumbers_list,
-                    content=file_contents_list,
+                    content=self._add_line_numbers(
+                        file_contents_list,
+                        start=section.start_line,
+                        end=section.end_line,
+                        suffix=suffix,
+                    ),
                 )
             )
 
             line = section.end_line + 1
 
-        end_line = len(linenumbers_list)
+        end_line = len(file_contents_list)
         if line <= end_line:
             # Add any remaining lines of the file
             rendered_sections.append(
-                self._create_section(
+                LogFileWidget.RenderedSection(
                     start_line=line,
-                    end_line=end_line,
+                    title=None,
                     expanded=True,
-                    linenumbers=linenumbers_list,
-                    content=file_contents_list,
+                    content=self._add_line_numbers(
+                        file_contents_list,
+                        start=line,
+                        end=end_line,
+                        suffix=suffix,
+                    ),
                 )
             )
 
@@ -382,11 +367,7 @@ class LogFileWidget(FileWidget):
         """Add context data for text files."""
         context = super().get_context_data()
 
-        context["file_contents_div_id"] = (
-            f"file-contents-{self.file_tag}"
-            if self.file_tag is not None
-            else "file-contents"
-        )
+        context["file_contents_div_id"] = f"file-contents{self.suffix()}"
 
         if not self.file_in_artifact.complete:
             return context
@@ -400,7 +381,8 @@ class LogFileWidget(FileWidget):
         file_contents = raw_str.replace("\r", "^M")
         file_contents = file_contents.rstrip("\n")
 
-        context["sections"] = self._render(file_contents)
+        context["sections"] = self._render(file_contents, self.suffix())
+        context["suffix"] = self.suffix()
 
         return context
 
@@ -416,12 +398,7 @@ class TextFileWidget(FileWidget):
         """Add context data for text files."""
         context = super().get_context_data()
 
-        if self.file_tag is None:
-            div_id = "file-contents"
-        else:
-            div_id = f"file-contents-{self.file_tag}"
-
-        context["file_contents_div_id"] = div_id
+        context["file_contents_div_id"] = f"file-contents{self.suffix()}"
 
         if self.file_in_artifact.complete:
             # Generate CSS with:
@@ -456,14 +433,12 @@ class TextFileWidget(FileWidget):
             formatter = LinenoHtmlFormatter(
                 cssclass="file_highlighted",
                 linenos=False,
+                suffix=self.suffix(),
             )
 
             with self._open_file() as stream:
                 formatted = pygments.highlight(stream.read(), lexer, formatter)
 
-            context["file_linenumbers"] = SafeString(
-                formatter.render_linenos(self.file_tag)
-            )
             context["file_content"] = SafeString(formatted)
 
         return context
