@@ -13,8 +13,17 @@ from typing import Any, TypeVar
 
 from django.db.models import Model, Q, QuerySet
 
-from debusine.db.models import CollectionItem, WorkRequest, Worker, WorkerPool
+from debusine.db.models import (
+    Artifact,
+    CollectionItem,
+    User,
+    WorkRequest,
+    Worker,
+    WorkerPool,
+    WorkflowTemplate,
+)
 from debusine.db.models.auth import GroupAuditLog, GroupMembership
+from debusine.db.models.work_requests import WorkRequestQuerySet
 from debusine.web.forms import WorkflowFilterForm
 from debusine.web.views.table import (
     Column,
@@ -101,6 +110,7 @@ class WorkerTable(Table[Worker]):
         )
         for pk, name in (
             WorkerPool.objects.filter(worker__in=self.queryset)
+            .order_by("name")
             .distinct()
             .values_list("id", "name")
         ):
@@ -109,6 +119,7 @@ class WorkerTable(Table[Worker]):
             )
 
         # Add options to status filter
+        connected = Worker.objects.connected()
         self.filters["status"].add_option(
             "disabled",
             "Disabled",
@@ -117,21 +128,21 @@ class WorkerTable(Table[Worker]):
         self.filters["status"].add_option(
             "disconnected",
             "Disconnected",
-            Q(token__enabled=True, connected_at__isnull=True),
+            Q(token__enabled=True) & ~Q(pk__in=connected),
         )
         self.filters["status"].add_option(
             "running",
             "Running",
             Q(
                 token__enabled=True,
-                connected_at__isnull=False,
+                pk__in=connected,
                 assigned_work_requests__status=(WorkRequest.Statuses.RUNNING),
             ),
         )
         self.filters["status"].add_option(
             "idle",
             "Idle",
-            Q(token__enabled=True, connected_at__isnull=False)
+            Q(token__enabled=True, pk__in=connected)
             & ~Q(
                 assigned_work_requests__status=(WorkRequest.Statuses.RUNNING),
             ),
@@ -200,14 +211,16 @@ class FilterFailedWorkRequests(FilterToggle):
         """Filter the queryset."""
         if not value:
             return queryset
-        # This filter calls workflow_work_requests_failure
-        # for all the workflows. This might need to be changed
-        # (cached in the DB?)
-        work_requests_ids_relevant = []
-        for work_request in queryset:
-            if getattr(work_request, "workflow_work_requests_failure", 0) > 0:
-                work_requests_ids_relevant.append(work_request.pk)
-        return queryset.filter(id__in=work_requests_ids_relevant)
+        assert isinstance(queryset, WorkRequestQuerySet)
+        return queryset.filter(
+            id__in=(
+                # mypy doesn't understand that workflow_flattened returns a
+                # queryset annotated with ancestor_id.
+                queryset.workflow_flattened()  # type: ignore[misc]
+                .filter(result=WorkRequest.Results.FAILURE)
+                .values("ancestor_id")
+            )
+        )
 
 
 class WorkflowTable(Table[WorkRequest]):
@@ -231,10 +244,8 @@ class WorkflowTable(Table[WorkRequest]):
 
     filter_statuses = FilterStatuses()
     filter_results = FilterField(q_field="result__in")
-    filter_workflow_templates = FilterField(
-        q_field="workflow_data_json__workflow_template_name__in"
-    )
-    filter_started_by = FilterField(q_field="created_by__username__in")
+    filter_workflow_templates = FilterSelectOne("Workflow template")
+    filter_started_by = FilterSelectOne("Started by")
     filter_with_failed_work_requests = FilterFailedWorkRequests(
         "With failed work requests"
     )
@@ -243,6 +254,29 @@ class WorkflowTable(Table[WorkRequest]):
     default_order = "id.desc"
     filter_form_class = WorkflowFilterForm
 
+    def init_filters(self) -> None:
+        """Add filter options based on database contents."""
+        super().init_filters()
+        for wt in (
+            WorkflowTemplate.objects.filter(
+                pk__in=self.queryset.values("workflow_template")
+            )
+            .order_by("name")
+            .distinct()
+        ):
+            self.filters["workflow_templates"].add_option(
+                wt.name, wt.name, query=Q(workflow_template=wt)
+            )
+
+        for user in (
+            User.objects.filter(pk__in=self.queryset.values("created_by"))
+            .order_by("first_name", "last_name")
+            .distinct()
+        ):
+            self.filters["started_by"].add_option(
+                user.username, user.get_full_name(), query=Q(created_by=user)
+            )
+
 
 class CollectionItemTable(Table[CollectionItem]):
     """Table set up for CollectionItem."""
@@ -254,3 +288,17 @@ class CollectionItemTable(Table[CollectionItem]):
 
     template_name = "web/_collection_item-list.html"
     default_order = "category"
+
+
+class ArtifactTable(Table[Artifact]):
+    """Table set up for Artifact."""
+
+    category = StringColumn(
+        "Type",
+        ordering=Ordering(asc=["category", "id"], desc=["-category", "id"]),
+    )
+    name = StringColumn("Name")
+    links = Column("")
+
+    template_name = "web/_artifact-list.html"
+    default_order = "category"
