@@ -10,11 +10,11 @@
 """Tests for the artifact views."""
 
 import abc
+import datetime as dt
 import io
 import json
 import os.path
 import tarfile
-from datetime import timedelta
 from pathlib import Path
 from typing import Any, ClassVar, cast
 
@@ -24,7 +24,7 @@ from django.core.files.uploadedfile import SimpleUploadedFile
 from django.db.models import Max, TextChoices
 from django.db.models.functions import Lower
 from django.http.response import HttpResponseBase
-from django.test import Client
+from django.test import Client, RequestFactory
 from django.urls import reverse
 from django.utils.http import http_date
 from django.utils.timesince import timesince
@@ -50,7 +50,11 @@ from debusine.test.django import (
     override_permission,
 )
 from debusine.web.views import ui_shortcuts
-from debusine.web.views.artifacts import ArtifactDetailView, FileView
+from debusine.web.views.artifacts import (
+    ArtifactDetailView,
+    FileDetailView,
+    FileDetailViewRaw,
+)
 from debusine.web.views.tests.utils import ViewTestMixin, date_format
 
 
@@ -313,11 +317,21 @@ class ArtifactDetailViewTests(ViewTestMixin, PermissionTests):
             "get_context_data",
         )
 
-    def test_get_title(self) -> None:
-        """Test get_title method."""
+    def test_get_place(self) -> None:
         view = ArtifactDetailView()
         view.object = self.artifact
-        self.assertEqual(view.get_title(), "Artifact debusine:test")
+        view.request = RequestFactory().get("/")
+        place = view.get_place()
+        self.assertEqual(place.title, "Artifact debusine:test")
+        self.assertEqual(place.url, self.artifact.get_absolute_url())
+        self.assertEqual(place.icon, "file")
+        self.assertEqual(
+            place.description, "debusine:test artifact debusine:test"
+        )
+        assert place.parent is not None
+        self.assertEqual(
+            place.parent.url, self.artifact.workspace.get_absolute_url()
+        )
 
     def test_invalid_artifact_id(self) -> None:
         """Test viewing an artifact ID that does not exist."""
@@ -411,7 +425,7 @@ class ArtifactDetailViewTests(ViewTestMixin, PermissionTests):
         """Test HTML output with no user and expiration."""
         with context.disable_permission_checks():
             self.artifact.created_by = self.user
-            self.artifact.expiration_delay = timedelta(0)
+            self.artifact.expiration_delay = dt.timedelta(0)
             self.artifact.save()
         response = self._get()
         self.assertEqual(response.status_code, status.HTTP_200_OK)
@@ -720,40 +734,6 @@ class ArtifactDetailViewTests(ViewTestMixin, PermissionTests):
         )
 
 
-class FileViewTests(TestCase):
-    """Test :py:class:`FileView`."""
-
-    contents: ClassVar[dict[str, bytes]]
-    artifact: ClassVar[Artifact]
-    file: ClassVar[FileInArtifact]
-
-    @classmethod
-    def setUpTestData(cls) -> None:
-        """Set up the common test fixture."""
-        super().setUpTestData()
-        cls.contents = {
-            "file.md": b"Line 1\nLine 2\n",
-        }
-        cls.artifact, _ = cls.playground.create_artifact(
-            paths=cls.contents,
-            create_files=True,
-        )
-        cls.file = cls.artifact.fileinartifact_set.get(path="file.md")
-
-    def test_get_title(self) -> None:
-        """Test get_title method."""
-        view = FileView()
-        view.object = self.file
-        self.assertEqual(view.get_title(), self.file.path)
-
-    def test_get_title_incomplete(self) -> None:
-        """Test get_title method with an incomplete file."""
-        self.file.complete = False
-        view = FileView()
-        view.object = self.file
-        self.assertEqual(view.get_title(), self.file.path + " (incomplete)")
-
-
 class FileDetailViewTests(ViewTestMixin, PermissionTests):
     """Test FileDetailView."""
 
@@ -809,6 +789,27 @@ class FileDetailViewTests(ViewTestMixin, PermissionTests):
             "get_context_data",
         )
 
+    def test_get_place(self) -> None:
+        view = FileDetailView()
+        view.object = self.file
+        view.request = RequestFactory().get("/")
+        place = view.get_place()
+        self.assertEqual(place.title, self.file.path)
+        self.assertEqual(place.url, self.file.get_absolute_url())
+        self.assertIsNone(place.icon)
+        self.assertIsNone(place.description)
+
+    def test_get_place_incomplete_file(self) -> None:
+        self.file.complete = False
+        view = FileDetailView()
+        view.object = self.file
+        view.request = RequestFactory().get("/")
+        place = view.get_place()
+        self.assertEqual(place.title, self.file.path + " (incomplete)")
+        self.assertEqual(place.url, self.file.get_absolute_url())
+        self.assertIsNone(place.icon)
+        self.assertIsNone(place.description)
+
     def test_invalid_artifact_id(self) -> None:
         """Test viewing an artifact ID that does not exist."""
         artifact_id = Artifact.objects.aggregate(Max("id"))['id__max'] + 1
@@ -862,7 +863,9 @@ class FileDetailViewTests(ViewTestMixin, PermissionTests):
         )
         self.assertIn("file", response.context)
         title = self.assertHasElement(tree, "head/title")
-        self.assertTextContentEqual(title, f"Debusine - {self.file.path}")
+        self.assertTextContentEqual(
+            title, f"debusine - System - debusine:test - {self.file.path}"
+        )
 
     def test_get_incomplete(self) -> None:
         """Test a get of an incomplete file."""
@@ -881,7 +884,9 @@ class FileDetailViewTests(ViewTestMixin, PermissionTests):
         self.assertIn("file", response.context)
         title = self.assertHasElement(tree, "head/title")
         self.assertTextContentEqual(
-            title, f"Debusine - {self.file.path} (incomplete)"
+            title,
+            "debusine - System - debusine:test"
+            f" - {self.file.path} (incomplete)",
         )
 
 
@@ -942,6 +947,27 @@ class FileDetailViewRawTests(ViewTestMixin, PermissionTests):
             "get",
         )
 
+    def test_get_place(self) -> None:
+        view = FileDetailViewRaw()
+        view.object = self.file
+        view.request = RequestFactory().get("/")
+        place = view.get_place()
+        self.assertEqual(place.title, self.file.path)
+        self.assertEqual(place.url, self.file.get_absolute_url_raw())
+        self.assertIsNone(place.icon)
+        self.assertIsNone(place.description)
+
+    def test_get_place_incomplete_file(self) -> None:
+        self.file.complete = False
+        view = FileDetailViewRaw()
+        view.object = self.file
+        view.request = RequestFactory().get("/")
+        place = view.get_place()
+        self.assertEqual(place.title, self.file.path + " (incomplete)")
+        self.assertEqual(place.url, self.file.get_absolute_url_raw())
+        self.assertIsNone(place.icon)
+        self.assertIsNone(place.description)
+
     def test_invalid_file_in_artifact_path(self) -> None:
         """Test viewing a file_path that does not exist."""
         response = self.client.get(
