@@ -10,10 +10,11 @@
 """Tests for the collections views."""
 
 import datetime as dt
-from typing import ClassVar
+from typing import ClassVar, override
 from unittest import mock
 
 from django.db.models import Max
+from django.template.response import SimpleTemplateResponse
 from django.test import RequestFactory
 from django.urls import reverse
 from rest_framework import status
@@ -23,9 +24,9 @@ from debusine.artifacts.models import (
     BareDataCategory,
     CollectionCategory,
 )
-from debusine.db.models import Collection, CollectionItem
+from debusine.db.models import Collection, CollectionItem, Workspace
 from debusine.db.playground import scenarios
-from debusine.test.django import TestCase
+from debusine.test.django import AllowAll, TestCase, override_permission
 from debusine.web.templatetags.debusine import (
     ui_shortcuts as template_ui_shortcuts,
 )
@@ -44,7 +45,7 @@ class CollectionViewsTestsBase(ViewTestMixin, TestCase):
 
     scenario = scenarios.DefaultContext(set_current=True)
     default_collections: ClassVar[list[Collection]]
-    bookworm: ClassVar[Collection]
+    sid: ClassVar[Collection]
     trixie: ClassVar[Collection]
     environments: ClassVar[Collection]
 
@@ -53,8 +54,8 @@ class CollectionViewsTestsBase(ViewTestMixin, TestCase):
         """Set up a database layout for views."""
         super().setUpTestData()
         cls.default_collections = list(cls.scenario.workspace.collections.all())
-        cls.bookworm = cls.playground.create_collection(
-            "bookworm",
+        cls.sid = cls.playground.create_collection(
+            "sid",
             CollectionCategory.SUITE,
         )
         cls.trixie = cls.playground.create_collection(
@@ -117,7 +118,7 @@ class CollectionListViewTests(CollectionViewsTestsBase):
         self.assertQuerySetEqual(
             queryset,
             self.default_collections
-            + [self.bookworm, self.trixie, self.environments],
+            + [self.sid, self.trixie, self.environments],
             ordered=False,
         )
 
@@ -184,7 +185,7 @@ class CollectionCategoryListViewTests(CollectionViewsTestsBase):
         queryset = response.context["collection_list"]
         self.assertQuerySetEqual(
             queryset,
-            [self.bookworm, self.trixie],
+            [self.sid, self.trixie],
             ordered=False,
         )
 
@@ -317,7 +318,7 @@ class CollectionDetailViewTests(CollectionViewsTestsBase):
                 kwargs={
                     "wname": self.scenario.workspace.name,
                     "ccat": self.trixie.category,
-                    "cname": "sid",
+                    "cname": "bookworm",
                 },
             )
         )
@@ -395,6 +396,181 @@ class CollectionDetailViewTests(CollectionViewsTestsBase):
         )
 
 
+class CollectionCreateTests(CollectionViewsTestsBase):
+    """Tests for :py:class:`CollectionCreate` view."""
+
+    scenario = scenarios.DefaultContext()
+
+    @override
+    def setUp(self) -> None:
+        super().setUp()
+        self.url = reverse(
+            "workspaces:collections:create",
+            kwargs={"wname": self.scenario.workspace.name},
+        )
+
+    def test_permissions(self) -> None:
+        """Test permissions."""
+        self.assertSetsCurrentWorkspace(self.scenario.workspace, self.url)
+        self.assertEnforcesPermission(
+            self.scenario.workspace.can_create_collections,
+            self.url,
+            "get_context_data",
+        )
+
+    @override_permission(Workspace, "can_create_collections", AllowAll)
+    def test_configure(self) -> None:
+        response = self.client.post(
+            self.url,
+            {
+                "name": "test",
+                "category": CollectionCategory.SUITE,
+                "full_history_retention_period": 2,
+                "metadata_only_retention_period": 3,
+                "data": "created: true",
+            },
+        )
+        coll = Collection.objects.get(
+            workspace=self.scenario.workspace,
+            name="test",
+            category=CollectionCategory.SUITE,
+        )
+        self.assertRedirects(response, coll.get_absolute_url())
+
+        self.assertEqual(coll.name, "test")
+        self.assertEqual(
+            coll.retains_artifacts, Collection.RetainsArtifacts.ALWAYS
+        )
+        self.assertEqual(
+            coll.full_history_retention_period, dt.timedelta(days=2)
+        )
+        self.assertEqual(
+            coll.metadata_only_retention_period, dt.timedelta(days=3)
+        )
+        self.assertEqual(coll.data, {"created": True})
+
+    @override_permission(Workspace, "can_create_collections", AllowAll)
+    def test_get(self) -> None:
+        response = self.client.get(self.url)
+        self.assertResponseHTML(response)
+        assert isinstance(response, SimpleTemplateResponse)
+
+
+class CollectionUpdateTests(CollectionViewsTestsBase):
+    """Tests for :py:class:`CollectionUpdate` view."""
+
+    scenario = scenarios.DefaultContext()
+
+    def url(self, collection: Collection) -> str:
+        """Return the update URL for a collection."""
+        return reverse(
+            "workspaces:collections:update",
+            kwargs={
+                "wname": collection.workspace.name,
+                "cname": collection.name,
+                "ccat": collection.category,
+            },
+        )
+
+    def test_permissions(self) -> None:
+        """Test permissions."""
+        self.assertSetsCurrentWorkspace(
+            self.scenario.workspace, self.url(self.sid)
+        )
+        self.assertEnforcesPermission(
+            self.sid.can_configure, self.url(self.sid), "get_context_data"
+        )
+
+    @override_permission(Collection, "can_configure", AllowAll)
+    def test_configure(self) -> None:
+        response = self.client.post(
+            self.url(self.sid),
+            {
+                "name": "test",
+                "full_history_retention_period": 2,
+                "metadata_only_retention_period": 3,
+                "data": "changed: true",
+            },
+        )
+        self.sid.refresh_from_db()
+        self.assertRedirects(response, self.sid.get_absolute_url())
+
+        self.assertEqual(self.sid.name, "test")
+        self.assertEqual(
+            self.sid.full_history_retention_period, dt.timedelta(days=2)
+        )
+        self.assertEqual(
+            self.sid.metadata_only_retention_period, dt.timedelta(days=3)
+        )
+        self.assertEqual(self.sid.data, {"changed": True})
+
+    @override_permission(Collection, "can_configure", AllowAll)
+    def test_initial_form(self) -> None:
+        response = self.client.get(self.url(self.sid))
+        self.assertResponseHTML(response)
+        assert isinstance(response, SimpleTemplateResponse)
+        assert response.context_data is not None
+        form = response.context_data["form"]
+        self.assertIsNone(form["full_history_retention_period"].initial)
+        self.assertEqual(form["name"].initial, "sid")
+
+
+class CollectionDeleteTests(CollectionViewsTestsBase):
+    """Tests for :py:class:`CollectionDelete` view."""
+
+    scenario = scenarios.DefaultContext()
+
+    def url(self, collection: Collection) -> str:
+        """Return the delete URL for a collection."""
+        return reverse(
+            "workspaces:collections:delete",
+            kwargs={
+                "wname": collection.workspace.name,
+                "cname": collection.name,
+                "ccat": collection.category,
+            },
+        )
+
+    def test_permissions(self) -> None:
+        """Test permissions."""
+        self.assertSetsCurrentWorkspace(
+            self.scenario.workspace, self.url(self.sid)
+        )
+        self.assertEnforcesPermission(
+            self.sid.can_delete, self.url(self.sid), "get_context_data"
+        )
+
+    @override_permission(Collection, "can_delete", AllowAll)
+    def test_configure(self) -> None:
+        with mock.patch(
+            "debusine.db.models.Collection.delete_collection", autospec=True
+        ) as delete_collection:
+            response = self.client.post(self.url(self.sid), {})
+        delete_collection.assert_called_once_with(self.sid)
+        self.assertRedirects(
+            response, self.scenario.workspace.get_absolute_url()
+        )
+
+    @override_permission(Collection, "can_delete", AllowAll)
+    def test_form_empty_collection(self) -> None:
+        response = self.client.get(self.url(self.sid))
+        tree = self.assertResponseHTML(response)
+        self.assertHasElement(tree, "//p[@id='collection-is-empty']")
+
+    @override_permission(Collection, "can_delete", AllowAll)
+    def test_form_collection_with_items(self) -> None:
+        source = self.playground.create_source_artifact()
+        self.sid.manager.add_artifact(
+            source,
+            user=self.scenario.user,
+            variables={"component": "main", "section": "extra"},
+        )
+
+        response = self.client.get(self.url(self.sid))
+        tree = self.assertResponseHTML(response)
+        self.assertHasElement(tree, "//p[@id='collection-contains-items']")
+
+
 class CollectionSearchViewTests(CollectionViewsTestsBase):
     """Tests for :py:class:`CollectionSearchView`."""
 
@@ -473,7 +649,7 @@ class CollectionSearchViewTests(CollectionViewsTestsBase):
                 kwargs={
                     "wname": self.scenario.workspace.name,
                     "ccat": self.trixie.category,
-                    "cname": "sid",
+                    "cname": "bookworm",
                 },
             )
         )
@@ -537,7 +713,7 @@ class CollectionSearchViewTests(CollectionViewsTestsBase):
         data = {
             "work_request_id": 1,
             "vendor": "debian",
-            "codename": "bookworm",
+            "codename": "sid",
             "architecture": "amd64",
             "srcpkg_name": "hello",
             "srcpkg_version": "1.0-1",
