@@ -9,9 +9,8 @@
 
 """Tests for the enroll views."""
 
-import datetime
+import datetime as dt
 import secrets
-from datetime import timezone as tz
 
 from asgiref.sync import async_to_sync
 from channels.layers import get_channel_layer
@@ -151,7 +150,7 @@ class ConfirmViewTests(ViewTestMixin, TestCase):
         ce = ClientEnroll.objects.create(
             nonce=self.payload.nonce, payload=self.payload.dict()
         )
-        ce.created_at = datetime.datetime(2025, 1, 1, tzinfo=tz.utc)
+        ce.created_at = dt.datetime(2025, 1, 1, tzinfo=dt.timezone.utc)
         ce.save()
 
         self.client.force_login(self.scenario.user)
@@ -192,6 +191,7 @@ class ConfirmViewTests(ViewTestMixin, TestCase):
 
         # A confirmation message is sent via channels
         channel_layer = get_channel_layer()
+        assert channel_layer is not None
         channel_name = f"enroll.{self.payload.nonce}"
         msg = async_to_sync(channel_layer.receive)(channel_name)
         confirm_payload = EnrollConfirmPayload.parse_obj(msg)
@@ -228,6 +228,7 @@ class ConfirmViewTests(ViewTestMixin, TestCase):
 
         # A confirmation message is sent via channels
         channel_layer = get_channel_layer()
+        assert channel_layer is not None
         channel_name = f"enroll.{self.payload.nonce}"
         msg = async_to_sync(channel_layer.receive)(channel_name)
         confirm_payload = EnrollConfirmPayload.parse_obj(msg)
