@@ -21,6 +21,7 @@ from django.http.response import HttpResponseBase, HttpResponseRedirectBase
 from django.template import engines
 from django.test import override_settings
 from django.utils.datastructures import CaseInsensitiveMapping
+from django.utils.safestring import SafeString, mark_safe
 from rest_framework import status
 
 from debusine.db.models import File, FileInArtifact
@@ -31,9 +32,11 @@ from debusine.web.views.files import (
     FileDownloadMixin,
     FileUI,
     FileWidget,
+    LinenoHtmlFormatter,
     LogFileWidget,
     MAX_FILE_SIZE,
     PathMixin,
+    add_line_number,
 )
 from debusine.web.views.tests.utils import ViewTestMixin
 
@@ -575,8 +578,6 @@ class FileWidgetTests(ViewTestMixin, TestCase):
             with self.subTest(file=file.path):
                 context = FileWidget.create(file).get_context_data()
 
-                file_linenumbers = context.pop("file_linenumbers")
-                self.assertIn("L1", file_linenumbers)
                 file_content = context.pop("file_content")
                 self.assertIn(expected_content, file_content)
 
@@ -596,7 +597,6 @@ class FileWidgetTests(ViewTestMixin, TestCase):
         file.save()
         context = FileWidget.create(file).get_context_data()
 
-        self.assertNotIn("file_linenumbers", context)
         self.assertNotIn("file_content", context)
 
         self.assertEqual(
@@ -620,6 +620,22 @@ class FileWidgetTests(ViewTestMixin, TestCase):
             },
         )
 
+    def test_text_file_line_numbers(self) -> None:
+        """Test a text file has the line numbers."""
+        file = self.text
+        context = FileWidget.create(file).get_context_data()
+        self.assertHTMLEqual(
+            context["file_content"],
+            textwrap.dedent(
+                """\
+                <div class="file_highlighted">
+                <a id="L1" href="#L1">1</a><div># Title\n</div>
+                <a id="L2" href="#L2">2</a><div>Text\n</div>
+                </div>\n
+                """
+            ),
+        )
+
     def test_context_data_binary(self) -> None:
         """Test context for binary files."""
         file = self.binary
@@ -713,32 +729,23 @@ class LogFileWidgetTests(ViewTestMixin, TestCase):
             context["sections"],
             [
                 {
-                    "content": (
-                        f"{self.before_contents}"
-                        f"first line\n"
-                        f"second line^Mprogress^Mprogress^Mfinished"
-                        f"{self.after_contents}"
-                    ),
                     "expanded": True,
-                    "id": "L1-section",
-                    'linenumbers': (
-                        "<a id='L1' href='#L1'>1</a>\n"
-                        "<a id='L2' href='#L2'>2</a>"
+                    "content": add_line_numbers(
+                        start=1,
+                        lines=[
+                            "first line",
+                            "second line^Mprogress^Mprogress^Mfinished",
+                        ],
                     ),
+                    "start_line": 1,
                     "title": "Start",
                 },
                 {
-                    "content": (
-                        f"{self.before_contents}"
-                        f"third line\nfourth line"
-                        f"{self.after_contents}"
-                    ),
                     "expanded": True,
-                    "id": "L3-section",
-                    "linenumbers": (
-                        "<a id='L3' href='#L3'>3</a>\n"
-                        "<a id='L4' href='#L4'>4</a>"
+                    "content": add_line_numbers(
+                        start=3, lines=["third line", "fourth line"]
                     ),
+                    "start_line": 3,
                     "title": "End",
                 },
             ],
@@ -749,7 +756,13 @@ class LogFileWidgetTests(ViewTestMixin, TestCase):
         self.assertEqual(context["file_contents_div_id"], "file-contents")
         self.assertCountEqual(
             context.keys(),
-            {"file_contents_div_id", "file_in_artifact", "file_ui", "sections"},
+            {
+                "file_contents_div_id",
+                "file_in_artifact",
+                "file_ui",
+                "sections",
+                "suffix",
+            },
         )
 
     def test_get_context_without_sections(self) -> None:
@@ -760,20 +773,17 @@ class LogFileWidgetTests(ViewTestMixin, TestCase):
             context["sections"],
             [
                 {
-                    "content": f"{self.before_contents}"
-                    "first line\n"
-                    "second line^Mprogress^Mprogress^Mfinished\n"
-                    "third line\n"
-                    "fourth line"
-                    f"{self.after_contents}",
-                    "expanded": True,
-                    "id": "L1-section",
-                    "linenumbers": (
-                        "<a id='L1' href='#L1'>1</a>\n"
-                        "<a id='L2' href='#L2'>2</a>\n"
-                        "<a id='L3' href='#L3'>3</a>\n"
-                        "<a id='L4' href='#L4'>4</a>"
+                    "start_line": 1,
+                    "content": add_line_numbers(
+                        start=1,
+                        lines=[
+                            "first line",
+                            "second line^Mprogress^Mprogress^Mfinished",
+                            "third line",
+                            "fourth line",
+                        ],
                     ),
+                    "expanded": True,
                     "title": None,
                 }
             ],
@@ -800,50 +810,49 @@ class LogFileWidgetTests(ViewTestMixin, TestCase):
             context["sections"],
             [
                 {
-                    "content": (
-                        f"{self.before_contents}"
-                        f"first line"
-                        f"{self.after_contents}"
+                    "content": add_line_numbers(
+                        start=1, suffix="-tag", lines=["first line"]
                     ),
                     "expanded": True,
-                    "id": "L1-tag-section",
-                    "linenumbers": "<a id='L1-tag' href='#L1-tag'>1</a>",
+                    "start_line": 1,
                     "title": None,
                 },
                 {
-                    "content": (
-                        f"{self.before_contents}"
-                        f"second line^Mprogress^Mprogress^Mfinished\nthird line"
-                        f"{self.after_contents}"
+                    "content": add_line_numbers(
+                        start=2,
+                        suffix="-tag",
+                        lines=[
+                            "second line^Mprogress^Mprogress^Mfinished",
+                            "third line",
+                        ],
                     ),
                     "expanded": True,
-                    "id": "L2-tag-section",
-                    "linenumbers": (
-                        "<a id='L2-tag' href='#L2-tag'>2</a>\n"
-                        "<a id='L3-tag' href='#L3-tag'>3</a>"
-                    ),
+                    "start_line": 2,
                     "title": "Middle",
                 },
                 {
-                    "content": (
-                        f"{self.before_contents}"
-                        f"fourth line"
-                        f"{self.after_contents}"
+                    "content": add_line_numbers(
+                        start=4, suffix="-tag", lines=["fourth line"]
                     ),
                     "expanded": True,
-                    "id": "L4-tag-section",
-                    "linenumbers": "<a id='L4-tag' href='#L4-tag'>4</a>",
+                    "start_line": 4,
                     "title": None,
                 },
             ],
         )
-
+        self.assertEqual(context["suffix"], "-tag")
         self.assertIsInstance(context["file_ui"], FileUI)
         self.assertEqual(context["file_in_artifact"], self.file)
         self.assertEqual(context["file_contents_div_id"], "file-contents-tag")
         self.assertCountEqual(
             context.keys(),
-            {"file_contents_div_id", "file_in_artifact", "file_ui", "sections"},
+            {
+                "file_contents_div_id",
+                "file_in_artifact",
+                "file_ui",
+                "sections",
+                "suffix",
+            },
         )
 
     def test_contents_displayed_sections(self) -> None:
@@ -861,7 +870,6 @@ class LogFileWidgetTests(ViewTestMixin, TestCase):
             )
         )
 
-        self.assertIn('<div class="single-horizontal-bar">', rendered)
         self.assertIn("first line", rendered)
 
     def test_incomplete(self) -> None:
@@ -877,3 +885,67 @@ class LogFileWidgetTests(ViewTestMixin, TestCase):
                 "file_in_artifact": self.file,
             },
         )
+
+    def test_add_line_numbers(self) -> None:
+        self.assertHTMLEqual(
+            LogFileWidget._add_line_numbers(
+                [
+                    "first line",
+                    "second <b>line</b>",
+                    "third <b>line</b>",
+                    "fourth <b>line</b>",
+                ],
+                start=2,
+                end=3,
+                suffix="-big",
+            ),
+            textwrap.dedent(
+                """
+                <a id="L2-big" href="#L2-big">2</a>
+                <div>second &lt;b&gt;line&lt;/b&gt;</div>
+                <a id="L3-big" href="#L3-big">3</a>
+                <div>third &lt;b&gt;line&lt;/b&gt;</div>"""
+            ),
+        )
+
+
+class LinenoHtmlFormatterTests(TestCase):
+    """Tests for LinenoHtmlFormatter class."""
+
+    def test_wrap(self) -> None:
+        formatter = LinenoHtmlFormatter(suffix="-suffix")
+
+        with mock.patch(
+            "debusine.web.views.files.add_line_number"
+        ) as add_line_numbers:
+            self.assertEqual(
+                list(formatter.wrap([(0, "<pre>")])), [(0, "<pre>")]
+            )
+
+        add_line_numbers.assert_not_called()
+
+        with mock.patch(
+            "debusine.web.views.files.add_line_number", return_value="CHANGED"
+        ) as add_line_numbers:
+            self.assertEqual(
+                list(formatter.wrap([(1, "foo")])), [(1, "CHANGED")]
+            )
+
+        add_line_numbers.assert_called_once_with(
+            line_number=1, suffix="-suffix", line="foo"
+        )
+
+
+def add_line_numbers(
+    *, start: int, lines: list[str], suffix: str = ""
+) -> SafeString:
+    """Format lines."""
+    formatted = [
+        add_line_number(
+            line_number=line_number,
+            suffix=suffix,
+            line=SafeString(line),
+        )
+        for line_number, line in enumerate(lines, start=start)
+    ]
+    return mark_safe("".join(formatted))
