@@ -14,7 +14,7 @@ import json
 import textwrap
 from collections.abc import Iterable
 from itertools import zip_longest
-from typing import Any, ClassVar, Literal
+from typing import Any, ClassVar, Literal, override
 from unittest import mock
 
 import django
@@ -102,9 +102,9 @@ class WorkRequestDetailViewTests(ViewTestMixin, TestCase):
 
     INTERNAL_COLLECTION_ID = "internal-collection"
 
+    @override
     @classmethod
     def setUpTestData(cls) -> None:
-        """Set up common objects."""
         super().setUpTestData()
         started_at = timezone.now()
         duration = 73
@@ -761,6 +761,7 @@ class WorkRequestDetailViewTests(ViewTestMixin, TestCase):
                     display_name=f"Lintian {i + 1}",
                     step=f"lintian{i + 1}",
                 ),
+                mark_pending=True,
             )
             self.make_work_request_lintian(wr)
             grandchildren.append(wr)
@@ -829,7 +830,7 @@ class WorkRequestDetailViewTests(ViewTestMixin, TestCase):
             template, task_data={}, mark_running=True
         )
         work_request = WorkRequest.objects.create_synchronization_point(
-            parent=root, step="test", status=WorkRequest.Statuses.BLOCKED
+            parent=root, step="test"
         )
         work_request.unblock_strategy = WorkRequest.UnblockStrategy.MANUAL
         workflow_data = work_request.workflow_data
@@ -926,7 +927,7 @@ class WorkRequestDetailViewTests(ViewTestMixin, TestCase):
             task_name="externaldebsign",
             task_data=ExternalDebsignData(unsigned=self.source.pk),
             created_by=user,
-            status=WorkRequest.Statuses.RUNNING,
+            mark_running=True,
             workflow_data=WorkRequestWorkflowData(needs_input=True),
         )
 
@@ -971,7 +972,7 @@ class WorkRequestDetailViewTests(ViewTestMixin, TestCase):
         work_request = self.playground.create_wait_task(
             task_name="externaldebsign",
             task_data=ExternalDebsignData(unsigned=self.source.pk),
-            status=WorkRequest.Statuses.RUNNING,
+            mark_running=True,
             workflow_data=WorkRequestWorkflowData(needs_input=True),
         )
         other_user = get_user_model().objects.create_user(
@@ -1400,12 +1401,10 @@ class WorkRequestListViewTests(ViewTestMixin, TestCase):
         """Test get_queryset() with status=XX returns expected WorkRequests."""
         work_requests = {
             "pending": self.playground.create_worker_task(
-                status=WorkRequest.Statuses.PENDING,
-                task_name="noop",
+                task_name="noop", mark_pending=True
             ),
             "running": self.playground.create_worker_task(
-                status=WorkRequest.Statuses.RUNNING,
-                task_name="noop",
+                task_name="noop", mark_running=True
             ),
             "completed": self.playground.create_worker_task(
                 task_name="noop", result=WorkRequest.Results.SUCCESS
@@ -1413,10 +1412,7 @@ class WorkRequestListViewTests(ViewTestMixin, TestCase):
             "aborted": self.playground.create_worker_task(
                 task_name="noop", mark_aborted=True
             ),
-            "blocked": self.playground.create_worker_task(
-                status=WorkRequest.Statuses.BLOCKED,
-                task_name="noop",
-            ),
+            "blocked": self.playground.create_worker_task(task_name="noop"),
         }
 
         for work_request_status, work_request in work_requests.items():
@@ -1441,8 +1437,7 @@ class WorkRequestListViewTests(ViewTestMixin, TestCase):
         It also adds a Message to the page.
         """
         work_request = self.playground.create_worker_task(
-            status=WorkRequest.Statuses.PENDING,
-            task_name="noop",
+            task_name="noop", mark_pending=True
         )
 
         response = self.client.get(
@@ -1474,8 +1469,7 @@ class WorkRequestListViewTests(ViewTestMixin, TestCase):
     def test_get_queryset_filter_by_architecture_wrong_status(self) -> None:
         """Test get_queryset() filters by architecture only for pending."""
         work_request = self.playground.create_worker_task(
-            status=WorkRequest.Statuses.RUNNING,
-            task_name="noop",
+            task_name="noop", mark_running=True
         )
 
         response = self.client.get(
@@ -1509,16 +1503,15 @@ class WorkRequestListViewTests(ViewTestMixin, TestCase):
         """Test filtering by architecture."""
         # Create two amd64 work requests
         work_request_amd64_1 = self.playground.create_worker_task(
-            status=WorkRequest.Statuses.PENDING,
+            mark_pending=True,
             task_name="noop",
-            task_data={"build_architecture": "amd64"},
-            validate=False,
+            task_data=NoopData(build_architecture="amd64"),
         )
         bootstrap_repository = SystemBootstrapRepository(
             mirror="https://deb.debian.org/debian", suite="stable"
         )
         work_request_amd64_2 = self.playground.create_worker_task(
-            status=WorkRequest.Statuses.PENDING,
+            mark_pending=True,
             task_name="mmdebstrap",
             task_data=MmDebstrapData(
                 bootstrap_options=MmDebstrapBootstrapOptions(
@@ -1534,14 +1527,13 @@ class WorkRequestListViewTests(ViewTestMixin, TestCase):
 
         # Create two i386 work requests
         self.playground.create_worker_task(
-            status=WorkRequest.Statuses.PENDING,
+            mark_pending=True,
             task_name="noop",
-            task_data={"build_architecture": "i386"},
-            validate=False,
+            task_data=NoopData(build_architecture="i386"),
         )
 
         self.playground.create_worker_task(
-            status=WorkRequest.Statuses.PENDING,
+            mark_pending=True,
             task_name="mmdebstrap",
             task_data=MmDebstrapData(
                 bootstrap_options=MmDebstrapBootstrapOptions(
@@ -1554,9 +1546,7 @@ class WorkRequestListViewTests(ViewTestMixin, TestCase):
         # Create a work request for which "WorkRequest.get_task()" raise
         # TaskConfigError.
         self.playground.create_worker_task(
-            status=WorkRequest.Statuses.PENDING,
-            task_name="sbuild",
-            validate=False,
+            mark_pending=True, task_name="sbuild", validate=False
         )
 
         response = self.client.get(
@@ -1648,11 +1638,42 @@ class WorkRequestCreateViewTests(ViewTestMixin, TestCase):
             )
         self.assertEqual(response.status_code, status.HTTP_302_FOUND)
 
+        # The work request got created
+        work_request = WorkRequest.objects.latest("id")
+        self.assertIsNotNone(work_request.id)
+        self.assertEqual(work_request.created_by, self.scenario.user)
+        self.assertEqual(work_request.status, WorkRequest.Statuses.PENDING)
+
+        # the browser got redirected to the work_requests:detail
+        self.assertRedirects(response, work_request.get_absolute_url())
+
+    @override_permission(Workspace, "can_create_work_requests", AllowAll)
+    def test_create_work_request_empty_task_data(self) -> None:
+        self.playground.create_debian_environment(codename="bookworm")
+        self.client.force_login(self.scenario.user)
+        workspace = self.scenario.workspace
+        name = "noop"
+        self.assertEqual(WorkRequest.objects.count(), 0)
+
+        response = self.client.post(
+            reverse(
+                "workspaces:work_requests:create",
+                kwargs={"wname": workspace.name},
+            ),
+            {
+                "workspace": workspace.id,
+                "task_name": name,
+                "task_data": "",
+            },
+        )
+        self.assertEqual(response.status_code, status.HTTP_302_FOUND)
+
         # The work request got created
         work_request = WorkRequest.objects.latest("id")
         self.assertIsNotNone(work_request.id)
         # and has the user assigned
         self.assertEqual(work_request.created_by, self.scenario.user)
+        self.assertEqual(work_request.task_data, {})
 
         # the browser got redirected to the work_requests:detail
         self.assertRedirects(response, work_request.get_absolute_url())
@@ -1724,9 +1745,9 @@ class WorkRequestRetryViewTests(ViewTestMixin, TestCase):
 
     work_request: ClassVar[WorkRequest]
 
+    @override
     @classmethod
     def setUpTestData(cls) -> None:
-        """Initialize class data."""
         super().setUpTestData()
         cls.work_request = cls.playground.create_worker_task(
             task_name="noop",
@@ -1811,9 +1832,9 @@ class WorkRequestAbortViewTests(ViewTestMixin, TestCase):
 
     work_request: ClassVar[WorkRequest]
 
+    @override
     @classmethod
     def setUpTestData(cls) -> None:
-        """Initialize class data."""
         super().setUpTestData()
         cls.work_request = cls.playground.create_worker_task(
             task_name="noop", task_data=NoopData()
@@ -1926,18 +1947,16 @@ class WorkRequestUnblockViewTests(ViewTestMixin, TestCase):
     workflow: ClassVar[WorkRequest]
     work_request: ClassVar[WorkRequest]
 
+    @override
     @classmethod
     def setUpTestData(cls) -> None:
-        """Initialize class data."""
         super().setUpTestData()
         template = cls.playground.create_workflow_template("test", "noop")
         cls.workflow = cls.playground.create_workflow(
             template, task_data={}, mark_running=True
         )
         cls.work_request = WorkRequest.objects.create_synchronization_point(
-            parent=cls.workflow,
-            step="test",
-            status=WorkRequest.Statuses.BLOCKED,
+            parent=cls.workflow, step="test"
         )
         cls.work_request.unblock_strategy = WorkRequest.UnblockStrategy.MANUAL
         cls.work_request.save()
@@ -2108,9 +2127,9 @@ class WorkRequestConfirmViewTests(ViewTestMixin, TestCase):
     workflow: ClassVar[WorkRequest]
     work_request: ClassVar[WorkRequest]
 
+    @override
     @classmethod
     def setUpTestData(cls) -> None:
-        """Initialize class data."""
         super().setUpTestData()
         cls.workflow = cls.playground.create_workflow(mark_running=True)
         cls.work_request = cls.playground.create_wait_task(
@@ -2247,6 +2266,14 @@ class WorkRequestConfirmViewTests(ViewTestMixin, TestCase):
 class WorkRequestPluginTests(TestCase):
     """Tests for WorkRequestPlugin."""
 
+    work_request: ClassVar[WorkRequest]
+
+    @override
+    @classmethod
+    def setUpTestData(cls) -> None:
+        super().setUpTestData()
+        cls.work_request = cls.playground.create_worker_task(task_name="noop")
+
     def setUp(self) -> None:
         super().setUp()
 
@@ -2260,40 +2287,56 @@ class WorkRequestPluginTests(TestCase):
 
             task_type = TaskTypes.WORKER
             task_name = "noop"
+            task: Noop
 
-        work_request = self.playground.create_worker_task(task_name="noop")
-        self.plugin = NoopPlugin(work_request)
+        self.plugin_class = NoopPlugin
 
     def tearDown(self) -> None:
         # WorkRequestPlugins as before
         WorkRequestPlugin._work_request_plugins = self._work_request_plugin_orig
         super().tearDown()
 
+    def test_plugin_for(self) -> None:
+        plugin = WorkRequestPlugin.plugin_for(self.work_request)
+        assert plugin is not None
+        self.assertIsInstance(plugin, self.plugin_class)
+        self.assertEqual(plugin.work_request, self.work_request)
+        self.assertIsInstance(plugin.task, Noop)
+
+    def test_plugin_for_not_enabled(self) -> None:
+        with mock.patch.object(
+            self.plugin_class, "is_enabled", return_value=False
+        ):
+            plugin = WorkRequestPlugin.plugin_for(self.work_request)
+        self.assertIsNone(plugin)
+
     def test_get_description_data(self) -> None:
         """Return value from do_get_description_data() (dynamic_data is set)."""
-        self.plugin.task.dynamic_data = BaseDynamicTaskData()
+        plugin = self.plugin_class(self.work_request)
+        plugin.task.dynamic_data = BaseDynamicTaskData()
 
         with mock.patch.object(
-            self.plugin, "do_get_description_data", return_value="X"
+            plugin, "do_get_description_data", return_value="X"
         ) as mocked:
-            self.assertEqual(self.plugin.get_description_data(), "X")
+            self.assertEqual(plugin.get_description_data(), "X")
             mocked.assert_called_once()
 
     def test_get_description_data_dynamic_data_is_none(self) -> None:
         """Return empty dictionary (dynamic data is None)."""
-        self.plugin.task.dynamic_data = None
+        plugin = self.plugin_class(self.work_request)
+        plugin.task.dynamic_data = None
 
-        with mock.patch.object(
-            self.plugin, "do_get_description_data"
-        ) as mocked:
-            result = self.plugin.get_description_data()
+        with mock.patch.object(plugin, "do_get_description_data") as mocked:
+            result = plugin.get_description_data()
             self.assertEqual(result, {})
             mocked.assert_not_called()
 
     def test_do_get_description_data(self) -> None:
         """Default implementation do_get_description_data() return {}."""
-        self.assertEqual(self.plugin.do_get_description_data(), {})
+        plugin = self.plugin_class(self.work_request)
+        self.assertEqual(plugin.do_get_description_data(), {})
 
     def test_task(self) -> None:
         """WorkRequestPlugin.task return the Task."""
-        self.assertIsInstance(self.plugin.task, Noop)
+        plugin = self.plugin_class(self.work_request)
+        self.assertIsInstance(plugin.task, Noop)
