@@ -181,9 +181,7 @@ class WorkRequestDetailView(
         """Return context for this view's tab."""
         context = super().get_context_data(**kwargs)
 
-        plugin_view = WorkRequestPlugin.plugin_for(
-            self.object.task_type, self.object.task_name
-        )
+        plugin_view = WorkRequestPlugin.plugin_for(self.object)
 
         context["description_template"] = (
             "web/_work_request-generic-description.html"
@@ -195,7 +193,7 @@ class WorkRequestDetailView(
         if plugin_view is not None:
             # Add specific specialized plugin information
             context.update(
-                **plugin_view(self.object).get_context_data(),
+                **plugin_view.get_context_data(),
                 **{"specialized_view": True},
             )
 
@@ -372,6 +370,7 @@ class WorkRequestPlugin(abc.ABC):
     model = WorkRequest
     task_type: str
     task_name: str
+    task: BaseTask[Any, Any]
     template_name: str
 
     #: Maps (task_type, task_name) tuples to their associated plugin class.
@@ -386,24 +385,31 @@ class WorkRequestPlugin(abc.ABC):
         """Store the work_request to display."""
         self.work_request = work_request
 
-    @classmethod
-    def plugin_for(
-        cls, task_type: str, task_name: str
-    ) -> type["WorkRequestPlugin"] | None:
-        """Return WorkRequestPlugin for task_name or None."""
-        return cls._work_request_plugins.get((task_type, task_name))
-
-    @functools.cached_property
-    def task(self) -> BaseTask[Any, Any]:
-        """Return task for the work request self.work_request."""
         task_cls = BaseTask.class_from_name(
             TaskTypes(self.task_type), self.task_name
         )
-        return task_cls(
+        self.task = task_cls(
             task_data=self.work_request.used_task_data,
             dynamic_task_data=self.work_request.dynamic_task_data,
         )
 
+    @classmethod
+    def plugin_for(cls, instance: WorkRequest) -> "WorkRequestPlugin | None":
+        """Return WorkRequestPlugin for task_name or None."""
+        plugin_class = cls._work_request_plugins.get(
+            (instance.task_type, instance.task_name)
+        )
+        if plugin_class is None:
+            return None
+        plugin = plugin_class(instance)
+        if not plugin.is_enabled():
+            return None
+        return plugin
+
+    def is_enabled(self) -> bool:
+        """Check if the tab should appear."""
+        return True
+
     def get_description_data(self) -> dict[str, Any]:
         """Return data for the description or {} if dynamic_data is None."""
         if self.task.dynamic_data is None:
@@ -565,6 +571,7 @@ class WorkRequestCreateView(
             form.add_error("task_data", f"Invalid task data: {exc}")
             return self.form_invalid(form)
 
+        self.object.mark_pending()
         return super().form_valid(form)
 
 
