@@ -19,6 +19,7 @@ from debusine.db.models import WorkRequest, Worker
 from debusine.db.models.workers import WorkerQuerySet
 from debusine.tasks.models import WorkerType
 from debusine.web.views.base import BaseUIView, DetailViewBase, ListViewBase
+from debusine.web.views.places import Place
 from debusine.web.views.table import TableMixin
 from debusine.web.views.tables import WorkRequestTable, WorkerTable
 
@@ -64,9 +65,9 @@ class WorkerDetailView(BaseUIView, DetailViewBase[Worker]):
         queryset = self.get_queryset().exclude(worker_type=WorkerType.CELERY)
         return get_object_or_404(queryset, name=self.kwargs["name"])
 
-    def get_title(self) -> str:
-        """Get the title for the page."""
-        return f"Worker {self.object.name}"
+    def get_place(self) -> Place:
+        """Return the Place for this page."""
+        return self.object.ui(self.request).place
 
     def get_context_data(self, **kwargs: Any) -> dict[str, Any]:
         """Add to context data."""
