@@ -176,7 +176,7 @@ class DebusineAioHTTPTestCase(AioHTTPTestCase):
                     "source_artifact": 5,
                 },
                 "environment": "debian/match:codename=bullseye",
-                "host_architecture": "amd64",
+                "build_architecture": "amd64",
                 "build_components": [
                     "any",
                     "all",
